#<profile4xi profile = "12" />
# =================================================================================================
#<program4xi program = "mpu2image4xi.py">
#<abstract4xi abstract = "MPU 2D image management"/>
#<refer4xi reference = "image2d4xi.py"/>
#<verify4xi verification = "10" target = "mpu2image4xi5test.py">
#<state4xi status = "ACK4review; ACK4tagging; ACK4moduletesting; ACK4systemtesting; [Rel 7.21]"/>
# ================================================================================================= 

# -------------------------------------------------------------------------------------------------
# Public API
# -------------------------------------------------------------------------------------------------
#<interface4xi interface = " 
__all__ = [ 'MPU2Image4Xi',                        # ACK 5.31  p76 
            
            'copy2image4mpu',                      # ACK 5.3.3 p88
            'copy2imagetobuffer4mpu',              # ACK 5.3.4 p90
            'copy2buffertoimage4mpu',              # ACK 5.3.4 p92
            'create2image4mpu',                    # ACK 5.31  p76            

            'get2imageinfo4mpu',                   # ACK 5.3.6 p97 nb: modernized so that you can also access only selected elements, if you want           
            'get2supportedimageformats4mpu',       # ACK 5.3.2 p82 nb: a separate function to get format whereas using get2imageinfo4mpu you can access any element by key such as 'FORMAT' 
                              
            'manoeuvre2copyimage4mpu',             # ACK modernized 
            'manoeuvre2copyimagetobuffer4mpu',     # ACK modernized        
            'manoeuvre2copybuffertoimage4mpu',     # ACK modernized
            'manoeuvre2mapimage4mpu',              # ACK modernized                                           
            'manoeuvre2readimage4mpu',             # ACK modernized
            'manoeuvre2writeimage4mpu',            # ACK modernized            
            'map2image4mpu',                       # ACK 5.3.5 p94 
            
            'read2image4mpu',                      # ACK 5.3.3 p84  
            'retain2image4mpu',                    # ACK 5.4.1 p99
            'release2image4mpu',                   # ACK 5.4.1 p99

            'set2imagecallback4mpu',               # ACK modernized
            'set2image4mpu',                       # ACK modernized nb: with memorymap features you can process 2D images as files!                    

            'trace2image4mpu',                     # ACK modernized 
                                    
            'wait2image4mpu',                      # ACK modernized nb: checks Queue object and thereby causes delay doing something useful
            'write2image4mpu'                      # ACK 5.3.3 p84                 
          ] 
#"
#/>


# -------------------------------------------------------------------------------------------------
# Standard Imports                          
# -------------------------------------------------------------------------------------------------
#<standardize4xi standard = "
import os
from platform import processor
import sys
import threading                
import _thread   
from decimal import *
#"
#/>


# -------------------------------------------------------------------------------------------------
# Scope4Xi imports                               
# -------------------------------------------------------------------------------------------------
#<dragonize4xi dragonization = "
import version4xi

# -------------------------------------------------------------------------------------------------
# Buffer module                           
# -------------------------------------------------------------------------------------------------
from mpu2buffer4xi import MPU2Buffer4Xi
from mpu2buffer4xi import create2buffer4mpu
from mpu2buffer4xi import create2subbuffer4mpu
from mpu2buffer4xi import get2bufferinfo4mpu
from mpu2buffer4xi import release2buffer4mpu
from mpu2buffer4xi import retain2buffer4mpu
from mpu2buffer4xi import set2buffer4mpu
from mpu2buffer4xi import trace2buffer4mpu

# -------------------------------------------------------------------------------------------------
# Command queue module                           
# -------------------------------------------------------------------------------------------------
from mpu2commandqueue4xi import MPU2CommandQueue4Xi
from mpu2commandqueue4xi import create2commandqueue4mpu 
from mpu2commandqueue4xi import flush2commandqueue4mpu
from mpu2commandqueue4xi import finish2commandqueue4mpu
from mpu2commandqueue4xi import get2commandqueueinfo4mpu
from mpu2commandqueue4xi import release2commandqueue4mpu
from mpu2commandqueue4xi import retain2commandqueue4mpu
from mpu2commandqueue4xi import set2commandqueue4mpu
from mpu2commandqueue4xi import trace2commandqueue4mpu

# -------------------------------------------------------------------------------------------------
# Context module                           
# -------------------------------------------------------------------------------------------------
from mpu2context4xi import MPU2Context4Xi 
from mpu2context4xi import create2context4mpu
from mpu2context4xi import get2context4mpu
from mpu2context4xi import release2context4mpu
from mpu2context4xi import retain2context4mpu
from mpu2context4xi import set2context4mpu


# -------------------------------------------------------------------------------------------------
# Device module                        
# -------------------------------------------------------------------------------------------------
from mpu2device4xi import MPU4Xi
from mpu2device4xi import init2mpu4xi 
from mpu2device4xi import set2mpu4xi
from mpu2device4xi import get2mpu4xi

# -------------------------------------------------------------------------------------------------
# Event module                           
# -------------------------------------------------------------------------------------------------
from mpu2event4xi import MPU2Event4Xi
from mpu2event4xi import create2event4mpu
from mpu2event4xi import get2eventinfo4mpu
from mpu2event4xi import release2event4mpu
from mpu2event4xi import retain2event4mpu
from mpu2event4xi import set2eventcallback4mpu 
from mpu2event4xi import set2event4mpu
from mpu2event4xi import trace2event4mpu
from mpu2event4xi import wait2event4mpu


# -------------------------------------------------------------------------------------------------
# Platform module                           
# -------------------------------------------------------------------------------------------------
from platformids4xi  import get2platformids4xi
from platforminfo4xi import PlatformInfo4Xi                                                                        
from platforminfo4xi import init2platforminfo4xi
from platforminfo4xi import set2platforminfo4xi
from platforminfo4xi import get2platforminfo4xi
#"
#/>

# -------------------------------------------------------------------------------------------------
# Multiprocessing Imports                          
# -------------------------------------------------------------------------------------------------
#<process4xi process = "
import _multiprocessing
#"
#/>


# -------------------------------------------------------------------------------------------------
# DragonXi imports                              
# -------------------------------------------------------------------------------------------------
#<dragonize4xi dragonization = "
from ace_object4xi import ace_object4xi
from arena4xi import Arena4Xi
from cpu_corecount4xi import cpu_corecount4xi
from dummyprocess4xi import DummyProcess4Xi
from print4xi import print4xi
from pipe4xi import Pipe4Xi
from queue4xi import Queue4Xi  
from time4xi import time4xi
from reason_object4xi import reason_object4xi
from sense_object4xi import sense_object4xi


#"
#/>

# -------------------------------------------------------------------------------------------------
# Global constants                            
# ------------------------------------------------------------------------------------------------
#<globalize4xi globalization = "
global mpu2image4xi                                                     # 2D-IMAGES FOR MPU

global mpu2trace4xi                                                     # TRACE

global mpu2buffer4xi                                                    # BUFFER FOR MPU

global control2readimage4mpu                                            # COMMAND AND CONTROL TO READ  2D IMAGES WITH MPU  
global state2read4mpu                                                   # STATE FOR COMMAND AND CONTROL
global max2readstate4mpu                                                # MAX FOR STATE
global ack2readinfo4mpu                                                 # ACK FOR IMAGE INFO
global message2readget4mpu                                              # QUEUE GET
global message2readput4mpu                                              # QUEUE PUT
global message2readreceive4mpu                                          # PIPE RECEIVE
global message2readsend4mpu                                             # PIPE SEND

global control2writeimage4mpu                                           # COMMAND AND CONTROL TO WRITE 2D IMAGES WITH MPU 
global state2write4mpu                                                  # STATE FOR COMMAND AND CONTROL
global max2writestate4mpu                                               # MAX FOR STATE
global ack2writeinfo4mpu                                                # ACK FOR IMAGE INFO
global message2writeget4mpu                                             # QUEUE GET
global message2writeput4mpu                                             # QUEUE PUT
global message2writereceive4mpu                                         # PIPE RECEIVE
global message2writesend4mpu                                            # PIPE SEND

global control2copyimage4mpu                                            # COMMAND AND CONTROL TO COPY 2D IMAGES WITH MPU 
global state2copyimage4mpu                                              # STATE FOR COMMAND AND CONTROL
global max2copyimagestate4mpu                                           # MAX FOR STATE
global ack2copyimageinfo4mpu                                            # ACK FOR IMAGE INFO
global message2copyimageget4mpu                                         # QUEUE GET
global message2copyimageput4mpu                                         # QUEUE PUT
global message2copyimagereceive4mpu                                     # PIPE RECEIVE
global message2copyimagesend4mpu                                        # PIPE SEND

global control2copyimagetobuffer4mpu                                    # COMMAND AND CONTROL TO COPY 2D IMAGES TO BUFFER WITH MPU 
global state2copyimagetobuffer4mpu                                      # STATE FOR COMMAND AND CONTROL
global max2copyimagetobufferstate4mpu                                   # MAX FOR STATE
global ack2copyimagetobufferinfo4mpu                                    # ACK FOR IMAGE INFO
global message2copyimagetobufferget4mpu                                 # QUEUE GET
global message2copyimagetobufferput4mpu                                 # QUEUE PUT
global message2copyimagetobufferreceive4mpu                             # PIPE RECEIVE
global message2copyimagetobuffersend4mpu                                # PIPE SEND

global control2copybuffertoimage4mpu                                    # COMMAND AND CONTROL TO COPY BUFFER TO 2D IMAGE WITH MPU 
global state2copybuffertoimage4mpu                                      # STATE FOR COMMAND AND CONTROL
global max2copybuffertoimagestate4mpu                                   # MAX FOR STATE
global ack2copybuffertoimageinfo4mpu                                    # ACK FOR IMAGE INFO
global message2copybuffertoimageget4mpu                                 # QUEUE GET
global message2copybuffertoimageput4mpu                                 # QUEUE PUT
global message2copybuffertoimagereceive4mpu                             # PIPE RECEIVE
global message2copybuffertoimagesend4mpu                                # PIPE SEND                      

global control2mapimage4mpu                                            # COMMAND AND CONTROL TO READ  2D IMAGES WITH MPU  
global state2map4mpu                                                   # STATE FOR COMMAND AND CONTROL
global max2mapstate4mpu                                                # MAX FOR STATE
global ack2mapinfo4mpu                                                 # ACK FOR IMAGE INFO
global message2mapget4mpu                                              # QUEUE GET
global message2mapput4mpu                                              # QUEUE PUT
global message2mapreceive4mpu                                          # PIPE RECEIVE
global message2mapsend4mpu                                             # PIPE SEND
            
#"
#/>

# -------------------------------------------------------------------------------------------------
# Settings
# -------------------------------------------------------------------------------------------------
#<set4xi setting = "

#"
#"/>

# -------------------------------------------------------------------------------------------------
#<function4xi function = "trace2image4mpu" usage = "
#
#   print('=' *33)          
#   print('(Step 15A/25) OF MPU2IMAGE4XI5TEST - TRYING TO TRACE MPU: ', time4xi())      
#   print('=' *33)      
#   trace2mpu = None
#   try:
#       trace2mpu = trace2image4mpu() 
#       print('(Step 15B/25) OF MPU2IMAGE4XI5TEST - TRACE2MPU: ', trace2mpu)  
#       # {'MESSAGE': '{(Step 2Z0/33) failed; trace2image4mpu-failed {}; time: 01:39:15 10/06/11 Pacific Daylight Time}', 'CLASS': 'trace2image4mpu', 'METHOD': 'trace2image4mpu'}
#   except:
#       print('(Step 15Z1/25) OF MPU2IMAGE4XI5TEST *** FAILED TO TRACE MPU: ', trace2mpu)       
#">

# -------------------------------------------------------------------------------------------------   
def trace2image4mpu(task4trace2mpu=None):  

    '''
    creates callback to mpu2image4xi
    '''
    
    global mpu2trace4xi
    
    
    # -----------------------------------------------------------------------------------------
    # (Step 1/33)         
    # -----------------------------------------------------------------------------------------          
    class2print = "trace2image4mpu" 
    method2print = "trace2image4mpu"

    message2print = "{(Step 1A/33) started" + "; trace2image4mpu-started: " + str(trace2image4mpu)  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                              
    # <function trace2image4mpu at 0x0000000003AF0348> 
  
             
    message2print = "{(Step 1B/33) passed" + "; task4trace2mpu-to-trace2image4mpu: " + str(task4trace2mpu) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                                                              # 
    # None 

    code2ack  = ' '
    code2nack = ' '    
    
    mpu2trace4xi = {}                                                                                               # INITIALIZES TO EMPTY DICTIONARY
      
    if task4trace2mpu is None:
        # -----------------------------------------------------------------------------------------
        # NO PARAMETERS GIVEN BY MPUIZED CALLBACK !
        # -----------------------------------------------------------------------------------------    
        task4trace2mpu = {}                                                                                            
            
        task4trace2mpu['CLASS']   = "trace2image4mpu"                                                               # MPUIZE ZPU TRACE!
        task4trace2mpu['METHOD']  = "trace2image4mpu" 
        task4trace2mpu['MESSAGE'] = "{(Step 2Z0/33) failed" + "; trace2image4mpu-failed " + str(mpu2trace4xi)  + "; time: " + time4xi() + "}"               
  
        
        message2print = "{(Step 2A/33) set" + "; task4trace2mpu-set-by-trace2image4mpu to: " + str(task4trace2mpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # {'MESSAGE': '{(Step 2Z0/33) failed; trace2image4mpu-failed {}; time: 08:14:33 09/06/11 Pacific Daylight Time}',
        #  'CLASS': 'trace2image4mpu',
        #  'METHOD': 'trace2image4mpu'}
        #    ; time: 08:14:33 09/06/11 Pacific Daylight Time}
 
    
        code2ack = code2ack + '2A-'            


    # -----------------------------------------------------------------------------------------
    # PARAMETER GIVEN BY MPUIZED CALLBACK !
    # -----------------------------------------------------------------------------------------          
    message2print = "{(Step 3A/33) try" + "; task4trace2mpu-try-by-trace2image4mpu: " + str(task4trace2mpu)  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)    
    #  { trace2image4mpu : trace2image4mpu ; (Step 3A/33) try; task4trace2mpu-try-by-trace2image4mpu: 
    # {'MESSAGE': '{(Step 2Z0/33) failed; trace2image4mpu-failed {}; time: 08:14:33 09/06/11 Pacific Daylight Time}',
    #  'CLASS': 'trace2image4mpu',
    #  'METHOD': 'trace2image4mpu'}
    #    ; time: 08:14:33 09/06/11 Pacific Daylight Time}

    try:
        mpu2trace4xi ['CLASS']  = task4trace2mpu['CLASS']                                                      # MPUIZE MPU TRACE!
        mpu2trace4xi ['METHOD']  = task4trace2mpu['METHOD']
        mpu2trace4xi ['MESSAGE'] = task4trace2mpu['MESSAGE']
        
        message2print = "{(Step 3B/33) set" + "; mpu2trace4xi-set-by-trace2image4mpu to: " + str(mpu2trace4xi)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # { trace2image4mpu : trace2image4mpu ; (Step 3B/33) set; mpu2trace4xi-set-by-trace2image4mpu to: 
        #                     {'MESSAGE': '{(Step 2Z0/33) failed; trace2image4mpu-failed {}; time: 23:41:37 09/06/11 Pacific Daylight Time}',
        #                      'CLASS': 'trace2image4mpu',
        #                      'METHOD': 'trace2image4mpu'}
        #                     ; time: 23:41:37 09/06/11 Pacific Daylight Time}

        code2ack = code2ack + '3B-'         
    except:
        message2print = "{(Step 3Z1/33) failed" + "; mpu-trace-failed-by-trace2image4mpu: " + str(task4trace2mpu) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                    
        code2nack = code2nack + '3Z1-' 
        
    try:
        # =================================
        # SIMULATE MPU TRACE
        # =================================
        class2print   = task4trace2mpu['CLASS']                                                                # MPUIZE MPU TRACE!
        method2print  = task4trace2mpu['METHOD']
        
        message2print = task4trace2mpu['MESSAGE']
        print4xi(class2print, method2print, message2print)                                                     # MPUIZED TRACE FOR OPENCL CALLBACK!
        # { trace2image4mpu : trace2image4mpu ; (Step 2Z0/33) failed; trace2image4mpu-failed {}; time: 23:41:37 09/06/11 Pacific Daylight Time}
        
        code2ack = code2ack + '3C-'         
    except:
        class2print = "trace2image4mpu"                                                                        # SET TRACE BACK TO THIS FUNCTION!
        method2print = "trace2image4mpu"   
        message2print = "{(Step 3Z2/33) failed" + "; simulation-of-mpu-trace-failed-by-trace2image4mpu: " + str(task4trace2mpu) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                    
        code2nack = code2nack + '3Z2-' 
        
    try:
        # =================================
        # NORMAL TRACE
        # =================================                     
        class2print = "trace2image4mpu"                                                                        # SET TRACE BACK TO THIS FUNCTION!
        method2print = "trace2image4mpu"                
        message2print = "{(Step 3D/33) performed" + "; mpu-trace-performed-by-trace2image4mpu: " + str(task4trace2mpu) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # { trace2image4mpu : trace2image4mpu ; (Step 3D/33) performed; mpu-trace-performed-by-trace2image4mpu: 
        # {'MESSAGE': '{(Step 2Z0/33) failed; trace2image4mpu-failed {}; time: 23:41:37 09/06/11 Pacific Daylight Time}', 
        # 'CLASS': 'trace2image4mpu', 'METHOD': 'trace2image4mpu'}; time: 23:41:37 09/06/11 Pacific Daylight Time}

        code2ack = code2ack + '3D-' 
    except:
        message2print = "{(Step 3Z3/33) failed" + "; mpu-trace-failed-by-trace2image4mpu: " + str(task4trace2mpu) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                    
        code2nack = code2nack + '3Z3-' 

    # -----------------------------------------------------------------------------------------
    # (Step 33/3)           
    # -----------------------------------------------------------------------------------------
    message2print = "{(Step 33/33) returning" + "; mpu2trace4xi-returning-by-trace2image4mpu: " + str(mpu2trace4xi)  + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)        
    # {'MESSAGE': '{(Step 2Z0/33) failed; trace2image4mpu-failed {}; time: 23:41:37 09/06/11 Pacific Daylight Time}', 
    #  'CLASS': 'trace2image4mpu', 
    #  'METHOD': 'trace2image4mpu'}; time: 23:41:37 09/06/11 Pacific Daylight Time}

    
    return mpu2trace4xi
            
#</function4xi> 

# -------------------------------------------------------------------------------------------------
#<function4xi function = "set2imagecallback4mpu" usage = "
#
#   print('(Step 14A/700) OF MPU2IMAGE4XI5TEST - TRYING TO SET 2D IMAGE CALLBACK OF MPU DEVICE ', time4xi())  
#   try:                               
#       response4setcallback2image = set2imagecallback4mpu()                                                                                                 
#       print('(Step 14B/700) OF MPU2IMAGE4XI5TEST - SET 2D IMAGE CALLBACK OF MPU DEVICE TO: ', response4setcallback2image)   
#       # <function trace2image4mpu at 0x0000000004E2CAC8>
#   except:
#       print('(Step 14Z1/700) OF MPU2IMAGE4XI5TEST *** FAILED TO SET 2D IMAGE CALLBACK OF MPU DEVICE ', time4xi())  
# 
#">

# ------------------------------------------------------------------------------------------------- 
 
def set2imagecallback4mpu(task4set2imagecallback=None):

    '''
    registers callback function to mpu2image4xi
    '''
    global mpu2image4xi
    
    class2print = "set2imagecallback4mpu" 
    method2print = "set2imagecallback4mpu"
    
    message2print = "{(Step 1A/70) started" + "; set2imagecallback4mpu-started: " + str(set2imagecallback4mpu)  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)
    # <function set2imagecallback4mpu at 0x000000000390B748>
    
    try:
    
        mpu2image4xi ['CALLBACK'] = trace2image4mpu
        
        message2print = "{(Step 1B/70) set callback2mpu" + "; imagecallback4mpu-set-by-set2imagecallback4mpu: " + str(mpu2image4xi ['CALLBACK']) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        # <function trace2image4mpu at 0x0000000003A9E348> 
        
        message2print = "{(Step 1C/70) returning" + "; callback-of-mpu2image4xi-returning-by-set2imagecallback4mpu: " + str(mpu2image4xi ['CALLBACK']) + "; time: " + time4xi() + "}"  
        print4xi(class2print, method2print, message2print) 
        # <function trace2image4mpu at 0x0000000003864A48> 
        
        return mpu2image4xi ['CALLBACK']      
        
    except:
        message2print = "{(Step 1Z1/70) failed" + "; callback2image-failed-by-set2imagecallback4mpu: " + str(callback2image) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)         

        message2print = "{(Step 1Z2/70) returning" + "; trace2image4mpu-returning-by-set2imagecallback4mpu: " + str(trace2image4mpu) + "; time: " + time4xi() + "}"  
        print4xi(class2print, method2print, message2print)      
            
        return trace2image4mpu
            
#</function4xi> 




# -------------------------------------------------------------------------------------------------
#<function4xi function = "create2image4mpu" usage = "
#   #1     
#   print('=' *33)          
#   print('(Step 2A/700) OF MPU2IMAGE4XI5TEST - TRYING TO CREATE 2D IMAGE4MPU ', time4xi())      
#   print('=' *33)        
#   # 
#   try:
#       response4create2Dimage = create2image4mpu()                                                                 
#       print('(Step 2B/700) OF MPU2IMAGE4XI5TEST - CREATED 2D IMAGE4MPU ', time4xi())
#       keys2response = response4create2Dimage.keys()
#       for key2response in sorted(keys2response):
#           print(key2response, ' : ', response4create2Dimage[key2response])
#           # 
#   except:
#       print('(Step 2Z1/700) OF MPU2IMAGE4XI5TEST *** FAILED TO CREATE 2D IMAGE4MPU *** ', time4xi())
#   #2
#   print('(Step 123A/700) OF MPU2IMAGE4XI5TEST - TRYING TO CREATE 2D IMAGE for MPU ', time4xi()) 
#   # 
#   try:
#       xi2mpu = MPU2Image4Xi()                            
#       response4create2Dimage = xi2mpu.create2image()                                  
#       keys2response = response4create2Dimage.keys()
#       print('(Step 122B/700) OF MPU2IMAGE4XI5TEST - CREATED 2D IMAGE ', time4xi())  
#       for key2response in sorted(keys2response):
#           print(key2response, ' : ', response4create2Dimage[key2response])
#           # 
#   except:
#       print('(Step 123Z/700) OF MPU2IMAGE4XI5TEST *** FAILED TO CREATE 2D IMAGE for MPU ', time4xi())    
#">

# -------------------------------------------------------------------------------------------------   
def create2image4mpu(task4imagecreate2mpu=None):

    '''
    creates 2D image for MPU
    '''
    
    global mpu2image4xi                                                                                              
    
    # -----------------------------------------------------------------------------------------
    # (Step 1/70) - CHECK PARAMETERS           
    # -----------------------------------------------------------------------------------------          
    class2print = "create2image4mpu" 
    method2print = "create2image4mpu"

    message2print = "{(Step 1A/70) started" + "; create2image4mpu-started: " + str(create2image4mpu)  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)
    # <function create2image4mpu at 0x00000000038D97C8> 
  
             
    message2print = "{(Step 1B/70) passed" + "; task4imagecreate2mpu-to-create2image4mpu: " + str(task4imagecreate2mpu) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                                                              
    # None 

    code2ack  = ' '
    code2nack = ' '    
    
    mpu2image4xi = {}                                                                                               # INITIALIZES TO EMPTY DICTIONARY
               
    try:             
        if task4imagecreate2mpu is None:
            # -----------------------------------------------------------------------------------------
            # NO PARAMETERS GIVEN BY CALLER FOR CREATION - BUILD TASK TO MPU! 
            # -----------------------------------------------------------------------------------------    
            
            task4imagecreate2mpu = {}                                                                                                                                                                                       
            
            message2print = "{(Step 1C/70) initialized" + "; task4imagecreate2mpu-created-by-create2image4mpu to: " + str(task4imagecreate2mpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)               
            # {}
            
            function2callback4mpu = trace2image4mpu                                                                 # CALLBACK FUNCTION FOR MPU
            
            message2print = "{(Step 1D/70) initialized" + "; function2callback4mpu-created-by-create2image4mpu to: " + str(function2callback4mpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)            
            # <function trace2image4mpu at 0x00000000038D9348>
            
            code2ack  = code2ack + '1D-'
           
            args2callback = {}                                                                                      # CREATE ARGUMENT DICTIONARY FOR CALLBACK FUNCTION             
            args2callback['CLASS']   = "create2image4mpu"                                                          # MODULE OF MPUIZED OPENCL MODULE
            args2callback['METHOD']  = "create2image4mpu"                                                          # FUNCTION OF MPUIZED OPENCL MODULE
            args2callback['MESSAGE'] = "{(Step 1Z1/33) failed" + "; create2image4mpu-failed " + "; time: " + time4xi() + "}" # ERROR MESSAGE FOR MPUIZED OPENCL TO RETURN    

            message2print = "{(Step 1E/70) initialized" + "; args2callback-created-by-create2image4mpu to: " + str(args2callback)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)               
            # {'MESSAGE': '{(Step 1Z1/33) failed; create2image4mpu-failed ; time: 00:54:15 09/06/11 Pacific Daylight Time}',
            #  'CLASS': 'create2image4mpu',
            #  'METHOD': 'create2image4mpu'}; time: 00:54:15 09/06/11 Pacific Daylight Time}

            code2ack  = code2ack + '1E-'
            
            # ---------------------------------------------------------------------------------------------
            # (1) CALLBACKS - EXTRA                      
            # ---------------------------------------------------------------------------------------------              
            task4imagecreate2mpu['DATA2CALLBACK'] = args2callback                                                   # USER DATA PASSED TO MPUIZED OPENCL - SEVERAL ARGS PASSED IN ONE DICTIONARY
            task4imagecreate2mpu['FUNCTION2CALLBACK'] = function2callback4mpu                                       # CALLBACK FUNCTION TO CALLED BY OPENCL IN EXCEPTIONS
            code2ack  = code2ack + '2C-'
            message2print = "{(Step 1F/70) initialized" + "; task4imagecreate2mpu-created-by-create2image4mpu to: " + str(task4imagecreate2mpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            # {'FUNCTION2CALLBACK': <function trace2image4mpu at 0x00000000038D9348>,
            #  'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2image4mpu-failed ; time: 00:54:15 09/06/11 Pacific Daylight Time}',
            #  'CLASS': 'create2image4mpu',
            #  'METHOD': 'create2image4mpu'}}; time: 00:54:15 09/06/11 Pacific Daylight Time}
            
            # ---------------------------------------------------------------------------------------------
            # (2) PLATFORM - EXTRA                      
            # ---------------------------------------------------------------------------------------------             
            platform2xi = {}
            message2print = "{(Step 2A/70) initialized" + "; platform2xi-created-by-create2image4mpu to: " + str(platform2xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)   
            # {}
            
            try:
                platform2xi = get2platformids4xi()                                                                  # PLATFORM IS COMMON TO ALL DEVICES I.E TO DEVICE XI
                message2print = "{(Step 2B/70) got" + "; platform2xi-got-by-create2image4mpu: " + str(platform2xi)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                # {'list2platforms': ['DragonXi'], 'ack2return': ' 2C-2E-2F-2G-', 'nack2return': ' ', 'available2platform': 1, 'max2entries': 1}; time: 00:54:15 09/06/11 Pacific Daylight Time}
                 
                code2ack  = code2ack + '2B-'
            except:
                message2print = "{(Step 2Z/70) failed" + "; platform2xi-failed-by-create2image4mpu: " + str(failed)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                                
                code2nack  = code2nack + '2Z-'
                 
            task4imagecreate2mpu['PLATFORM'] = platform2xi                                                          # PLATFORM OF SYSTEM 
              
            
            # ---------------------------------------------------------------------------------------------
            # (3) CONTEXT - REQUIRED                         
            # ---------------------------------------------------------------------------------------------    
            context2mpu = {}
            message2print = "{(Step 3A/70) trying" + "; context2mpu-trying-by-create2image4mpu: " + str(context2mpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            try:
                context2mpu = get2context4mpu()                                                                     # NO PARAMETERS - THE SYSTEM CREATES CONTEXT IF REQUIRED  
                
                message2print = "{(Step 3B/70) got" + "; context2mpu-got-by-create2image4mpu: " + str(context2mpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # {'COUNT2REFERENCE': 0,
                #  'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2context4mpu-failed ; time: 00:54:15 09/06/11 Pacific Daylight Time}',
                #  'CLASS': 'create2context4mpu',
                #  'METHOD': 'create2context4mpu'},
                #  'FUNCTION2CALLBACK': <function trace2context4mpu at 0x00000000037D0148>,
                #  'ACK2RETURN': ' 2A-2C-2D-2E-2B-3B-4B-5B-6B-7B-7B-',
                #  'INFO2PRIVATE': None,
                #  'PLATFORM': {'list2platforms': ['DragonXi'], 'ack2return': ' 2C-2E-2F-2G-', 'nack2return': ' ', 'available2platform': 1, 'max2entries': 1},
                #  'NACK2RETURN': ' ',
                #  'SIZE2PRIVATE': 0, 'PROPERTIES': None}; time: 00:54:15 09/06/11 Pacific Daylight Time}

                
                code2ack  = code2ack + '3B-'         
            except:
                message2print = "{(Step 3Z/70) failed" + "; context2mpu-failed-by-create2image4mpu: " + str(context2mpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '3Z-'
                
            task4imagecreate2mpu ['CONTEXT'] = context2mpu           
            
            # ---------------------------------------------------------------------------------------------
            # (4) DEVICE - EXTRA                            
            # ---------------------------------------------------------------------------------------------    
            device2mpu = {}
            
            message2print = "{(Step 4A/70) trying" + "; device2mpu-trying-by-create2image4mpu: " + str(device2mpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            try:
                device2mpu = get2mpu4xi()                                                                           # THE SYSTEM CREATES DEVICE INFO, IF NECESSARY    
                
                message2print = "{(Step 4B/70) got" + "; device2mpu-got-by-create2image4mpu: " + str(device2mpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # {'RESOLUTION2TIMER4PROFILING': 318,
                #  'ID2PLATFORM': '0000000001509568',
                #  'PROFILING2QUEUE': 'ACK',
                #  'EXECUTE2KERNELS4OPENCL': 'ACK',
                #  'NATIVE2WIDTHLONG4VECTOR': 2,
                #  'MAX2UNITS4COMPUTE': 4,
                #  'SIZE2MEMORY4GLOBAL': 3221225472,
                #  'NATIVE2WIDTHFLOAT4VECTOR': 4,
                #  'MAX2SIZE4WORKGROUP': 1024,
                #  'PREFERRED2WIDTHCHAR4VECTOR': 16,
                #  'SIZE2CACHE': 65536,
                #  'IEEE754-2008FUSED2MULTIPLY-ADD': 'NACK',
                #  'TYPE4CACHE': 'Read/Write',
                #  'PREFERRED2WIDTHINT4VECTOR': 4,
                #  'SIZE2PREFERREDWORKGROUPMULTIPLE4KERNEL': 1,
                #  'NATIVE2WIDTHINT4VECTOR': 4,
                #  'SIZE2BUFFER4CONSTANT': 65536,
                #  'EXECUTE2FUNCTION4NATIVE': 'ACK',
                #  'PREFERRED2WIDTHDOUBLE4VECTOR': 0,
                #  'MAX2NUMBER4CONSTANTARGS': 8,
                #  'VERSION2DRIVER': '2.0',
                #  'EXTENSIONS': 'cl_amd_fp64 cl_khr_global_int32_base_atomics cl_khr_global_int32_extended_atomics cl_khr_local_int32_base_atomics 
              
                #  'AVAILABLE2COMPILER': 'ACK',
                #  'MAX2DIMENSIONS4WORKITEMS': 3,
                #  'SUPPORT2CORRECTION4ERROR': 0,
                #  'BITS2ADDRESS': 64,
                #  'AVAILABLE': 'ACK',
                #  'SIZE2MEMORY4LOCAL': 32768,
                #  'VENDOR': 'AuthenticAMD',
                #  'SUPPORT2IMAGE': 'NACK',
                #  'MAX2SIZE4KERNELARGUMENT': 4096,
                #  'ACK2RETURN': ' 1C-1D-1E-1F-1D-2A-
                #  'OUT-OF-ORDER2QUEUE': 'NACK',
                #  'ALIGNMENT2ADDRESS4BASE': 1024,
                #  'PROFILE': 'FULL_PROFILE',
                #  'TYPE2MEMORY4LOCAL': 'Global',
                #  'PREFERRED2WIDTHFLOAT4VECTOR': 4,
                #  'PREFERRED2WIDTHLONG4VECTOR': 2,
                #  'MAX2FREQUENCY4CLOCK': 3210,
                #  'PREFERRED2WIDTHSHORT4VECTOR': 8,
                #  'ID': 4033,
                #  'MAX2WORKITEM2': 1024,
                #  'NATIVE2WIDTHDOUBLE4VECTOR': 0,
                #  'ROUND2ZERO': 'ACK',
                #  'NAME': 'AMD64 Family 16 Model 4 Stepping 2, AuthenticAMD',
                #  'MAX2WORKITEM1': 1024,
                #  'MAX2WORKITEM0': 1024,
                #  'SIZE2LINE4CACHE': 64,
                #  'MAX2ALLOCATION4MEMORY': 1073741824,
                #  'NATIVE2WIDTHCHAR4VECTOR': 16,
                #  'QUIET2NANS': 'ACK',
                #  'NATIVE2WIDTHSHORT4VECTOR': 8,
                #  'ROUND2NEAREST4EVEN': 'ACK',
                #  'VERSION': 'OpenCL 1.1 ATI-Stream-v2.3 (451)',
                #  'NACK2RETURN': ' ',
                #  'MEMORY2UNIFIED4HOSTANDDEVICE': 1,
                #  'ENDIANESS2DEVICE': 'Little',
                #  'MINIMUM2ALIGNMENT4DATATYPE': 128,
                #  'ROUND2+VE3INFINITY': 'ACK',
                #  'TYPE': 'CL_DEVICE_TYPE_MPU',
                #  'DENORMS2SINGLEPRECISION4FLOATINGPOINTCAPABILITY': 'ACK'}; time: 00:54:15 09/06/11 Pacific Daylight Time}

                
                code2ack  = code2ack + '4B-'  
            except:
                message2print = "{(Step 4Z/70) failed" + "; device2mpu-failed-by-create2image4mpu: " + str(device2mpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '4Z-'
                
            task4imagecreate2mpu ['DEVICE'] = device2mpu
            
                   
            # ---------------------------------------------------------------------------------------------
            # (5) ERROR CODE - REQUIRED                            
            # ---------------------------------------------------------------------------------------------         
            task4imagecreate2mpu ['CODE2ERROR'] = 0             
            code2ack  = code2ack + '5B-'              

            # ---------------------------------------------------------------------------------------------
            # (6) QUEUE - EXTRA                          
            # ---------------------------------------------------------------------------------------------         
            queue2mpu = None
            
            message2print = "{(Step 6A/70) trying" + "; queue2mpu-trying-by-create2image4mpu: " + str(queue2mpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
            try:
                queue2mpu = Queue4Xi()                                                                                  # THE SYSTEM CREATES QUEUE    
                
                message2print = "{(Step 6B/70) got" + "; queue2mpu-got-by-create2image4mpu: " + str(queue2mpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # <queue4xi.Queue4Xi object at 0x0000000003991B70> 
                
                code2ack  = code2ack + '6B-'  
            except:
                message2print = "{(Step 6Z/70) failed" + "; queue2mpu-failed-by-create2image4mpu: " + str(queue2mpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '6Z-'
                
            task4imagecreate2mpu ['QUEUE'] = queue2mpu

            
            # ---------------------------------------------------------------------------------------------
            # (7) PIPE - EXTRA                              
            # ---------------------------------------------------------------------------------------------         
            pipe2mpu = None
           
            message2print = "{(Step 7A/70) trying" + "; pipe2mpu-trying-by-create2image4mpu: " + str(pipe2mpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
            try:
                pipe2mpu = Pipe4Xi()                                                                                    # THE SYSTEM CREATES PIPE    
                
                message2print = "{(Step 7B/70) got" + "; pipe2mpu-got-by-create2image4mpu: " + str(pipe2mpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # (<read-write PipeConnection, handle 388>, <read-write PipeConnection, handle 392>)
                
                code2ack  = code2ack + '7B-'  
            except:
                message2print = "{(Step 7Z/70) failed" + "; pipe2mpu-failed-by-create2image4mpu: " + str(pipe2mpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '7Z-'
                
            task4imagecreate2mpu ['PIPE'] = pipe2mpu 
            
            # ---------------------------------------------------------------------------------------------
            # (8) COMMAND QUEUE INFO - EXTRA                              
            # ---------------------------------------------------------------------------------------------             
            commandqueueinfo2mpu = {}
           
            message2print = "{(Step 8A/70) trying" + "; commandqueueinfo2mpu-trying-by-create2image4mpu: " + str(commandqueueinfo2mpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print)  
                        
            try:
                commandqueueinfo2mpu = get2commandqueueinfo4mpu()                                                    # SYSTEM CREATES COMMAND QUEUE, IF NECESSARY   
                message2print = "{(Step 8B/70) got" + "; commandqueueinfo2mpu-got-by-create2image4mpu: " + str(commandqueueinfo2mpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # {'CODE2ERROR': 0,
                #  'COUNT2REFERENCE': 0,
                #  'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2commandqueue4mpu-failed ; time: 05:04:51 09/06/11 Pacific Daylight Time}',
                #  'CLASS': 'create2commandqueue4mpu',
                #  'METHOD': 'create2commandqueue4mpu'},
                #  'FUNCTION2CALLBACK': <function trace2commandqueue4mpu at 0x000000000372C848>,
                #  'ACK2RETURN': ' 1D-1E-2C-2B-3B-4B-8A-9B-10B-9B-10B-11B-12B-13B-14B-15B-16B-17B-18B-',
                #  'INFO2PRIVATE': None,
                #  'QUEUE': <queue4xi.Queue4Xi object at 0x0000000003A7A6D8>,
                #  'NACK2RETURN': ' ',
                #  'CONTEXT': {'COUNT2REFERENCE': 0 ... 
                #  'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2context4mpu-failed ; time: 05:04:51 09/06/11 Pacific Daylight Time}', 
                #  'CLASS': 'create2context4mpu', 
                #  'METHOD': 'create2context4mpu'}, 
                #  'FUNCTION2CALLBACK': <function trace2context4mpu at 0x00000000036E0148>, 
                #  'ACK2RETURN': ' 2A-2C-2D-2E-2B-3B-4B-5B-6B-7B-7B-', 
                #  'INFO2PRIVATE': None, 
                #  'PLATFORM': {'list2platforms': ['DragonXi'], 'ack2return': ' 2C-2E-2F-2G-', 'nack2return': ' ', 'available2platform': 1, 'max2entries': 1}, 
                #  'NACK2RETURN': ' ', 
                #  'SIZE2PRIVATE': 0, 
                #  'PROPERTIES': None}, 
                #  'DEVICE': {'RESOLUTION2TIMER4PROFILING': 318, ...
                
                code2ack  = code2ack + '8B-'  
            except:
                message2print = "{(Step 8Z/70) failed" + "; commandqueueinfo2mpu-failed-by-create2image4mpu: " + str(commandqueueinfo2mpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '8Z-'
        
            task4imagecreate2mpu ['COMMANDQUEUE'] = commandqueueinfo2mpu
            
            
            
            # ---------------------------------------------------------------------------------------------
            # (9) BUFFER INFO - EXTRA                              
            # ---------------------------------------------------------------------------------------------             
            bufferinfo2mpu = {}
           
            message2print = "{(Step 9A/70) trying" + "; bufferinfo2mpu-trying-by-create2image4mpu: " + str(bufferinfo2mpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                bufferinfo2mpu = get2bufferinfo4mpu()                                                                   # SYSTEM CREATES BUFFER FOR MPU, IF NECESSARY
                message2print = "{(Step 9B/70) got" + "; bufferinfo2mpu-got-by-create2image4mpu: " + str(bufferinfo2mpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # {'CODE2ERROR': 0,
                #  'COUNT2REFERENCE': 0,
                #  'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2buffer4mpu-failed ; time: 05:04:51 09/06/11 Pacific Daylight Time}',
                #                    'CLASS': 'create2buffer4mpu',
                #                    'METHOD': 'create2buffer4mpu'},
                #  'FUNCTION2CALLBACK': <function trace2buffer4mpu at 0x0000000003B2B348>,
                #  'POINTER2BUFFER': <mmap.mmap object at 0x0000000003ADD2F0>,
                #  'FLAGS': 0,
                #  'CONTEXT': {'COUNT2REFERENCE': 0, 
                #  'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2context4mpu-failed ; time: 05:04:51 09/06/11 Pacific Daylight Time}', 
                #  'CLASS': 'create2context4mpu', 
                #  'METHOD': 'create2context4mpu'}, 
                #  'FUNCTION2CALLBACK': <function trace2context4mpu at 0x00000000036E0148>, 
                #  'ACK2RETURN': ' 2A-2C-2D-2E-2B-3B-4B-5B-6B-7B-7B-', 
                #  'INFO2PRIVATE': None, 
                #  'PLATFORM': {'list2platforms': ['DragonXi'], 'ack2return': ' 2C-2E-2F-2G-', 'nack2return': ' ', 'available2platform': 1, 'max2entries': 1}, 
                #  'NACK2RETURN': ' ', 
                #  'SIZE2PRIVATE': 0, 
                #  'PROPERTIES': None}, 
                #  'DEVICE': {'RESOLUTION2TIMER4PROFILING': 318, ...
                
                
                code2ack  = code2ack + '9B-' 
            except:
                message2print = "{(Step 9Z/70) failed" + "; bufferinfo2mpu-failed-by-create2image4mpu: " + str(bufferinfo2mpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '9Z-'
        
            task4imagecreate2mpu ['BUFFER'] = bufferinfo2mpu 
            
            # ---------------------------------------------------------------------------------------------
            # (10) EVENT INFO - EXTRA                              
            # ---------------------------------------------------------------------------------------------             
            eventinfo2mpu = {}
           
            message2print = "{(Step 10A/70) trying" + "; eventinfo2mpu-trying-by-create2image4mpu: " + str(eventinfo2mpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                eventinfo2mpu = get2eventinfo4mpu()                                                                     # SYSTEM CREATES EVENT IF NECESSAY   
                message2print = "{(Step 10B/70) got" + "; eventinfo2mpu-got-by-create2image4mpu: " + str(eventinfo2mpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # {'CODE2ERROR': 0,
                #  'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2event4mpu-failed ; time: 05:10:51 09/06/11 Pacific Daylight Time}',
                #                    'CLASS': 'create2event4mpu',
                #                    'METHOD': 'create2event4mpu'},
                #  'FUNCTION2CALLBACK': <function trace2event4mpu at 0x0000000003747D48>,
                #  'ACK2RETURN': ' 1D-1E-2C-2B-3B-4B-6B-7B-8B-9B-10B-11B-12B-14B-15B-16B-',
                #  'QUEUE': <queue4xi.Queue4Xi object at 0x0000000003A02898>,
                #  'NACK2RETURN': ' ',
                #  'CONTEXT': {'COUNT2REFERENCE': 0, 'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2context4mpu-failed ; time: 05:10:51 09/06/11 Pacific Daylight Time}', 
                #                                                      'CLASS': 'create2context4mpu', 'METHOD': 'create2context4mpu'}, 
                #  'FUNCTION2CALLBACK': <function trace2context4mpu at 0x00000000036F0148>,
                #  'ACK2RETURN': ' 2A-2C-2D-2E-2B-3B-4B-5B-6B-7B-7B-',
                #  'INFO2PRIVATE': None,
                #  'PLATFORM': {'list2platforms': ['DragonXi'], 'ack2return': ' 2C-2E-2F-2G-', 'nack2return': ' ', 'available2platform': 1, 'max2entries': 1}, 
                #  'NACK2RETURN': ' ',
                #  'SIZE2PRIVATE': 0,
                #  'PROPERTIES': None},
                #  'COMMANDQUEUE': {'CODE2ERROR': 0, 'COUNT2REFERENCE': 0, 'DATA2CALLBACK': ...
                
                
                code2ack  = code2ack + '10B-' 
            except:
                message2print = "{(Step 10Z/70) failed" + "; eventinfo2mpu-failed-by-create2image4mpu: " + str(eventinfo2mpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '10Z-'
        
            task4imagecreate2mpu ['EVENT'] = eventinfo2mpu             
            
            # ---------------------------------------------------------------------------------------------
            # (11) FLAGS - REQUIRED                              
            # ---------------------------------------------------------------------------------------------             
            flags2mpu = 0
           
            message2print = "{(Step 11A/70) trying" + "; flags2mpu-trying-by-create2image4mpu: " + str(flags2mpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                flags2mpu = 0                                                              
                message2print = "{(Step 11B/70) set" + "; flags2mpu-set-by-create2image4mpu to: " + str(flags2mpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                code2ack  = code2ack + '11B-' 
            except:
                message2print = "{(Step 11Z/70) failed" + "; flags2mpu-failed-by-create2image4mpu: " + str(flags2mpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '11Z-'
        
            task4imagecreate2mpu ['FLAGS'] = flags2mpu             
            
            # ---------------------------------------------------------------------------------------------
            # (14) WIDTH - REQUIRED                              
            # ---------------------------------------------------------------------------------------------             
            width2mpu = 0

            quantity2core = 1                                                                      # ORIGIN 
            quantity2skeleton = 1                                                                  # SKELETON WITH DISTANCE 1
            quantity2layers = 33                                                                   # LAYERS 10, 100 ... 900, 1000 ... 3300
            quantity2frame = 1                                                                     # FRAME
            quantity2extra = 4                                                                     # CONTROLS FOR GUI ETC...
            width2layer = 10                                                                       # SCALED TO 10 EACH LAYER            
            
            message2print = "{(Step 14A/70) trying" + "; width2mpu-trying-by-create2image4mpu: " + str(width2mpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:                                                                                    
                width2mpu = int(width2layer * (quantity2core + quantity2skeleton + quantity2layers + quantity2frame + quantity2extra))
                width2mpu = 2 * width2mpu                                                          # POS and NEG X  2*400 = 800 PIXELS
                                                                                                                                         
                message2print = "{(Step 14B/70) set" + "; width2mpu-set-by-create2image4mpu to: " + str(width2mpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 800 
                code2ack  = code2ack + '14B-' 
            except:
                message2print = "{(Step 14Z/70) failed" + "; width2mpu-failed-by-create2image4mpu: " + str(width2mpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '14Z-'
        
            task4imagecreate2mpu ['WIDTH'] = width2mpu
            
            
            # ---------------------------------------------------------------------------------------------
            # (15) HEIGHT - REQUIRED                               
            # ---------------------------------------------------------------------------------------------             
            height2mpu = 0
            quantity2core = 1                                                                      # ORIGIN 
            quantity2skeleton = 1                                                                  # SKELETON WITH DISTANCE 1
            quantity2layers = 33                                                                   # LAYERS 10, 100 ... 900, 1000 ... 3300
            quantity2frame = 1                                                                     # FRAME
            quantity2extra = 4                                                                     # CONTROLS FOR GUI
            
            quantity2total = quantity2core + quantity2skeleton + quantity2layers + quantity2frame + quantity2extra
            height2layer = 10                                                                      # SCALED TO 10 
                       
            message2print = "{(Step 15A/70) trying" + "; height2mpu-trying-by-create2image4mpu: " + str(height2mpu)  + "; time: " + time4xi() + "}"  
            print4xi(class2print, method2print, message2print) 
                        
            try:
                height2mpu = height2layer * quantity2total
                
                height2mpu = 2 * height2mpu                                                        # POS and NEG Y   2*400 = 800 PIXELS
                                                                                                                                         
                message2print = "{(Step 15B2/70) set" + "; height2mpu-set-by-create2image4mpu to: " + str(height2mpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 800 
                code2ack  = code2ack + '15B-' 
            except:
                message2print = "{(Step 15Z/70) failed" + "; height2mpu-failed-by-create2image4mpu: " + str(height2mpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                # 
                
                code2nack  = code2nack + '15Z-'
             
                        
            task4imagecreate2mpu ['HEIGHT'] = height2mpu 
            
            # ---------------------------------------------------------------------------------------------
            # (16) ROWPITCH (STRIDE) - REQUIRED                              
            # ---------------------------------------------------------------------------------------------             
            # One can assume that memory representing an image or screen will have data laid out as one row of pixels followed directly by the next row of pixels. 
            # However many APIs (e.g. Direct3D) that return a pointer to image data add an extra block of data at the end of each row. 
            # int offset = x * bytesPerPixel + y * pitch;
            # When a video image is stored in memory, the memory buffer might contain extra padding bytes after each row of pixels. 
            # The padding bytes affect how the image is stored in memory, but do not affect how the image is displayed.
            # The stride is the number of bytes from one row of pixels in memory to the next row of pixels in memory. 
            # Stride is also called pitch. 
            # If padding bytes are present, the stride is wider than the width of the image, as shown in the following illustration.


            rowpitch2mpu = 0
            size2element = 1                                                                                        # VOXEL (X,Y,Z) = SHOT IN PIXELS, R=1 PIXELS
            voxelquantity2layer = 1                                                                                 # MAX NUMBER OF VOXELS IN EACH 2D LAYER 
            quantity2direction = 1                                                                                  # ONE DIRECTIONS IN 2D
            
            message2print = "{(Step 16A/70) trying" + "; rowpitch2mpu-trying-by-create2image4mpu: " + str(rowpitch2mpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
            # 
                        
            try:                                                                                                    # X-AXEL IS THE WORST CASE TO SCAN WITH Y=0, Z=0
                rowpitch2mpu = int(width2mpu * size2element * voxelquantity2layer / quantity2direction)             # (800 * 8 * 10) / 8 = 8K PIXELS MAX TO SCAN 
                                                                                                                    # SPHERE IS SPARSE HAVING FEW SHOTS PER SCAN LINE 
                                                                                                                                         
                message2print = "{(Step 16B/70) set" + "; rowpitch2mpu-set-by-create2image4mpu to: " + str(rowpitch2mpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 800                                                                                             
                 
                code2ack  = code2ack + '16B-' 
            except:
                message2print = "{(Step 16Z/70) failed" + "; rowpitch2mpu-failed-by-create2image4mpu: " + str(rowpitch2mpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                # 
                
                code2nack  = code2nack + '16Z-'
        
            task4imagecreate2mpu ['ROWPITCH'] = rowpitch2mpu     
            
            # ---------------------------------------------------------------------------------------------
            # (17) POINTER - REQUIRED                           
            # ---------------------------------------------------------------------------------------------             
            pointer2mpu = None
            size2mpu = int(width2mpu * height2mpu)                                                                  # CALCULATED MEMORY
            if size2mpu == 0:
                size2mpu = 4400                                                                                     # MINIMUM MEMORY
            
            message2print = "{(Step 17A/70) trying" + "; pointer2mpu-trying-by-create2image4mpu: " + str(pointer2mpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
            # 
                        
            try:
                arena2mpu = Arena4Xi(size2mpu)                                                                      # SYSTEM RESERVER MEMORY FOR IMAGE                             
                                                                                                                                         
                message2print = "{(Step 17B1/70) got" + "; arena2mpu-got-by-create2image4mpu: " + str(arena2mpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # <arena4xi.Arena4Xi object at 0x00000000041C0D68>
                
                size2arena = arena2mpu.size                                                                         # 
                              
                message2print = "{(Step 17B2/70) got" + "; size2arena-got-by-create2buffer4mpu: " + str(size2arena)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                # 640000 
                
                buffer2arena = arena2mpu.buffer
                
                message2print = "{(Step 17B3/70) got" + "; buffer2arena-got-by-create2buffer4mpu: " + str(buffer2arena)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                
                # <mmap.mmap object at 0x0000000003AAF298> 
                
                code2ack  = code2ack + '17B-' 
                
                if size2arena < size2mpu:
                    size2mpu = size2arena                                                                           # AVAILABLE MEMORY FOR IMAGE
                    message2print = "{(Step 17C/70) set" + "; size2mpu-set-by-create2buffer4mpu to: " + str(size2mpu)  + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)
                    # 
                    
                    code2ack  = code2ack + '17C-'
                else:
                    pass
                    
                pointer2mpu = buffer2arena
                message2print = "{(Step 17D/70) set" + "; pointer2mpu-set-by-create2buffer4mpu to: " + str(pointer2mpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                # <mmap.mmap object at 0x0000000003AAF298>
                
                code2ack  = code2ack + '17D-'       
            except:
                message2print = "{(Step 17Z/70) failed" + "; pointer2mpu-failed-by-create2image4mpu: " + str(pointer2mpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                # 
                
                code2nack  = code2nack + '17Z-'
        
            task4imagecreate2mpu ['POINTER'] = pointer2mpu                            
            task4imagecreate2mpu ['SIZE']    = size2mpu                                                             # EXTRA
            
            
            # ---------------------------------------------------------------------------------------------
            # (18) FORMAT - REQUIRED                              
            # ---------------------------------------------------------------------------------------------             
            format2mpu = {}
           
            message2print = "{(Step 18A/70) trying" + "; format2mpu-trying-by-create2image4mpu: " + str(format2mpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                format2mpu['ORDER'] = 'CL_RGBA'                                                              
                message2print = "{(Step 18B/70) set" + "; format2mpu-set-by-create2image4mpu to: " + str(format2mpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # {'ORDER': 'CL_RGBA'}
                 
                code2ack  = code2ack + '18B-'
                
                format2mpu['TYPE'] = 'CL_UNORM_INT8'                                                              
                message2print = "{(Step 18C/70) set" + "; format2mpu-set-by-create2image4mpu to: " + str(format2mpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # {'TYPE': 'CL_UNORM_INT8', 'ORDER': 'CL_RGBA'}
                
                code2ack  = code2ack + '18C-'                
                 
            except:
                message2print = "{(Step 18Z/70) failed" + "; format2mpu-failed-by-create2image4mpu: " + str(format2mpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '18Z-'
        
            task4imagecreate2mpu ['FORMAT'] = format2mpu                  
            
            # ---------------------------------------------------------------------------------------------
            # (19) DEPTH - MUST BE 0                               
            # ---------------------------------------------------------------------------------------------             
            depth2mpu = 0
            message2print = "{(Step 19A/70) set" + "; depth2mpu-set-by-create2image4mpu to: " + str(depth2mpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            #  
            code2ack  = code2ack + '19A-' 
            task4imagecreate2mpu ['DEPTH'] = depth2mpu
                                                                                                                                       
        else:
            pass                                                                                                    # PARAMETER DATA RECEIVED                       
    except: 
        message2print = "{(Step 1Z1/70) yielded" + "; task4imagecreate2mpu-yielded-by-create2image4mpu: " + str(task4imagecreate2mpu) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # 
        
        code2nack  = code2nack + '1Z1-' 

    # ---------------------------------------------------------------------------------------------
    # TASK EITHER GENERATED BY THIS PROGRAM (NONE GIVEN) OR GIVEN BY USER    
    # ---------------------------------------------------------------------------------------------
    keys2task = task4imagecreate2mpu.keys()        
    message2print = "{(Step 19A/70) using" + "; keys2task-using-by-create2image4mpu: " + str(sorted(keys2task))  + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)    
    # ['BUFFER',
    #  'CODE2ERROR',
    #  'COMMANDQUEUE',
    #  'CONTEXT',
    #  'DATA2CALLBACK',
    #  'DEVICE',
    #  'EVENT',
    #  'FLAGS',
    #  'FORMAT',
    #  'FUNCTION2CALLBACK',
    #  'HEIGHT',
    #  'PIPE',
    #  'PLATFORM',
    #  'POINTER',
    #  'QUEUE',
    #  'ROWPITCH',
    #  'SIZE',
    #  'WIDTH']; time: 00:53:55 10/05/11 Pacific Daylight Time}
  
    
    # ---------------------------------------------------------------------------------------------
    # (Step 20A/70) - CREATE 2D IMAGE FOR MPU! 
    # ---------------------------------------------------------------------------------------------    
    mpu2image4xi = {}                                                                                                                                                                               
    message2print = "{(Step 20A/70) initialized" + "; mpu2image4xi-initialized-by-create2image4mpu to: " + str(mpu2image4xi)  + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)
    # {} 
    
    message2print = "{(Step 21A0/70) try" + "; data2callback-try-by-create2image4mpu: {}" + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)      
    # 
    try:
        # -----------------------------------------------------------------------------------------
        # (Step 21A/70) - DATA2CALLBACK - EXTRA 
        # -----------------------------------------------------------------------------------------               
        mpu2image4xi['DATA2CALLBACK'] = task4imagecreate2mpu['DATA2CALLBACK']
        
        message2print = "{(Step 21A1/70) initialized" + "; data2callback-created-by-create2image4mpu to: " + str(mpu2image4xi['DATA2CALLBACK'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # {'MESSAGE': '{(Step 1Z1/33) failed; create2image4mpu-failed ; time: 05:56:13 09/06/11 Pacific Daylight Time}',
        #  'CLASS': 'create2image4mpu',
        #  'METHOD': 'create2image4mpu'}; time: 05:56:13 09/06/11 Pacific Daylight Time}

        
        code2ack  = code2ack + '21A1-'
    except: 
        message2print = "{(Step 21Z1/70) yielded" + "; data2callback-mpu2image4xi-yielded-by-create2image4mpu: " + str(mpu2image4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        code2nack  = code2nack + '21Z1-'
    
    message2print = "{(Step 22B0/70) try" + "; function2callback-try-by-create2image4mpu: None" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)        
    # 
    try:
        # -----------------------------------------------------------------------------------------
        # (Step 22B/70) - FUNCTION2CALLBACK - EXTRA
        # -----------------------------------------------------------------------------------------               
        mpu2image4xi['FUNCTION2CALLBACK'] = task4imagecreate2mpu['FUNCTION2CALLBACK']
        
        message2print = "{(Step 22B1/70) initialized" + "; function2callback-created-by-create2image4mpu to: " + str(mpu2image4xi['FUNCTION2CALLBACK'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # <function trace2image4mpu at 0x0000000003B04348>  
        code2ack  = code2ack + '22B1-'
    except: 
        message2print = "{(Step 22Z2/70) yielded" + "; function2callback-mpu2image4xi-yielded-by-create2image4mpu: " + str(mpu2image4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        code2nack  = code2nack + '22Z2-' 
    
    message2print = "{(Step 22C0/70) try" + "; platform-try-by-create2image4mpu: {}" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)        
    # 
    try:
        # -----------------------------------------------------------------------------------------
        # (Step 22C/70) - PLATFORM - EXTRA
        # -----------------------------------------------------------------------------------------               
        mpu2image4xi['PLATFORM'] = task4imagecreate2mpu['PLATFORM']
        
        message2print = "{(Step 22C1/70) initialized" + "; platform-created-by-create2image4mpu to: " + str(mpu2image4xi['PLATFORM'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # {'list2platforms': ['DragonXi'], 'ack2return': ' 2C-2E-2F-2G-', 'nack2return': ' ', 'available2platform': 1, 'max2entries': 1}; time: 05:56:13 09/06/11 Pacific Daylight Time}
  
        code2ack  = code2ack + '22C1-'
    except: 
        message2print = "{(Step 22Z3/70) yielded" + "; platform-mpu2image4xi-yielded-by-create2image4mpu: " + str(mpu2image4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        code2nack  = code2nack + '22Z3-' 
        
        
      
    message2print = "{(Step 23A/70) try" + "; context2mpu-try-by-create2image4mpu: {}" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)       
    try:
        # -----------------------------------------------------------------------------------------
        # (Step 23/70) - CONTEXT - REQUIRED
        # -----------------------------------------------------------------------------------------               
        mpu2image4xi['CONTEXT'] = task4imagecreate2mpu['CONTEXT']
        
        message2print = "{(Step 23B/70) initialized" + "; context-created-by-create2image4mpu to: " + str(mpu2image4xi['CONTEXT'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # {'COUNT2REFERENCE': 0,
        #  'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2context4mpu-failed ; time: 05:56:12 09/06/11 Pacific Daylight Time}', 
        #  'CLASS': 'create2context4mpu',
        #  'METHOD': 'create2context4mpu'},
        #  'FUNCTION2CALLBACK': <function trace2context4mpu at 0x00000000036B0148>,
        #  'ACK2RETURN': ' 2A-2C-2D-2E-2B-3B-4B-5B-6B-7B-7B-',
        #  'INFO2PRIVATE': None,
        #  'PLATFORM': {'list2platforms': ['DragonXi'], 'ack2return': ' 2C-2E-2F-2G-', 'nack2return': ' ', 'available2platform': 1, 'max2entries': 1}, 
        #  'NACK2RETURN': ' ',
        #  'SIZE2PRIVATE': 0,
        #  'PROPERTIES': None}; time: 05:56:13 09/06/11 Pacific Daylight Time}

        
        code2ack  = code2ack + '23B-'
    except: 
        message2print = "{(Step 23Z1/70) yielded" + "; context-mpu2image4xi-yielded-by-create2image4mpu: " + str(mpu2image4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
    
        code2nack  = code2nack + '23Z1-' 
        
    message2print = "{(Step 24A/70) try" + "; device-try-by-create2image4mpu: {}" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)    
    try:
        # -----------------------------------------------------------------------------------------
        # (Step 24/70) - DEVICE - EXTRA
        # -----------------------------------------------------------------------------------------               
        mpu2image4xi['DEVICE'] = task4imagecreate2mpu['DEVICE']
        
        message2print = "{(Step 24B/70) initialized" + "; device-created-by-create2image4mpu to: " + str(mpu2image4xi['DEVICE'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # {'RESOLUTION2TIMER4PROFILING': 318,
        #  'ID2PLATFORM': '0000000001509568',
        #  'PROFILING2QUEUE': 'ACK',
        #  'EXECUTE2KERNELS4OPENCL': 'ACK',
        #  'NATIVE2WIDTHLONG4VECTOR': 2,
        #  'MAX2UNITS4COMPUTE': 4,
        #  'SIZE2MEMORY4GLOBAL': 3221225472,
        #  'NATIVE2WIDTHFLOAT4VECTOR': 4,
        #  'MAX2SIZE4WORKGROUP': 1024,
        #  'PREFERRED2WIDTHCHAR4VECTOR': 16,
        #  'SIZE2CACHE': 65536,
        #  'IEEE754-2008FUSED2MULTIPLY-ADD': 'NACK',
        #  'TYPE4CACHE': 'Read/Write',
        #  'PREFERRED2WIDTHINT4VECTOR': 4,
        #  'SIZE2PREFERREDWORKGROUPMULTIPLE4KERNEL': 1,
        #  'NATIVE2WIDTHINT4VECTOR': 4,
        #  'SIZE2BUFFER4CONSTANT': 65536,
        #  'EXECUTE2FUNCTION4NATIVE': 'ACK',
        #  'PREFERRED2WIDTHDOUBLE4VECTOR': 0,
        #  'MAX2NUMBER4CONSTANTARGS': 8,
        #  'VERSION2DRIVER': '2.0', 'EXTENSIONS': 'cl_amd_fp64 ...
        
        
        code2ack  = code2ack + '24B-'
    except: 
        message2print = "{(Step 24Z1/70) yielded" + "; device-mpu2image4xi-yielded-by-create2image4mpu: " + str(mpu2image4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        code2nack  = code2nack + '24Z1-'
                 
    message2print = "{(Step 25A/70) try" + "; code2error-try-by-create2image4mpu: None" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print) 
    try:
        # -----------------------------------------------------------------------------------------
        # (Step 25/70) - ERROR CODE - REQUIRED
        # -----------------------------------------------------------------------------------------               
        mpu2image4xi['CODE2ERROR'] = task4imagecreate2mpu['CODE2ERROR']
        
        message2print = "{(Step 25B/70) initialized" + "; code2error-created-by-create2image4mpu to: " + str(mpu2image4xi['CODE2ERROR'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # 0 
        code2ack  = code2ack + '25B-'
    except: 
        message2print = "{(Step 25Z1/70) yielded" + "; code2error-mpu2image4xi-yielded-by-create2image4mpu: " + str(mpu2image4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        code2nack  = code2nack + '25Z1-'                
                
          
    message2print = "{(Step 26A/70) try" + "; queue-try-by-create2image4mpu: None" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                
    try:
        # -----------------------------------------------------------------------------------------
        # (Step 26/70) - QUEUE - EXTRA
        # -----------------------------------------------------------------------------------------               
        mpu2image4xi['QUEUE'] = task4imagecreate2mpu['QUEUE']
        
        message2print = "{(Step 26B/70) initialized" + "; queue-created-by-create2image4mpu to: " + str(mpu2image4xi['QUEUE'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        # <queue4xi.Queue4Xi object at 0x0000000003ED99E8>  
        code2ack  = code2ack + '26B-'
    except: 
        message2print = "{(Step 26Z1/70) yielded" + "; queue-mpu2image4xi-yielded-by-create2image4mpu: " + str(mpu2image4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        code2nack  = code2nack + '26Z1-' 
                                
    message2print = "{(Step 27A/70) try" + "; pipe-try-by-create2image4mpu: None" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                     
    try:
        # -----------------------------------------------------------------------------------------
        # (Step 27/70) - PIPE - EXTRA
        # -----------------------------------------------------------------------------------------               
        mpu2image4xi['PIPE'] = task4imagecreate2mpu['PIPE']
        
        message2print = "{(Step 27B/70) initialized" + "; pipe-created-by-create2image4mpu to: " + str(mpu2image4xi['PIPE'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        # (<read-write PipeConnection, handle 376>, <read-write PipeConnection, handle 380>)  
        
        code2ack  = code2ack + '27B-'
    except: 
        message2print = "{(Step 27Z1/70) yielded" + "; pipe-mpu2image4xi-yielded-by-create2image4mpu: " + str(mpu2image4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        code2nack  = code2nack + '27Z1-'   

    message2print = "{(Step 28A/70) try" + "; commandqueue-try-by-create2image4mpu: {}" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)        
    try:            
        # ---------------------------------------------------------------------------------------------
        # (28) COMMAND QUEUE - EXTRA                              
        # ---------------------------------------------------------------------------------------------             
                        
        mpu2image4xi['COMMANDQUEUE']  = task4imagecreate2mpu['COMMANDQUEUE']  
 
        message2print = "{(Step 28B/70) initialized" + "; commandqueue-created-by-create2image4mpu to: " + str(mpu2image4xi['COMMANDQUEUE'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        # {'CODE2ERROR': 0,
        #  'COUNT2REFERENCE': 0,
        #  'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2commandqueue4mpu-failed ; time: 07:13:59 09/06/11 Pacific Daylight Time}',
        #  'CLASS': 'create2commandqueue4mpu',
        #  'METHOD': 'create2commandqueue4mpu'},
        #  'FUNCTION2CALLBACK': <function trace2commandqueue4mpu at 0x00000000037BC848>,
        #  'ACK2RETURN': ' 1D-1E-2C-2B-3B-4B-8A-9B-10B-9B-10B-11B-12B-13B-14B-15B-16B-17B-18B-',
        #  'INFO2PRIVATE': None,
        #  'QUEUE': <queue4xi.Queue4Xi object at 0x000000000432B3C8>,
        #  'NACK2RETURN': ' ',
        #  'CONTEXT': {'COUNT2REFERENCE': 0, 'DATA2CALLBACK': {'MESSAGE': ...
        
        code2ack  = code2ack + '28B-'
    except: 
        message2print = "{(Step 28Z1/70) yielded" + "; commandqueue-yielded-by-create2image4mpu: " + str(mpu2image4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # 
              
        code2nack  = code2nack + '28Z1-'    

    message2print = "{(Step 29A/70) try" + "; bufferinfo-try-by-create2image4mpu: {}" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)        
    try:            
        # ---------------------------------------------------------------------------------------------
        # (29) BUFFER INFO - EXTRA                              
        # ---------------------------------------------------------------------------------------------             
                        
        mpu2image4xi['BUFFER']  = task4imagecreate2mpu['BUFFER']  
 
        message2print = "{(Step 29B/70) initialized" + "; bufferinfo-created-by-create2image4mpu to: " + str(mpu2image4xi['BUFFER'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        # {'CODE2ERROR': 0,
        #  'COUNT2REFERENCE': 0,
        #  'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2buffer4mpu-failed ; time: 05:56:12 09/06/11 Pacific Daylight Time}', 
        #  'CLASS': 'create2buffer4mpu',
        #  'METHOD': 'create2buffer4mpu'},
        #  'FUNCTION2CALLBACK': <function trace2buffer4mpu at 0x0000000003AFC348>,
        #  'POINTER2BUFFER': <mmap.mmap object at 0x0000000003AAF2F0>,
        #  'FLAGS': 0,
        #  'CONTEXT': {'COUNT2REFERENCE': 0, 'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2context4mpu-failed ...
        
        code2ack  = code2ack + '29B-'
    except: 
        message2print = "{(Step 29Z1/70) yielded" + "; bufferinfo-yielded-by-create2image4mpu: " + str(mpu2image4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        code2nack  = code2nack + '29Z1-'            

    message2print = "{(Step 30A/70) try" + "; eventinfo-try-by-create2image4mpu: {}" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)        
    try:            
        # ---------------------------------------------------------------------------------------------
        # (30) EVENT INFO - EXTRA                              
        # ---------------------------------------------------------------------------------------------             
                        
        mpu2image4xi['EVENT']  = task4imagecreate2mpu['EVENT']  
 
        message2print = "{(Step 30B/70) initialized" + "; eventinfo-created-by-create2image4mpu to: " + str(mpu2image4xi['EVENT'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        # {'CODE2ERROR': 0,
        #  'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2event4mpu-failed ; time: 05:56:12 09/06/11 Pacific Daylight Time}',
        #                     'CLASS': 'create2event4mpu',
        #                     'METHOD': 'create2event4mpu'},
        #  'FUNCTION2CALLBACK': <function trace2event4mpu at 0x0000000003707D48>,
        #  'ACK2RETURN': ' 1D-1E-2C-2B-3B-4B-6B-7B-8B-9B-10B-11B-12B-14B-15B-16B-',
        #  'QUEUE': <queue4xi.Queue4Xi object at 0x00000000039C3860>,
        #  'NACK2RETURN': ' ',
        #  'CONTEXT': {'COUNT2REFERENCE': 0, 'DATA2CALLBACK': {'MESSAGE': ...
        
        code2ack  = code2ack + '30B-'
    except: 
        message2print = "{(Step 30Z1/70) yielded" + "; eventinfo-yielded-by-create2image4mpu: " + str(mpu2image4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        code2nack  = code2nack + '30Z1-'            
                

    message2print = "{(Step 31A/70) try" + "; flags-try-by-create2image4mpu: None" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)        
    try:            
        # ---------------------------------------------------------------------------------------------
        # (31) FLAGS - REQUIRED                              
        # ---------------------------------------------------------------------------------------------             
                        
        mpu2image4xi['FLAGS']  = task4imagecreate2mpu['FLAGS']  
 
        message2print = "{(Step 31B/70) initialized" + "; flags-created-by-create2image4mpu to: " + str(mpu2image4xi['FLAGS'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        # 0
        code2ack  = code2ack + '31B-'
    except: 
        message2print = "{(Step 31Z1/70) yielded" + "; flags-yielded-by-create2image4mpu: " + str(mpu2image4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        code2nack  = code2nack + '31Z1-'     

    message2print = "{(Step 34A/70) try" + "; width-try-by-create2image4mpu: None" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)        
    try:            
        # ---------------------------------------------------------------------------------------------
        # (34) WIDTH - REQUIRED                              
        # ---------------------------------------------------------------------------------------------      
        mpu2image4xi['WIDTH']  = task4imagecreate2mpu['WIDTH']  
 
        message2print = "{(Step 34H/70) initialized" + "; width-created-by-create2image4mpu to: " + str(mpu2image4xi['WIDTH'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        # 800 
        code2ack  = code2ack + '34H-'
    except: 
        message2print = "{(Step 34Z1/70) yielded" + "; width-yielded-by-create2image4mpu: " + str(mpu2image4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        code2nack  = code2nack + '34Z1-'   


    message2print = "{(Step 35A/70) try" + "; height-try-by-create2image4mpu: None" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)        
    try:            
        # ---------------------------------------------------------------------------------------------
        # (35) HEIGHT - REQUIRED                              
        # ---------------------------------------------------------------------------------------------      
        mpu2image4xi['HEIGHT']  = task4imagecreate2mpu['HEIGHT']  
 
        message2print = "{(Step 35H/70) initialized" + "; height-created-by-create2image4mpu to: " + str(mpu2image4xi['HEIGHT'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        # 800 
        code2ack  = code2ack + '35H-'
    except: 
        message2print = "{(Step 35Z1/70) yielded" + "; height-yielded-by-create2image4mpu: " + str(mpu2image4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        code2nack  = code2nack + '35Z1-'   
 

    message2print = "{(Step 36A/70) try" + "; rowpitch-try-by-create2image4mpu: None" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)        
    try:            
        # ---------------------------------------------------------------------------------------------
        # (36) ROWPITCH - REQUIRED                              
        # ---------------------------------------------------------------------------------------------             
                        
        mpu2image4xi['ROWPITCH']  = task4imagecreate2mpu['ROWPITCH']  
 
        message2print = "{(Step 36B/70) initialized" + "; rowpitch-created-by-create2image4mpu to: " + str(mpu2image4xi['ROWPITCH'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        # 800
        code2ack  = code2ack + '36B-'
    except: 
        message2print = "{(Step 36Z1/70) yielded" + "; rowpitch-yielded-by-create2image4mpu: " + str(mpu2image4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        code2nack  = code2nack + '36Z1-'

    message2print = "{(Step 37A/70) try" + "; pointer-try-by-create2image4mpu: None"  + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)        
    try:            
        # ---------------------------------------------------------------------------------------------
        # (37) POINTER - REQUIRED                              
        # ---------------------------------------------------------------------------------------------             
                        
        mpu2image4xi['POINTER']  = task4imagecreate2mpu['POINTER']  
 
        message2print = "{(Step 37B/70) initialized" + "; pointer-created-by-create2image4mpu to: " + str(mpu2image4xi['POINTER'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        # <mmap.mmap object at 0x0000000003C050E0>
         
        code2ack  = code2ack + '37B-'
    except: 
        message2print = "{(Step 37Z1/70) yielded" + "; pointer-yielded-by-create2image4mpu: " + str(mpu2image4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        code2nack  = code2nack + '37Z1-'

    message2print = "{(Step 38A/70) try" + "; format-try-by-create2image4mpu: None" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)        
    try:            
        # ---------------------------------------------------------------------------------------------
        # (38) FORMAT - REQUIRED                              
        # ---------------------------------------------------------------------------------------------             
                        
        mpu2image4xi['FORMAT']  = task4imagecreate2mpu['FORMAT']  
 
        message2print = "{(Step 38B/70) initialized" + "; format-created-by-create2image4mpu to: " + str(mpu2image4xi['FORMAT'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        # {'TYPE': 'CL_UNORM_INT8', 'ORDER': 'CL_RGBA'} 
        code2ack  = code2ack + '38B-'
    except: 
        message2print = "{(Step 38Z1/70) yielded" + "; format-yielded-by-create2image4mpu: " + str(mpu2image4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        code2nack  = code2nack + '38Z1-'

    message2print = "{(Step 39A/70) try" + "; depth-try-by-create2image4mpu: None" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)        
    try:            
        # ---------------------------------------------------------------------------------------------
        # (39) DEPTH - MUST BE 0                              
        # ---------------------------------------------------------------------------------------------             
                        
        mpu2image4xi['DEPTH']  = task4imagecreate2mpu['DEPTH']  
 
        message2print = "{(Step 39B/70) initialized" + "; depth-created-by-create2image4mpu to: " + str(mpu2image4xi['DEPTH'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        #  
        code2ack  = code2ack + '39B-'
    except: 
        message2print = "{(Step 39Z1/70) yielded" + "; depth-yielded-by-create2image4mpu: " + str(mpu2image4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        code2nack  = code2nack + '39Z1-'
                                                                                                               
    # -----------------------------------------------------------------------------------------
    # (Step 70/70)           
    # -----------------------------------------------------------------------------------------
    mpu2image4xi ['ACK2RETURN']  = code2ack
    mpu2image4xi ['NACK2RETURN'] = code2nack  
    
    message2print = "{(Step 70/70) returning" + "; mpu2image4xi-returning-by-create2image4mpu: " + str(mpu2image4xi) + "; time: " + time4xi() + "}"  
    print4xi(class2print, method2print, message2print)                                                              
    # 
    
    # 'DEPTH'
    # 'WIDTH'
    # 'HEIGHT'
    return mpu2image4xi
            
#</function4xi> 


# -------------------------------------------------------------------------------------------------
#<function4xi function = "set2image4mpu" usage = "
#    
#   print('(Step 4B/33) OF MPU2IMAGE4XI5TEST - TRYING TO GET PLATFORMIDS4XI', time4xi())
#   platform2xi = {}
#   try:
#       platform2xi = get2platformids4xi()
#       print('(Step 4B/33) of CONTEXT4MPU5TEST - GOT PLATFORMIDS4XI', platform2xi) 
#   except:
#       print('(Step 4Z/33) of CONTEXT4MPU5TEST *** FAILED TO GET PLATFORMIDS4XI', time4xi())     
#   
#   print('(Step 2A/33) OF DEVICE4MPU5TEST - TRYING TO INITIALIZE DEVICE4MPU ', time4xi())   
#   print('=' *33)        
#   try:
#       device2mpu = init2mpu4xi()                                                                                
#       print('(Step 2B/33) OF DEVICE4MPU5TEST - INITIALIZED DEVICE4MPU TO: ', device2mpu)        
#   except:
#       print('(Step 2Z/33) OF DEVICE4MPU5TEST *** FAILED TO INITIALIZE DEVICE4MPU', time4xi()) 
#   print('(Step 3A/33) OF MPU2IMAGE4XI5TEST - TRYING TO CREATE CONTEXT4MPU ', time4xi())    
#   print('=' *33)        
#   try:
#       context2mpu = create2context4mpu()                                                                               
#       print('(Step 3B/33) OF MPU2IMAGE4XI5TEST - CREATED CONTEXT4MPU: ', context2mpu)                                  
#   except:
#       print('(Step 3Z/33) OF MPU2IMAGE4XI5TEST *** FAILED TO CREATE CONTEXT4MPU', time4xi()) 
#          
#   task4set2image = {}
#   task4set2image  ['CONTEXT'] = context2mpu  
# 
#   ... other data for set operation ...
#    
#        
#   response4set2Dimage = {} 
#   print('(Step 11A/700) OF MPU2IMAGE4XI5TEST - TRYING TO SET 2D IMAGE4MPU ', time4xi())   
#   try:                               
#       response4set2Dimage = set2image4mpu(task4set2image)                                                                                                         
#                                                                                     
#       print('(Step 11B/700) OF MPU2IMAGE4XI5TEST - SET 2D IMAGE4MPU ', time4xi())
#       keys2response = response4set2Dimage.keys()
#       for key2response in sorted(keys2response):
#           print(key2response, ' : ', response4set2Dimage[key2response])
#           # 
#   except:
#       print('(Step 11Z/700) OF MPU2IMAGE4XI5TEST *** FAILED TO SET 2D IMAGE4MPU *** ', time4xi()) 
#
#">

# -------------------------------------------------------------------------------------------------   
def set2image4mpu(task4imageset2mpu=None):

    '''
    sets 2D images for MPU
    '''
    global mpu2image4xi  
    
    # -----------------------------------------------------------------------------------------
    # (Step 1/33) PARAMETERS           
    # -----------------------------------------------------------------------------------------          
    class2print = "set2image4mpu"
    method2print = "set2image4mpu"

    message2print = "{(Step 1A/33) started" + "; set2image4mpu-started: " + str(set2image4mpu)  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                                     #  
    # <function set2image4mpu at 0x0000000003A62848> 
    
    message2print = "{(Step 1B/33) passed" + "; task4imageset2mpu-to-set2image4mpu: " + str(task4imageset2mpu) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                                                              
    # {'CODE2ERROR': 0,
    #  'QUEUE': <queue4xi.Queue4Xi object at 0x0000000003D4A710>,
    #  'CONTEXT': {'COUNT2REFERENCE': 0,
    #              'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2context4mpu-failed ; time: 07:13:59 09/06/11 Pacific Daylight Time}',
    #              'CLASS': 'create2context4mpu',
    #              'METHOD': 'create2context4mpu'},
    #              'FUNCTION2CALLBACK': <function trace2context4mpu at 0x0000000003770148>,
    #              'ACK2RETURN': ' 2A-2C-2D-2E-2B-3B-4B-5B-6B-7B-7B-',
    #              'INFO2PRIVATE': None,
    #              'PLATFORM': {'list2platforms': ['DragonXi'], 'ack2return': ' 2C-2E-2F-2G-', 'nack2return': ' ', 'available2platform': 1, 'max2entries': 1}, 
    #              'NACK2RETURN': ' ',
    #              'SIZE2PRIVATE': 0,
    #              'PROPERTIES': None},
    #  'COMMANDQUEUE': {'CODE2ERROR': 0,
    #              'COUNT2REFERENCE': 0,
    #              'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2commandqueue4mpu-failed ; time: 07:13:59 09/06/11 Pacific Daylight Time}',
    #              'CLASS': 'create2commandqueue4mpu',
    #              'METHOD': 'create2commandqueue4mpu'},
    #  'FUNCTION2CALLBACK': <function trace2commandqueue4mpu at 0x00000000037BC848>,
    #  'ACK2RETURN': ' 1D-1E-2C-2B-3B-4B-8A-9B-10B-9B-10B-11B-12B-13B-14B-15B-16B-17B-18B-',
    #  'INFO2PRIVATE': None,
    #  'QUEUE': <queue4xi.Queue4Xi object at 0x0000000003EDA780>,
    #  'NACK2RETURN': ' ',
    #  'CONTEXT': {'COUNT2REFERENCE': 0,
    #              'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2context4mpu-failed ; time: 07:13:59 09/06/11 Pacific Daylight Time}',
    #              'CLASS': 'create2context4mpu',
    #              'METHOD': 'create2context4mpu'},
    #              'FUNCTION2CALLBACK': <function trace2context4mpu at 0x0000000003770148>,
    #              'ACK2RETURN': ' 2A-2C-2D-2E-2B-3B-4B-5B-6B-7B-7B-',
    #              'INFO2PRIVATE': None,
    #              'PLATFORM': {'list2platforms': ['DragonXi'],
    #                           'ack2return': ' 2C-2E-2F-2G-',
    #                           'nack2return': ' ', 'available2platform': 1, 'max2entries': 1},
    #              'NACK2RETURN': ' ',
    #              'SIZE2PRIVATE': 0,
    #              'PROPERTIES': None},
    #              'DEVICE': {'RESOLUTION2TIMER4PROFILING': 318,
    #                          'ID2PLATFORM': '0000000001509568',
    #  'PROPERTIES': None,
    #  'PIPE': (<read-write PipeConnection, handle 552>, <read-write PipeConnection, handle 556>)},
    #  'DEVICE': {'RESOLUTION2TIMER4PROFILING': 318, 'ID2PLATFORM': '0000000001509568', 'PROFILING2QUEUE': 'ACK', 'EXECUTE2KERNELS4OPENCL': 'ACK',
    #              'NATIVE2WIDTHLONG4VECTOR': 2, 'MAX2UNITS4COMPUTE': 4, 'SIZE2MEMORY4GLOBAL': 3221225472, 'NATIVE2WIDTHFLOAT4VECTOR': 4, 'MAX2SIZE4WORKGROUP': 1024, 'PREFERRED2WIDTHCHAR4VECTOR': 16, 'SIZE2CACHE': 65536, 
    #              'IEEE754-2008FUSED2MULTIPLY-ADD': 'NACK', 'TYPE4CACHE': 'Read/Write', 'PREFERRED2WIDTHINT4VECTOR': 4, 'SIZE2PREFERREDWORKGROUPMULTIPLE4KERNEL': 1, 'NATIVE2WIDTHINT4VECTOR': 4, 
    #              'SIZE2BUFFER4CONSTANT': 65536, 'EXECUTE2FUNCTION4NATIVE': 'ACK', 'PREFERRED2WIDTHDOUBLE4VECTOR': 0, 'MAX2NUMBER4CONSTANTARGS': 8, 'VERSION2DRIVER': '2.0', 
            
    code2ack  = ' '
    code2nack = ' '
    # -----------------------------------------------------------------------------------------
    # CREATE IF NECESSARY       
    # ----------------------------------------------------------------------------------------- 
    try:
        if mpu2image4xi is None:
            mpu2image4xi = create2image4mpu()                                                                              # INITIALIZE IMAGE FOR MPU 
            message2print = "{(Step 1C/70) set" + "; mpu2image4xi-set-by-set2image4mpu to: " + str(mpu2image4xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #             
        else:
            message2print = "{(Step 1D/70) skipped" + "; create2image4mpu-skipped-by-set2image4mpu: " + str(create2image4mpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            # <function create2image4mpu at 0x0000000003A627C8>;                                  
    except:
        try:
            mpu2image4xi = create2image4mpu()
            message2print = "{(Step 1E/70) set" + "; mpu2image4xi-set-by-set2image4mpu to: " + str(mpu2image4xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #             
        except:
            mpu2image4xi = {}
            message2print = "{(Step 1Z/70) yielded" + "; mpu2image4xi-yielded-by-set2image4mpu: " + str(mpu2image4xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            code2nack  = code2nack + '1Z-' 
                
    # ---------------------------------------------------------------------------------------------
    # SET            
    # ---------------------------------------------------------------------------------------------     
    message2print = "{(Step 22A0/70) try" + "; data2callback-try-by-set2image4mpu: None" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)        
    #     
    try:
        # -----------------------------------------------------------------------------------------
        # (Step 21A/70) - DATA2CALLBACK - EXTRA
        # -----------------------------------------------------------------------------------------               
        mpu2image4xi['DATA2CALLBACK'] = task4imageset2mpu['DATA2CALLBACK']
                                        
        message2print = "{(Step 21A1/70) set" + "; data2callback-set-by-set2image4mpu to: " + str(mpu2image4xi['DATA2CALLBACK'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # 
        
        code2ack  = code2ack + '21A1-'
    except: 
        message2print = "{(Step 21Z1/70) yielded" + "; data2callback-mpu2image4xi-yielded-by-set2image4mpu: " + str(mpu2image4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # yielded
        code2nack  = code2nack + '21Z1-'
    
    message2print = "{(Step 22B0/70) try" + "; function2callback-try-by-set2image4mpu: None" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)        
    # 
    try:
        # -----------------------------------------------------------------------------------------
        # (Step 22B/70) - FUNCTION2CALLBACK - EXTRA
        # -----------------------------------------------------------------------------------------               
        mpu2image4xi['FUNCTION2CALLBACK'] = task4imageset2mpu['FUNCTION2CALLBACK']
        
        message2print = "{(Step 22B1/70) set" + "; function2callback-set-by-set2image4mpu to: " + str(mpu2image4xi['FUNCTION2CALLBACK'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        #  
        code2ack  = code2ack + '22B1-'
    except: 
        message2print = "{(Step 22Z2/70) yielded" + "; function2callback-mpu2image4xi-yielded-by-set2image4mpu: " + str(mpu2image4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # yielded
        code2nack  = code2nack + '22Z2-' 
    
    message2print = "{(Step 22C0/70) try" + "; platform2callback-try-by-set2image4mpu: None" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)        
    # 
    try:
        # -----------------------------------------------------------------------------------------
        # (Step 22C/70) - PLATFORM - EXTRA
        # -----------------------------------------------------------------------------------------               
        mpu2image4xi['PLATFORM'] = task4imageset2mpu['PLATFORM']
        
        message2print = "{(Step 2C1/70) set" + "; platform-set-by-set2image4mpu to: " + str(mpu2image4xi['PLATFORM'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # yielded  
        code2ack  = code2ack + '2C1-'
    except: 
        message2print = "{(Step 22Z3/70) yielded" + "; platform-mpu2image4xi-yielded-by-set2image4mpu: " + str(mpu2image4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        code2nack  = code2nack + '22Z3-' 
        
        
      
    message2print = "{(Step 23A/70) try" + "; context2mpu-try-by-set2image4mpu: {}" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)       
    try:
        # -----------------------------------------------------------------------------------------
        # (Step 23/70) - CONTEXT - REQUIRED
        # -----------------------------------------------------------------------------------------               
        mpu2image4xi['CONTEXT'] = task4imageset2mpu['CONTEXT']
        
        message2print = "{(Step 23B/70) set" + "; context-set-by-set2image4mpu to: " + str(mpu2image4xi['CONTEXT'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # {'COUNT2REFERENCE': 0,
        #  'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2context4mpu-failed ; time: 07:13:59 09/06/11 Pacific Daylight Time}',
        #                    'CLASS': 'create2context4mpu',
        #                    'METHOD': 'create2context4mpu'},
        #  'FUNCTION2CALLBACK': <function trace2context4mpu at 0x0000000003770148>,
        #  'ACK2RETURN': ' 2A-2C-2D-2E-2B-3B-4B-5B-6B-7B-7B-',
        #  'INFO2PRIVATE': None,
        #  'PLATFORM': {'list2platforms': ['DragonXi'], 'ack2return': ' 2C-2E-2F-2G-', 'nack2return': ' ', 'available2platform': 1, 'max2entries': 1}, 'NACK2RETURN': ' ',
        #  'SIZE2PRIVATE': 0,
        #  'PROPERTIES': None}; time: 07:13:59 09/06/11 Pacific Daylight Time} 

        
        code2ack  = code2ack + '23B-'
    except: 
        message2print = "{(Step 23Z1/70) yielded" + "; context-mpu2image4xi-yielded-by-set2image4mpu: " + str(mpu2image4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
    
        code2nack  = code2nack + '23Z1-' 
        
    message2print = "{(Step 24A/70) try" + "; device-try-by-set2image4mpu: {}" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)    
    try:
        # -----------------------------------------------------------------------------------------
        # (Step 24/70) - DEVICE - EXTRA
        # -----------------------------------------------------------------------------------------               
        mpu2image4xi['DEVICE'] = task4imageset2mpu['DEVICE']
        
        message2print = "{(Step 24B/70) set" + "; device-set-by-set2image4mpu to: " + str(mpu2image4xi['DEVICE'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # {'RESOLUTION2TIMER4PROFILING': 318,
        #  'ID2PLATFORM': '0000000001509568',
        #  'PROFILING2QUEUE': 'ACK',
        #  'EXECUTE2KERNELS4OPENCL': 'ACK',
        #  'NATIVE2WIDTHLONG4VECTOR': 2,
        #  'MAX2UNITS4COMPUTE': 4,
        #  'SIZE2MEMORY4GLOBAL': 3221225472,
        #  'NATIVE2WIDTHFLOAT4VECTOR': 4,
        #  'MAX2SIZE4WORKGROUP': 1024,
        #  'PREFERRED2WIDTHCHAR4VECTOR': 16,
        #  'SIZE2CACHE': 65536,
        #  'IEEE754-2008FUSED2MULTIPLY-ADD': 'NACK',
        #  'TYPE4CACHE': 'Read/Write',
        #  'PREFERRED2WIDTHINT4VECTOR': 4,
        #  'SIZE2PREFERREDWORKGROUPMULTIPLE4KERNEL': 1,
        #  'NATIVE2WIDTHINT4VECTOR': 4,
        #  'SIZE2BUFFER4CONSTANT': 65536,
        #  'EXECUTE2FUNCTION4NATIVE': 'ACK',
        #  'PREFERRED2WIDTHDOUBLE4VECTOR': 0,
        #  'MAX2NUMBER4CONSTANTARGS': 8,
        #  'VERSION2DRIVER': '2.0',
        #  'EXTENSIONS': 'cl_amd_fp64 cl_khr_global_int32_base_atomics ...
        #  'AVAILABLE2COMPILER': 'ACK',
        #  'MAX2DIMENSIONS4WORKITEMS': 3,
        #  'SUPPORT2CORRECTION4ERROR': 0,
        #  'BITS2ADDRESS': 64,
        #  'AVAILABLE': 'ACK',
        #  'SIZE2MEMORY4LOCAL': 32768,
        #  'VENDOR': 'AuthenticAMD',
        #  'SUPPORT2IMAGE': 'NACK',
        #  'MAX2SIZE4KERNELARGUMENT': 4096,
        #  'ACK2RETURN': ' 1C-1D-1E-1F-1D-2A-3A-4A-5A-6A-7A-8A-9A-10A-11A-...
        #  'OUT-OF-ORDER2QUEUE': 'NACK',
        #  'ALIGNMENT2ADDRESS4BASE': 1024,
        #  'PROFILE': 'FULL_PROFILE',
        #  'TYPE2MEMORY4LOCAL': 'Global',
        #  'PREFERRED2WIDTHFLOAT4VECTOR': 4,
        #  'PREFERRED2WIDTHLONG4VECTOR': 2,
        #  'MAX2FREQUENCY4CLOCK': 3210,
        #  'PREFERRED2WIDTHSHORT4VECTOR': 8,
        #  'ID': 4033,
        #  'MAX2WORKITEM2': 1024,
        #  'NATIVE2WIDTHDOUBLE4VECTOR': 0,
        #  'ROUND2ZERO': 'ACK',
        #  'NAME': 'AMD64 Family 16 Model 4 Stepping 2, AuthenticAMD',
        #  'MAX2WORKITEM1': 1024,
        #  'MAX2WORKITEM0': 1024,
        #  'SIZE2LINE4CACHE': 64,
        #  'MAX2ALLOCATION4MEMORY': 1073741824,
        #  'NATIVE2WIDTHCHAR4VECTOR': 16,
        #  'QUIET2NANS': 'ACK',
        #  'NATIVE2WIDTHSHORT4VECTOR': 8,
        #  'ROUND2NEAREST4EVEN': 'ACK',
        #  'VERSION': 'OpenCL 1.1 ATI-Stream-v2.3 (451)',
        #  'NACK2RETURN': ' ',
        #  'MEMORY2UNIFIED4HOSTANDDEVICE': 1,
        #  'ENDIANESS2DEVICE': 'Little',
        #  'MINIMUM2ALIGNMENT4DATATYPE': 128,
        #  'ROUND2+VE3INFINITY': 'ACK',
        #  'TYPE': 'CL_DEVICE_TYPE_MPU',
        #  'DENORMS2SINGLEPRECISION4FLOATINGPOINTCAPABILITY': 'ACK'}; time: 07:13:59 09/06/11 Pacific Daylight Time}

        
        code2ack  = code2ack + '24B-'
    except: 
        message2print = "{(Step 24Z1/70) yielded" + "; device-mpu2image4xi-yielded-by-set2image4mpu: " + str(mpu2image4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        code2nack  = code2nack + '24Z1-'
                 
    message2print = "{(Step 25A/70) try" + "; code2error-try-by-set2image4mpu: None" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print) 
    try:
        # -----------------------------------------------------------------------------------------
        # (Step 25/70) - ERROR CODE - REQUIRED
        # -----------------------------------------------------------------------------------------               
        mpu2image4xi['CODE2ERROR'] = task4imageset2mpu['CODE2ERROR']
        
        message2print = "{(Step 25B/70) set" + "; code2error-set-by-set2image4mpu to: " + str(mpu2image4xi['CODE2ERROR'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # 0 
        code2ack  = code2ack + '25B-'
    except: 
        message2print = "{(Step 25Z1/70) yielded" + "; code2error-mpu2image4xi-yielded-by-set2image4mpu: " + str(mpu2image4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        code2nack  = code2nack + '25Z1-'                
                
          
    message2print = "{(Step 26A/70) try" + "; queue-try-by-set2image4mpu: None" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                
    try:
        # -----------------------------------------------------------------------------------------
        # (Step 26/70) - QUEUE - EXTRA
        # -----------------------------------------------------------------------------------------               
        mpu2image4xi['QUEUE'] = task4imageset2mpu['QUEUE']
        
        message2print = "{(Step 26B/70) set" + "; queue-set-by-set2image4mpu to: " + str(mpu2image4xi['QUEUE'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        # <queue4xi.Queue4Xi object at 0x0000000003D4A710> 
        code2ack  = code2ack + '26B-'
    except: 
        message2print = "{(Step 26Z1/70) yielded" + "; queue-mpu2image4xi-yielded-by-set2image4mpu: " + str(mpu2image4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        code2nack  = code2nack + '26Z1-' 
                                
    message2print = "{(Step 27A/70) try" + "; pipe-try-by-set2image4mpu: None" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                     
    try:
        # -----------------------------------------------------------------------------------------
        # (Step 27/70) - PIPE - EXTRA
        # -----------------------------------------------------------------------------------------               
        mpu2image4xi['PIPE'] = task4imageset2mpu['PIPE']
        
        message2print = "{(Step 27B/70) set" + "; pipe-set-by-set2image4mpu to: " + str(mpu2image4xi['PIPE'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        # (<read-write PipeConnection, handle 520>, <read-write PipeConnection, handle 524>)  
        
        code2ack  = code2ack + '27B-'
    except: 
        message2print = "{(Step 27Z1/70) yielded" + "; pipe-mpu2image4xi-yielded-by-set2image4mpu: " + str(mpu2image4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        code2nack  = code2nack + '27Z1-'   

    message2print = "{(Step 28A/70) try" + "; commandqueue-try-by-set2image4mpu: {}" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)        
    try:            
        # ---------------------------------------------------------------------------------------------
        # (28) COMMAND QUEUE - EXTRA                              
        # ---------------------------------------------------------------------------------------------             
                        
        mpu2image4xi['COMMANDQUEUE']  = task4imageset2mpu['COMMANDQUEUE']  
 
        message2print = "{(Step 28B/70) set" + "; commandqueue-set-by-set2image4mpu to: " + str(mpu2image4xi['COMMANDQUEUE'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        # {'CODE2ERROR': 0, 'COUNT2REFERENCE': 0, 'DATA2CALLBACK': {'MESSAGE': ...
        
        code2ack  = code2ack + '28B-'
    except: 
        message2print = "{(Step 28Z1/70) yielded" + "; commandqueue-yielded-by-set2image4mpu: " + str(mpu2image4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        code2nack  = code2nack + '28Z1-'    

    message2print = "{(Step 29A/70) try" + "; bufferinfo-try-by-set2image4mpu: {}" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)        
    try:            
        # ---------------------------------------------------------------------------------------------
        # (29) BUFFER INFO - EXTRA                              
        # ---------------------------------------------------------------------------------------------             
                        
        mpu2image4xi['BUFFER']  = task4imageset2mpu['BUFFER']  
 
        message2print = "{(Step 29B/70) set" + "; bufferinfo-set-by-set2image4mpu to: " + str(mpu2image4xi['BUFFER'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        #  
        code2ack  = code2ack + '29B-'
    except: 
        message2print = "{(Step 29Z1/70) yielded" + "; bufferinfo-yielded-by-set2image4mpu: " + str(mpu2image4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # yielded
        code2nack  = code2nack + '29Z1-'            

    message2print = "{(Step 30A/70) try" + "; eventinfo-try-by-set2image4mpu: {}" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)        
    try:            
        # ---------------------------------------------------------------------------------------------
        # (30) EVENT INFO - EXTRA                              
        # ---------------------------------------------------------------------------------------------             
                        
        mpu2image4xi['EVENT']  = task4imageset2mpu['EVENT']  
 
        message2print = "{(Step 30B/70) set" + "; eventinfo-set-by-set2image4mpu to: " + str(mpu2image4xi['EVENT'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        # 
        code2ack  = code2ack + '30B-'
    except: 
        message2print = "{(Step 30Z1/70) yielded" + "; eventinfo-yielded-by-set2image4mpu: " + str(mpu2image4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # yielded
        
        code2nack  = code2nack + '30Z1-'            
                

    message2print = "{(Step 31A/70) try" + "; flags-try-by-set2image4mpu: None" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)        
    try:            
        # ---------------------------------------------------------------------------------------------
        # (31) FLAGS - REQUIRED                              
        # ---------------------------------------------------------------------------------------------             
                        
        mpu2image4xi['FLAGS']  = task4imageset2mpu['FLAGS']  
 
        message2print = "{(Step 31B/70) set" + "; flags-set-by-set2image4mpu to: " + str(mpu2image4xi['FLAGS'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        # 
        code2ack  = code2ack + '31B-'
    except: 
        message2print = "{(Step 31Z1/70) yielded" + "; flags-yielded-by-set2image4mpu: " + str(mpu2image4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # yielded
        
        code2nack  = code2nack + '31Z1-'     

    message2print = "{(Step 34A/70) try" + "; width-try-by-set2image4mpu: None" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)        
    try:            
        # ---------------------------------------------------------------------------------------------
        # (34) WIDTH - REQUIRED                              
        # ---------------------------------------------------------------------------------------------      
        mpu2image4xi['WIDTH']  = task4imageset2mpu['WIDTH']  
 
        message2print = "{(Step 34H/70) set" + "; width-set-by-set2image4mpu to: " + str(mpu2image4xi['WIDTH'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        # 
        code2ack  = code2ack + '34H-'
    except: 
        message2print = "{(Step 34Z1/70) yielded" + "; width-yielded-by-set2image4mpu: " + str(mpu2image4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        code2nack  = code2nack + '34Z1-'   


    message2print = "{(Step 35A/70) try" + "; height-try-by-set2image4mpu: None"  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)        
    try:            
        # ---------------------------------------------------------------------------------------------
        # (35) HEIGHT - REQUIRED                              
        # ---------------------------------------------------------------------------------------------      
        mpu2image4xi['HEIGHT']  = task4imageset2mpu['HEIGHT']  
 
        message2print = "{(Step 35H/70) set" + "; height-set-by-set2image4mpu to: " + str(mpu2image4xi['HEIGHT'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        #  
        code2ack  = code2ack + '35H-'
    except: 
        message2print = "{(Step 35Z1/70) yielded" + "; height-yielded-by-set2image4mpu: " + str(mpu2image4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # yielded 
        code2nack  = code2nack + '35Z1-'   
 

    message2print = "{(Step 36A/70) try" + "; rowpitch-try-by-set2image4mpu: None" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)        
    try:            
        # ---------------------------------------------------------------------------------------------
        # (36) ROWPITCH - REQUIRED                              
        # ---------------------------------------------------------------------------------------------             
                        
        mpu2image4xi['ROWPITCH']  = task4imageset2mpu['ROWPITCH']  
 
        message2print = "{(Step 36B/70) set" + "; rowpitch-set-by-set2image4mpu to: " + str(mpu2image4xi['ROWPITCH'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        # 
        code2ack  = code2ack + '36B-'
    except: 
        message2print = "{(Step 36Z1/70) yielded" + "; rowpitch-yielded-by-set2image4mpu: " + str(mpu2image4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # yielded 
        code2nack  = code2nack + '36Z1-'

    message2print = "{(Step 37A/70) try" + "; pointer-try-by-set2image4mpu: None" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)        
    try:            
        # ---------------------------------------------------------------------------------------------
        # (37) POINTER - REQUIRED                              
        # ---------------------------------------------------------------------------------------------             
                        
        mpu2image4xi['POINTER']  = task4imageset2mpu['POINTER']  
 
        message2print = "{(Step 37B/70) set" + "; pointer-set-by-set2image4mpu to: " + str(mpu2image4xi['POINTER'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        # 
        code2ack  = code2ack + '37B-'
    except: 
        message2print = "{(Step 37Z1/70) yielded" + "; pointer-yielded-by-set2image4mpu: " + str(mpu2image4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # yielded 
        code2nack  = code2nack + '37Z1-'

    message2print = "{(Step 38A/70) try" + "; format-try-by-set2image4mpu: None" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)        
    try:            
        # ---------------------------------------------------------------------------------------------
        # (38) FORMAT - REQUIRED                              
        # ---------------------------------------------------------------------------------------------             
                        
        mpu2image4xi['FORMAT']  = task4imageset2mpu['FORMAT']  
 
        message2print = "{(Step 38B/70) set" + "; format-set-by-set2image4mpu to: " + str(mpu2image4xi['FORMAT'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        # 
        code2ack  = code2ack + '38B-'
    except: 
        message2print = "{(Step 38Z1/70) yielded" + "; format-yielded-by-set2image4mpu: " + str(mpu2image4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # yielded 
        code2nack  = code2nack + '38Z1-'
                        
    # -----------------------------------------------------------------------------------------
    # (Step 70/70)           
    # -----------------------------------------------------------------------------------------
    mpu2image4xi ['ACK2RETURN']  = code2ack
    mpu2image4xi ['NACK2RETURN'] = code2nack 
    
    message2print = "{(Step 70/70) returning" + "; mpu2image4xi-returning-by-set2image4mpu: " + str(mpu2image4xi) + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                             
    # {'CODE2ERROR': 0, 'ACK2RETURN': ' 23B-24B-25B-26B-27B-28B-', 'ROWPITCH': 800, 'DATA2CALLBACK': ...
    
    
    return mpu2image4xi

           
#</function4xi> 


# -------------------------------------------------------------------------------------------------
#<function4xi function = "copy2image4mpu" usage = "
#                                                                                                              
#   # -----------------------------------------------------------------------------------------------
#   # (16) ESTABLISH CONTROL & COMMAND WITH COPY2IMAGE4MPU                      
#   # -----------------------------------------------------------------------------------------------    
#   print('(Step 16A/700) OF MPU2IMAGE4XI5TEST - TRYING TO ESTABLISH CONTROL & COMMAND WITH COPY2IMAGE4MPU ', time4xi())    
#   response4copy2image = {} 
#   task4copy2image = None
#   try:                               
#       response4copy2image = copy2image4mpu(task4copy2image)                                                                                                          
#       print('(Step 16B/700) OF MPU2IMAGE4XI5TEST - ESTABLISHED CONTROL & COMMAND WITH COPY2IMAGE4MPU : ', response4copy2image)
#       queue2copy4mpu      = response4copy2image['QUEUE']
#       channel2copypublish4mpu = response4copy2image['PUBLISHERCHANNEL']
#       channel2copyconsume4mpu = response4copy2image['CONSUMERCHANNEL']
#   except:
#       print('(Step 16Z/700) OF MPU2IMAGE4XI5TEST *** FAILED TO ESTABLISH CONTROL & COMMAND WITH COPY2IMAGE4MPU ', time4xi()) 
#   # -----------------------------------------------------------------------------------------------
#   # (17) SEND CONTROL & COMMAND COPY VIA PIPE FOR MANOEUVRE2IMAGE4MPU TO RECEIVE                 
#   # -----------------------------------------------------------------------------------------------      
#   print('(Step 17A/700) OF MPU2IMAGE4XI5TEST - TRYING TO SEND CONTROL & COMMAND VIA PIPE WITH COPY2IMAGE4MPU ', time4xi())     
#   try:                               
#       message2copyimagesend4mpu = "<copy4xi>copy2copy4mpu</copy4xi>"
#       channel2copypublish4mpu = response4copy2image['PUBLISHERCHANNEL']
#       channel2copypublish4mpu.send(message2copyimagesend4mpu)
#       print("(Step 17B/700) OF MPU2IMAGE4XI5TEST - SENT TO CHANNEL2PUBLISH4MPU: ", channel2copypublish4mpu, " MESSAGE2SEND4MPU: " , message2copyimagesend4mpu)                                                                                                
#   except:
#       print('(Step 17Z1/700) OF MPU2IMAGE4XI5TEST *** FAILED TO SEND CONTROL & COMMAND VIA PIPE WITH COPY2IMAGE4MPU ', time4xi())                                                
#   # -----------------------------------------------------------------------------------------------
#   # (18) LET MANOEUVRE2IMAGE4MPU TO RECEIVE COPY AND AND SEND ROGER VIA PIPE                
#   # -----------------------------------------------------------------------------------------------       
#   print("(Step 18C/700) OF MPU2IMAGE4XI5TEST - TRYING TO PERFORM TARGET FUNCTION MANOEUVRE2IMAGE4MPU ", time4xi()) 
#   try:        
#       response2manoeuvre4mpu = manoeuvre2copyimage4mpu(queue2copy4mpu, channel2copyconsume4mpu, channel2copypublish4mpu)
#       print("(Step 18D/700) OF MPU2IMAGE4XI5TEST - PERFORMED TARGET FUNCTION MANOEUVRE2IMAGE4MPU ", manoeuvre2copyimage4mpu)
#   except:
#       print("(Step 18Z/700) OF MPU2IMAGE4XI5TEST *** FAILED TO PERFORM TARGET FUNCTION MANOEUVRE2IMAGE4MPU ", time4xi())     
#   else:
#       pass # trust activated thread    
#   # -----------------------------------------------------------------------------------------------
#   # (19) RECEIVE CONTROL & COMMAND ROGER VIA PIPE                   
#   # -----------------------------------------------------------------------------------------------          
#   print('(Step 19A/700) OF MPU2IMAGE4XI5TEST - TRYING TO RECEIVE CONTROL & COMMAND VIA PIPE WITH COPY2IMAGE4MPU ', time4xi())     
#   try:
#       channel2copyconsume4mpu = response4copy2image['CONSUMERCHANNEL']                              
#       message2copyimagereceive4mpu = channel2copyconsume4mpu.recv()
#       print("(Step 19B/700) OF MPU2IMAGE4XI5TEST - RECEIVED FROM CHANNEL2CONSUME4MPU: ", channel2copyconsume4mpu, " CONTROL & COMMAND: ", message2copyimagereceive4mpu)                                                                                                  
#       # <roger4xi>roger2copy4mpu</roger4xi>
#   except:
#       print('(Step 19Z/700) OF MPU2IMAGE4XI5TEST *** FAILED TO RECEIVE CONTROL & COMMAND VIA PIPE  WITH COPY2IMAGE4MPU ', time4xi())         
#   # -----------------------------------------------------------------------------------------------
#   # (20) SEND CONTROL & COMMAND READY VIA PIPE FOR MANOEUVRE2IMAGE4MPU TO RECEIVE                
#   # -----------------------------------------------------------------------------------------------      
#   print('(Step 20A/700) OF MPU2IMAGE4XI5TEST - TRYING TO SEND CONTROL & COMMAND VIA PIPE WITH COPY2IMAGE4MPU ', time4xi())     
#   try:                               
#       message2copyimagesend4mpu = "<copy4xi>copy2copy4mpu</copy4xi>"
#       channel2copypublish4mpu = response4copy2image['PUBLISHERCHANNEL']
#       channel2copypublish4mpu.send(message2copyimagesend4mpu)
#       print("(Step 20B/700) OF MPU2IMAGE4XI5TEST - SENT TO CHANNEL2PUBLISH4MPU: ", channel2copypublish4mpu, " MESSAGE2SEND4MPU: " , message2copyimagesend4mpu)                                                                                                
#       # <copy4xi>copy2copy4mpu</copy4xi> 
#   except:
#        print('(Step 20Z/700) OF MPU2IMAGE4XI5TEST *** FAILED TO SEND CONTROL & COMMAND VIA PIPE WITH COPY2IMAGE4MPU ', time4xi())                                                
#   # -----------------------------------------------------------------------------------------------
#   # (21) LET MANOEUVRE2IMAGE4MPU TO RECEIVE READY AND TO SEND READY                
#   # -----------------------------------------------------------------------------------------------       
#   print("(Step 21A/700) OF MPU2IMAGE4XI5TEST - TRYING TO PERFORM TARGET FUNCTION MANOEUVRE2IMAGE4MPU ", time4xi()) 
#   try:        
#       response2manoeuvre4mpu = manoeuvre2copyimage4mpu(queue2copy4mpu, channel2copyconsume4mpu, channel2copypublish4mpu)
#       print("(Step 21B/700) OF MPU2IMAGE4XI5TEST - PERFORMED TARGET FUNCTION MANOEUVRE2IMAGE4MPU ", manoeuvre2copyimage4mpu)
#   except:
#       print("(Step 21Z/700) OF MPU2IMAGE4XI5TEST *** FAILED TO PERFORM TARGET FUNCTION MANOEUVRE2IMAGE4MPU ", time4xi())     
#   # -----------------------------------------------------------------------------------------------
#   # (22) RECEIVE CONTROL & COMMAND READY VIA PIPE                      
#   # -----------------------------------------------------------------------------------------------          
#   print('(Step 22A/700) OF MPU2IMAGE4XI5TEST - TRYING TO RECEIVE CONTROL & COMMAND VIA PIPE WITH COPY2IMAGE4MPU ', time4xi())     
#   try:
#       channel2copyconsume4mpu = response4copy2image['CONSUMERCHANNEL']                              
#       message2copyimagereceive4mpu = channel2copyconsume4mpu.recv()
#       print("(Step 22B/700) OF MPU2IMAGE4XI5TEST - RECEIVED FROM CHANNEL2CONSUME4MPU: ", channel2copyconsume4mpu, " CONTROL & COMMAND: ", message2copyimagereceive4mpu)                                                                                                  
#       # <copy4xi>copy2copy4mpu</copy4xi> 
#   except:
#       print('(Step 22Z/700) OF MPU2IMAGE4XI5TEST *** FAILED TO RECEIVE CONTROL & COMMAND VIA PIPE  WITH COPY2IMAGE4MPU ', time4xi())          
#   # ----------------------------------------------------------------------------------------------
#   # (24) PUT IMAGE PATH TO QUEUE
#   # ----------------------------------------------------------------------------------------------                          
#   print("(Step 24A/700) OF MPU2IMAGE4XI5TEST - TRYING TO PUT IMAGE PATH TO QUEUE ", time4xi())      
#   try:    
#       queue2copy4dragon.put(path2dragon)             
#       print("(Step 24B/700) OF DRAGON2IMAGE4XI5TEST - PUT TO QUEUE IMAGE PATH: ", path2dragon) 
#       # ..//..//..//..//..//web//images//png//dragonxi.png  
#   except:
#       print("(Step 24Z/700) OF MPU2IMAGE4XI5TEST *** FAILED TO PUT IMAGE PATH TO QUEUE ", time4xi()) 
#   # -----------------------------------------------------------------------------------------------
#   # (25) LET MANOEUVRE2IMAGE4MPU TO GET FROM QUEUE AND TO SEND EITHER ACK OR NACK VIA PIPE               
#   # -----------------------------------------------------------------------------------------------       
#   print("(Step 25A/700) OF MPU2IMAGE4XI5TEST - TRYING TO PERFORM TARGET FUNCTION MANOEUVRE2IMAGE4MPU ", time4xi()) 
#   try:        
#       response2manoeuvre4mpu = manoeuvre2copyimage4mpu(queue2copy4mpu, channel2copyconsume4mpu, channel2copypublish4mpu)
#       print("(Step 25B/700) OF MPU2IMAGE4XI5TEST - PERFORMED TARGET FUNCTION MANOEUVRE2IMAGE4MPU ", manoeuvre2copyimage4mpu)
#   except:
#       print("(Step 25Z/700) OF MPU2IMAGE4XI5TEST *** FAILED TO PERFORM TARGET FUNCTION MANOEUVRE2IMAGE4MPU ", time4xi())     
#   # -----------------------------------------------------------------------------------------------
#   # (26) RECEIVE ACK OR NACK ABOUT IMAGE OPERATION VIA PIPE                
#   # -----------------------------------------------------------------------------------------------          
#   print('(Step 26A/700) OF MPU2IMAGE4XI5TEST - TRYING TO RECEIVE CONTROL & COMMAND ABOUT IMAGE OPERTION VIA PIPE ', time4xi())      
#   try: 
#       message2copyimagereceive4mpu = channel2copyconsume4mpu.recv()
#       print("(Step 26D/700) OF MPU2IMAGE4XI5TEST - RECEIVED FROM CHANNEL2CONSUME4MPU: ", channel2copyconsume4mpu, " CONTROL & COMMAND: ", message2copyimagereceive4mpu)                                                                                                   
#       # <ack4xi>ack2imageinfo4mpu</ack4xi>
#   except:
#       print('(Step 26Z/700) OF MPU2IMAGE4XI5TEST *** FAILED TO RECEIVE CONTROL & COMMAND ABOUT IMAGE OPERTION VIA PIPE ', time4xi())               
#   # -----------------------------------------------------------------------------------------------
#   (27) LET MANOEUVRE2IMAGE4MPU GO TO STATE 0               
#   # -----------------------------------------------------------------------------------------------       
#   print("(Step 27A/700) OF MPU2IMAGE4XI5TEST - TRYING TO PERFORM TARGET FUNCTION MANOEUVRE2IMAGE4MPU ", time4xi()) 
#   try:        
#       response2manoeuvre4mpu = manoeuvre2copyimage4mpu(queue2copy4mpu, channel2copyconsume4mpu, channel2copypublish4mpu)
#       print("(Step 27B/700) OF MPU2IMAGE4XI5TEST - GOT RESPONSE2MANOEUVRE4MPU ", response2manoeuvre4mpu)
#   except:
#       print("(Step 27Z/700) OF MPU2IMAGE4XI5TEST *** FAILED TO PERFORM TARGET FUNCTION MANOEUVRE2IMAGE4MPU ", time4xi())   

#">

# -------------------------------------------------------------------------------------------------   
def copy2image4mpu(task4imagecopy2mpu=None):

    '''
    copies 2D images for MPU
    '''
    global mpu2image4xi                                                                                             # 2D-IMAGES FOR MPU  
    global control2copyimage4mpu                                                                                    # COMMAND AND CONTROL FOR MPU IMAGES 
    
    # -----------------------------------------------------------------------------------------
    # (Step 1/33) PARAMETERS           
    # -----------------------------------------------------------------------------------------          
    class2print = "copy2image4mpu"
    method2print = "copy2image4mpu"

    message2print = "{(Step 0A/33) started" + "; copy2image4mpu-started: " + str(copy2image4mpu)  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                               
    # <function copy2image4mpu at 0x00000000038538C8>
    
    message2print = "{(Step 0B/33) passed" + "; task4imagecopy2mpu-to-copy2image4mpu: " + str(task4imagecopy2mpu) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                                                              
    # None
            
    code2ack  = ' '
    code2nack = ' '
    
    # -----------------------------------------------------------------------------------------
    # CREATE MPU2IMAGE4XI IF NECESSARY       
    # ----------------------------------------------------------------------------------------- 
    try:
        if mpu2image4xi is None:
            mpu2image4xi = create2image4mpu()                                                                       # INITIALIZE IMAGE FOR MPU 
            message2print = "{(Step 0C/70) created" + "; mpu2image4xi-created-by-copy2image4mpu to: " + str(mpu2image4xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #              
        else:
            message2print = "{(Step 0D/70) skipped" + "; create2image4mpu-skipped-by-copy2image4mpu: " + str(copy2image4mpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            # <function copy2image4mpu at 0x00000000038538C8>                                 
    except:
        try:
            mpu2image4xi = create2image4mpu()
            message2print = "{(Step 0E/70) set" + "; mpu2image4xi-set-by-copy2image4mpu to: " + str(mpu2image4xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #             
        except:
            mpu2image4xi = {}
            message2print = "{(Step 0Z/70) yielded" + "; mpu2image4xi-yielded-by-copy2image4mpu: " + str(mpu2image4xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            code2nack  = code2nack + '0Z-' 
    
    # -----------------------------------------------------------------------------------------
    # (Step 1/70) - CHECK PARAMETERS           
    # -----------------------------------------------------------------------------------------          
    try:             
        if task4imagecopy2mpu is None:
            # -----------------------------------------------------------------------------------------
            # NO PARAMETERS GIVEN BY CALLER FOR COPY - BUILD TASK! 
            # -----------------------------------------------------------------------------------------    
            
            task4imagecopy2mpu = {}                                                                                                                                                                                       
            
            message2print = "{(Step 1A/70) initialized" + "; task4imagecopy2mpu-created-by-copy2image4mpu to: " + str(task4imagecopy2mpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)               
            # {} 
            
            # ---------------------------------------------------------------------------------------------
            # (1B) TRACE - AUTOMATIC                      
            # ---------------------------------------------------------------------------------------------              
            function2callback4mpu = trace2image4mpu                                                                 # CALLBACK FUNCTION FOR MPU
            
            message2print = "{(Step 1B1/70) initialized" + "; function2callback4mpu-created-by-copy2image4mpu to: " + str(function2callback4mpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)            
            # <function trace2image4mpu at 0x000000000384E9C8> 
            
            code2ack  = code2ack + '1B1-'
           
            args2callback = {}                                                                                      # CREATE ARGUMENT DICTIONARY FOR CALLBACK FUNCTION             
            args2callback['CLASS']   = "copy2image4mpu"                                                            # MODULE OF MPUIZED OPENCL MODULE
            args2callback['METHOD']  = "copy2image4mpu"                                                            # FUNCTION OF MPUIZED OPENCL MODULE
            args2callback['MESSAGE'] = "{(Step 1Z1/33) failed" + "; copy2image4mpu-failed " + "; time: " + time4xi() + "}" # ERROR MESSAGE FOR MPUIZED OPENCL TO RETURN    

            message2print = "{(Step 1B2/70) initialized" + "; args2callback-created-by-copy2image4mpu to: " + str(args2callback)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)               
            # {'MESSAGE': '{(Step 1Z1/33) failed; copy2image4mpu-failed ; time: 07:01:44 10/04/11 Pacific Daylight Time}',
            #  'CLASS': 'copy2image4mpu',
            #  'METHOD': 'copy2image4mpu'}; time: 07:01:44 10/04/11 Pacific Daylight Time}


            code2ack  = code2ack + '1B2-'
            
            # ---------------------------------------------------------------------------------------------
            # (1C) CALLBACK - AUTOMATIC                      
            # ---------------------------------------------------------------------------------------------              
            task4imagecopy2mpu['DATA2CALLBACK'] = args2callback                                                    # USER DATA PASSED TO MPUIZED OPENCL - SEVERAL ARGS PASSED IN ONE DICTIONARY
            task4imagecopy2mpu['FUNCTION2CALLBACK'] = function2callback4mpu                                        # CALLBACK FUNCTION TO CALLED BY OPENCL IN EXCEPTIONS
            
            code2ack  = code2ack + '1C-'
            message2print = "{(Step 1C/70) initialized" + "; task4imagecopy2mpu-created-by-copy2image4mpu to: " + str(task4imagecopy2mpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            # {'FUNCTION2CALLBACK': <function trace2image4mpu at 0x000000000384E9C8>,
            #  'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; copy2image4mpu-failed ; time: 07:01:44 10/04/11 Pacific Daylight Time}',
            #  'CLASS': 'copy2image4mpu',
            #  'METHOD': 'copy2image4mpu'}}; time: 07:01:44 10/04/11 Pacific Daylight Time}

            
            # ---------------------------------------------------------------------------------------------
            # (2) PLATFORM - AUTOMATIC                   
            # ---------------------------------------------------------------------------------------------             
            platform2xi = {}
            message2print = "{(Step 2A/70) initialized" + "; platform2xi-created-by-copy2image4mpu to: " + str(platform2xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)   
            # {}
            
            try:
                platform2xi = get2platformids4xi()                                                                  # PLATFORM IS COMMON TO ALL DEVICES I.E TO DEVICE XI
                message2print = "{(Step 2B/70) got" + "; platform2xi-got-by-copy2image4mpu: " + str(platform2xi)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                # {'list2platforms': ['DragonXi'], 'ack2return': ' 2C-2E-2F-2G-', 'nack2return': ' ', 'available2platform': 1, 'max2entries': 1}; time: 07:01:44 10/04/11 Pacific Daylight Time}
                 
                code2ack  = code2ack + '2B-'
            except:
                message2print = "{(Step 2Z/70) failed" + "; platform2xi-failed-by-copy2image4mpu: " + str(failed)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                                
                code2nack  = code2nack + '2Z-'
                 
            task4imagecopy2mpu['PLATFORM'] = platform2xi                                                           # PLATFORM OF SYSTEM 
              
            
            # ---------------------------------------------------------------------------------------------
            # (3) CONTEXT - AUTOMATIC                 
            # ---------------------------------------------------------------------------------------------    
            context2mpu = {}
            message2print = "{(Step 3A/70) trying" + "; context2mpu-trying-by-copy2image4mpu: " + str(context2mpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            try:
                context2mpu = get2context4mpu()                                                                     # NO PARAMETERS - THE SYSTEM CREATES CONTEXT IF REQUIRED  
                
                message2print = "{(Step 3B/70) got" + "; context2mpu-got-by-copy2image4mpu: " + str(context2mpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # {'COUNT2REFERENCE': 0, 'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) ...
                
                code2ack  = code2ack + '3B-'         
            except:
                message2print = "{(Step 3Z/70) failed" + "; context2mpu-failed-by-copy2image4mpu: " + str(context2mpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '3Z-'
                
            task4imagecopy2mpu ['CONTEXT'] = context2mpu           
            
            # ---------------------------------------------------------------------------------------------
            # (4) DEVICE - AUTOMATIC                       
            # ---------------------------------------------------------------------------------------------    
            device2mpu = {}
            
            message2print = "{(Step 4A/70) trying" + "; device2mpu-trying-by-copy2image4mpu: " + str(device2mpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            try:
                device2mpu = get2mpu4xi()                                                                           # THE SYSTEM CREATES DEVICE INFO, IF NECESSARY    
                
                message2print = "{(Step 4B/70) got" + "; device2mpu-got-by-copy2image4mpu: " + str(device2mpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # {'RESOLUTION2TIMER4PROFILING': 318, 'ID2PLATFORM': '0000000001509568', 'PROFILING2QUEUE': 'ACK', ...
                
                code2ack  = code2ack + '4B-'  
            except:
                message2print = "{(Step 4Z/70) failed" + "; device2mpu-failed-by-copy2image4mpu: " + str(device2mpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '4Z-'
            
            task4imagecopy2mpu ['DEVICE'] = device2mpu
            
                   
            # ---------------------------------------------------------------------------------------------
            # (5) IMAGE SOURCE OBJECT - REQUIRED                         
            # ---------------------------------------------------------------------------------------------         
            imagesourceinfo2mpu = {}
             
            message2print = "{(Step 5A/70) trying" + "; imagesourceinfo2mpu-trying-by-copy2image4mpu: " + str(imagesourceinfo2mpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            try:
                imagesourceinfo2mpu = get2imageinfo4mpu()                                                                         
                
                message2print = "{(Step 5B/70) got" + "; imagesourceinfo2mpu-got-by-copy2image4mpu: " + str(imagesourceinfo2mpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # {'SIZE2QUEUE': 2147483646, 
                #  'FORMAT': {'TYPE': 'CL_UNORM_INT8', 
                #  'ORDER': 'CL_RGBA'}, 
                #  'QUEUE': <queue4xi.Queue4Xi object at 0x0000000004472BE0>, 
                #  'WIDTH': 800, 
                #  'CONTEXT': {'COUNT2REFERENCE': 0, ...
                
                code2ack  = code2ack + '5B-'  
            except:
                message2print = "{(Step 5Z/70) failed" + "; imagesourceinfo2mpu-failed-by-copy2image4mpu: " + str(imagesourceinfo2mpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '5Z-'          
                      
            task4imagecopy2mpu ['IMAGESOURCE'] = imagesourceinfo2mpu         
                   
            # ---------------------------------------------------------------------------------------------
            # (6) IMAGE DESTINATION OBJECT - REQUIRED                        
            # ---------------------------------------------------------------------------------------------                         
            imagedestinationinfo2mpu = {}
             
            message2print = "{(Step 6A/70) trying" + "; imagedestinationinfo2mpu-trying-by-copy2image4mpu: " + str(imagedestinationinfo2mpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            try:
                imagedestinationinfo2mpu = get2imageinfo4mpu()                                                                         
                
                message2print = "{(Step 6B/70) got" + "; imagedestinationinfo2mpu-got-by-copy2image4mpu: " + str(imagedestinationinfo2mpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # {'SIZE2QUEUE': 2147483646, 
                #  'FORMAT': {'TYPE': 'CL_UNORM_INT8', 
                #  'ORDER': 'CL_RGBA'}, 
                #  'QUEUE': <queue4xi.Queue4Xi object at 0x0000000004472BE0>, 
                #  'WIDTH': 800, 
                #  'CONTEXT': {'COUNT2REFERENCE': 0, ...
                # note: source and destination will be the same !             
                code2ack  = code2ack + '6B-'  
            except:
                message2print = "{(Step 6Z/70) failed" + "; imagedestinationinfo2mpu-failed-by-copy2image4mpu: " + str(imagedestinationinfo2mpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '6Z-'          
                      
            task4imagecopy2mpu ['IMAGEDESTINATION'] = imagedestinationinfo2mpu     
            
            # ---------------------------------------------------------------------------------------------
            # (7) QUEUE - REQUIRED BY MPUXI                          
            # ---------------------------------------------------------------------------------------------         
            queue2mpu = None
            
            message2print = "{(Step 7A/70) trying" + "; queue2mpu-trying-by-copy2image4mpu: " + str(queue2mpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
            try:
                queue2mpu = Queue4Xi()                                                                              # THE SYSTEM CREATES QUEUE    
                
                message2print = "{(Step 7B/70) got" + "; queue2mpu-got-by-copy2image4mpu: " + str(queue2mpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # <queue4xi.Queue4Xi object at 0x000000000453EB38>
                
                code2ack  = code2ack + '7B-'  
            except:
                message2print = "{(Step 7Z/70) failed" + "; queue2mpu-failed-by-copy2image4mpu: " + str(queue2mpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '7Z-'
                
            task4imagecopy2mpu ['QUEUE'] = queue2mpu

            
            # ---------------------------------------------------------------------------------------------
            # (8) PIPE - REQUIRED BY MPUXI                              
            # ---------------------------------------------------------------------------------------------         
            pipe2mpu = None
           
            message2print = "{(Step 8A/70) trying" + "; pipe2mpu-trying-by-copy2image4mpu: " + str(pipe2mpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
            try:
                pipe2mpu = Pipe4Xi()                                                                                # THE SYSTEM CREATES PIPE    
                
                message2print = "{(Step 8B/70) got" + "; pipe2mpu-got-by-copy2image4mpu: " + str(pipe2mpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # (<read-write PipeConnection, handle 612>, <read-write PipeConnection, handle 616>)
                
                code2ack  = code2ack + '8B-'  
            except:
                message2print = "{(Step 8Z/70) failed" + "; pipe2mpu-failed-by-copy2image4mpu: " + str(pipe2mpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '8Z-'
                
            task4imagecopy2mpu ['PIPE'] = pipe2mpu 
            
            # ---------------------------------------------------------------------------------------------
            # (9) COMMAND QUEUE - REQUIRED                              
            # ---------------------------------------------------------------------------------------------             
            commandqueueinfo2mpu = {}
           
            message2print = "{(Step 9A/70) trying" + "; commandqueueinfo2mpu-trying-by-copy2image4mpu: " + str(commandqueueinfo2mpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print)  
                        
            try:
                commandqueueinfo2mpu = get2commandqueueinfo4mpu()                                                   # SYSTEM CREATES COMMAND QUEUE, IF NECESSARY   
                message2print = "{(Step 9B/70) got" + "; commandqueueinfo2mpu-got-by-copy2image4mpu: " + str(commandqueueinfo2mpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # {'CODE2ERROR': 0, 
                #  'COUNT2REFERENCE': 0, 
                #  'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2commandqueue4mpu-failed ; time: 07:01:44 10/04/11 Pacific Daylight Time}', 
                #  'CLASS': 'create2commandqueue4mpu', 
                #  'METHOD': 'create2commandqueue4mpu'}, 
                #  'FUNCTION2CALLBACK': <function trace2commandqueue4mpu at 0x00000000037EDA48> ...
                
                code2ack  = code2ack + '9B-'  
            except:
                message2print = "{(Step 9Z/70) failed" + "; commandqueueinfo2mpu-failed-by-copy2image4mpu: " + str(commandqueueinfo2mpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '9Z-'
        
            task4imagecopy2mpu ['COMMANDQUEUE'] = commandqueueinfo2mpu
            
            
            
            # ---------------------------------------------------------------------------------------------
            # (10) BUFFER - AUTOMATIC BY MPUXI                               
            # ---------------------------------------------------------------------------------------------             
            bufferinfo2mpu = {}
           
            message2print = "{(Step 10A/70) trying" + "; bufferinfo2mpu-trying-by-copy2image4mpu: " + str(bufferinfo2mpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                bufferinfo2mpu = get2bufferinfo4mpu()                                                               # SYSTEM CREATES BUFFER FOR MPU, IF NECESSARY
                message2print = "{(Step 10B/70) got" + "; bufferinfo2mpu-got-by-copy2image4mpu: " + str(bufferinfo2mpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # {'CODE2ERROR': 0, 'COUNT2REFERENCE': 0, 'DATA2CALLBACK': {'MESSAGE' ...
                
                code2ack  = code2ack + '10B-' 
            except:
                message2print = "{(Step 10Z/70) failed" + "; bufferinfo2mpu-failed-by-copy2image4mpu: " + str(bufferinfo2mpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '10Z-'
        
            task4imagecopy2mpu ['BUFFER'] = bufferinfo2mpu 
            
            # ---------------------------------------------------------------------------------------------
            # (11) EVENT - REQUIRED                            
            # ---------------------------------------------------------------------------------------------             
            eventinfo2mpu = {}
           
            message2print = "{(Step 11A/70) trying" + "; eventinfo2mpu-trying-by-copy2image4mpu: " + str(eventinfo2mpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                eventinfo2mpu = get2eventinfo4mpu()                                                                    
                message2print = "{(Step 11B/70) got" + "; eventinfo2mpu-got-by-copy2image4mpu: " + str(eventinfo2mpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # {'CODE2ERROR': 0,
                #  'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2event4mpu-failed ; time: 07:01:44 10/04/11 Pacific Daylight Time}',
                #  'CLASS': 'create2event4mpu',
                #  'METHOD': 'create2event4mpu'},
                #  'FUNCTION2CALLBACK': <function trace2event4mpu at 0x0000000003803F48>,
                #  'ACK2RETURN': ' 1D-1E-2C-2B-3B-4B-6B-7B-8B-9B-10B-11B-12B-14B-15B-16B-',
                #  'QUEUE': <queue4xi.Queue4Xi object at 0x000000000423B668>,
                #  'NACK2RETURN': ' ',
                #  'CONTEXT': {'COUNT2REFERENCE': 0, 'DATA2CALLBACK': {'MESSAGE': ...
                
                code2ack  = code2ack + '11B-' 
            except:
                message2print = "{(Step 11Z/70) failed" + "; eventinfo2mpu-failed-by-copy2image4mpu: " + str(eventinfo2mpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '11Z-'
        
            task4imagecopy2mpu ['EVENT'] = eventinfo2mpu 
            
            
            # ---------------------------------------------------------------------------------------------
            # (12) EVENTWAITLIST - REQUIRED                              
            # ---------------------------------------------------------------------------------------------             
            eventwaitlist2mpu = {}
           
            message2print = "{(Step 12A/70) trying" + "; eventwaitlist2mpu-trying-by-copy2image4mpu: " + str(eventwaitlist2mpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                eventwaitlist2mpu = {}                                                                   
                message2print = "{(Step 12B/70) set" + "; eventwaitlist2mpu-set-by-copy2image4mpu: " + str(eventwaitlist2mpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # {} 
                
                code2ack  = code2ack + '12B-' 
            except:
                message2print = "{(Step 12Z/70) failed" + "; eventwaitlist2mpu-failed-by-copy2image4mpu: " + str(eventwaitlist2mpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '12Z-'
        
            task4imagecopy2mpu ['EVENTWAITLIST'] = eventwaitlist2mpu                     

            
            # ---------------------------------------------------------------------------------------------
            # (13) EVENTWAITLISTQUANTITY - REQUIRED                               
            # ---------------------------------------------------------------------------------------------             
            eventwaitlistquantity2mpu = 0
            
            message2print = "{(Step 13A/70) trying" + "; eventwaitlistquantity2mpu-trying-by-copy2image4mpu: " + str(eventwaitlistquantity2mpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                keys2event = eventwaitlist2mpu.keys()
                for key2event in keys2event:
                    eventwaitlistquantity2mpu = eventwaitlistquantity2mpu + 1    
                
                message2print = "{(Step 13B/70) set" + "; eventwaitlistquantity2mpu-set-by-copy2image4mpu: " + str(eventwaitlistquantity2mpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 0  
                
                code2ack  = code2ack + '13B-' 
            except:
                message2print = "{(Step 13Z/70) failed" + "; eventwaitlistquantity2mpu-failed-by-copy2image4mpu: " + str(eventwaitlistquantity2mpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '13Z-'
        
            task4imagecopy2mpu ['EVENTWAITLISTQUANTITY'] = eventwaitlistquantity2mpu       
                                                 
            
            # ---------------------------------------------------------------------------------------------
            # (14) ORIGIN FOR SOURCE - REQUIRED                             
            # ---------------------------------------------------------------------------------------------             
            origin4source2mpu = {}
            
            message2print = "{(Step 14A/70) trying" + "; origin4source2mpu-trying-by-copy2image4mpu: " + str(origin4source2mpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:                                                                                    
                origin4source2mpu['X'] = 0 
                origin4source2mpu['Y'] = 0  
                origin4source2mpu['Z'] = 0                                                                                                                          
                message2print = "{(Step 14B/70) set" + "; origin4source2mpu-set-by-copy2image4mpu to: " + str(origin4source2mpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                code2ack  = code2ack + '14B-' 
            except:
                message2print = "{(Step 14Z/70) failed" + "; origin4source2mpu-failed-by-copy2image4mpu: " + str(origin4source2mpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '14Z-'
        
            task4imagecopy2mpu ['SOURCEORIGIN'] = origin4source2mpu
            
            # ---------------------------------------------------------------------------------------------
            # (15) ORIGIN FOR DESTINATION - REQUIRED                             
            # ---------------------------------------------------------------------------------------------              
            origin4destination2mpu = {}
            
            message2print = "{(Step 15A/70) trying" + "; origin4destination2mpu-trying-by-copy2image4mpu: " + str(origin4destination2mpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:                                                                                    
                origin4destination2mpu['X'] = 100 
                origin4destination2mpu['Y'] = 100  
                origin4destination2mpu['Z'] = 100                                                                                                                          
                message2print = "{(Step 15B/70) set" + "; origin4destination2mpu-set-by-copy2image4mpu to: " + str(origin4destination2mpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                code2ack  = code2ack + '15B-' 
            except:
                message2print = "{(Step 15Z/70) failed" + "; origin4destination2mpu-failed-by-copy2image4mpu: " + str(origin4destination2mpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '15Z-'
        
            task4imagecopy2mpu ['DESTINATIONORIGIN'] = origin4destination2mpu
                        
            
            
            # --------------------------------------------------------------------------------------------------------
            # (16) REGION - REGION OF A IMAGE TO BE COPIED                         
            # --------------------------------------------------------------------------------------------------------             
            region4copy2mpu = {}                                                                                                         
           
            message2print = "{(Step 16A/70) trying" + "; region4copy2mpu-trying-by-copy2image4mpu: " + str(region4copy2mpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                region4copy2mpu['X'] = 10 
                region4copy2mpu['Y'] = 10  
                region4copy2mpu['Z'] = 10                                                                           
                message2print = "{(Step 16B/70) set" + "; region4copy2mpu-set-by-copy2image4mpu to: " + str(region4copy2mpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 

                code2ack  = code2ack + '16B-'                
                 
            except:
                message2print = "{(Step 16Z/70) failed" + "; region4copy2mpu-failed-by-copy2image4mpu: " + str(region4copy2mpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '16Z-'
            
            task4imagecopy2mpu ['REGION'] = region4copy2mpu              
            
            # ---------------------------------------------------------------------------------------------
            # EOF PARAMETER PROVIDING                
            # ---------------------------------------------------------------------------------------------                                                                                                                               
        else:
            pass
            
        # ---------------------------------------------------------------------------------------------
        # CHECK PARAMETERS - WHETHER AUTOMATICALLY GENERATED OR USER GIVEN 
        # ---------------------------------------------------------------------------------------------
        keys2task = task4imagecopy2mpu.keys()
        message2print = "{(Step 20/70) using" + "; keys2task-using-by-copy2image4mpu: " + str(sorted(keys2task))  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)             
        # ['BUFFER',
        #  'COMMANDQUEUE',
        #  'CONTEXT',
        #  'DATA2CALLBACK',
        #  'DESTINATIONORIGIN',
        #  'DEVICE',
        #  'EVENT',
        #  'EVENTWAITLIST',
        #  'EVENTWAITLISTQUANTITY',
        #  'FUNCTION2CALLBACK',
        #  'IMAGEDESTINATION',
        #  'IMAGESOURCE',
        #  'PIPE',
        #  'PLATFORM',
        #  'QUEUE',
        #  'REGION',
        #  'SOURCEORIGIN']; time: 01:21:58 10/05/11 Pacific Daylight Time}
 
        if (('DATA2CALLBACK' in keys2task) and ('FUNCTION2CALLBACK' in keys2task)):
            pass
        else:                
            # ---------------------------------------------------------------------------------------------
            # (21B) TRACE - AUTOMATIC                      
            # ---------------------------------------------------------------------------------------------              
            function2callback4mpu = trace2image4mpu                                                            # CALLBACK FUNCTION FOR MPU
            
            message2print = "{(Step 21B1/70) initialized" + "; function2callback4mpu-created-by-copy2image4mpu to: " + str(function2callback4mpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)            
            # 
            
            code2ack  = code2ack + '21B1-'
           
            args2callback = {}                                                                                 # CREATE ARGUMENT DICTIONARY FOR CALLBACK FUNCTION             
            args2callback['CLASS']   = "copy2image4mpu"                                                        # MODULE OF MPUIZED OPENCL MODULE
            args2callback['METHOD']  = "copy2image4mpu"                                                        # FUNCTION OF MPUIZED OPENCL MODULE
            args2callback['MESSAGE'] = "{(Step 1Z1/33) failed" + "; copy2image4mpu-failed " + "; time: " + time4xi() + "}" # ERROR MESSAGE FOR MPUIZED OPENCL TO RETURN    
            
            message2print = "{(Step 21B2/70) initialized" + "; args2callback-created-by-copy2image4mpu to: " + str(args2callback)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)               
            # 
    
            code2ack  = code2ack + '21B2-'
            
            # ---------------------------------------------------------------------------------------------
            # (21C) CALLBACK - AUTOMATIC                      
            # ---------------------------------------------------------------------------------------------              
            task4imagecopy2mpu['DATA2CALLBACK'] = args2callback                                                # USER DATA PASSED TO MPUIZED OPENCL - SEVERAL ARGS PASSED IN ONE DICTIONARY
            task4imagecopy2mpu['FUNCTION2CALLBACK'] = function2callback4mpu                                    # CALLBACK FUNCTION TO CALLED BY OPENCL IN EXCEPTIONS
                
            code2ack  = code2ack + '21C-'
            message2print = "{(Step 21C/70) initialized" + "; task4imagecopy2mpu-created-by-copy2image4mpu to: " + str(task4imagecopy2mpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            #
                 
        if 'PLATFORM' in keys2task:
            pass
        else:  
            # ---------------------------------------------------------------------------------------------
            # (22) PLATFORM - AUTOMATIC                   
            # ---------------------------------------------------------------------------------------------             
            platform2xi = {}
            message2print = "{(Step 22A/70) initialized" + "; platform2xi-created-by-copy2image4mpu to: " + str(platform2xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)   
            # 
                
            try:
                platform2xi = get2platformids4xi()                                                              # PLATFORM IS COMMON TO ALL DEVICES I.E TO DEVICE XI
                message2print = "{(Step 22B/70) got" + "; platform2xi-got-by-copy2image4mpu: " + str(platform2xi)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                # 
                 
                code2ack  = code2ack + '22B-'
            except:
                message2print = "{(Step 22Z/70) failed" + "; platform2xi-failed-by-copy2image4mpu: " + str(failed)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                                
                code2nack  = code2nack + '22Z-'
                 
            task4imagecopy2mpu['PLATFORM'] = platform2xi                                                       # PLATFORM OF SYSTEM 
          
        if 'CONTEXT' in keys2task:
            pass
        else:              
            # ---------------------------------------------------------------------------------------------
            # (23) CONTEXT - AUTOMATIC                 
            # ---------------------------------------------------------------------------------------------    
            context2mpu = {}
            message2print = "{(Step 23A/70) trying" + "; context2mpu-trying-by-copy2image4mpu: " + str(context2mpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            try:
                context2mpu = get2context4mpu()                                                                 # NO PARAMETERS - THE SYSTEM CREATES CONTEXT IF REQUIRED  
                
                message2print = "{(Step 23B/70) got" + "; context2mpu-got-by-copy2image4mpu: " + str(context2mpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # 
                
                code2ack  = code2ack + '23B-'         
            except:
                message2print = "{(Step 23Z/70) failed" + "; context2mpu-failed-by-copy2image4mpu: " + str(context2mpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '23Z-'
                
            task4imagecopy2mpu ['CONTEXT'] = context2mpu           
        
        if 'DEVICE' in keys2task:
            pass
        else:                   
            # ---------------------------------------------------------------------------------------------
            # (24) DEVICE - AUTOMATIC                       
            # ---------------------------------------------------------------------------------------------    
            device2mpu = {}
            
            message2print = "{(Step 24A/70) trying" + "; device2mpu-trying-by-copy2image4mpu: " + str(device2mpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            try:
                device2mpu = get2mpu4xi()                                                                       # THE SYSTEM CREATES DEVICE INFO, IF NECESSARY    
                
                message2print = "{(Step 24B/70) got" + "; device2mpu-got-by-copy2image4mpu: " + str(device2mpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # 
                
                code2ack  = code2ack + '24B-'  
            except:
                message2print = "{(Step 24Z/70) failed" + "; device2mpu-failed-by-copy2image4mpu: " + str(device2mpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '24Z-'
            
            task4imagecopy2mpu ['DEVICE'] = device2mpu
        
        if 'IMAGESOURCE' in keys2task:
            pass
        else:                      
            # ---------------------------------------------------------------------------------------------
            # (25) SOURCE IMAGE - REQUIRED                            
            # ---------------------------------------------------------------------------------------------         
            sourceimageinfo2mpu = {}
             
            message2print = "{(Step 25A/70) trying" + "; sourceimageinfo2mpu-trying-by-copy2image4mpu: " + str(sourceimageinfo2mpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            try:
                sourceimageinfo2mpu = get2imageinfo4mpu()                                                                         
                
                message2print = "{(Step 25B/70) got" + "; sourceimageinfo2mpu-got-by-copy2image4mpu: " + str(sourceimageinfo2mpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                #                 
                code2ack  = code2ack + '25B-'  
            except:
                message2print = "{(Step 25Z/70) failed" + "; sourceimageinfo2mpu-failed-by-copy2image4mpu: " + str(sourceimageinfo2mpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '25Z-'          
                      
            task4imagecopy2mpu ['IMAGESOURCE'] = sourceimageinfo2mpu         
        
        if 'IMAGEDESTINATION' in keys2task:
            pass
        else:                      
            # ---------------------------------------------------------------------------------------------
            # (26) DESTINATION IMAGE - REQUIRED                            
            # ---------------------------------------------------------------------------------------------         
            destinationimageinfo2mpu = {}
             
            message2print = "{(Step 26A/70) trying" + "; destinationimageinfo2mpu-trying-by-copy2image4mpu: " + str(destinationimageinfo2mpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            try:
                destinationimageinfo2mpu = get2imageinfo4mpu()                                                                         
                
                message2print = "{(Step 26B/70) got" + "; destinationimageinfo2mpu-got-by-copy2image4mpu: " + str(destinationimageinfo2mpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                #                 
                code2ack  = code2ack + '26B-'  
            except:
                message2print = "{(Step 26Z/70) failed" + "; destinationimageinfo2mpu-failed-by-copy2image4mpu: " + str(destinationimageinfo2mpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '26Z-'          
                      
            task4imagecopy2mpu ['IMAGEDESTINATION'] = destinationimageinfo2mpu    
                                    
        if 'QUEUE' in keys2task:
            pass
        else:   
            # ---------------------------------------------------------------------------------------------
            # (27) QUEUE - REQUIRED BY MPUXI                          
            # ---------------------------------------------------------------------------------------------         
            queue2mpu = None
            
            message2print = "{(Step 27A/70) trying" + "; queue2mpu-trying-by-copy2image4mpu: " + str(queue2mpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
            try:
                queue2mpu = Queue4Xi()                                                                          # THE SYSTEM CREATES QUEUE    
                
                message2print = "{(Step 27B/70) got" + "; queue2mpu-got-by-copy2image4mpu: " + str(queue2mpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # 
                
                code2ack  = code2ack + '27B-'  
            except:
                message2print = "{(Step 27Z/70) failed" + "; queue2mpu-failed-by-copy2image4mpu: " + str(queue2mpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '27Z-'
                
            task4imagecopy2mpu ['QUEUE'] = queue2mpu

        if 'PIPE' in keys2task:
            pass
        else:              
            # ---------------------------------------------------------------------------------------------
            # (28) PIPE - REQUIRED BY MPUXI                              
            # ---------------------------------------------------------------------------------------------         
            pipe2mpu = None
           
            message2print = "{(Step 28A/70) trying" + "; pipe2mpu-trying-by-copy2image4mpu: " + str(pipe2mpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
            try:
                pipe2mpu = Pipe4Xi()                                                                            # THE SYSTEM CREATES PIPE    
                
                message2print = "{(Step 28B/70) got" + "; pipe2mpu-got-by-copy2image4mpu: " + str(pipe2mpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # 
                
                code2ack  = code2ack + '28B-'  
            except:
                message2print = "{(Step 28Z/70) failed" + "; pipe2mpu-failed-by-copy2image4mpu: " + str(pipe2mpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '28Z-'
                
            task4imagecopy2mpu ['PIPE'] = pipe2mpu 
            
        if 'COMMANDQUEUE' in keys2task:
            pass
        else:              
            # ---------------------------------------------------------------------------------------------
            # (28) COMMAND QUEUE - REQUIRED                              
            # ---------------------------------------------------------------------------------------------             
            commandqueueinfo2mpu = {}
           
            message2print = "{(Step 29A/70) trying" + "; commandqueueinfo2mpu-trying-by-copy2image4mpu: " + str(commandqueueinfo2mpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print)  
                        
            try:
                commandqueueinfo2mpu = get2commandqueueinfo4mpu()                                               # SYSTEM CREATES COMMAND QUEUE, IF NECESSARY   
                message2print = "{(Step 29B/70) got" + "; commandqueueinfo2mpu-got-by-copy2image4mpu: " + str(commandqueueinfo2mpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                
                code2ack  = code2ack + '29B-'  
            except:
                message2print = "{(Step 29Z/70) failed" + "; commandqueueinfo2mpu-failed-by-copy2image4mpu: " + str(commandqueueinfo2mpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '29Z-'
        
            task4imagecopy2mpu ['COMMANDQUEUE'] = commandqueueinfo2mpu
        
        
        if 'BUFFER' in keys2task:
            pass
        else:               
            # ---------------------------------------------------------------------------------------------
            # (30) BUFFER - AUTOMATIC BY MPUXI                               
            # ---------------------------------------------------------------------------------------------             
            bufferinfo2mpu = {}
           
            message2print = "{(Step 30A/70) trying" + "; bufferinfo2mpu-trying-by-copy2image4mpu: " + str(bufferinfo2mpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                bufferinfo2mpu = get2bufferinfo4mpu()                                                           # SYSTEM CREATES BUFFER FOR MPU, IF NECESSARY
                message2print = "{(Step 30B/70) got" + "; bufferinfo2mpu-got-by-copy2image4mpu: " + str(bufferinfo2mpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                
                code2ack  = code2ack + '30B-' 
            except:
                message2print = "{(Step 30Z/70) failed" + "; bufferinfo2mpu-failed-by-copy2image4mpu: " + str(bufferinfo2mpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '30Z-'
        
            task4imagecopy2mpu ['BUFFER'] = bufferinfo2mpu
             
        if 'EVENT' in keys2task:
            pass
        else:               
            # ---------------------------------------------------------------------------------------------
            # (31) EVENT - REQUIRED                            
            # ---------------------------------------------------------------------------------------------             
            eventinfo2mpu = {}
           
            message2print = "{(Step 31A/70) trying" + "; eventinfo2mpu-trying-by-copy2image4mpu: " + str(eventinfo2mpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                eventinfo2mpu = get2eventinfo4mpu()                                                                    
                message2print = "{(Step 31B/70) got" + "; eventinfo2mpu-got-by-copy2image4mpu: " + str(eventinfo2mpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                
                code2ack  = code2ack + '31B-' 
            except:
                message2print = "{(Step 31Z/70) failed" + "; eventinfo2mpu-failed-by-copy2image4mpu: " + str(eventinfo2mpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '31Z-'
        
            task4imagecopy2mpu ['EVENT'] = eventinfo2mpu 
        
             
        if 'EVENTWAITLIST' in keys2task:
            pass
        else:               
            # ---------------------------------------------------------------------------------------------
            # (32) EVENTWAITLIST - REQUIRED                              
            # ---------------------------------------------------------------------------------------------             
            eventwaitlist2mpu = {}
           
            message2print = "{(Step 32A/70) trying" + "; eventwaitlist2mpu-trying-by-copy2image4mpu: " + str(eventwaitlist2mpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                eventwaitlist2mpu = {}                                                                   
                message2print = "{(Step 32B/70) set" + "; eventwaitlist2mpu-set-by-copy2image4mpu: " + str(eventwaitlist2mpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                
                code2ack  = code2ack + '32B-' 
            except:
                message2print = "{(Step 32Z/70) failed" + "; eventwaitlist2mpu-failed-by-copy2image4mpu: " + str(eventwaitlist2mpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '32Z-'
        
            task4imagecopy2mpu ['EVENTWAITLIST'] = eventwaitlist2mpu                     
              
        if 'EVENTWAITLISTQUANTITY' in keys2task:
            pass
        else:               
            # ---------------------------------------------------------------------------------------------
            # (33) EVENTWAITLISTQUANTITY - REQUIRED                              
            # ---------------------------------------------------------------------------------------------
            eventwaitlistquantity2mpu = 0
            
            message2print = "{(Step 33A/70) trying" + "; eventwaitlistquantity2mpu-trying-by-copy2image4mpu: " + str(eventwaitlistquantity2mpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                keys2event = eventwaitlist2mpu.keys()
                for key2event in keys2event:
                    eventwaitlistquantity2mpu = eventwaitlistquantity2mpu + 1    
                
                message2print = "{(Step 33B/70) set" + "; eventwaitlistquantity2mpu-set-by-copy2image4mpu: " + str(eventwaitlistquantity2mpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 0 
                
                code2ack  = code2ack + '33B-' 
            except:
                message2print = "{(Step 33Z/70) failed" + "; eventwaitlistquantity2mpu-failed-by-copy2image4mpu: " + str(eventwaitlistquantity2mpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '33Z-'
                    
            task4imagecopy2mpu ['EVENTWAITLISTQUANTITY'] = eventwaitlistquantity2mpu       
                                             
                    
        if 'SOURCEORIGIN' in keys2task:
            pass
        else:                    
            # ---------------------------------------------------------------------------------------------
            # (34) SOURCEORIGIN - REQUIRED                             
            # ---------------------------------------------------------------------------------------------             
            source4origin2mpu = {}
            
            message2print = "{(Step 34A/70) trying" + "; source4origin2mpu-trying-by-copy2image4mpu: " + str(source4origin2mpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:                                                                                    
                source4origin2mpu['X'] = 0 
                source4origin2mpu['Y'] = 0  
                source4origin2mpu['Z'] = 0                                                                                                                          
                message2print = "{(Step 34B/70) set" + "; source4origin2mpu-set-by-copy2image4mpu to: " + str(source4origin2mpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                code2ack  = code2ack + '34B-' 
            except:
                message2print = "{(Step 34Z/70) failed" + "; source4origin2mpu-failed-by-copy2image4mpu: " + str(source4origin2mpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '34Z-'
        
            task4imagecopy2mpu ['SOURCEORIGIN'] = source4origin2mpu
             
        if 'DESTINATIONORIGIN' in keys2task:
            pass
        else:                    
            # ---------------------------------------------------------------------------------------------
            # (35) DESTINATIONORIGIN - REQUIRED                             
            # ---------------------------------------------------------------------------------------------             
            destination4origin2mpu = {}
            
            message2print = "{(Step 35A/70) trying" + "; destination4origin2mpu-trying-by-copy2image4mpu: " + str(destination4origin2mpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:                                                                                    
                destination4origin2mpu['X'] = 100 
                destination4origin2mpu['Y'] = 100  
                destination4origin2mpu['Z'] = 100                                                                                                                          
                message2print = "{(Step 35B/70) set" + "; destination4origin2mpu-set-by-copy2image4mpu to: " + str(destination4origin2mpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                code2ack  = code2ack + '35B-' 
            except:
                message2print = "{(Step 35Z/70) failed" + "; destination4origin2mpu-failed-by-copy2image4mpu: " + str(destination4origin2mpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '35Z-'
        
            task4imagecopy2mpu ['DESTINATIONSOURCEORIGIN'] = destination4origin2mpu                
             
        if 'REGION' in keys2task:
            pass
        else:            
            # ---------------------------------------------------------------------------------------------
            # (36) REGION - REQUIRED                         
            # ---------------------------------------------------------------------------------------------             
            region2mpu = {}
            
            message2print = "{(Step 36A/70) trying" + "; region2mpu-trying-by-copy2image4mpu: " + str(region2mpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:                                                                                    
                region2mpu['X'] = 160                                                                               # WIDTH  IN PIXELS TO WRITE 
                region2mpu['Y'] = 90                                                                                # HEIGHT IN PIXELS TO WRITE
                region2mpu['Z'] = 0                                                                                 # DEPTH  IN PIXELS TO WRITE                                                                                                                          
                message2print = "{(Step 36B/70) set" + "; region2mpu-set-by-copy2image4mpu to: " + str(region2mpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                code2ack  = code2ack + '36B-' 
            except:
                message2print = "{(Step 36Z/70) failed" + "; region2mpu-failed-by-copy2image4mpu: " + str(region2mpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '36Z-'
        
            task4imagecopy2mpu ['REGION'] = region2mpu
        
        # ---------------------------------------------------------------------------------------------
        #  EOF PARAMETER CHECKING AND PROVIDING FOR MISSING ONES
        # ---------------------------------------------------------------------------------------------
        keys2task = task4imagecopy2mpu.keys()
        message2print = "{(Step 39/70) set" + "; keys2task-set-by-copy2image4mpu to: " + str(sorted(keys2task))  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
        # ['BUFFER',
        #  'COMMANDQUEUE',
        #  'CONTEXT',
        #  'DATA2CALLBACK',
        #  'DESTINATIONORIGIN',
        #  'DEVICE',
        #  'EVENT',
        #  'EVENTWAITLIST',
        #  'EVENTWAITLISTQUANTITY',
        #  'FUNCTION2CALLBACK',
        #  'IMAGEDESTINATION',
        #  'IMAGESOURCE',
        #  'PIPE',
        #  'PLATFORM',
        #  'QUEUE',
        #  'REGION',
        #  'SOURCEORIGIN']; time: 01:28:44 10/05/11 Pacific Daylight Time}
    except: 
        message2print = "{(Step 39Z/70) yielded" + "; task4imagecopy2mpu-yielded-by-copy2image4mpu: " + str(task4imagecopy2mpu) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # 
        
        code2nack  = code2nack + '39Z-' 

    # ---------------------------------------------------------------------------------------------
    # CONTROL AND COMMAND
    # ---------------------------------------------------------------------------------------------
    control2copyimage4mpu = {}       
                                
    # ---------------------------------------------------------------------------------------------
    # CONSUMER AND PUBLISHER CHANNEL FOR CONTROL AND COMMAND
    # ---------------------------------------------------------------------------------------------
    ack4channels2mpu = False
    channels2mpu = ()
    message2print = "{(Step 40A/70) trying" + "channels2mpu-trying-by-copy2image4mpu: " + str(channels2mpu)  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)     
    try:
        # 
        consumerchannel2mpu, publisherchannel2mpu = task4imagecopy2mpu ['PIPE']

        message2print = "{(Step 40B/70) got" + "; consumerchannel2mpu-got-by-copy2image4mpu: " + str(consumerchannel2mpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
        # <read-write PipeConnection, handle 612> 

        message2print = "{(Step 40C/70) got" + "; publisherchannel2mpu-got-by-copy2image4mpu: " + str(publisherchannel2mpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
        # <read-write PipeConnection, handle 616>
                              
        ack4channels2mpu = True
        
        control2copyimage4mpu['CONSUMERCHANNEL']  = consumerchannel2mpu
        control2copyimage4mpu['PUBLISHERCHANNEL'] = publisherchannel2mpu
        
        code2ack  = code2ack + '40D-' 
        
    except:
        ack4channels2mpu = False
        code2nack  = code2nack + '40Z-'      
                        
    # ---------------------------------------------------------------------------------------------
    # QUEUE FOR CONTROL AND COMMAND
    # ---------------------------------------------------------------------------------------------
    ack4queue2mpu = False
    queue2mpu = None
    message2print = "{(Step 41A/70) trying" + "queue2mpu-trying-by-copy2image4mpu: " + str(queue2mpu)  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)     
    try:
        queue2mpu = task4imagecopy2mpu ['QUEUE']

        message2print = "{(Step 41B/70) got" + "; queue2mpu-got-by-copy2image4mpu: " + str(queue2mpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
        # <queue4xi.Queue4Xi object at 0x0000000004721CF8>
              
        ack4queue2mpu = True
        control2copyimage4mpu['QUEUE']  = queue2mpu        
        code2ack  = code2ack + '41C-'
    except:
        ack4queue2mpu = False  
        code2nack  = code2nack + '41Z-'
    
    
    # -----------------------------------------------------------------------------------------    
    # DUMMY PROCESS OBJECT = THREAD CREATION
    # -----------------------------------------------------------------------------------------           
    # dummy2image4mpu = None
    # message2print = "{(Step 69A/70) trying" + "dummy2image4mpu-trying-by-copy2image4mpu: " + str(dummy2image4mpu)  + "; time: " + time4xi() + "}"   
    # print4xi(class2print, method2print, message2print)     
    # try:                            
        # dummy2image4mpu = DummyProcess4Xi(target=manoeuvre2copyimage4mpu,                                               # in this module to handle queue and pipe channels
        #                                   args =(queue2mpu,                                                         # caller puts and gets - does NOT call any functions
        #                                          consumerchannel2mpu, publisherchannel2mpu))                        # caller sends and receives - does NOT call any functions
        # message2print = "{(Step 69B/70) created" + "dummy2image4mpu-created-by-copy2image4mpu: " + str(dummy2image4mpu)  + "; time: " + time4xi() + "}" 
        # print4xi(class2print, method2print, message2print)  
        # code2ack  = code2ack + '69B-'
        
    # except:
        # code2nack  = code2nack + '69Z-'
        
    # ---------------------------------------------------------------------------------------------
    # RETURN CONTROL AND COMMAND DICTIONARY
    # ---------------------------------------------------------------------------------------------          
    message2print = "{(Step 70A/70) set" + "; code2ack-set-by-copy2image4mpu: " + str(code2ack) + "; time: " + time4xi() + "}"  
    print4xi(class2print, method2print, message2print)                                                             
    # 1B1-1B2-1C-2B-3B-4B-5B-6B-7B-8B-9B-10B-11B-12B-13B-14B-15B-16B-40D-41C- 
    message2print = "{(Step 70B/70) set" + "; code2nack-set-by-copy2image4mpu: " + str(code2nack) + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                             
    #  
        
    message2print = "{(Step 70C/70) returning" + "; control2copyimage4mpu-returning-by-copy2image4mpu: " + str(control2copyimage4mpu) + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                             
    # {'QUEUE': <queue4xi.Queue4Xi object at 0x0000000004721CF8>,
    #  'PUBLISHERCHANNEL': <read-write PipeConnection, handle 616>,
    #  'CONSUMERCHANNEL': <read-write PipeConnection, handle 612>}; time: 01:28:44 10/05/11 Pacific Daylight Time}
   
    return control2copyimage4mpu
                    
#</function4xi> 


# -------------------------------------------------------------------------------------------------
#<function4xi function = "copy2imagetobuffer4mpu" usage = "
#                                                                                                              
#   # ---------------------------------------------------------------------------------------------
#   # (60) ESTABLISH CONTROL & COMMAND WITH COPY2IMAGETOBUFFER4MPU                      
#   # ---------------------------------------------------------------------------------------------    
#   print('(Step 60A/700) OF MPU2IMAGE4XI5TEST - TRYING TO ESTABLISH CONTROL & COMMAND WITH COPY2IMAGETOBUFFER4MPU ', time4xi())    
#   response4copy2imagetobuffer = {} 
#   task4copy2imagetobuffer = None
#   try:                               
#       response4copy2imagetobuffer = copy2imagetobuffer4mpu(task4copy2imagetobuffer)                                                                                                          
#       print('(Step 60B/700) OF MPU2IMAGE4XI5TEST - ESTABLISHED CONTROL & COMMAND WITH COPY2IMAGETOBUFFER4MPU : ', response4copy2imagetobuffer)
#       queue2copyimagetobuffer4mpu      = response4copy2imagetobuffer['QUEUE']
#       channel2copyimagetobufferpublish4mpu = response4copy2imagetobuffer['PUBLISHERCHANNEL']
#       channel2copyimagetobufferconsume4mpu = response4copy2imagetobuffer['CONSUMERCHANNEL']
#   except:
#       print('(Step 60Z/700) OF MPU2IMAGE4XI5TEST *** FAILED TO ESTABLISH CONTROL & COMMAND WITH COPY2IMAGETOBUFFER4MPU ', time4xi()) 
#   # --------------------------------------------------------------------------------------------------
#   # (61) SEND CONTROL & COMMAND COPY VIA PIPE FOR MANOEUVRE2COPYIMAGETOBUFFER4MPU TO RECEIVE                 
#   # --------------------------------------------------------------------------------------------------      
#   print('(Step 61A/700) OF MPU2IMAGE4XI5TEST - TRYING TO SEND CONTROL & COMMAND VIA PIPE WITH COPY2IMAGETOBUFFER4MPU ', time4xi())     
#   try:                               
#       message2copyimagetobuffersend4mpu = "<copy4xi>copy2copyimagetobuffer4mpu</copy4xi>"
#       channel2copyimagetobufferpublish4mpu = response4copy2imagetobuffer['PUBLISHERCHANNEL']
#       channel2copyimagetobufferpublish4mpu.send(message2copyimagetobuffersend4mpu)
#       print("(Step 61B/700) OF MPU2IMAGE4XI5TEST - SENT TO CHANNEL2PUBLISH4MPU: ", channel2copyimagetobufferpublish4mpu, " MESSAGE2SEND4MPU: " , message2copyimagetobuffersend4mpu)                                                                                                
#   except:
#       print('(Step 61Z1/700) OF MPU2IMAGE4XI5TEST *** FAILED TO SEND CONTROL & COMMAND VIA PIPE WITH COPY2IMAGETOBUFFER4MPU ', time4xi())                                                
#   # ------------------------------------------------------------------------------------------
#   # (62) LET MANOEUVRE2COPYIMAGETOBUFFER4MPU TO RECEIVE COPY AND AND SEND ROGER VIA PIPE                
#   # ------------------------------------------------------------------------------------------       
#   print("(Step 62A/700) OF MPU2IMAGE4XI5TEST - TRYING TO PERFORM TARGET FUNCTION MANOEUVRE2COPYIMAGETOBUFFER4MPU ", time4xi()) 
#   try:        
#       response2manoeuvreimagecopytobuffer4mpu = manoeuvre2copyimagetobuffer4mpu(queue2copyimagetobuffer4mpu, channel2copyimagetobufferconsume4mpu, channel2copyimagetobufferpublish4mpu)
#       print("(Step 62B/700) OF MPU2IMAGE4XI5TEST - PERFORMED TARGET FUNCTION MANOEUVRE2COPYIMAGETOBUFFER4MPU ", manoeuvre2copyimagetobuffer4mpu)
#   except:
#       print("(Step 62Z/700) OF MPU2IMAGE4XI5TEST *** FAILED TO PERFORM TARGET FUNCTION MANOEUVRE2COPYIMAGETOBUFFER4MPU ", time4xi())     
#   else:
#       pass # trust activated thread    
#   # ---------------------------------------------------------------------------------------------
#   # (63) RECEIVE CONTROL & COMMAND ROGER VIA PIPE                   
#   # ---------------------------------------------------------------------------------------------          
#   print('(Step 63A/700) OF MPU2IMAGE4XI5TEST - TRYING TO RECEIVE CONTROL & COMMAND VIA PIPE WITH COPY2IMAGETOBUFFER4MPU ', time4xi())     
#   try:
#       channel2copyimagetobufferconsume4mpu = response4copy2imagetobuffer['CONSUMERCHANNEL']                              
#       message2copyimagereceive4mpu = channel2copyimagetobufferconsume4mpu.recv()
#       print("(Step 63B/700) OF MPU2IMAGE4XI5TEST - RECEIVED FROM CHANNEL2CONSUME4MPU: ", channel2copyimagetobufferconsume4mpu, " CONTROL & COMMAND: ", message2copyimagereceive4mpu)                                                                                                  
#       # 
#   except:
#       print('(Step 63Z/700) OF MPU2IMAGE4XI5TEST *** FAILED TO RECEIVE CONTROL & COMMAND VIA PIPE  WITH COPY2IMAGETOBUFFER4MPU ', time4xi())         
#   # ---------------------------------------------------------------------------------------------------
#   # (64) SEND CONTROL & COMMAND READY VIA PIPE FOR MANOEUVRE2COPYIMAGETOBUFFER4MPU TO RECEIVE                
#   # ---------------------------------------------------------------------------------------------------      
#   print('(Step 64A/700) OF MPU2IMAGE4XI5TEST - TRYING TO SEND CONTROL & COMMAND VIA PIPE WITH COPY2IMAGETOBUFFER4MPU ', time4xi())     
#   try:                               
#       message2copyimagetobuffersend4mpu = "<copy4xi>copy2copyimagetobuffer4mpu</copy4xi>"
#       channel2copyimagetobufferpublish4mpu = response4copy2imagetobuffer['PUBLISHERCHANNEL']
#       channel2copyimagetobufferpublish4mpu.send(message2copyimagetobuffersend4mpu)
#       print("(Step 64B/700) OF MPU2IMAGE4XI5TEST - SENT TO CHANNEL2PUBLISH4MPU: ", channel2copyimagetobufferpublish4mpu, " MESSAGE2SEND4MPU: " , message2copyimagetobuffersend4mpu)                                                                                                
#       #  
#   except:
#       print('(Step 64Z/700) OF MPU2IMAGE4XI5TEST *** FAILED TO SEND CONTROL & COMMAND VIA PIPE WITH COPY2IMAGETOBUFFER4MPU ', time4xi())                                                
#   # -----------------------------------------------------------------------------------
#   # (65) LET MANOEUVRE2COPYIMAGETOBUFFER4MPU TO RECEIVE READY AND TO SEND READY                
#   # -----------------------------------------------------------------------------------       
#   print("(Step 65A/700) OF MPU2IMAGE4XI5TEST - TRYING TO PERFORM TARGET FUNCTION MANOEUVRE2COPYIMAGETOBUFFER4MPU ", time4xi()) 
#   try:        
#       response2manoeuvreimagecopytobuffer4mpu = manoeuvre2copyimagetobuffer4mpu(queue2copyimagetobuffer4mpu, channel2copyimagetobufferconsume4mpu, channel2copyimagetobufferpublish4mpu)
#       print("(Step 65B/700) OF MPU2IMAGE4XI5TEST - PERFORMED TARGET FUNCTION MANOEUVRE2COPYIMAGETOBUFFER4MPU ", manoeuvre2copyimagetobuffer4mpu)
#   except:
#       print("(Step 65Z/700) OF MPU2IMAGE4XI5TEST *** FAILED TO PERFORM TARGET FUNCTION MANOEUVRE2COPYIMAGETOBUFFER4MPU ", time4xi())     
#   # ---------------------------------------------------------------------------------------
#   # (66) RECEIVE CONTROL & COMMAND READY VIA PIPE                      
#   # ---------------------------------------------------------------------------------------          
#   print('(Step 66A/700) OF MPU2IMAGE4XI5TEST - TRYING TO RECEIVE CONTROL & COMMAND VIA PIPE WITH COPY2IMAGETOBUFFER4MPU ', time4xi())     
#   try:
#       channel2copyimagetobufferconsume4mpu = response4copy2imagetobuffer['CONSUMERCHANNEL']                              
#       message2copyimagereceive4mpu = channel2copyimagetobufferconsume4mpu.recv()
#       print("(Step 66B/700) OF MPU2IMAGE4XI5TEST - RECEIVED FROM CHANNEL2CONSUME4MPU: ", channel2copyimagetobufferconsume4mpu, " CONTROL & COMMAND: ", message2copyimagereceive4mpu)                                                                                                  
#       # 
#   except:
#       print('(Step 66Z/700) OF MPU2IMAGE4XI5TEST *** FAILED TO RECEIVE CONTROL & COMMAND VIA PIPE  WITH COPY2IMAGETOBUFFER4MPU ', time4xi())          
#   # -----------------------------------------------------------------------------------------
#   # (67) PUT IMAGE PATH TO QUEUE
#   # -----------------------------------------------------------------------------------------                          
#   print("(Step 67A/700) OF MPU2IMAGE4XI5TEST - TRYING TO PUT IMAGE PATH TO QUEUE ", time4xi())      
#   try:    
#       queue2copyimagetobuffer4dragon.put(path2dragon)             
#       print("(Step 67B/700) OF DRAGON2IMAGE4XI5TEST - PUT TO QUEUE IMAGE PATH: ", path2dragon) 
#       # ..//..//..//..//..//web//images//png//dragonxi.png  
#   except:
#       print("(Step 67Z/700) OF MPU2IMAGE4XI5TEST *** FAILED TO PUT IMAGE PATH TO QUEUE ", time4xi()) 
#   # ------------------------------------------------------------------------------------------------------
#   # (68) LET MANOEUVRE2COPYIMAGETOBUFFER4MPU TO GET FROM QUEUE AND TO SEND EITHER ACK OR NACK VIA PIPE               
#   # -------------------------------------------------------------------------------------------------------       
#   print("(Step 68A/700) OF MPU2IMAGE4XI5TEST - TRYING TO PERFORM TARGET FUNCTION MANOEUVRE2COPYIMAGETOBUFFER4MPU ", time4xi()) 
#   try:        
#       response2manoeuvreimagecopytobuffer4mpu = manoeuvre2copyimagetobuffer4mpu(queue2copyimagetobuffer4mpu, channel2copyimagetobufferconsume4mpu, channel2copyimagetobufferpublish4mpu)
#       print("(Step 68B/700) OF MPU2IMAGE4XI5TEST - PERFORMED TARGET FUNCTION MANOEUVRE2COPYIMAGETOBUFFER4MPU ", manoeuvre2copyimagetobuffer4mpu)
#   except:
#       print("(Step 68Z/700) OF MPU2IMAGE4XI5TEST *** FAILED TO PERFORM TARGET FUNCTION MANOEUVRE2COPYIMAGETOBUFFER4MPU ", time4xi())     
#   # -----------------------------------------------------------------------------------------------
#   # (69) RECEIVE ACK OR NACK ABOUT IMAGE OPERATION VIA PIPE                
#   # -----------------------------------------------------------------------------------------------          
#   print('(Step 69A/700) OF MPU2IMAGE4XI5TEST - TRYING TO RECEIVE CONTROL & COMMAND ABOUT IMAGE OPERTION VIA PIPE ', time4xi())      
#   try: 
#       message2copyimagereceive4mpu = channel2copyimagetobufferconsume4mpu.recv()
#       print("(Step 69D/700) OF MPU2IMAGE4XI5TEST - RECEIVED FROM CHANNEL2CONSUME4MPU: ", channel2copyimagetobufferconsume4mpu, " CONTROL & COMMAND: ", message2copyimagereceive4mpu)                                                                                                   
#   except:
#       print('(Step 69Z/700) OF MPU2IMAGE4XI5TEST *** FAILED TO RECEIVE CONTROL & COMMAND ABOUT IMAGE OPERTION VIA PIPE ', time4xi())               
#   # -------------------------------------------------------------------------------------------------------
#   # (70) LET MANOEUVRE2COPYIMAGETOBUFFER4MPU GO TO STATE 0               
#   # -------------------------------------------------------------------------------------------------------       
#   print("(Step 70A/700) OF MPU2IMAGE4XI5TEST - TRYING TO PERFORM TARGET FUNCTION MANOEUVRE2COPYIMAGETOBUFFER4MPU ", time4xi()) 
#   try:        
#       response2manoeuvreimagecopytobuffer4mpu = manoeuvre2copyimagetobuffer4mpu(queue2copyimagetobuffer4mpu, channel2copyimagetobufferconsume4mpu, channel2copyimagetobufferpublish4mpu)
#       print("(Step 70B/700) OF MPU2IMAGE4XI5TEST - GOT RESPONSE2MANOEUVRE4MPU ", response2manoeuvreimagecopytobuffer4mpu)
#   except:
#       print("(Step 70Z/700) OF MPU2IMAGE4XI5TEST *** FAILED TO PERFORM TARGET FUNCTION MANOEUVRE2COPYIMAGETOBUFFER4MPU ", time4xi())   

#">

# -------------------------------------------------------------------------------------------------   
def copy2imagetobuffer4mpu(task4copyimagetobuffer2mpu=None):

    '''
    copies 2D images to buffer for MPU
    '''
    global mpu2image4xi                                                                                             # 2D-IMAGES FOR MPU  
    global mpu2buffer4xi                                                                                            # BUFFER FOR MPU
    
    global control2copyimagetobuffer4mpu                                                                            # COMMAND AND CONTROL FOR MPU IMAGES 
    
    # -----------------------------------------------------------------------------------------
    # (Step 1/33) PARAMETERS           
    # -----------------------------------------------------------------------------------------          
    class2print = "copy2imagetobuffer4mpu"
    method2print = "copy2imagetobuffer4mpu"

    message2print = "{(Step 1A/33) started" + "; copy2imagetobuffer4mpu-started: " + str(copy2imagetobuffer4mpu)  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                                 
    # <function copy2imagetobuffer4mpu at 0x000000000390B948>
    
    message2print = "{(Step 1B/33) passed" + "; task4copyimagetobuffer2mpu-to-copy2imagetobuffer4mpu: " + str(task4copyimagetobuffer2mpu) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                                                              
    # None
            
    code2ack  = ' '
    code2nack = ' '
    
    # -----------------------------------------------------------------------------------------
    # CREATE MPU2IMAGE4XI IF NECESSARY       
    # ----------------------------------------------------------------------------------------- 
    try:
        if mpu2image4xi is None:
            mpu2image4xi = create2image4mpu()                                                                              # INITIALIZE IMAGE FOR MPU 
            message2print = "{(Step 1C/70) created" + "; mpu2image4xi-created-by-copy2imagetobuffer4mpu to: " + str(mpu2image4xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #             
        else:
            message2print = "{(Step 1D/70) skipped" + "; create2image4mpu-skipped-by-copy2imagetobuffer4mpu: " + str(copy2imagetobuffer4mpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            # create2image4mpu-skipped-by-copy2imagetobuffer4mpu                                 
    except:
        try:
            mpu2image4xi = create2image4mpu()
            message2print = "{(Step 1E/70) set" + "; mpu2image4xi-set-by-copy2imagetobuffer4mpu to: " + str(mpu2image4xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #             
        except:
            mpu2image4xi = {}
            message2print = "{(Step 1Z1/70) yielded" + "; mpu2image4xi-yielded-by-copy2imagetobuffer4mpu: " + str(mpu2image4xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            code2nack  = code2nack + '1Z1-' 
    
    # -----------------------------------------------------------------------------------------
    # CREATE MPU2BUFFER4XI IF NECESSARY       
    # ----------------------------------------------------------------------------------------- 
    try:
        if mpu2buffer4xi is None:
            mpu2buffer4xi = create2buffer4mpu()                                                                     # INITIALIZE BUFFER FOR MPU 
            message2print = "{(Step 1F/70) created" + "; mpu2buffer4xi-created-by-copy2imagetobuffer4mpu to: " + str(mpu2buffer4xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #             
        else:
            message2print = "{(Step 1G/70) skipped" + "; create2buffer4mpu-skipped-by-copy2imagetobuffer4mpu: " + str(copy2imagetobuffer4mpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #                                  
    except:
        try:
            mpu2buffer4xi = create2buffer4mpu()
            message2print = "{(Step 1H/70) set" + "; mpu2buffer4xi-set-by-copy2imagetobuffer4mpu to: " + str(mpu2buffer4xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            # {'CODE2ERROR': 0,
            #  'COUNT2REFERENCE': 0,
            #   'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2buffer4mpu-failed ; time: 03:40:47 10/04/11 Pacific Daylight Time}',
            #   'CLASS': 'create2buffer4mpu',
            #   'METHOD': 'create2buffer4mpu'},
            #   'FUNCTION2CALLBACK': <function trace2buffer4mpu at 0x0000000004782B48>,
            #   'POINTER2BUFFER': <mmap.mmap object at 0x000000000423BD98>,
            #   'FLAGS': 0,
            #   'CONTEXT': {'COUNT2REFERENCE': 0,
            #   'DATA2CALLBACK': ...
            
        except:
            mpu2buffer4xi = {}
            message2print = "{(Step 1Z2/70) yielded" + "; mpu2buffer4xi-yielded-by-copy2imagetobuffer4mpu: " + str(mpu2buffer4xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            code2nack  = code2nack + '1Z2-' 
                            
    # -----------------------------------------------------------------------------------------
    # (Step 1/70) - ANALYZE TASK          
    # -----------------------------------------------------------------------------------------          
              
    if task4copyimagetobuffer2mpu is None:
        # -----------------------------------------------------------------------------------------
        # NO PARAMETERS GIVEN BY CALLER FOR COPY - BUILD TASK! 
        # -----------------------------------------------------------------------------------------    
            
        task4copyimagetobuffer2mpu = {}                                                                                                                                                                                       
            
        message2print = "{(Step 1A/70) initialized" + "; task4copyimagetobuffer2mpu-created-by-copy2imagetobuffer4mpu to: " + str(task4copyimagetobuffer2mpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)               
        # 
            
        # ---------------------------------------------------------------------------------------------
        # (1B) TRACE - AUTOMATIC                      
        # ---------------------------------------------------------------------------------------------              
        function2callback4mpu = trace2image4mpu                                                                 # CALLBACK FUNCTION FOR MPU
            
        message2print = "{(Step 1B1/70) initialized" + "; function2callback4mpu-created-by-copy2imagetobuffer4mpu to: " + str(function2callback4mpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
        # <function trace2image4mpu at 0x0000000004797548> 
            
        code2ack  = code2ack + '1B1-'
           
        args2callback = {}                                                                                      # CREATE ARGUMENT DICTIONARY FOR CALLBACK FUNCTION             
        args2callback['CLASS']   = "copy2imagetobuffer4mpu"                                                            # MODULE OF MPUIZED OPENCL MODULE
        args2callback['METHOD']  = "copy2imagetobuffer4mpu"                                                            # FUNCTION OF MPUIZED OPENCL MODULE
        args2callback['MESSAGE'] = "{(Step 1Z1/33) failed" + "; copy2imagetobuffer4mpu-failed " + "; time: " + time4xi() + "}" # ERROR MESSAGE FOR MPUIZED OPENCL TO RETURN    

        message2print = "{(Step 1B2/70) initialized" + "; args2callback-created-by-copy2imagetobuffer4mpu to: " + str(args2callback)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)               
        # {'MESSAGE': '{(Step 1Z1/33) failed; copy2imagetobuffer4mpu-failed; time: 03:40:47 10/04/11 Pacific Daylight Time}', 
        #  'CLASS': 'copy2imagetobuffer4mpu', 
        #  'METHOD': 'copy2imagetobuffer4mpu'}; time: 03:40:47 10/04/11 Pacific Daylight Time}


        code2ack  = code2ack + '1B2-'
            
        # ---------------------------------------------------------------------------------------------
        # (1C) CALLBACK - AUTOMATIC                      
        # ---------------------------------------------------------------------------------------------              
        task4copyimagetobuffer2mpu['DATA2CALLBACK'] = args2callback                                                    # USER DATA PASSED TO MPUIZED OPENCL - SEVERAL ARGS PASSED IN ONE DICTIONARY
        task4copyimagetobuffer2mpu['FUNCTION2CALLBACK'] = function2callback4mpu                                        # CALLBACK FUNCTION TO CALLED BY OPENCL IN EXCEPTIONS
        
        code2ack  = code2ack + '1C-'
        message2print = "{(Step 1C/70) initialized" + "; task4copyimagetobuffer2mpu-created-by-copy2imagetobuffer4mpu to: " + str(task4copyimagetobuffer2mpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # {'FUNCTION2CALLBACK': <function trace2image4mpu at 0x0000000004797548>,
        #   'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; copy2imagetobuffer4mpu-failed ; time: 03:40:47 10/04/11 Pacific Daylight Time}', 
        #  'CLASS': 'copy2imagetobuffer4mpu', 
        #  'METHOD': 'copy2imagetobuffer4mpu'}}; time: 03:40:47 10/04/11 Pacific Daylight Time}

            
        # ---------------------------------------------------------------------------------------------
        # (2) PLATFORM - AUTOMATIC                   
        # ---------------------------------------------------------------------------------------------             
        platform2xi = {}
        message2print = "{(Step 2A/70) initialized" + "; platform2xi-created-by-copy2imagetobuffer4mpu to: " + str(platform2xi)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
        # {}
            
        try:
            platform2xi = get2platformids4xi()                                                                  # PLATFORM IS COMMON TO ALL DEVICES I.E TO DEVICE XI
            message2print = "{(Step 2B/70) got" + "; platform2xi-got-by-copy2imagetobuffer4mpu: " + str(platform2xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            # {'list2platforms': ['DragonXi'], 'ack2return': ' 2C-2E-2F-2G-', 'nack2return': ' ', 'available2platform': 1, 'max2entries': 1}; time: 03:40:47 10/04/11 Pacific Daylight Time}
                 
            code2ack  = code2ack + '2B-'
        except:
            message2print = "{(Step 2Z/70) failed" + "; platform2xi-failed-by-copy2imagetobuffer4mpu: " + str(failed)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)                                                
            code2nack  = code2nack + '2Z-'
                 
        task4copyimagetobuffer2mpu['PLATFORM'] = platform2xi                                                           # PLATFORM OF SYSTEM 
              
            
        # ---------------------------------------------------------------------------------------------
        # (3) CONTEXT - AUTOMATIC                 
        # ---------------------------------------------------------------------------------------------    
        context2mpu = {}
        message2print = "{(Step 3A/70) trying" + "; context2mpu-trying-by-copy2imagetobuffer4mpu: " + str(context2mpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        try:
            context2mpu = get2context4mpu()                                                                     # NO PARAMETERS - THE SYSTEM CREATES CONTEXT IF REQUIRED  
                
            message2print = "{(Step 3B/70) got" + "; context2mpu-got-by-copy2imagetobuffer4mpu: " + str(context2mpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)                                  
            # {'COUNT2REFERENCE': 0, 
            #  'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2context4mpu-failed ; time: 03:40:47 10/04/11 Pacific Daylight Time}', 
            #  'CLASS': 'create2context4mpu', 
            #  'METHOD': 'create2context4mpu'}, 
            #  'FUNCTION2CALLBACK': <function trace2context4mpu at 0x000000000394F348>, 
            #  'ACK2RETURN': ' 2A-2C-2D-2E-2B-3B-4B-5B-6B-7B-7B-', 
            #  'INFO2PRIVATE': None, 
            #  'PLATFORM': {'list2platforms': ['DragonXi'],'ack2return': ' 2C-2E-2F-2G-', 'nack2return': ' ', 'available2platform': 1, 'max2entries': 1}, 
            #  'NACK2RETURN': ' ', 
            #  'SIZE2PRIVATE': 0, 
            #  'PROPERTIES': None}; time: 03:40:47 10/04/11 Pacific Daylight Time}
                
            code2ack  = code2ack + '3B-'         
        except:
            message2print = "{(Step 3Z/70) failed" + "; context2mpu-failed-by-copy2imagetobuffer4mpu: " + str(context2mpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '3Z-'
            
        task4copyimagetobuffer2mpu ['CONTEXT'] = context2mpu           
            
        # ---------------------------------------------------------------------------------------------
        # (4) DEVICE - AUTOMATIC                       
        # ---------------------------------------------------------------------------------------------    
        device2mpu = {}
            
        message2print = "{(Step 4A/70) trying" + "; device2mpu-trying-by-copy2imagetobuffer4mpu: " + str(device2mpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        try:
            device2mpu = get2mpu4xi()                                                                           # THE SYSTEM CREATES DEVICE INFO, IF NECESSARY    
            
            message2print = "{(Step 4B/70) got" + "; device2mpu-got-by-copy2imagetobuffer4mpu: " + str(device2mpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)                                  
            # {'RESOLUTION2TIMER4PROFILING': 318, 
            #  'ID2PLATFORM': '0000000001509568', 
            #  'PROFILING2QUEUE': 'ACK', 
            #  'EXECUTE2KERNELS4OPENCL': 'ACK', 
            #  'NATIVE2WIDTHLONG4VECTOR': 2, 
            #  'MAX2UNITS4COMPUTE': 4, 
            #  'SIZE2MEMORY4GLOBAL': 3221225472, 
            #  'NATIVE2WIDTHFLOAT4VECTOR': 4, 
            #  'MAX2SIZE4WORKGROUP': 1024, 
            #  'PREFERRED2WIDTHCHAR4VECTOR': 16, 
            #  'SIZE2CACHE': 65536, 
            #  'IEEE754-2008FUSED2MULTIPLY-ADD': 'NACK', 
            #  'TYPE4CACHE': 'Read/Write', 
            #  'PREFERRED2WIDTHINT4VECTOR': 4, 
            #  'SIZE2PREFERREDWORKGROUPMULTIPLE4KERNEL': 1, 
            #  'NATIVE2WIDTHINT4VECTOR': 4, 
            #  'SIZE2BUFFER4CONSTANT': 65536, 
            #  'EXECUTE2FUNCTION4NATIVE': 'ACK', 
            #  'PREFERRED2WIDTHDOUBLE4VECTOR': 0, 
            #  'MAX2NUMBER4CONSTANTARGS': 8, 
            #  'VERSION2DRIVER': '2.0', 
            #  'EXTENSIONS': 'cl_amd_fp64 ...
            #  'AVAILABLE2COMPILER': 'ACK', 
            #  'MAX2DIMENSIONS4WORKITEMS': 3, 
            #  'SUPPORT2CORRECTION4ERROR': 0, 
            #  'BITS2ADDRESS': 64, 
            #  'AVAILABLE': 'ACK', 
            #  'SIZE2MEMORY4LOCAL': 32768, 
            #  'VENDOR': 'AuthenticAMD', 
            #  'SUPPORT2IMAGE': 'NACK', 
            #  'MAX2SIZE4KERNELARGUMENT': 4096, 
            #  'ACK2RETURN': ' 1C-1D-1E-1F-1D-2A-...
            #  'OUT-OF-ORDER2QUEUE': 'NACK', 
            #  'ALIGNMENT2ADDRESS4BASE': 1024, 
            #  'PROFILE': 'FULL_PROFILE', 
            #  'TYPE2MEMORY4LOCAL': 'Global', 
            #  'PREFERRED2WIDTHFLOAT4VECTOR': 4, 
            #  'PREFERRED2WIDTHLONG4VECTOR': 2, 
            #  'MAX2FREQUENCY4CLOCK': 3210, 
            #  'PREFERRED2WIDTHSHORT4VECTOR': 8, 
            #  'ID': 4033, 
            #  'MAX2WORKITEM2': 1024, 
            #  'NATIVE2WIDTHDOUBLE4VECTOR': 0, 
            #  'ROUND2ZERO': 'ACK', 
            #  'NAME': 'AMD64 Family 16 Model 4 Stepping 2, 
            #   AuthenticAMD', 
            #  'MAX2WORKITEM1': 1024, 
            #  'MAX2WORKITEM0': 1024, 
            #  'SIZE2LINE4CACHE': 64, 
            #  'MAX2ALLOCATION4MEMORY': 1073741824, 
            #  'NATIVE2WIDTHCHAR4VECTOR': 16, 
            #  'QUIET2NANS': 'ACK', 
            #  'NATIVE2WIDTHSHORT4VECTOR': 8, 
            #  'ROUND2NEAREST4EVEN': 'ACK', 
            #  'VERSION': 'OpenCL 1.1 ATI-Stream-v2.3 (451)', 
            #  'NACK2RETURN': ' ', 
            #  'MEMORY2UNIFIED4HOSTANDDEVICE': 1, 
            #  'ENDIANESS2DEVICE': 'Little', 
            #  'MINIMUM2ALIGNMENT4DATATYPE': 128, 
            #  'ROUND2+VE3INFINITY': 'ACK', 
            #  'TYPE': 'CL_DEVICE_TYPE_MPU', 
            #  'DENORMS2SINGLEPRECISION4FLOATINGPOINTCAPABILITY': 'ACK'}; time: 03:40:47 10/04/11 Pacific Daylight Time}
            
            code2ack  = code2ack + '4B-'  
        except:
            message2print = "{(Step 4Z/70) failed" + "; device2mpu-failed-by-copy2imagetobuffer4mpu: " + str(device2mpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '4Z-'
        
        task4copyimagetobuffer2mpu ['DEVICE'] = device2mpu
        
               
        # ---------------------------------------------------------------------------------------------
        # (5) IMAGE SOURCE OBJECT - REQUIRED                         
        # ---------------------------------------------------------------------------------------------         
        imagesourceinfo2mpu = {}
         
        message2print = "{(Step 5A/70) trying" + "; imagesourceinfo2mpu-trying-by-copy2imagetobuffer4mpu: " + str(imagesourceinfo2mpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        try:
            imagesourceinfo2mpu = get2imageinfo4mpu()                                                                         
            
            message2print = "{(Step 5B/70) got" + "; imagesourceinfo2mpu-got-by-copy2imagetobuffer4mpu: " + str(imagesourceinfo2mpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)                                  
            # {'SIZE2QUEUE': 2147483646, 
            #  'FORMAT': {'TYPE': 'CL_UNORM_INT8', 
            #  'ORDER': 'CL_RGBA'}, 
            #  'QUEUE': <queue4xi.Queue4Xi object at 0x00000000042514E0>, 
            #  'WIDTH': 800, 
            #  'CONTEXT': ...

            code2ack  = code2ack + '5B-'  
        except:
            message2print = "{(Step 5Z/70) failed" + "; imagesourceinfo2mpu-failed-by-copy2imagetobuffer4mpu: " + str(imagesourceinfo2mpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '5Z-'          
                  
        task4copyimagetobuffer2mpu ['IMAGESOURCE'] = imagesourceinfo2mpu         
               
        # ---------------------------------------------------------------------------------------------
        # (6) BUFFER DESTINATION OBJECT - REQUIRED                        
        # ---------------------------------------------------------------------------------------------                         
        bufferdestinationinfo2mpu = {}
         
        message2print = "{(Step 6A/70) trying" + "; bufferdestinationinfo2mpu-trying-by-copy2imagetobuffer4mpu: " + str(bufferdestinationinfo2mpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        try:
            bufferdestinationinfo2mpu = get2bufferinfo4mpu()                                                                         
            
            message2print = "{(Step 6B/70) got" + "; bufferdestinationinfo2mpu-got-by-copy2imagetobuffer4mpu: " + str(bufferdestinationinfo2mpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)                                  
            # {'CODE2ERROR': 0, 
            #  'COUNT2REFERENCE': 0, 
            #  'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2buffer4mpu-failed ; time: 03:40:47 10/04/11 Pacific Daylight Time}', 
            #  'CLASS': 'create2buffer4mpu', 
            #  'METHOD': 'create2buffer4mpu'}, 
            #  'FUNCTION2CALLBACK': <function trace2buffer4mpu at 0x0000000004782B48>, 
            #  'POINTER2BUFFER': <mmap.mmap object at 0x000000000423BD98>, 
            #  'FLAGS': 0, 
            #  'CONTEXT': {'COUNT2REFERENCE': 0, ...
            code2ack  = code2ack + '6B-'  
        except:
            message2print = "{(Step 6Z/70) failed" + "; bufferdestinationinfo2mpu-failed-by-copy2imagetobuffer4mpu: " + str(bufferdestinationinfo2mpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '6Z-'          
                  
        task4copyimagetobuffer2mpu ['BUFFERDESTINATION'] = bufferdestinationinfo2mpu     
        
        # ---------------------------------------------------------------------------------------------
        # (7) QUEUE - REQUIRED BY MPUXI                          
        # ---------------------------------------------------------------------------------------------         
        queue2copyimagetobuffer4mpu = None
        
        message2print = "{(Step 7A/70) trying" + "; queue2copyimagetobuffer4mpu-trying-by-copy2imagetobuffer4mpu: " + str(queue2copyimagetobuffer4mpu)  + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
        try:
            queue2copyimagetobuffer4mpu = Queue4Xi()                                                                              # THE SYSTEM CREATES QUEUE    
            
            message2print = "{(Step 7B/70) got" + "; queue2copyimagetobuffer4mpu-got-by-copy2imagetobuffer4mpu: " + str(queue2copyimagetobuffer4mpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)                                  
            # <queue4xi.Queue4Xi object at 0x00000000042B2208> 
            
            code2ack  = code2ack + '7B-'  
        except:
            message2print = "{(Step 7Z/70) failed" + "; queue2copyimagetobuffer4mpu-failed-by-copy2imagetobuffer4mpu: " + str(queue2copyimagetobuffer4mpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '7Z-'
            
        task4copyimagetobuffer2mpu ['QUEUE'] = queue2copyimagetobuffer4mpu
         
        # ---------------------------------------------------------------------------------------------
        # (8) PIPE - REQUIRED BY MPUXI                              
        # ---------------------------------------------------------------------------------------------         
        pipe2mpu = None
       
        message2print = "{(Step 8A/70) trying" + "; pipe2mpu-trying-by-copy2imagetobuffer4mpu: " + str(pipe2mpu)  + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
        try:
            pipe2mpu = Pipe4Xi()                                                                                # THE SYSTEM CREATES PIPE    
            
            message2print = "{(Step 8B/70) got" + "; pipe2mpu-got-by-copy2imagetobuffer4mpu: " + str(pipe2mpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)                                  
            # (<read-write PipeConnection, handle 660>, <read-write PipeConnection, handle 664>)
            
            code2ack  = code2ack + '8B-'  
        except:
            message2print = "{(Step 8Z/70) failed" + "; pipe2mpu-failed-by-copy2imagetobuffer4mpu: " + str(pipe2mpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '8Z-'
            
        task4copyimagetobuffer2mpu ['PIPE'] = pipe2mpu 
        
        # ---------------------------------------------------------------------------------------------
        # (9) COMMAND QUEUE - REQUIRED                              
        # ---------------------------------------------------------------------------------------------             
        commandqueueinfo2mpu = {}
       
        message2print = "{(Step 9A/70) trying" + "; commandqueueinfo2mpu-trying-by-copy2imagetobuffer4mpu: " + str(commandqueueinfo2mpu)  + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print)  
                    
        try:
            commandqueueinfo2mpu = get2commandqueueinfo4mpu()                                                   # SYSTEM CREATES COMMAND QUEUE, IF NECESSARY   
            message2print = "{(Step 9B/70) got" + "; commandqueueinfo2mpu-got-by-copy2imagetobuffer4mpu: " + str(commandqueueinfo2mpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # {'CODE2ERROR': 0, 
            #  'COUNT2REFERENCE': 0, 
            #  'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2commandqueue4mpu-failed ; time: 03:40:47 10/04/11 Pacific Daylight Time}', 
            #  'CLASS': 'create2commandqueue4mpu', 
            #  'METHOD': 'create2commandqueue4mpu'}, 
            #  'FUNCTION2CALLBACK': <function trace2commandqueue4mpu at 0x000000000395EA48>, 
            #  'ACK2RETURN': ...
            
            code2ack  = code2ack + '9B-'  
        except:
            message2print = "{(Step 9Z/70) failed" + "; commandqueueinfo2mpu-failed-by-copy2imagetobuffer4mpu: " + str(commandqueueinfo2mpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '9Z-'
    
        task4copyimagetobuffer2mpu ['COMMANDQUEUE'] = commandqueueinfo2mpu
        
        
        
        # ---------------------------------------------------------------------------------------------
        # (11) EVENT - REQUIRED                            
        # ---------------------------------------------------------------------------------------------             
        eventinfo2mpu = {}
       
        message2print = "{(Step 11A/70) trying" + "; eventinfo2mpu-trying-by-copy2imagetobuffer4mpu: " + str(eventinfo2mpu)  + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
                    
        try:
            eventinfo2mpu = get2eventinfo4mpu()                                                                    
            message2print = "{(Step 11B/70) got" + "; eventinfo2mpu-got-by-copy2imagetobuffer4mpu: " + str(eventinfo2mpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # {'CODE2ERROR': 0, 
            #  'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2event4mpu-failed ; time: 03:40:47 10/04/11 Pacific Daylight Time}', 
            #  'CLASS': 'create2event4mpu', 
            #  'METHOD': 'create2event4mpu'}, 
            #  'FUNCTION2CALLBACK': <function trace2event4mpu at 0x0000000003971F48>, 
            #  'ACK2RETURN': ' 1D-1E-...
            code2ack  = code2ack + '11B-' 
        except:
            message2print = "{(Step 11Z/70) failed" + "; eventinfo2mpu-failed-by-copy2imagetobuffer4mpu: " + str(eventinfo2mpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '11Z-'
    
        task4copyimagetobuffer2mpu ['EVENT'] = eventinfo2mpu 
        
        
        # ---------------------------------------------------------------------------------------------
        # (12) EVENTWAITLIST - REQUIRED                              
        # ---------------------------------------------------------------------------------------------             
        eventwaitlist2mpu = {}
       
        message2print = "{(Step 12A/70) trying" + "; eventwaitlist2mpu-trying-by-copy2imagetobuffer4mpu: " + str(eventwaitlist2mpu)  + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
                    
        try:
            eventwaitlist2mpu = {}                                                                   
            message2print = "{(Step 12B/70) set" + "; eventwaitlist2mpu-set-by-copy2imagetobuffer4mpu: " + str(eventwaitlist2mpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # {}
            
            code2ack  = code2ack + '12B-' 
        except:
            message2print = "{(Step 12Z/70) failed" + "; eventwaitlist2mpu-failed-by-copy2imagetobuffer4mpu: " + str(eventwaitlist2mpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '12Z-'
    
        task4copyimagetobuffer2mpu ['EVENTWAITLIST'] = eventwaitlist2mpu                     
         
        # ---------------------------------------------------------------------------------------------
        # (13) EVENTWAITLISTQUANTITY - REQUIRED                              
        # ---------------------------------------------------------------------------------------------             
        eventwaitlistquantity2mpu = 0
       
        message2print = "{(Step 13A/70) trying" + "; eventwaitlistquantity2mpu-trying-by-copy2imagetobuffer4mpu: " + str(eventwaitlistquantity2mpu)  + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
                    
        try:
            keys2event = eventwaitlist2mpu.keys()
            for key2event in keys2event:
                eventwaitlistquantity2mpu = eventwaitlistquantity2mpu + 1                                                                      
            message2print = "{(Step 13B/70) set" + "; eventwaitlistquantity2mpu-set-by-copy2imagetobuffer4mpu: " + str(eventwaitlistquantity2mpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # 0 
            
            code2ack  = code2ack + '13B-' 
        except:
            message2print = "{(Step 13Z/70) failed" + "; eventwaitlistquantity2mpu-failed-by-copy2imagetobuffer4mpu: " + str(eventwaitlistquantity2mpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '13Z-'
    
        task4copyimagetobuffer2mpu ['EVENTWAITLISTQUANTITY'] = eventwaitlistquantity2mpu       
                                             
        
        # ---------------------------------------------------------------------------------------------
        # (14) ORIGIN FOR SOURCE - REQUIRED                             
        # ---------------------------------------------------------------------------------------------             
        origin4source2mpu = {}
        
        message2print = "{(Step 14A/70) trying" + "; origin4source2mpu-trying-by-copy2imagetobuffer4mpu: " + str(origin4source2mpu)  + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
                    
        try:                                                                                    
            origin4source2mpu['X'] = 0 
            origin4source2mpu['Y'] = 0  
            origin4source2mpu['Z'] = 0                                                                                                                          
            message2print = "{(Step 14B/70) set" + "; origin4source2mpu-set-by-copy2imagetobuffer4mpu to: " + str(origin4source2mpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # {'Y': 0, 'X': 0, 'Z': 0} 
            code2ack  = code2ack + '14B-' 
        except:
            message2print = "{(Step 14Z/70) failed" + "; origin4source2mpu-failed-by-copy2imagetobuffer4mpu: " + str(origin4source2mpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '14Z-'
    
        task4copyimagetobuffer2mpu ['SOURCEORIGIN'] = origin4source2mpu
        
       
        
        # --------------------------------------------------------------------------------------------------------
        # (16) REGION - REGION OF A IMAGE TO BE COPIED                         
        # --------------------------------------------------------------------------------------------------------             
        region4copy2mpu = {}                                                                                                         
       
        message2print = "{(Step 16A/70) trying" + "; region4copy2mpu-trying-by-copy2imagetobuffer4mpu: " + str(region4copy2mpu)  + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
                    
        try:
            region4copy2mpu['X'] = 10 
            region4copy2mpu['Y'] = 10  
            region4copy2mpu['Z'] = 10                                                                           
            message2print = "{(Step 16B/70) set" + "; region4copy2mpu-set-by-copy2imagetobuffer4mpu to: " + str(region4copy2mpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # {'Y': 10, 'X': 10, 'Z': 10} 
            code2ack  = code2ack + '16B-'                
             
        except:
            message2print = "{(Step 16Z/70) failed" + "; region4copy2mpu-failed-by-copy2imagetobuffer4mpu: " + str(region4copy2mpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '16Z-'
        
        task4copyimagetobuffer2mpu ['REGION'] = region4copy2mpu              
        
        # --------------------------------------------------------------------------------------------------------
        # (17) OFFSET - WHERE TO BEGIN COPYING DATA INTO DESTINATION BUFFER                     
        # --------------------------------------------------------------------------------------------------------             
        offset4copy2mpu = 0                                                                                                         
        message2print = "{(Step 17A/70) set" + "; offset4copy2mpu-set-by-copy2imagetobuffer4mpu to: " + str(offset4copy2mpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        # 0
        code2ack  = code2ack + '17A-'                
        task4copyimagetobuffer2mpu ['OFFSET'] = offset4copy2mpu            
                    
        # ---------------------------------------------------------------------------------------------
        # EOF PARAMETER PROVIDING                
        # ---------------------------------------------------------------------------------------------                                                                                                                               
    else:
        pass
                    
    # ---------------------------------------------------------------------------------------------
    # ANALYZE TASK - WHETHER AUTOMATICALLY GENERATED OR USER GIVEN 
    # ---------------------------------------------------------------------------------------------
    keys2task = task4copyimagetobuffer2mpu.keys()
    message2print = "{(Step 20/70) got" + "; keys2task-got-by-copy2imagetobuffer4mpu: " + str(sorted(keys2task))  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)            
    # ['BUFFERDESTINATION',
    #  'COMMANDQUEUE',
    #  'CONTEXT',
    #  'DATA2CALLBACK',
    #  'DEVICE',
    #  'EVENT',
    #  'EVENTWAITLIST',
    #  'EVENTWAITLISTQUANTITY',
    #  'FUNCTION2CALLBACK',
    #  'IMAGESOURCE',
    #  'OFFSET',
    #  'PIPE',
    #  'PLATFORM',
    #  'QUEUE',
    #  'REGION',
    #  'SOURCEORIGIN']; time: 01:28:44 10/05/11 Pacific Daylight Time}


    if (('DATA2CALLBACK' in keys2task) and ('FUNCTION2CALLBACK' in keys2task)):
        pass
    else:                
        # ---------------------------------------------------------------------------------------------
        # (21B) TRACE - AUTOMATIC                      
        # ---------------------------------------------------------------------------------------------              
        function2callback4mpu = trace2image4mpu                                                             # CALLBACK FUNCTION FOR MPU
        
        message2print = "{(Step 21B1/70) initialized" + "; function2callback4mpu-created-by-copy2imagetobuffer4mpu to: " + str(function2callback4mpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
        # 
            
        code2ack  = code2ack + '21B1-'
           
        args2callback = {}                                                                                  # CREATE ARGUMENT DICTIONARY FOR CALLBACK FUNCTION             
        args2callback['CLASS']   = "copy2imagetobuffer4mpu"                                                        # MODULE OF MPUIZED OPENCL MODULE
        args2callback['METHOD']  = "copy2imagetobuffer4mpu"                                                        # FUNCTION OF MPUIZED OPENCL MODULE
        args2callback['MESSAGE'] = "{(Step 1Z1/33) failed" + "; copy2imagetobuffer4mpu-failed " + "; time: " + time4xi() + "}" # ERROR MESSAGE FOR MPUIZED OPENCL TO RETURN    

        message2print = "{(Step 21B2/70) initialized" + "; args2callback-created-by-copy2imagetobuffer4mpu to: " + str(args2callback)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)               
        # 

        code2ack  = code2ack + '21B2-'
        
        # ---------------------------------------------------------------------------------------------
        # (21C) CALLBACK - AUTOMATIC                      
        # ---------------------------------------------------------------------------------------------              
        task4copyimagetobuffer2mpu['DATA2CALLBACK'] = args2callback                                                # USER DATA PASSED TO MPUIZED OPENCL - SEVERAL ARGS PASSED IN ONE DICTIONARY
        task4copyimagetobuffer2mpu['FUNCTION2CALLBACK'] = function2callback4mpu                                    # CALLBACK FUNCTION TO CALLED BY OPENCL IN EXCEPTIONS
        
        code2ack  = code2ack + '21C-'
        message2print = "{(Step 21C/70) initialized" + "; task4copyimagetobuffer2mpu-created-by-copy2imagetobuffer4mpu to: " + str(task4copyimagetobuffer2mpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        #
             
    if 'PLATFORM' in keys2task:
        pass
    else:  
        # ---------------------------------------------------------------------------------------------
        # (22) PLATFORM - AUTOMATIC                   
        # ---------------------------------------------------------------------------------------------             
        platform2xi = {}
        message2print = "{(Step 22A/70) initialized" + "; platform2xi-created-by-copy2imagetobuffer4mpu to: " + str(platform2xi)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
        # 
            
        try:
            platform2xi = get2platformids4xi()                                                              # PLATFORM IS COMMON TO ALL DEVICES I.E TO DEVICE XI
            message2print = "{(Step 22B/70) got" + "; platform2xi-got-by-copy2imagetobuffer4mpu: " + str(platform2xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            # 
                 
            code2ack  = code2ack + '22B-'
        except:
            message2print = "{(Step 22Z/70) failed" + "; platform2xi-failed-by-copy2imagetobuffer4mpu: " + str(failed)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)                                                
            code2nack  = code2nack + '22Z-'
                 
        task4copyimagetobuffer2mpu['PLATFORM'] = platform2xi                                                       # PLATFORM OF SYSTEM 
          
    if 'CONTEXT' in keys2task:
        pass
    else:              
        # ---------------------------------------------------------------------------------------------
        # (23) CONTEXT - AUTOMATIC                 
        # ---------------------------------------------------------------------------------------------    
        context2mpu = {}
        message2print = "{(Step 23A/70) trying" + "; context2mpu-trying-by-copy2imagetobuffer4mpu: " + str(context2mpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        try:
            context2mpu = get2context4mpu()                                                                 # NO PARAMETERS - THE SYSTEM CREATES CONTEXT IF REQUIRED  
            
            message2print = "{(Step 23B/70) got" + "; context2mpu-got-by-copy2imagetobuffer4mpu: " + str(context2mpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)                                  
            # 
            
            code2ack  = code2ack + '23B-'         
        except:
            message2print = "{(Step 23Z/70) failed" + "; context2mpu-failed-by-copy2imagetobuffer4mpu: " + str(context2mpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '23Z-'
            
        task4copyimagetobuffer2mpu ['CONTEXT'] = context2mpu           
    
    if 'DEVICE' in keys2task:
        pass
    else:                   
        # ---------------------------------------------------------------------------------------------
        # (24) DEVICE - AUTOMATIC                       
        # ---------------------------------------------------------------------------------------------    
        device2mpu = {}
        
        message2print = "{(Step 24A/70) trying" + "; device2mpu-trying-by-copy2imagetobuffer4mpu: " + str(device2mpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        try:
            device2mpu = get2mpu4xi()                                                                       # THE SYSTEM CREATES DEVICE INFO, IF NECESSARY    
            
            message2print = "{(Step 24B/70) got" + "; device2mpu-got-by-copy2imagetobuffer4mpu: " + str(device2mpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)                                  
            # 
                
            code2ack  = code2ack + '24B-'  
        except:
            message2print = "{(Step 24Z/70) failed" + "; device2mpu-failed-by-copy2imagetobuffer4mpu: " + str(device2mpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '24Z-'
        
        task4copyimagetobuffer2mpu ['DEVICE'] = device2mpu
    
    if 'IMAGESOURCE' in keys2task:
        pass
    else:                      
        # ---------------------------------------------------------------------------------------------
        # (25) SOURCE IMAGE - REQUIRED                            
        # ---------------------------------------------------------------------------------------------         
        sourceimageinfo2mpu = {}
         
        message2print = "{(Step 25A/70) trying" + "; sourceimageinfo2mpu-trying-by-copy2imagetobuffer4mpu: " + str(sourceimageinfo2mpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        try:
            sourceimageinfo2mpu = get2imageinfo4mpu()                                                                         
            
            message2print = "{(Step 25B/70) got" + "; sourceimageinfo2mpu-got-by-copy2imagetobuffer4mpu: " + str(sourceimageinfo2mpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)                                  
            #                 
            code2ack  = code2ack + '25B-'  
        except:
            message2print = "{(Step 25Z/70) failed" + "; sourceimageinfo2mpu-failed-by-copy2imagetobuffer4mpu: " + str(sourceimageinfo2mpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '25Z-'          
                  
        task4copyimagetobuffer2mpu ['IMAGESOURCE'] = sourceimageinfo2mpu         
    
    if 'BUFFERDESTINATION' in keys2task:
        pass
    else:                      
        # ---------------------------------------------------------------------------------------------
        # (26) DESTINATION BUFFER - REQUIRED                            
        # ---------------------------------------------------------------------------------------------         
        destinationbufferinfo2mpu = {}
         
        message2print = "{(Step 26A/70) trying" + "; destinationbufferinfo2mpu-trying-by-copy2imagetobuffer4mpu: " + str(destinationbufferinfo2mpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        try:
            destinationbufferinfo2mpu = get2bufferinfo4mpu()                                                                         
            
            message2print = "{(Step 26B/70) got" + "; destinationbufferinfo2mpu-got-by-copy2imagetobuffer4mpu: " + str(destinationbufferinfo2mpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)                                  
            #                 
            code2ack  = code2ack + '26B-'  
        except:
            message2print = "{(Step 26Z/70) failed" + "; destinationbufferinfo2mpu-failed-by-copy2imagetobuffer4mpu: " + str(destinationbufferinfo2mpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '26Z-'          
                  
        task4copyimagetobuffer2mpu ['BUFFERDESTINATION'] = destinationbufferinfo2mpu    
                                
    if 'QUEUE' in keys2task:
        pass
    else:   
        # ---------------------------------------------------------------------------------------------
        # (27) QUEUE - REQUIRED BY MPUXI                          
        # ---------------------------------------------------------------------------------------------         
        queue2copyimagetobuffer4mpu = None
        
        message2print = "{(Step 27A/70) trying" + "; queue2copyimagetobuffer4mpu-trying-by-copy2imagetobuffer4mpu: " + str(queue2copyimagetobuffer4mpu)  + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
        try:
            queue2copyimagetobuffer4mpu = Queue4Xi()                                                                          # THE SYSTEM CREATES QUEUE    
            
            message2print = "{(Step 27B/70) got" + "; queue2copyimagetobuffer4mpu-got-by-copy2imagetobuffer4mpu: " + str(queue2copyimagetobuffer4mpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)                                  
            # 
            
            code2ack  = code2ack + '27B-'  
        except:
            message2print = "{(Step 27Z/70) failed" + "; queue2copyimagetobuffer4mpu-failed-by-copy2imagetobuffer4mpu: " + str(queue2copyimagetobuffer4mpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '27Z-'
            
        task4copyimagetobuffer2mpu ['QUEUE'] = queue2copyimagetobuffer4mpu
        
    if 'PIPE' in keys2task:
        pass
    else:              
        # ---------------------------------------------------------------------------------------------
        # (28) PIPE - REQUIRED BY MPUXI                              
        # ---------------------------------------------------------------------------------------------         
        pipe2mpu = None
       
        message2print = "{(Step 28A/70) trying" + "; pipe2mpu-trying-by-copy2imagetobuffer4mpu: " + str(pipe2mpu)  + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
        try:
            pipe2mpu = Pipe4Xi()                                                                            # THE SYSTEM CREATES PIPE    
            
            message2print = "{(Step 28B/70) got" + "; pipe2mpu-got-by-copy2imagetobuffer4mpu: " + str(pipe2mpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)                                  
            # 
            
            code2ack  = code2ack + '28B-'  
        except:
            message2print = "{(Step 28Z/70) failed" + "; pipe2mpu-failed-by-copy2imagetobuffer4mpu: " + str(pipe2mpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '28Z-'
            
        task4copyimagetobuffer2mpu ['PIPE'] = pipe2mpu 
        
    if 'COMMANDQUEUE' in keys2task:
        pass
    else:              
        # ---------------------------------------------------------------------------------------------
        # (28) COMMAND QUEUE - REQUIRED                              
        # ---------------------------------------------------------------------------------------------             
        commandqueueinfo2mpu = {}
       
        message2print = "{(Step 29A/70) trying" + "; commandqueueinfo2mpu-trying-by-copy2imagetobuffer4mpu: " + str(commandqueueinfo2mpu)  + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print)  
                    
        try:
            commandqueueinfo2mpu = get2commandqueueinfo4mpu()                                               # SYSTEM CREATES COMMAND QUEUE, IF NECESSARY   
            message2print = "{(Step 29B/70) got" + "; commandqueueinfo2mpu-got-by-copy2imagetobuffer4mpu: " + str(commandqueueinfo2mpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # 
            
            code2ack  = code2ack + '29B-'  
        except:
            message2print = "{(Step 29Z/70) failed" + "; commandqueueinfo2mpu-failed-by-copy2imagetobuffer4mpu: " + str(commandqueueinfo2mpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '29Z-'
    
        task4copyimagetobuffer2mpu ['COMMANDQUEUE'] = commandqueueinfo2mpu
    
         
    if 'EVENT' in keys2task:
        pass
    else:               
        # ---------------------------------------------------------------------------------------------
        # (31) EVENT - REQUIRED                            
        # ---------------------------------------------------------------------------------------------             
        eventinfo2mpu = {}
       
        message2print = "{(Step 31A/70) trying" + "; eventinfo2mpu-trying-by-copy2imagetobuffer4mpu: " + str(eventinfo2mpu)  + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
                    
        try:
            eventinfo2mpu = get2eventinfo4mpu()                                                                    
            message2print = "{(Step 31B/70) got" + "; eventinfo2mpu-got-by-copy2imagetobuffer4mpu: " + str(eventinfo2mpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # 
            
            code2ack  = code2ack + '31B-' 
        except:
            message2print = "{(Step 31Z/70) failed" + "; eventinfo2mpu-failed-by-copy2imagetobuffer4mpu: " + str(eventinfo2mpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '31Z-'
    
        task4copyimagetobuffer2mpu ['EVENT'] = eventinfo2mpu 
    
         
    if 'EVENTWAITLIST' in keys2task:
        pass
    else:               
        # ---------------------------------------------------------------------------------------------
        # (32) EVENTWAITLIST - REQUIRED                              
        # ---------------------------------------------------------------------------------------------             
        eventwaitlist2mpu = {}
       
        message2print = "{(Step 32A/70) trying" + "; eventwaitlist2mpu-trying-by-copy2imagetobuffer4mpu: " + str(eventwaitlist2mpu)  + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
                    
        try:
            eventwaitlist2mpu = {}                                                                   
            message2print = "{(Step 32B/70) set" + "; eventwaitlist2mpu-set-by-copy2imagetobuffer4mpu: " + str(eventwaitlist2mpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # 
            
            code2ack  = code2ack + '32B-' 
        except:
            message2print = "{(Step 32Z/70) failed" + "; eventwaitlist2mpu-failed-by-copy2imagetobuffer4mpu: " + str(eventwaitlist2mpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '32Z-'
    
        task4copyimagetobuffer2mpu ['EVENTWAITLIST'] = eventwaitlist2mpu                     
          
    if 'EVENTWAITLISTQUANTITY' in keys2task:
        pass
    else:               
        # ---------------------------------------------------------------------------------------------
        # (33) EVENTWAITLISTQUANTITY - REQUIRED                              
        # ---------------------------------------------------------------------------------------------             
        eventwaitlistquantity2mpu = 0
       
        message2print = "{(Step 33A/70) trying" + "; eventwaitlistquantity2mpu-trying-by-copy2imagetobuffer4mpu: " + str(eventwaitlistquantity2mpu)  + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
                    
        try:
            keys2event = eventwaitlist2mpu.keys()
            for key2event in keys2event:
                eventwaitlistquantity2mpu = eventwaitlistquantity2mpu + 1                                                                   
            message2print = "{(Step 33B/70) set" + "; eventwaitlistquantity2mpu-set-by-copy2imagetobuffer4mpu: " + str(eventwaitlistquantity2mpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # 0 
            
            code2ack  = code2ack + '33B-' 
        except:
            message2print = "{(Step 33Z/70) failed" + "; eventwaitlistquantity2mpu-failed-by-copy2imagetobuffer4mpu: " + str(eventwaitlistquantity2mpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '33Z-'
    
        task4copyimagetobuffer2mpu ['EVENTWAITLISTQUANTITY'] = eventwaitlistquantity2mpu       
                                         
                
    if 'SOURCEORIGIN' in keys2task:
        pass
    else:                    
        # ---------------------------------------------------------------------------------------------
        # (34) SOURCEORIGIN - REQUIRED                             
        # ---------------------------------------------------------------------------------------------             
        source4origin2mpu = {}
        
        message2print = "{(Step 34A/70) trying" + "; source4origin2mpu-trying-by-copy2imagetobuffer4mpu: " + str(source4origin2mpu)  + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
                    
        try:                                                                                    
            source4origin2mpu['X'] = 0 
            source4origin2mpu['Y'] = 0  
            source4origin2mpu['Z'] = 0                                                                                                                          
            message2print = "{(Step 34B/70) set" + "; source4origin2mpu-set-by-copy2imagetobuffer4mpu to: " + str(source4origin2mpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # 
            code2ack  = code2ack + '34B-' 
        except:
            message2print = "{(Step 34Z/70) failed" + "; source4origin2mpu-failed-by-copy2imagetobuffer4mpu: " + str(source4origin2mpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '34Z-'
    
        task4copyimagetobuffer2mpu ['SOURCEORIGIN'] = source4origin2mpu
             
   
             
    if 'REGION' in keys2task:
        pass
    else:            
        # ---------------------------------------------------------------------------------------------
        # (36) REGION - REQUIRED                         
        # ---------------------------------------------------------------------------------------------             
        region2mpu = {}
        
        message2print = "{(Step 36A/70) trying" + "; region2mpu-trying-by-copy2imagetobuffer4mpu: " + str(region2mpu)  + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
                    
        try:                                                                                    
            region2mpu['X'] = 160                                                                               # WIDTH  IN PIXELS TO WRITE 
            region2mpu['Y'] = 90                                                                                # HEIGHT IN PIXELS TO WRITE
            region2mpu['Z'] = 0                                                                                 # DEPTH  IN PIXELS TO WRITE                                                                                                                          
            message2print = "{(Step 36B/70) set" + "; region2mpu-set-by-copy2imagetobuffer4mpu to: " + str(region2mpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # 
            code2ack  = code2ack + '36B-' 
        except:
            message2print = "{(Step 36Z/70) failed" + "; region2mpu-failed-by-copy2imagetobuffer4mpu: " + str(region2mpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '36Z-'
    
        task4copyimagetobuffer2mpu ['REGION'] = region2mpu
    
         
    if 'OFFSET' in keys2task:
        pass
    else:            
        # ---------------------------------------------------------------------------------------------
        # (37) OFFSET - WHERE TO BEGIN COPYING DATA INTO DESTINATION BUFFER - REQUIRED                         
        # ---------------------------------------------------------------------------------------------             
        offset2mpu = 0
        message2print = "{(Step 37A/70) set" + "; offset2mpu-set-by-copy2imagetobuffer4mpu to: " + str(offset2mpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        # 
        task4copyimagetobuffer2mpu ['OFFSET'] = offset2mpu        
        code2ack  = code2ack + '37A-' 
        
    # ---------------------------------------------------------------------------------------------
    # EOF TASK
    # ---------------------------------------------------------------------------------------------
    keys2task = task4copyimagetobuffer2mpu.keys()
    message2print = "{(Step 39/70) set" + "; keys2task-set-by-copy2imagetobuffer4mpu to: " + str(sorted(keys2task))  + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)            
    # ['BUFFERDESTINATION',
    #  'COMMANDQUEUE',
    #  'CONTEXT',
    #  'DATA2CALLBACK',
    #  'DEVICE',
    #  'EVENT',
    #  'EVENTWAITLIST',
    #  'EVENTWAITLISTQUANTITY',
    #  'FUNCTION2CALLBACK',
    #  'IMAGESOURCE',
    #  'OFFSET',
    #  'PIPE',
    #  'PLATFORM',
    #  'QUEUE',
    #  'REGION',
    #  'SOURCEORIGIN']; time: 01:28:44 10/05/11 Pacific Daylight Time}
               


    # ---------------------------------------------------------------------------------------------
    # CONTROL AND COMMAND
    # ---------------------------------------------------------------------------------------------
    control2copyimagetobuffer4mpu = {}       
                                
    # ---------------------------------------------------------------------------------------------
    # CONSUMER AND PUBLISHER CHANNEL FOR CONTROL AND COMMAND
    # ---------------------------------------------------------------------------------------------
    ack4channels2mpu = False
    channels2mpu = ()
    message2print = "{(Step 40A/70) trying" + "channels2mpu-trying-by-copy2imagetobuffer4mpu: " + str(channels2mpu)  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)     
    try:
        # 
        consumerchannel2copyimagetobuffer4mpu, publisherchannel2copyimagetobuffer4mpu = task4copyimagetobuffer2mpu ['PIPE']

        message2print = "{(Step 40B/70) got" + "; consumerchannel2copyimagetobuffer4mpu-got-by-copy2imagetobuffer4mpu: " + str(consumerchannel2copyimagetobuffer4mpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
        # <read-write PipeConnection, handle 660>

        message2print = "{(Step 40C/70) got" + "; publisherchannel2copyimagetobuffer4mpu-got-by-copy2imagetobuffer4mpu: " + str(publisherchannel2copyimagetobuffer4mpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
        # <read-write PipeConnection, handle 664>                     
        ack4channels2mpu = True
        
        control2copyimagetobuffer4mpu['CONSUMERCHANNEL']  = consumerchannel2copyimagetobuffer4mpu
        
        
        control2copyimagetobuffer4mpu['PUBLISHERCHANNEL'] = publisherchannel2copyimagetobuffer4mpu
        
        code2ack  = code2ack + '40D-' 
        
    except:
        ack4channels2mpu = False
        code2nack  = code2nack + '40Z-'      
                        
    # ---------------------------------------------------------------------------------------------
    # QUEUE FOR CONTROL AND COMMAND
    # ---------------------------------------------------------------------------------------------
    ack4queue2copyimagetobuffer4mpu = False
    queue2copyimagetobuffer4mpu = None
    message2print = "{(Step 41A/70) trying" + "queue2copyimagetobuffer4mpu-trying-by-copy2imagetobuffer4mpu: " + str(queue2copyimagetobuffer4mpu)  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)     
    try:
        queue2copyimagetobuffer4mpu = task4copyimagetobuffer2mpu ['QUEUE']

        message2print = "{(Step 41B/70) got" + "; queue2copyimagetobuffer4mpu-got-by-copy2imagetobuffer4mpu: " + str(queue2copyimagetobuffer4mpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
        # <queue4xi.Queue4Xi object at 0x0000000004465E48>
              
        ack4queue2copyimagetobuffer4mpu = True
        control2copyimagetobuffer4mpu['QUEUE']  = queue2copyimagetobuffer4mpu        
        code2ack  = code2ack + '41C-'
    except:
        ack4queue2copyimagetobuffer4mpu = False  
        code2nack  = code2nack + '41Z-'
    
    
    # -----------------------------------------------------------------------------------------    
    # DUMMY PROCESS OBJECT = THREAD CREATION
    # -----------------------------------------------------------------------------------------           
    # dummy2image4mpu = None
    # message2print = "{(Step 69A/70) trying" + "dummy2image4mpu-trying-by-copy2imagetobuffer4mpu: " + str(dummy2image4mpu)  + "; time: " + time4xi() + "}"   
    # print4xi(class2print, method2print, message2print)     
    # try:                            
        # dummy2image4mpu = DummyProcess4Xi(target=manoeuvre2copyimagetobuffer4mpu,                                               # in this module to handle queue and pipe channels
        #                                   args =(queue2copyimagetobuffer4mpu,                                                         # caller puts and gets - does NOT call any functions
        #                                          consumerchannel2copyimagetobuffer4mpu, publisherchannel2copyimagetobuffer4mpu))                        # caller sends and receives - does NOT call any functions
        # message2print = "{(Step 69B/70) created" + "dummy2image4mpu-created-by-copy2imagetobuffer4mpu: " + str(dummy2image4mpu)  + "; time: " + time4xi() + "}" 
        # print4xi(class2print, method2print, message2print)  
        # code2ack  = code2ack + '69B-'
        
    # except:
        # code2nack  = code2nack + '69Z-'
        
    # ---------------------------------------------------------------------------------------------
    # RETURN CONTROL AND COMMAND DICTIONARY
    # ---------------------------------------------------------------------------------------------          
    message2print = "{(Step 70A/70) set" + "; code2ack-set-by-copy2imagetobuffer4mpu: " + str(code2ack) + "; time: " + time4xi() + "}"  
    print4xi(class2print, method2print, message2print)                                                             
    # 1B1-1B2-1C-2B-3B-4B-5B-6B-7B-8B-9B-11B-12B-13B-14B-16B-17A-40D-41C- 
    message2print = "{(Step 70B/70) set" + "; code2nack-set-by-copy2imagetobuffer4mpu: " + str(code2nack) + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                             
    #  
        
    message2print = "{(Step 70C/70) returning" + "; control2copyimagetobuffer4mpu-returning-by-copy2imagetobuffer4mpu: " + str(control2copyimagetobuffer4mpu) + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                             
    # {'QUEUE': <queue4xi.Queue4Xi object at 0x0000000004465E48>,
    #  'PUBLISHERCHANNEL': <read-write PipeConnection, handle 664>,
    #  'CONSUMERCHANNEL': <read-write PipeConnection, handle 660>}; time: 01:28:44 10/05/11 Pacific Daylight Time}

        
    return control2copyimagetobuffer4mpu
                    
#</function4xi> 

# -------------------------------------------------------------------------------------------------
#<function4xi function = "copy2buffertoimage4mpu" usage = "
#                                                                                                              
#   # ---------------------------------------------------------------------------------------------
#   # (80) ESTABLISH CONTROL & COMMAND WITH COPYBUFFERTOIMAGE4MPU                      
#   # ---------------------------------------------------------------------------------------------    
#   print('(Step 80A/700) OF MPU2IMAGE4XI5TEST - TRYING TO ESTABLISH CONTROL & COMMAND WITH COPYBUFFERTOIMAGE4MPU ', time4xi())    
#   response4copy2buffertoimage = {} 
#   task4copy2buffertoimage = None
#   try:                               
#       response4copy2buffertoimage = copy2buffertoimage4mpu(task4copy2buffertoimage)                                                                                                          
#       print('(Step 80B/700) OF MPU2IMAGE4XI5TEST - ESTABLISHED CONTROL & COMMAND WITH COPYBUFFERTOIMAGE4MPU : ', response4copy2buffertoimage)
#       queue2copybuffertoimage4mpu      = response4copy2buffertoimage['QUEUE']
#       channel2copybuffertoimagepublish4mpu = response4copy2buffertoimage['PUBLISHERCHANNEL']
#       channel2copybuffertoimageconsume4mpu = response4copy2buffertoimage['CONSUMERCHANNEL']
#   except:
#       print('(Step 80Z/700) OF MPU2IMAGE4XI5TEST *** FAILED TO ESTABLISH CONTROL & COMMAND WITH COPYBUFFERTOIMAGE4MPU ', time4xi()) 
#   # --------------------------------------------------------------------------------------------------
#   # (81) SEND CONTROL & COMMAND COPY VIA PIPE FOR MANOEUVRE2COPYIMAGETOBUFFER4MPU TO RECEIVE                 
#   # --------------------------------------------------------------------------------------------------      
#   print('(Step 81A/700) OF MPU2IMAGE4XI5TEST - TRYING TO SEND CONTROL & COMMAND VIA PIPE WITH COPYBUFFERTOIMAGE4MPU ', time4xi())     
#   try:                               
#       message2copybuffertoimagesend4mpu = "<copy4xi>copy2copybuffertoimage4mpu</copy4xi>"
#       channel2copybuffertoimagepublish4mpu = response4copy2buffertoimage['PUBLISHERCHANNEL']
#       channel2copybuffertoimagepublish4mpu.send(message2copybuffertoimagesend4mpu)
#       print("(Step 81B/700) OF MPU2IMAGE4XI5TEST - SENT TO CHANNEL2PUBLISH4MPU: ", channel2copybuffertoimagepublish4mpu, " MESSAGE2SEND4MPU: " , message2copybuffertoimagesend4mpu)                                                                                                
#   except:
#       print('(Step 81Z/700) OF MPU2IMAGE4XI5TEST *** FAILED TO SEND CONTROL & COMMAND VIA PIPE WITH COPYBUFFERTOIMAGE4MPU ', time4xi())                                                
#   # ------------------------------------------------------------------------------------------
#   # (82) LET MANOEUVRE2COPYIMAGETOBUFFER4MPU TO RECEIVE COPY AND AND SEND ROGER VIA PIPE                
#   # ------------------------------------------------------------------------------------------       
#   print("(Step 82C/700) OF MPU2IMAGE4XI5TEST - TRYING TO PERFORM TARGET FUNCTION MANOEUVRE2COPYIMAGETOBUFFER4MPU ", time4xi()) 
#   try:        
#       response2manoeuvreimagecopytobuffer4mpu = manoeuvre2copybuffertoimage4mpu(queue2copybuffertoimage4mpu, channel2copybuffertoimageconsume4mpu, channel2copybuffertoimagepublish4mpu)
#       print("(Step 82D/700) OF MPU2IMAGE4XI5TEST - PERFORMED TARGET FUNCTION MANOEUVRE2COPYIMAGETOBUFFER4MPU ", manoeuvre2copybuffertoimage4mpu)
#   except:
#       print("(Step 82Z/700) OF MPU2IMAGE4XI5TEST *** FAILED TO PERFORM TARGET FUNCTION MANOEUVRE2COPYIMAGETOBUFFER4MPU ", time4xi())     
#   else:
#       pass # trust activated thread    
#   # ---------------------------------------------------------------------------------------------
#   # (83) RECEIVE CONTROL & COMMAND ROGER VIA PIPE                   
#   # ---------------------------------------------------------------------------------------------          
#   print('(Step 83A/700) OF MPU2IMAGE4XI5TEST - TRYING TO RECEIVE CONTROL & COMMAND VIA PIPE WITH COPYBUFFERTOIMAGE4MPU ', time4xi())     
#   try:
#       channel2copybuffertoimageconsume4mpu = response4copy2buffertoimage['CONSUMERCHANNEL']                              
#       message2copyimagereceive4mpu = channel2copybuffertoimageconsume4mpu.recv()
#       print("(Step 83B/700) OF MPU2IMAGE4XI5TEST - RECEIVED FROM CHANNEL2CONSUME4MPU: ", channel2copybuffertoimageconsume4mpu, " CONTROL & COMMAND: ", message2copyimagereceive4mpu)                                                                                                  
#       # 
#   except:
#       print('(Step 83Z/700) OF MPU2IMAGE4XI5TEST *** FAILED TO RECEIVE CONTROL & COMMAND VIA PIPE  WITH COPYBUFFERTOIMAGE4MPU ', time4xi())         
#   # ---------------------------------------------------------------------------------------------------
#   # (84) SEND CONTROL & COMMAND READY VIA PIPE FOR MANOEUVRE2COPYIMAGETOBUFFER4MPU TO RECEIVE                
#   # ---------------------------------------------------------------------------------------------------      
#   print('(Step 84A/700) OF MPU2IMAGE4XI5TEST - TRYING TO SEND CONTROL & COMMAND VIA PIPE WITH COPYBUFFERTOIMAGE4MPU ', time4xi())     
#   try:                               
#       message2copybuffertoimagesend4mpu = "<copy4xi>copy2copybuffertoimage4mpu</copy4xi>"
#       channel2copybuffertoimagepublish4mpu = response4copy2buffertoimage['PUBLISHERCHANNEL']
#       channel2copybuffertoimagepublish4mpu.send(message2copybuffertoimagesend4mpu)
#       print("(Step 84B/700) OF MPU2IMAGE4XI5TEST - SENT TO CHANNEL2PUBLISH4MPU: ", channel2copybuffertoimagepublish4mpu, " MESSAGE2SEND4MPU: " , message2copybuffertoimagesend4mpu)                                                                                                
#       #  
#   except:
#        print('(Step 84Z/700) OF MPU2IMAGE4XI5TEST *** FAILED TO SEND CONTROL & COMMAND VIA PIPE WITH COPYBUFFERTOIMAGE4MPU ', time4xi())                                                
#   # -----------------------------------------------------------------------------------
#   # (85) LET MANOEUVRE2COPYIMAGETOBUFFER4MPU TO RECEIVE READY AND TO SEND READY                
#   # -----------------------------------------------------------------------------------       
#   print("(Step 85A/700) OF MPU2IMAGE4XI5TEST - TRYING TO PERFORM TARGET FUNCTION MANOEUVRE2COPYIMAGETOBUFFER4MPU ", time4xi()) 
#   try:        
#       response2manoeuvreimagecopytobuffer4mpu = manoeuvre2copybuffertoimage4mpu(queue2copybuffertoimage4mpu, channel2copybuffertoimageconsume4mpu, channel2copybuffertoimagepublish4mpu)
#       print("(Step 85B/700) OF MPU2IMAGE4XI5TEST - PERFORMED TARGET FUNCTION MANOEUVRE2COPYIMAGETOBUFFER4MPU ", manoeuvre2copybuffertoimage4mpu)
#   except:
#       print("(Step 85Z/700) OF MPU2IMAGE4XI5TEST *** FAILED TO PERFORM TARGET FUNCTION MANOEUVRE2COPYIMAGETOBUFFER4MPU ", time4xi())     
#   # ---------------------------------------------------------------------------------------
#   # (86) RECEIVE CONTROL & COMMAND READY VIA PIPE                      
#   # ---------------------------------------------------------------------------------------          
#   print('(Step 86A/700) OF MPU2IMAGE4XI5TEST - TRYING TO RECEIVE CONTROL & COMMAND VIA PIPE WITH COPYBUFFERTOIMAGE4MPU ', time4xi())     
#   try:
#       channel2copybuffertoimageconsume4mpu = response4copy2buffertoimage['CONSUMERCHANNEL']                              
#       message2copyimagereceive4mpu = channel2copybuffertoimageconsume4mpu.recv()
#       print("(Step 86B/700) OF MPU2IMAGE4XI5TEST - RECEIVED FROM CHANNEL2CONSUME4MPU: ", channel2copybuffertoimageconsume4mpu, " CONTROL & COMMAND: ", message2copyimagereceive4mpu)                                                                                                  
#       # 
#   except:
#       print('(Step 86Z/700) OF MPU2IMAGE4XI5TEST *** FAILED TO RECEIVE CONTROL & COMMAND VIA PIPE  WITH COPYBUFFERTOIMAGE4MPU ', time4xi())          
#   # -----------------------------------------------------------------------------------------
#   # (87) PUT IMAGE PATH TO QUEUE
#   # -----------------------------------------------------------------------------------------                          
#   print("(Step 87A/700) OF MPU2IMAGE4XI5TEST - TRYING TO PUT IMAGE PATH TO QUEUE ", time4xi())      
#   try:    
#       queue2copybuffertoimage4dragon.put(path2dragon)             
#       print("(Step 87B/700) OF DRAGON2IMAGE4XI5TEST - PUT TO QUEUE IMAGE PATH: ", path2dragon) 
#       # ..//..//..//..//..//web//images//png//dragonxi.png  
#   except:
#       print("(Step 87Z/700) OF MPU2IMAGE4XI5TEST *** FAILED TO PUT IMAGE PATH TO QUEUE ", time4xi()) 
#   # ------------------------------------------------------------------------------------------------------
#   # (88) LET MANOEUVRE2COPYIMAGETOBUFFER4MPU TO GET FROM QUEUE AND TO SEND EITHER ACK OR NACK VIA PIPE               
#   # -------------------------------------------------------------------------------------------------------       
#   print("(Step 88A/700) OF MPU2IMAGE4XI5TEST - TRYING TO PERFORM TARGET FUNCTION MANOEUVRE2COPYIMAGETOBUFFER4MPU ", time4xi()) 
#   try:        
#       response2manoeuvreimagecopytobuffer4mpu = manoeuvre2copybuffertoimage4mpu(queue2copybuffertoimage4mpu, channel2copybuffertoimageconsume4mpu, channel2copybuffertoimagepublish4mpu)
#       print("(Step 88B/700) OF MPU2IMAGE4XI5TEST - PERFORMED TARGET FUNCTION MANOEUVRE2COPYIMAGETOBUFFER4MPU ", manoeuvre2copybuffertoimage4mpu)
#   except:
#       print("(Step 88Z/700) OF MPU2IMAGE4XI5TEST *** FAILED TO PERFORM TARGET FUNCTION MANOEUVRE2COPYIMAGETOBUFFER4MPU ", time4xi())     
#   # -----------------------------------------------------------------------------------------------
#   # (89) RECEIVE ACK OR NACK ABOUT IMAGE OPERATION VIA PIPE                
#   # -----------------------------------------------------------------------------------------------          
#   print('(Step 89A/700) OF MPU2IMAGE4XI5TEST - TRYING TO RECEIVE CONTROL & COMMAND ABOUT IMAGE OPERTION VIA PIPE ', time4xi())      
#   try: 
#       message2copyimagereceive4mpu = channel2copybuffertoimageconsume4mpu.recv()
#       print("(Step 89D/700) OF MPU2IMAGE4XI5TEST - RECEIVED FROM CHANNEL2CONSUME4MPU: ", channel2copybuffertoimageconsume4mpu, " CONTROL & COMMAND: ", message2copyimagereceive4mpu)                                                                                                   
#   except:
#       print('(Step 89Z/700) OF MPU2IMAGE4XI5TEST *** FAILED TO RECEIVE CONTROL & COMMAND ABOUT IMAGE OPERTION VIA PIPE ', time4xi())               
#   # -------------------------------------------------------------------------------------------------------
#   # (90) LET MANOEUVRE2COPYIMAGETOBUFFER4MPU GO TO STATE 0               
#   # -------------------------------------------------------------------------------------------------------       
#   print("(Step 90A/700) OF MPU2IMAGE4XI5TEST - TRYING TO PERFORM TARGET FUNCTION MANOEUVRE2COPYIMAGETOBUFFER4MPU ", time4xi()) 
#   try:        
#       response2manoeuvreimagecopytobuffer4mpu = manoeuvre2copybuffertoimage4mpu(queue2copybuffertoimage4mpu, channel2copybuffertoimageconsume4mpu, channel2copybuffertoimagepublish4mpu)
#       print("(Step 90B/700) OF MPU2IMAGE4XI5TEST - GOT RESPONSE2MANOEUVRE4MPU ", response2manoeuvreimagecopytobuffer4mpu)
#   except:
#       print("(Step 90Z/700) OF MPU2IMAGE4XI5TEST *** FAILED TO PERFORM TARGET FUNCTION MANOEUVRE2COPYIMAGETOBUFFER4MPU ", time4xi())   

#">

# -------------------------------------------------------------------------------------------------   
def copy2buffertoimage4mpu(task4copybuffertoimage2mpu=None):

    '''
    copies buffer to image for MPU
    '''
    global mpu2image4xi                                                                                             # 2D-IMAGES FOR MPU  
    global mpu2buffer4xi                                                                                            # BUFFER FOR MPU
    
    global control2copybuffertoimage4mpu                                                                            # COMMAND AND CONTROL FOR MPU IMAGES 
    
    # -----------------------------------------------------------------------------------------
    # (Step 1/33) PARAMETERS           
    # -----------------------------------------------------------------------------------------          
    class2print = "copy2buffertoimage4mpu"
    method2print = "copy2buffertoimage4mpu"

    message2print = "{(Step 1A/33) started" + "; copy2buffertoimage4mpu-started: " + str(copy2buffertoimage4mpu)  + "; time: " + time4xi() + "}"  
    print4xi(class2print, method2print, message2print)                                                                
    # <function copy2buffertoimage4mpu at 0x0000000003869548>
    
    message2print = "{(Step 1B/33) passed" + "; task4copybuffertoimage2mpu-to-copy2buffertoimage4mpu: " + str(task4copybuffertoimage2mpu) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                                                              
    # None
            
    code2ack  = ' '
    code2nack = ' '
    
    # -----------------------------------------------------------------------------------------
    # CREATE MPU2IMAGE4XI IF NECESSARY       
    # ----------------------------------------------------------------------------------------- 
    try:
        if mpu2image4xi is None:
            mpu2image4xi = create2image4mpu()                                                                              # INITIALIZE IMAGE FOR MPU 
            message2print = "{(Step 1C/70) created" + "; mpu2image4xi-created-by-copy2buffertoimage4mpu to: " + str(mpu2image4xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #             
        else:
            message2print = "{(Step 1D/70) skipped" + "; create2image4mpu-skipped-by-copy2buffertoimage4mpu: " + str(copy2buffertoimage4mpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            # skipped                                  
    except:
        try:
            mpu2image4xi = create2image4mpu()
            message2print = "{(Step 1E/70) set" + "; mpu2image4xi-set-by-copy2buffertoimage4mpu to: " + str(mpu2image4xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #             
        except:
            mpu2image4xi = {}
            message2print = "{(Step 1Z1/70) yielded" + "; mpu2image4xi-yielded-by-copy2buffertoimage4mpu: " + str(mpu2image4xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            code2nack  = code2nack + '1Z1-' 
    
    # -----------------------------------------------------------------------------------------
    # CREATE MPU2BUFFER4XI IF NECESSARY       
    # ----------------------------------------------------------------------------------------- 
    try:
        if mpu2buffer4xi is None:
            mpu2buffer4xi = create2buffer4mpu()                                                                     # INITIALIZE BUFFER FOR MPU 
            message2print = "{(Step 1F/70) created" + "; mpu2buffer4xi-created-by-copy2buffertoimage4mpu to: " + str(mpu2buffer4xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #             
        else:
            message2print = "{(Step 1G/70) skipped" + "; create2buffer4mpu-skipped-by-copy2buffertoimage4mpu: " + str(copy2buffertoimage4mpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            # skipped                                  
    except:
        try:
            mpu2buffer4xi = create2buffer4mpu()
            message2print = "{(Step 1H/70) set" + "; mpu2buffer4xi-set-by-copy2buffertoimage4mpu to: " + str(mpu2buffer4xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #             
        except:
            mpu2buffer4xi = {}
            message2print = "{(Step 1Z2/70) yielded" + "; mpu2buffer4xi-yielded-by-copy2buffertoimage4mpu: " + str(mpu2buffer4xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            code2nack  = code2nack + '1Z2-' 
                            
    # -----------------------------------------------------------------------------------------
    # (Step 1/70) - ANALYZE TASKS           
    # -----------------------------------------------------------------------------------------          
    if task4copybuffertoimage2mpu is None:
        # -----------------------------------------------------------------------------------------
        # NO PARAMETERS GIVEN BY CALLER FOR COPY - BUILD TASK! 
        # -----------------------------------------------------------------------------------------    
        
        task4copybuffertoimage2mpu = {}                                                                                                                                                                                       
        
        message2print = "{(Step 1A/70) initialized" + "; task4copybuffertoimage2mpu-created-by-copy2buffertoimage4mpu to: " + str(task4copybuffertoimage2mpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)               
        # 
        
        # ---------------------------------------------------------------------------------------------
        # (1B) TRACE - AUTOMATIC                      
        # ---------------------------------------------------------------------------------------------              
        function2callback4mpu = trace2image4mpu                                                                 # CALLBACK FUNCTION FOR MPU
        
        message2print = "{(Step 1B1/70) initialized" + "; function2callback4mpu-created-by-copy2buffertoimage4mpu to: " + str(function2callback4mpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
        # <function trace2image4mpu at 0x0000000004645548>
        
        code2ack  = code2ack + '1B1-'
       
        args2callback = {}                                                                                      # CREATE ARGUMENT DICTIONARY FOR CALLBACK FUNCTION             
        args2callback['CLASS']   = "copy2buffertoimage4mpu"                                                     # MODULE OF MPUIZED OPENCL MODULE
        args2callback['METHOD']  = "copy2buffertoimage4mpu"                                                     # FUNCTION OF MPUIZED OPENCL MODULE
        args2callback['MESSAGE'] = "{(Step 1Z1/33) failed" + "; copy2buffertoimage4mpu-failed " + "; time: " + time4xi() + "}" # ERROR MESSAGE FOR MPUIZED OPENCL TO RETURN    

        message2print = "{(Step 1B2/70) initialized" + "; args2callback-created-by-copy2buffertoimage4mpu to: " + str(args2callback)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)               
        # {'MESSAGE': '{(Step 1Z1/33) failed; copy2buffertoimage4mpu-failed ; time: 04:41:15 10/04/11 Pacific Daylight Time}',
        #  'CLASS': 'copy2buffertoimage4mpu', 
        #  'METHOD': 'copy2buffertoimage4mpu'}; time: 04:41:15 10/04/11 Pacific Daylight Time}

        code2ack  = code2ack + '1B2-'
            
        # ---------------------------------------------------------------------------------------------
        # (1C) CALLBACK - AUTOMATIC                      
        # ---------------------------------------------------------------------------------------------              
        task4copybuffertoimage2mpu['DATA2CALLBACK'] = args2callback                                             # USER DATA PASSED TO MPUIZED OPENCL - SEVERAL ARGS PASSED IN ONE DICTIONARY
        task4copybuffertoimage2mpu['FUNCTION2CALLBACK'] = function2callback4mpu                                 # CALLBACK FUNCTION TO CALLED BY OPENCL IN EXCEPTIONS
            
        code2ack  = code2ack + '1C-'
        message2print = "{(Step 1C/70) initialized" + "; task4copybuffertoimage2mpu-created-by-copy2buffertoimage4mpu to: " + str(task4copybuffertoimage2mpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # {'FUNCTION2CALLBACK': <function trace2image4mpu at 0x0000000004645548>, 
        #  'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; copy2buffertoimage4mpu-failed ; time: 04:41:15 10/04/11 Pacific Daylight Time}', 
        #  'CLASS': 'copy2buffertoimage4mpu', 
        #  'METHOD': 'copy2buffertoimage4mpu'}}; time: 04:41:15 10/04/11 Pacific Daylight Time}

            
        # ---------------------------------------------------------------------------------------------
        # (2) PLATFORM - AUTOMATIC                   
        # ---------------------------------------------------------------------------------------------             
        platform2xi = {}
        message2print = "{(Step 2A/70) initialized" + "; platform2xi-created-by-copy2buffertoimage4mpu to: " + str(platform2xi)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
        # {}
            
        try:
            platform2xi = get2platformids4xi()                                                                  # PLATFORM IS COMMON TO ALL DEVICES I.E TO DEVICE XI
            message2print = "{(Step 2B/70) got" + "; platform2xi-got-by-copy2buffertoimage4mpu: " + str(platform2xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            # {'list2platforms': ['DragonXi'], 
            #  'ack2return': ' 2C-2E-2F-2G-', 
            #  'nack2return': ' ', 
            #  'available2platform': 1, 
            #  'max2entries': 1}; time: 04:41:15 10/04/11 Pacific Daylight Time}
                 
            code2ack  = code2ack + '2B-'
        except:
            message2print = "{(Step 2Z/70) failed" + "; platform2xi-failed-by-copy2buffertoimage4mpu: " + str(failed)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)                                                
            code2nack  = code2nack + '2Z-'
                 
        task4copybuffertoimage2mpu['PLATFORM'] = platform2xi                                                    # PLATFORM OF SYSTEM 
              
            
        # ---------------------------------------------------------------------------------------------
        # (3) CONTEXT - AUTOMATIC                 
        # ---------------------------------------------------------------------------------------------    
        context2mpu = {}
        message2print = "{(Step 3A/70) trying" + "; context2mpu-trying-by-copy2buffertoimage4mpu: " + str(context2mpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        try:
            context2mpu = get2context4mpu()                                                                     # NO PARAMETERS - THE SYSTEM CREATES CONTEXT IF REQUIRED  
                
            message2print = "{(Step 3B/70) got" + "; context2mpu-got-by-copy2buffertoimage4mpu: " + str(context2mpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)                                  
            # {'COUNT2REFERENCE': 0, 'DATA2CALLBACK': {'MESSAGE': ...
                
            code2ack  = code2ack + '3B-'         
        except:
            message2print = "{(Step 3Z/70) failed" + "; context2mpu-failed-by-copy2buffertoimage4mpu: " + str(context2mpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '3Z-'
                
        task4copybuffertoimage2mpu ['CONTEXT'] = context2mpu           
            
        # ---------------------------------------------------------------------------------------------
        # (4) DEVICE - AUTOMATIC                       
        # ---------------------------------------------------------------------------------------------    
        device2mpu = {}
            
        message2print = "{(Step 4A/70) trying" + "; device2mpu-trying-by-copy2buffertoimage4mpu: " + str(device2mpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        try:
            device2mpu = get2mpu4xi()                                                                           # THE SYSTEM CREATES DEVICE INFO, IF NECESSARY    
            
            message2print = "{(Step 4B/70) got" + "; device2mpu-got-by-copy2buffertoimage4mpu: " + str(device2mpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)                                  
            # {'RESOLUTION2TIMER4PROFILING': 318, 
            #  'ID2PLATFORM': '0000000001509568', 
            #  'PROFILING2QUEUE': 'ACK', 
            #  'EXECUTE2KERNELS4OPENCL': 'ACK', 
            #  'NATIVE2WIDTHLONG4VECTOR': 2, 
            #  'MAX2UNITS4COMPUTE': 4, 
            #  'SIZE2MEMORY4GLOBAL': 3221225472, ...
               
            code2ack  = code2ack + '4B-'  
        except:
            message2print = "{(Step 4Z/70) failed" + "; device2mpu-failed-by-copy2buffertoimage4mpu: " + str(device2mpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '4Z-'
            
        task4copybuffertoimage2mpu ['DEVICE'] = device2mpu
            
                   
        # ---------------------------------------------------------------------------------------------
        # (5) IMAGE DESTINATION OBJECT - REQUIRED                         
        # ---------------------------------------------------------------------------------------------         
        imagedestinationinfo2mpu = {}
             
        message2print = "{(Step 5A/70) trying" + "; imagedestinationinfo2mpu-trying-by-copy2buffertoimage4mpu: " + str(imagedestinationinfo2mpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        try:
            imagedestinationinfo2mpu = get2imageinfo4mpu()                                                                         
            
            message2print = "{(Step 5B/70) got" + "; imagedestinationinfo2mpu-got-by-copy2buffertoimage4mpu: " + str(imagedestinationinfo2mpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)                                  
            # {'SIZE2QUEUE': 2147483646, 
            #  'FORMAT': {'TYPE': 'CL_UNORM_INT8', 
            #  'ORDER': 'CL_RGBA'}, 
            #  'QUEUE': <queue4xi.Queue4Xi object at 0x0000000004105358>, 
            #  'WIDTH': 800, 
            #  'CONTEXT': {'COUNT2REFERENCE': 0, 
            #  'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2context4mpu-failed ; time: 04:41:15 10/04/11 Pacific Daylight Time}', 
            #  'CLASS': 'create2context4mpu', 
            #  'METHOD': 'create2context4mpu'}, 
            #  'FUNCTION2CALLBACK': <function trace2context4mpu at 0x00000000037FF348>, 
            #  'ACK2RETURN': ' 2A-2C-2D-2E-2B-3B-4B-5B-6B-7B-7B-', 
            #  'INFO2PRIVATE': None, 
            #  'PLATFORM': {'list2platforms': ['DragonXi'],'ack2return': ' 2C-2E-2F-2G-', 'nack2return': ' ', 'available2platform': 1, 'max2entries': 1}, 
            #  'NACK2RETURN': ' ', 
            #  'SIZE2PRIVATE': 0, 
            #  'PROPERTIES': None}, 
            #  'PIPE': (<read-write PipeConnection, handle 524>, <read-write PipeConnection, handle 528>), 
            #  'COUNT2REFERENCE': 1, 
            #  'FUNCTION2CALLBACK': <function trace2image4mpu at 0x0000000004645548>, 
            #  'COMMANDQUEUE': ...
            
            code2ack  = code2ack + '5B-'  
        except:
            message2print = "{(Step 5Z/70) failed" + "; imagedestinationinfo2mpu-failed-by-copy2buffertoimage4mpu: " + str(imagedestinationinfo2mpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '5Z-'          
                  
        task4copybuffertoimage2mpu ['IMAGEDESTINATION'] = imagedestinationinfo2mpu         
               
        # ---------------------------------------------------------------------------------------------
        # (6) BUFFER SOURCE OBJECT - REQUIRED                        
        # ---------------------------------------------------------------------------------------------                         
        buffersourceinfo2mpu = {}
         
        message2print = "{(Step 6A/70) trying" + "; buffersourceinfo2mpu-trying-by-copy2buffertoimage4mpu: " + str(buffersourceinfo2mpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        try:
            buffersourceinfo2mpu = get2bufferinfo4mpu()                                                                         
            
            message2print = "{(Step 6B/70) got" + "; buffersourceinfo2mpu-got-by-copy2buffertoimage4mpu: " + str(buffersourceinfo2mpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)                                  
            # {'CODE2ERROR': 0, 
            #  'COUNT2REFERENCE': 0, 
            #  'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2buffer4mpu-failed ; time: 04:41:15 10/04/11 Pacific Daylight Time}', 
            #  'CLASS': 'create2buffer4mpu', 
            #  'METHOD': 'create2buffer4mpu'}, 
            #  'FUNCTION2CALLBACK': <function trace2buffer4mpu at 0x0000000004631B48>, 
            #  'POINTER2BUFFER': <mmap.mmap object at 0x00000000040EBD98>, 
            #  'FLAGS': 0, 
            #  'CONTEXT': ...
            
            code2ack  = code2ack + '6B-'  
        except:
            message2print = "{(Step 6Z/70) failed" + "; buffersourceinfo2mpu-failed-by-copy2buffertoimage4mpu: " + str(buffersourceinfo2mpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '6Z-'          
                  
        task4copybuffertoimage2mpu ['BUFFERSOURCE'] = buffersourceinfo2mpu     
        
        # ---------------------------------------------------------------------------------------------
        # (7) QUEUE - REQUIRED BY MPUXI                          
        # ---------------------------------------------------------------------------------------------         
        queue2copybuffertoimage4mpu = None
        
        message2print = "{(Step 7A/70) trying" + "; queue2copybuffertoimage4mpu-trying-by-copy2buffertoimage4mpu: " + str(queue2copybuffertoimage4mpu)  + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
        try:
            queue2copybuffertoimage4mpu = Queue4Xi()                                                                              # THE SYSTEM CREATES QUEUE    
            
            message2print = "{(Step 7B/70) got" + "; queue2copybuffertoimage4mpu-got-by-copy2buffertoimage4mpu: " + str(queue2copybuffertoimage4mpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)                                  
            # <queue4xi.Queue4Xi object at 0x000000000492B208> 
            
            code2ack  = code2ack + '7B-'  
        except:
            message2print = "{(Step 7Z/70) failed" + "; queue2copybuffertoimage4mpu-failed-by-copy2buffertoimage4mpu: " + str(queue2copybuffertoimage4mpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '7Z-'
            
        task4copybuffertoimage2mpu ['QUEUE'] = queue2copybuffertoimage4mpu
         
        # ---------------------------------------------------------------------------------------------
        # (8) PIPE - REQUIRED BY MPUXI                              
        # ---------------------------------------------------------------------------------------------         
        pipe2mpu = None
       
        message2print = "{(Step 8A/70) trying" + "; pipe2mpu-trying-by-copy2buffertoimage4mpu: " + str(pipe2mpu)  + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
        try:
            pipe2mpu = Pipe4Xi()                                                                                # THE SYSTEM CREATES PIPE    
            
            message2print = "{(Step 8B/70) got" + "; pipe2mpu-got-by-copy2buffertoimage4mpu: " + str(pipe2mpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)                                  
            # (<read-write PipeConnection, handle 708>, <read-write PipeConnection, handle 712>) 
            
            code2ack  = code2ack + '8B-'  
        except:
            message2print = "{(Step 8Z/70) failed" + "; pipe2mpu-failed-by-copy2buffertoimage4mpu: " + str(pipe2mpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '8Z-'
            
        task4copybuffertoimage2mpu ['PIPE'] = pipe2mpu 
        
        # ---------------------------------------------------------------------------------------------
        # (9) COMMAND QUEUE - REQUIRED                              
        # ---------------------------------------------------------------------------------------------             
        commandqueueinfo2mpu = {}
       
        message2print = "{(Step 9A/70) trying" + "; commandqueueinfo2mpu-trying-by-copy2buffertoimage4mpu: " + str(commandqueueinfo2mpu)  + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print)  
                    
        try:
            commandqueueinfo2mpu = get2commandqueueinfo4mpu()                                                   # SYSTEM CREATES COMMAND QUEUE, IF NECESSARY   
            message2print = "{(Step 9B/70) got" + "; commandqueueinfo2mpu-got-by-copy2buffertoimage4mpu: " + str(commandqueueinfo2mpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # {'CODE2ERROR': 0, 'COUNT2REFERENCE': 0, 'DATA2CALLBACK': ...
            
            code2ack  = code2ack + '9B-'  
        except:
            message2print = "{(Step 9Z/70) failed" + "; commandqueueinfo2mpu-failed-by-copy2buffertoimage4mpu: " + str(commandqueueinfo2mpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '9Z-'
    
        task4copybuffertoimage2mpu ['COMMANDQUEUE'] = commandqueueinfo2mpu
        
        
        
        # ---------------------------------------------------------------------------------------------
        # (11) EVENT - REQUIRED                            
        # ---------------------------------------------------------------------------------------------             
        eventinfo2mpu = {}
       
        message2print = "{(Step 11A/70) trying" + "; eventinfo2mpu-trying-by-copy2buffertoimage4mpu: " + str(eventinfo2mpu)  + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
                    
        try:
            eventinfo2mpu = get2eventinfo4mpu()                                                                    
            message2print = "{(Step 11B/70) got" + "; eventinfo2mpu-got-by-copy2buffertoimage4mpu: " + str(eventinfo2mpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # {'CODE2ERROR': 0, 'DATA2CALLBACK': ...
            
            code2ack  = code2ack + '11B-' 
        except:
            message2print = "{(Step 11Z/70) failed" + "; eventinfo2mpu-failed-by-copy2buffertoimage4mpu: " + str(eventinfo2mpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '11Z-'
    
        task4copybuffertoimage2mpu ['EVENT'] = eventinfo2mpu 
        
        
        # ---------------------------------------------------------------------------------------------
        # (12) EVENTWAITLIST - REQUIRED                              
        # ---------------------------------------------------------------------------------------------             
        eventwaitlist2mpu = {}
       
        message2print = "{(Step 12A/70) trying" + "; eventwaitlist2mpu-trying-by-copy2buffertoimage4mpu: " + str(eventwaitlist2mpu)  + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
                    
        try:
            eventwaitlist2mpu = {}                                                                   
            message2print = "{(Step 12B/70) set" + "; eventwaitlist2mpu-set-by-copy2buffertoimage4mpu: " + str(eventwaitlist2mpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # {}
            
            code2ack  = code2ack + '12B-' 
        except:
            message2print = "{(Step 12Z/70) failed" + "; eventwaitlist2mpu-failed-by-copy2buffertoimage4mpu: " + str(eventwaitlist2mpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '12Z-'
    
        task4copybuffertoimage2mpu ['EVENTWAITLIST'] = eventwaitlist2mpu                     
         
        # ---------------------------------------------------------------------------------------------
        # (13) EVENTWAITLISTQUANTITY - REQUIRED                              
        # ---------------------------------------------------------------------------------------------             
        eventwaitlistquantity2mpu = 0
       
        message2print = "{(Step 13A/70) trying" + "; eventwaitlistquantity2mpu-trying-by-copy2buffertoimage4mpu: " + str(eventwaitlistquantity2mpu)  + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
                    
        try:
            keys2event = eventwaitlist2mpu.keys()
            for key2event in keys2event:
                eventwaitlistquantity2mpu = eventwaitlistquantity2mpu + 1                                                                     
            message2print = "{(Step 13B/70) set" + "; eventwaitlistquantity2mpu-set-by-copy2buffertoimage4mpu: " + str(eventwaitlistquantity2mpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # 0 
            
            code2ack  = code2ack + '13B-' 
        except:
            message2print = "{(Step 13Z/70) failed" + "; eventwaitlistquantity2mpu-failed-by-copy2buffertoimage4mpu: " + str(eventwaitlistquantity2mpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '13Z-'
    
        task4copybuffertoimage2mpu ['EVENTWAITLISTQUANTITY'] = eventwaitlistquantity2mpu       
                                             
        
        # ---------------------------------------------------------------------------------------------
        # (15) ORIGIN FOR DESTINATION - REQUIRED                             
        # ---------------------------------------------------------------------------------------------              
        origin4destination2mpu = {}
        
        message2print = "{(Step 15A/70) trying" + "; origin4destination2mpu-trying-by-copy2buffertoimage4mpu: " + str(origin4destination2mpu)  + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
                    
        try:                                                                                    
            origin4destination2mpu['X'] = 100 
            origin4destination2mpu['Y'] = 100  
            origin4destination2mpu['Z'] = 100                                                                                                                          
            message2print = "{(Step 15B/70) set" + "; origin4destination2mpu-set-by-copy2buffertoimage4mpu to: " + str(origin4destination2mpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # {'Y': 100, 'X': 100, 'Z': 100}
            code2ack  = code2ack + '15B-' 
        except:
            message2print = "{(Step 15Z/70) failed" + "; origin4destination2mpu-failed-by-copy2buffertoimage4mpu: " + str(origin4destination2mpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '15Z-'
    
        task4copybuffertoimage2mpu ['DESTINATIONORIGIN'] = origin4destination2mpu
                    
        
        
        # --------------------------------------------------------------------------------------------------------
        # (16) REGION - REGION OF A IMAGE TO BE COPIED                         
        # --------------------------------------------------------------------------------------------------------             
        region4copy2mpu = {}                                                                                                         
       
        message2print = "{(Step 16A/70) trying" + "; region4copy2mpu-trying-by-copy2buffertoimage4mpu: " + str(region4copy2mpu)  + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
                    
        try:
            region4copy2mpu['X'] = 10 
            region4copy2mpu['Y'] = 10  
            region4copy2mpu['Z'] = 10                                                                           
            message2print = "{(Step 16B/70) set" + "; region4copy2mpu-set-by-copy2buffertoimage4mpu to: " + str(region4copy2mpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # {'Y': 10, 'X': 10, 'Z': 10}
            code2ack  = code2ack + '16B-'                
             
        except:
            message2print = "{(Step 16Z/70) failed" + "; region4copy2mpu-failed-by-copy2buffertoimage4mpu: " + str(region4copy2mpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '16Z-'
        
        task4copybuffertoimage2mpu ['REGION'] = region4copy2mpu              
        
        # --------------------------------------------------------------------------------------------------------
        # (17) OFFSET - WHERE TO BEGIN COPYING DATA FROM SOURCE BUFFER                   
        # --------------------------------------------------------------------------------------------------------             
        offset4copy2mpu = 0                                                                                                         
        message2print = "{(Step 17A/70) set" + "; offset4copy2mpu-set-by-copy2buffertoimage4mpu to: " + str(offset4copy2mpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        # 0
        
        task4copybuffertoimage2mpu ['OFFSET'] = offset4copy2mpu            
                                
        # ---------------------------------------------------------------------------------------------
        # EOF PARAMETER PROVIDING                
        # ---------------------------------------------------------------------------------------------                                                                                                                               
        
        # ---------------------------------------------------------------------------------------------
        # ANALYZE TASKS - WHETHER AUTOMATICALLY GENERATED OR USER GIVEN 
        # ---------------------------------------------------------------------------------------------
        keys2task = task4copybuffertoimage2mpu.keys()
        message2print = "{(Step 20/70) got" + "; keys2task-got-by-copy2buffertoimage4mpu: " + str(sorted(keys2task))  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
        # ['BUFFERSOURCE',
        #  'COMMANDQUEUE',
        #  'CONTEXT',
        #  'DATA2CALLBACK',
        #  'DESTINATIONORIGIN',
        #  'DEVICE',
        #  'EVENT',
        #  'EVENTWAITLIST',
        #  'EVENTWAITLISTQUANTITY',
        #  'FUNCTION2CALLBACK',
        #  'IMAGEDESTINATION',
        #  'OFFSET',
        #  'PIPE',
        #  'PLATFORM',
        #  'QUEUE',
        #  'REGION']; time: 04:41:15 10/04/11 Pacific Daylight Time}
 
        if (('DATA2CALLBACK' in keys2task) and ('FUNCTION2CALLBACK' in keys2task)):
            pass
        else:                
            # ---------------------------------------------------------------------------------------------
            # (21B) TRACE - AUTOMATIC                      
            # ---------------------------------------------------------------------------------------------              
            function2callback4mpu = trace2image4mpu                                                             # CALLBACK FUNCTION FOR MPU
            
            message2print = "{(Step 21B1/70) initialized" + "; function2callback4mpu-created-by-copy2buffertoimage4mpu to: " + str(function2callback4mpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)            
            # 
            
            code2ack  = code2ack + '21B1-'
           
            args2callback = {}                                                                                  # CREATE ARGUMENT DICTIONARY FOR CALLBACK FUNCTION             
            args2callback['CLASS']   = "copy2buffertoimage4mpu"                                                        # MODULE OF MPUIZED OPENCL MODULE
            args2callback['METHOD']  = "copy2buffertoimage4mpu"                                                        # FUNCTION OF MPUIZED OPENCL MODULE
            args2callback['MESSAGE'] = "{(Step 1Z1/33) failed" + "; copy2buffertoimage4mpu-failed " + "; time: " + time4xi() + "}" # ERROR MESSAGE FOR MPUIZED OPENCL TO RETURN    

            message2print = "{(Step 21B2/70) initialized" + "; args2callback-created-by-copy2buffertoimage4mpu to: " + str(args2callback)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)               
            # 

            code2ack  = code2ack + '21B2-'
        
            # ---------------------------------------------------------------------------------------------
            # (21C) CALLBACK - AUTOMATIC                      
            # ---------------------------------------------------------------------------------------------              
            task4copybuffertoimage2mpu['DATA2CALLBACK'] = args2callback                                                # USER DATA PASSED TO MPUIZED OPENCL - SEVERAL ARGS PASSED IN ONE DICTIONARY
            task4copybuffertoimage2mpu['FUNCTION2CALLBACK'] = function2callback4mpu                                    # CALLBACK FUNCTION TO CALLED BY OPENCL IN EXCEPTIONS
            
            code2ack  = code2ack + '21C-'
            message2print = "{(Step 21C/70) initialized" + "; task4copybuffertoimage2mpu-created-by-copy2buffertoimage4mpu to: " + str(task4copybuffertoimage2mpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            #
             
        if 'PLATFORM' in keys2task:
            pass
        else:  
            # ---------------------------------------------------------------------------------------------
            # (22) PLATFORM - AUTOMATIC                   
            # ---------------------------------------------------------------------------------------------             
            platform2xi = {}
            message2print = "{(Step 22A/70) initialized" + "; platform2xi-created-by-copy2buffertoimage4mpu to: " + str(platform2xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)   
            # 
            
            try:
                platform2xi = get2platformids4xi()                                                              # PLATFORM IS COMMON TO ALL DEVICES I.E TO DEVICE XI
                message2print = "{(Step 22B/70) got" + "; platform2xi-got-by-copy2buffertoimage4mpu: " + str(platform2xi)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                # 
                 
                code2ack  = code2ack + '22B-'
            except:
                message2print = "{(Step 22Z/70) failed" + "; platform2xi-failed-by-copy2buffertoimage4mpu: " + str(failed)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                                
                code2nack  = code2nack + '22Z-'
                 
            task4copybuffertoimage2mpu['PLATFORM'] = platform2xi                                                       # PLATFORM OF SYSTEM 
          
        if 'CONTEXT' in keys2task:
            pass
        else:              
            # ---------------------------------------------------------------------------------------------
            # (23) CONTEXT - AUTOMATIC                 
            # ---------------------------------------------------------------------------------------------    
            context2mpu = {}
            message2print = "{(Step 23A/70) trying" + "; context2mpu-trying-by-copy2buffertoimage4mpu: " + str(context2mpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            try:
                context2mpu = get2context4mpu()                                                                 # NO PARAMETERS - THE SYSTEM CREATES CONTEXT IF REQUIRED  
                
                message2print = "{(Step 23B/70) got" + "; context2mpu-got-by-copy2buffertoimage4mpu: " + str(context2mpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # 
                
                code2ack  = code2ack + '23B-'         
            except:
                message2print = "{(Step 23Z/70) failed" + "; context2mpu-failed-by-copy2buffertoimage4mpu: " + str(context2mpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '23Z-'
                
            task4copybuffertoimage2mpu ['CONTEXT'] = context2mpu           
        
        if 'DEVICE' in keys2task:
            pass
        else:                   
            # ---------------------------------------------------------------------------------------------
            # (24) DEVICE - AUTOMATIC                       
            # ---------------------------------------------------------------------------------------------    
            device2mpu = {}
            
            message2print = "{(Step 24A/70) trying" + "; device2mpu-trying-by-copy2buffertoimage4mpu: " + str(device2mpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            try:
                device2mpu = get2mpu4xi()                                                                       # THE SYSTEM CREATES DEVICE INFO, IF NECESSARY    
                
                message2print = "{(Step 24B/70) got" + "; device2mpu-got-by-copy2buffertoimage4mpu: " + str(device2mpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # 
                
                code2ack  = code2ack + '24B-'  
            except:
                message2print = "{(Step 24Z/70) failed" + "; device2mpu-failed-by-copy2buffertoimage4mpu: " + str(device2mpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '24Z-'
            
            task4copybuffertoimage2mpu ['DEVICE'] = device2mpu
        
        if 'IMAGEDESTINATION' in keys2task:
            pass
        else:                      
            # ---------------------------------------------------------------------------------------------
            # (25) DESTINATION IMAGE - REQUIRED                            
            # ---------------------------------------------------------------------------------------------         
            destinationimageinfo2mpu = {}
             
            message2print = "{(Step 25A/70) trying" + "; destinationimageinfo2mpu-trying-by-copy2buffertoimage4mpu: " + str(destinationimageinfo2mpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            try:
                destinationimageinfo2mpu = get2imageinfo4mpu()                                                                         
                
                message2print = "{(Step 25B/70) got" + "; destinationimageinfo2mpu-got-by-copy2buffertoimage4mpu: " + str(destinationimageinfo2mpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                #                 
                code2ack  = code2ack + '25B-'  
            except:
                message2print = "{(Step 25Z/70) failed" + "; destinationimageinfo2mpu-failed-by-copy2buffertoimage4mpu: " + str(destinationimageinfo2mpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '25Z-'          
                      
            task4copybuffertoimage2mpu ['IMAGEDESTINATION'] = destinationimageinfo2mpu         
        
        if 'BUFFERSOURCE' in keys2task:
            pass
        else:                      
            # ---------------------------------------------------------------------------------------------
            # (26) SOURCE BUFFER - REQUIRED                            
            # ---------------------------------------------------------------------------------------------         
            sourcebufferinfo2mpu = {}
             
            message2print = "{(Step 26A/70) trying" + "; sourcebufferinfo2mpu-trying-by-copy2buffertoimage4mpu: " + str(sourcebufferinfo2mpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            try:
                sourcebufferinfo2mpu = get2bufferinfo4mpu()                                                                         
                
                message2print = "{(Step 26B/70) got" + "; sourcebufferinfo2mpu-got-by-copy2buffertoimage4mpu: " + str(sourcebufferinfo2mpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                #                 
                code2ack  = code2ack + '26B-'  
            except:
                message2print = "{(Step 26Z/70) failed" + "; sourcebufferinfo2mpu-failed-by-copy2buffertoimage4mpu: " + str(sourcebufferinfo2mpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '26Z-'          
                      
            task4copybuffertoimage2mpu ['BUFFERSOURCE'] = sourcebufferinfo2mpu    
                                    
        if 'QUEUE' in keys2task:
            pass
        else:   
            # ---------------------------------------------------------------------------------------------
            # (27) QUEUE - REQUIRED BY MPUXI                          
            # ---------------------------------------------------------------------------------------------         
            queue2copybuffertoimage4mpu = None
            
            message2print = "{(Step 27A/70) trying" + "; queue2copybuffertoimage4mpu-trying-by-copy2buffertoimage4mpu: " + str(queue2copybuffertoimage4mpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
            try:
                queue2copybuffertoimage4mpu = Queue4Xi()                                                                          # THE SYSTEM CREATES QUEUE    
                
                message2print = "{(Step 27B/70) got" + "; queue2copybuffertoimage4mpu-got-by-copy2buffertoimage4mpu: " + str(queue2copybuffertoimage4mpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # 
                
                code2ack  = code2ack + '27B-'  
            except:
                message2print = "{(Step 27Z/70) failed" + "; queue2copybuffertoimage4mpu-failed-by-copy2buffertoimage4mpu: " + str(queue2copybuffertoimage4mpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '27Z-'
                
            task4copybuffertoimage2mpu ['QUEUE'] = queue2copybuffertoimage4mpu
            
        if 'PIPE' in keys2task:
            pass
        else:              
            # ---------------------------------------------------------------------------------------------
            # (28) PIPE - REQUIRED BY MPUXI                              
            # ---------------------------------------------------------------------------------------------         
            pipe2mpu = None
           
            message2print = "{(Step 28A/70) trying" + "; pipe2mpu-trying-by-copy2buffertoimage4mpu: " + str(pipe2mpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
            try:
                pipe2mpu = Pipe4Xi()                                                                            # THE SYSTEM CREATES PIPE    
                
                message2print = "{(Step 28B/70) got" + "; pipe2mpu-got-by-copy2buffertoimage4mpu: " + str(pipe2mpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # 
                
                code2ack  = code2ack + '28B-'  
            except:
                message2print = "{(Step 28Z/70) failed" + "; pipe2mpu-failed-by-copy2buffertoimage4mpu: " + str(pipe2mpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '28Z-'
                
            task4copybuffertoimage2mpu ['PIPE'] = pipe2mpu 
            
        if 'COMMANDQUEUE' in keys2task:
            pass
        else:              
            # ---------------------------------------------------------------------------------------------
            # (28) COMMAND QUEUE - REQUIRED                              
            # ---------------------------------------------------------------------------------------------             
            commandqueueinfo2mpu = {}
           
            message2print = "{(Step 29A/70) trying" + "; commandqueueinfo2mpu-trying-by-copy2buffertoimage4mpu: " + str(commandqueueinfo2mpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print)  
                        
            try:
                commandqueueinfo2mpu = get2commandqueueinfo4mpu()                                               # SYSTEM CREATES COMMAND QUEUE, IF NECESSARY   
                message2print = "{(Step 29B/70) got" + "; commandqueueinfo2mpu-got-by-copy2buffertoimage4mpu: " + str(commandqueueinfo2mpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                
                code2ack  = code2ack + '29B-'  
            except:
                message2print = "{(Step 29Z/70) failed" + "; commandqueueinfo2mpu-failed-by-copy2buffertoimage4mpu: " + str(commandqueueinfo2mpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '29Z-'
        
            task4copybuffertoimage2mpu ['COMMANDQUEUE'] = commandqueueinfo2mpu
        
             
        if 'EVENT' in keys2task:
            pass
        else:               
            # ---------------------------------------------------------------------------------------------
            # (31) EVENT - REQUIRED                            
            # ---------------------------------------------------------------------------------------------             
            eventinfo2mpu = {}
           
            message2print = "{(Step 31A/70) trying" + "; eventinfo2mpu-trying-by-copy2buffertoimage4mpu: " + str(eventinfo2mpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                eventinfo2mpu = get2eventinfo4mpu()                                                                    
                message2print = "{(Step 31B/70) got" + "; eventinfo2mpu-got-by-copy2buffertoimage4mpu: " + str(eventinfo2mpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                
                code2ack  = code2ack + '31B-' 
            except:
                message2print = "{(Step 31Z/70) failed" + "; eventinfo2mpu-failed-by-copy2buffertoimage4mpu: " + str(eventinfo2mpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '31Z-'
        
            task4copybuffertoimage2mpu ['EVENT'] = eventinfo2mpu 
        
             
        if 'EVENTWAITLIST' in keys2task:
            pass
        else:               
            # ---------------------------------------------------------------------------------------------
            # (32) EVENTWAITLIST - REQUIRED                              
            # ---------------------------------------------------------------------------------------------             
            eventwaitlist2mpu = {}
           
            message2print = "{(Step 32A/70) trying" + "; eventwaitlist2mpu-trying-by-copy2buffertoimage4mpu: " + str(eventwaitlist2mpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                eventwaitlist2mpu = {}                                                                   
                message2print = "{(Step 32B/70) set" + "; eventwaitlist2mpu-set-by-copy2buffertoimage4mpu: " + str(eventwaitlist2mpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                
                code2ack  = code2ack + '32B-' 
            except:
                message2print = "{(Step 32Z/70) failed" + "; eventwaitlist2mpu-failed-by-copy2buffertoimage4mpu: " + str(eventwaitlist2mpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '32Z-'
        
            task4copybuffertoimage2mpu ['EVENTWAITLIST'] = eventwaitlist2mpu                     
              
        if 'EVENTWAITLISTQUANTITY' in keys2task:
            pass
        else:               
            # ---------------------------------------------------------------------------------------------
            # (33) EVENTWAITLISTQUANTITY - REQUIRED                              
            # ---------------------------------------------------------------------------------------------             
            eventwaitlistquantity2mpu = 0
           
            message2print = "{(Step 33A/70) trying" + "; eventwaitlistquantity2mpu-trying-by-copy2buffertoimage4mpu: " + str(eventwaitlistquantity2mpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                keys2event = eventwaitlist2mpu.keys()
                for key2event in keys2event:
                    eventwaitlistquantity2mpu = eventwaitlistquantity2mpu + 1                                                                     
                message2print = "{(Step 33B/70) set" + "; eventwaitlistquantity2mpu-set-by-copy2buffertoimage4mpu: " + str(eventwaitlistquantity2mpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 0 
                
                code2ack  = code2ack + '33B-' 
            except:
                message2print = "{(Step 33Z/70) failed" + "; eventwaitlistquantity2mpu-failed-by-copy2buffertoimage4mpu: " + str(eventwaitlistquantity2mpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '33Z-'
        
            task4copybuffertoimage2mpu ['EVENTWAITLISTQUANTITY'] = eventwaitlistquantity2mpu       
                                             
  
             
        if 'DESTINATIONORIGIN' in keys2task:
            pass
        else:                    
            # ---------------------------------------------------------------------------------------------
            # (34) DESTINATIONORIGIN - REQUIRED                             
            # ---------------------------------------------------------------------------------------------             
            destination4origin2mpu = {}
            
            message2print = "{(Step 34A/70) trying" + "; destination4origin2mpu-trying-by-copy2buffertoimage4mpu: " + str(destination4origin2mpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:                                                                                    
                destination4origin2mpu['X'] = 0 
                destination4origin2mpu['Y'] = 0  
                destination4origin2mpu['Z'] = 0                                                                                                                          
                message2print = "{(Step 34B/70) set" + "; destination4origin2mpu-set-by-copy2buffertoimage4mpu to: " + str(destination4origin2mpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                code2ack  = code2ack + '34B-' 
            except:
                message2print = "{(Step 34Z/70) failed" + "; destination4origin2mpu-failed-by-copy2buffertoimage4mpu: " + str(destination4origin2mpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '34Z-'
        
            task4copybuffertoimage2mpu ['DESTINATIONORIGIN'] = destination4origin2mpu
             
     
             
        if 'REGION' in keys2task:
            pass
        else:            
            # ---------------------------------------------------------------------------------------------
            # (36) REGION - REQUIRED                         
            # ---------------------------------------------------------------------------------------------             
            region2mpu = {}
            
            message2print = "{(Step 36A/70) trying" + "; region2mpu-trying-by-copy2buffertoimage4mpu: " + str(region2mpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:                                                                                    
                region2mpu['X'] = 160                                                                               # WIDTH  IN PIXELS TO WRITE 
                region2mpu['Y'] = 90                                                                                # HEIGHT IN PIXELS TO WRITE
                region2mpu['Z'] = 0                                                                                 # DEPTH  IN PIXELS TO WRITE                                                                                                                          
                message2print = "{(Step 36B/70) set" + "; region2mpu-set-by-copy2buffertoimage4mpu to: " + str(region2mpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                code2ack  = code2ack + '36B-' 
            except:
                message2print = "{(Step 36Z/70) failed" + "; region2mpu-failed-by-copy2buffertoimage4mpu: " + str(region2mpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '36Z-'
        
            task4copybuffertoimage2mpu ['REGION'] = region2mpu
        
             
        if 'OFFSET' in keys2task:
            pass
        else:            
            # ---------------------------------------------------------------------------------------------
            # (37) OFFSET - WHERE TO BEGIN COPYING DATA FROM SOURCE BUFFER - REQUIRED                         
            # ---------------------------------------------------------------------------------------------             
            offset2mpu = 0
            message2print = "{(Step 37A/70) set" + "; offset2mpu-set-by-copy2buffertoimage4mpu to: " + str(offset2mpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # 
            task4copybuffertoimage2mpu ['OFFSET'] = offset2mpu        
            code2ack  = code2ack + '37A-' 
                    
        # ---------------------------------------------------------------------------------------------
        # EOF TASK ANALYSIS
        # ---------------------------------------------------------------------------------------------
        keys2task = task4copybuffertoimage2mpu.keys()
        message2print = "{(Step 39/70) set" + "; keys2task-set-by-copy2buffertoimage4mpu to: " + str(sorted(keys2task))  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
        # ['BUFFERSOURCE',
        #  'COMMANDQUEUE',
        #  'CONTEXT',
        #  'DATA2CALLBACK',
        #  'DESTINATIONORIGIN',
        #  'DEVICE',
        #  'EVENT',
        #  'EVENTWAITLIST',
        #  'EVENTWAITLISTQUANTITY',
        #  'FUNCTION2CALLBACK',
        #  'IMAGEDESTINATION',
        #  'OFFSET',
        #  'PIPE',
        #  'PLATFORM',
        #  'QUEUE',
        #  'REGION']; time: 04:52:11 10/04/11 Pacific Daylight Time}
                


    # ---------------------------------------------------------------------------------------------
    # CONTROL AND COMMAND
    # ---------------------------------------------------------------------------------------------
    control2copybuffertoimage4mpu = {}       
                                
    # ---------------------------------------------------------------------------------------------
    # CONSUMER AND PUBLISHER CHANNEL FOR CONTROL AND COMMAND
    # ---------------------------------------------------------------------------------------------
    ack4channels2mpu = False
    channels2mpu = ()
    message2print = "{(Step 40A/70) trying" + "channels2mpu-trying-by-copy2buffertoimage4mpu: " + str(channels2mpu)  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)     
    try:
        # 
        consumerchannel2copybuffertoimage4mpu, publisherchannel2copybuffertoimage4mpu = task4copybuffertoimage2mpu ['PIPE']

        message2print = "{(Step 40B/70) got" + "; consumerchannel2copybuffertoimage4mpu-got-by-copy2buffertoimage4mpu: " + str(consumerchannel2copybuffertoimage4mpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
        # <read-write PipeConnection, handle 704>

        message2print = "{(Step 40C/70) got" + "; publisherchannel2copybuffertoimage4mpu-got-by-copy2buffertoimage4mpu: " + str(publisherchannel2copybuffertoimage4mpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
        # <read-write PipeConnection, handle 708>                      
        ack4channels2mpu = True
        
        control2copybuffertoimage4mpu['CONSUMERCHANNEL']  = consumerchannel2copybuffertoimage4mpu
        
        
        control2copybuffertoimage4mpu['PUBLISHERCHANNEL'] = publisherchannel2copybuffertoimage4mpu
        
        code2ack  = code2ack + '40D-' 
        
    except:
        ack4channels2mpu = False
        code2nack  = code2nack + '40Z-'      
                        
    # ---------------------------------------------------------------------------------------------
    # QUEUE FOR CONTROL AND COMMAND
    # ---------------------------------------------------------------------------------------------
    ack4queue2copybuffertoimage4mpu = False
    queue2copybuffertoimage4mpu = None
    message2print = "{(Step 41A/70) trying" + "queue2copybuffertoimage4mpu-trying-by-copy2buffertoimage4mpu: " + str(queue2copybuffertoimage4mpu)  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)     
    try:
        queue2copybuffertoimage4mpu = task4copybuffertoimage2mpu ['QUEUE']

        message2print = "{(Step 41B/70) got" + "; queue2copybuffertoimage4mpu-got-by-copy2buffertoimage4mpu: " + str(queue2copybuffertoimage4mpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
        # <queue4xi.Queue4Xi object at 0x0000000003DF2198>
              
        ack4queue2copybuffertoimage4mpu = True
        control2copybuffertoimage4mpu['QUEUE']  = queue2copybuffertoimage4mpu        
        code2ack  = code2ack + '41C-'
    except:
        ack4queue2copybuffertoimage4mpu = False  
        code2nack  = code2nack + '41Z-'
    
    
    # -----------------------------------------------------------------------------------------    
    # DUMMY PROCESS OBJECT = THREAD CREATION
    # -----------------------------------------------------------------------------------------           
    # dummy2image4mpu = None
    # message2print = "{(Step 69A/70) trying" + "dummy2image4mpu-trying-by-copy2buffertoimage4mpu: " + str(dummy2image4mpu)  + "; time: " + time4xi() + "}"   
    # print4xi(class2print, method2print, message2print)     
    # try:                            
        # dummy2image4mpu = DummyProcess4Xi(target=manoeuvre2copybuffertoimage4mpu,                                               # in this module to handle queue and pipe channels
        #                                   args =(queue2copybuffertoimage4mpu,                                                         # caller puts and gets - does NOT call any functions
        #                                          consumerchannel2copybuffertoimage4mpu, publisherchannel2copybuffertoimage4mpu))                        # caller sends and receives - does NOT call any functions
        # message2print = "{(Step 69B/70) created" + "dummy2image4mpu-created-by-copy2buffertoimage4mpu: " + str(dummy2image4mpu)  + "; time: " + time4xi() + "}" 
        # print4xi(class2print, method2print, message2print)  
        # code2ack  = code2ack + '69B-'
        
    # except:
        # code2nack  = code2nack + '69Z-'
        
    # ---------------------------------------------------------------------------------------------
    # RETURN CONTROL AND COMMAND DICTIONARY
    # ---------------------------------------------------------------------------------------------          
    message2print = "{(Step 70A/70) set" + "; code2ack-set-by-copy2buffertoimage4mpu: " + str(code2ack) + "; time: " + time4xi() + "}"  
    print4xi(class2print, method2print, message2print)                                                             
    # 1B1-1B2-1C-2B-3B-4B-5B-6B-7B-8B-9B-11B-12B-13B-15B-16B-40D-41C- 
    message2print = "{(Step 70B/70) set" + "; code2nack-set-by-copy2buffertoimage4mpu: " + str(code2nack) + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                             
    # 
        
    message2print = "{(Step 70C/70) returning" + "; control2copybuffertoimage4mpu-returning-by-copy2buffertoimage4mpu: " + str(control2copybuffertoimage4mpu) + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                             
    # {'QUEUE': <queue4xi.Queue4Xi object at 0x0000000003DF2198>,
    #  'PUBLISHERCHANNEL': <read-write PipeConnection, handle 708>,
    #  'CONSUMERCHANNEL': <read-write PipeConnection, handle 704>}; time: 04:52:11 10/04/11 Pacific Daylight Time}

        
    return control2copybuffertoimage4mpu
                    
#</function4xi> 


# -------------------------------------------------------------------------------------------------
#<function4xi function = "read2image4mpu" usage = "
#                                                                                                              
#   # ---------------------------------------------------------------------------------------------
#   # (16) ESTABLISH CONTROL & COMMAND WITH READ2IMAGE4MPU                      
#   # ---------------------------------------------------------------------------------------------    
#   print('(Step 16A/700) OF MPU2IMAGE4XI5TEST - TRYING TO ESTABLISH CONTROL & COMMAND WITH READ2IMAGE4MPU ', time4xi())    
#   respon4read2image = {} 
#   task4read2image = None
#   try:                               
#       response4read2image = read2image4mpu(task4read2image)                                                                                                          
#       print('(Step 16B/700) OF MPU2IMAGE4XI5TEST - ESTABLISHED CONTROL & COMMAND WITH READ2IMAGE4MPU : ', response4read2image)
#       queue2read4mpu      = response4read2image['QUEUE']
#       channel2readpublish4mpu = response4read2image['PUBLISHERCHANNEL']
#       channel2readconsume4mpu = response4read2image['CONSUMERCHANNEL']
#   except:
#       print('(Step 16Z/700) OF MPU2IMAGE4XI5TEST *** FAILED TO ESTABLISH CONTROL & COMMAND WITH READ2IMAGE4MPU ', time4xi()) 
#   # --------------------------------------------------------------------------------------------------
#   # (17) SEND CONTROL & COMMAND COPY VIA PIPE FOR MANOEUVRE2IMAGE4MPU TO RECEIVE                 
#   # --------------------------------------------------------------------------------------------------      
#   print('(Step 17A/700) OF MPU2IMAGE4XI5TEST - TRYING TO SEND CONTROL & COMMAND VIA PIPE WITH READ2IMAGE4MPU ', time4xi())     
#   try:                               
#       message2readsend4mpu = "<copy4xi>copy2read4mpu</copy4xi>"
#       channel2readpublish4mpu = response4read2image['PUBLISHERCHANNEL']
#       channel2readpublish4mpu.send(message2readsend4mpu)
#       print("(Step 17B/700) OF MPU2IMAGE4XI5TEST - SENT TO CHANNEL2PUBLISH4MPU: ", channel2readpublish4mpu, " MESSAGE2SEND4MPU: " , message2readsend4mpu)                                                                                                
#   except:
#       print('(Step 17Z1/700) OF MPU2IMAGE4XI5TEST *** FAILED TO SEND CONTROL & COMMAND VIA PIPE WITH READ2IMAGE4MPU ', time4xi())                                                
#   # ------------------------------------------------------------------------------------------
#   # (18) LET MANOEUVRE2IMAGE4MPU TO RECEIVE COPY AND AND SEND ROGER VIA PIPE                
#   # ------------------------------------------------------------------------------------------       
#   print("(Step 18C/700) OF MPU2IMAGE4XI5TEST - TRYING TO PERFORM TARGET FUNCTION MANOEUVRE2IMAGE4MPU ", time4xi()) 
#   try:        
#       response2manoeuvre4mpu = manoeuvre2readimage4mpu(queue2read4mpu, channel2readconsume4mpu, channel2readpublish4mpu)
#       print("(Step 18D/700) OF MPU2IMAGE4XI5TEST - PERFORMED TARGET FUNCTION MANOEUVRE2IMAGE4MPU ", manoeuvre2readimage4mpu)
#   except:
#       print("(Step 18Z/700) OF MPU2IMAGE4XI5TEST *** FAILED TO PERFORM TARGET FUNCTION MANOEUVRE2IMAGE4MPU ", time4xi())     
#   else:
#       pass # trust activated thread    
#   # ---------------------------------------------------------------------------------------------
#   # (19) RECEIVE CONTROL & COMMAND ROGER VIA PIPE                   
#   # ---------------------------------------------------------------------------------------------          
#   print('(Step 19A/700) OF MPU2IMAGE4XI5TEST - TRYING TO RECEIVE CONTROL & COMMAND VIA PIPE WITH READ2IMAGE4MPU ', time4xi())     
#   try:
#       channel2readconsume4mpu = response4read2image['CONSUMERCHANNEL']                              
#       message2readreceive4mpu = channel2readconsume4mpu.recv()
#       print("(Step 19B/700) OF MPU2IMAGE4XI5TEST - RECEIVED FROM CHANNEL2CONSUME4MPU: ", channel2readconsume4mpu, " CONTROL & COMMAND: ", message2readreceive4mpu)                                                                                                  
#       # <roger4xi>roger2read4mpu</roger4xi>
#   except:
#       print('(Step 19Z/700) OF MPU2IMAGE4XI5TEST *** FAILED TO RECEIVE CONTROL & COMMAND VIA PIPE  WITH READ2IMAGE4MPU ', time4xi())         
#   # ---------------------------------------------------------------------------------------------------
#   # (20) SEND CONTROL & COMMAND READY VIA PIPE FOR MANOEUVRE2IMAGE4MPU TO RECEIVE                
#   # ---------------------------------------------------------------------------------------------------      
#   print('(Step 20A/700) OF MPU2IMAGE4XI5TEST - TRYING TO SEND CONTROL & COMMAND VIA PIPE WITH READ2IMAGE4MPU ', time4xi())     
#   try:                               
#       message2readsend4mpu = "<ready4xi>ready2read4mpu</ready4xi>"
#       channel2readpublish4mpu = response4read2image['PUBLISHERCHANNEL']
#       channel2readpublish4mpu.send(message2readsend4mpu)
#       print("(Step 20B/700) OF MPU2IMAGE4XI5TEST - SENT TO CHANNEL2PUBLISH4MPU: ", channel2readpublish4mpu, " MESSAGE2SEND4MPU: " , message2readsend4mpu)                                                                                                
#       # <ready4xi>ready2read4mpu</ready4xi> 
#   except:
#        print('(Step 20Z/700) OF MPU2IMAGE4XI5TEST *** FAILED TO SEND CONTROL & COMMAND VIA PIPE WITH READ2IMAGE4MPU ', time4xi())                                                
#   # -----------------------------------------------------------------------------------
#   # (21) LET MANOEUVRE2IMAGE4MPU TO RECEIVE READY AND TO SEND READY                
#   # -----------------------------------------------------------------------------------       
#   print("(Step 21A/700) OF MPU2IMAGE4XI5TEST - TRYING TO PERFORM TARGET FUNCTION MANOEUVRE2IMAGE4MPU ", time4xi()) 
#   try:        
#       response2manoeuvre4mpu = manoeuvre2readimage4mpu(queue2read4mpu, channel2readconsume4mpu, channel2readpublish4mpu)
#       print("(Step 21B/700) OF MPU2IMAGE4XI5TEST - PERFORMED TARGET FUNCTION MANOEUVRE2IMAGE4MPU ", manoeuvre2readimage4mpu)
#   except:
#       print("(Step 21Z/700) OF MPU2IMAGE4XI5TEST *** FAILED TO PERFORM TARGET FUNCTION MANOEUVRE2IMAGE4MPU ", time4xi())     
#   # ---------------------------------------------------------------------------------------
#   # (22) RECEIVE CONTROL & COMMAND READY VIA PIPE                      
#   # ---------------------------------------------------------------------------------------          
#   print('(Step 22A/700) OF MPU2IMAGE4XI5TEST - TRYING TO RECEIVE CONTROL & COMMAND VIA PIPE WITH READ2IMAGE4MPU ', time4xi())     
#   try:
#       channel2readconsume4mpu = response4read2image['CONSUMERCHANNEL']                              
#       message2readreceive4mpu = channel2readconsume4mpu.recv()
#       print("(Step 22B/700) OF MPU2IMAGE4XI5TEST - RECEIVED FROM CHANNEL2CONSUME4MPU: ", channel2readconsume4mpu, " CONTROL & COMMAND: ", message2readreceive4mpu)                                                                                                  
#       # <ready4xi>ready2read4mpu</ready4xi> 
#   except:
#       print('(Step 22Z/700) OF MPU2IMAGE4XI5TEST *** FAILED TO RECEIVE CONTROL & COMMAND VIA PIPE  WITH READ2IMAGE4MPU ', time4xi())          
#   # -----------------------------------------------------------------------------------------
#   # (24) PUT IMAGE PATH TO QUEUE
#   # -----------------------------------------------------------------------------------------                          
#   print("(Step 24A/700) OF MPU2IMAGE4XI5TEST - TRYING TO PUT IMAGE PATH TO QUEUE ", time4xi())      
#   try:    
#       queue2read4dragon.put(path2dragon)             
#       print("(Step 24B/700) OF DRAGON2IMAGE4XI5TEST - PUT TO QUEUE IMAGE PATH: ", path2dragon) 
#       # ..//..//..//..//..//web//images//png//dragonxi.png  
#   except:
#       print("(Step 24Z/700) OF MPU2IMAGE4XI5TEST *** FAILED TO PUT IMAGE PATH TO QUEUE ", time4xi()) 
#   # ------------------------------------------------------------------------------------------------------
#   # (25) LET MANOEUVRE2IMAGE4MPU TO GET FROM QUEUE AND TO SEND EITHER ACK OR NACK VIA PIPE               
#   # -------------------------------------------------------------------------------------------------------       
#   print("(Step 25A/700) OF MPU2IMAGE4XI5TEST - TRYING TO PERFORM TARGET FUNCTION MANOEUVRE2IMAGE4MPU ", time4xi()) 
#   try:        
#       response2manoeuvre4mpu = manoeuvre2readimage4mpu(queue2read4mpu, channel2readconsume4mpu, channel2readpublish4mpu)
#       print("(Step 25B/700) OF MPU2IMAGE4XI5TEST - PERFORMED TARGET FUNCTION MANOEUVRE2IMAGE4MPU ", manoeuvre2readimage4mpu)
#   except:
#       print("(Step 25Z/700) OF MPU2IMAGE4XI5TEST *** FAILED TO PERFORM TARGET FUNCTION MANOEUVRE2IMAGE4MPU ", time4xi())     
#   # -----------------------------------------------------------------------------------------------
#   # (26) RECEIVE ACK OR NACK ABOUT IMAGE OPERATION VIA PIPE                
#   # -----------------------------------------------------------------------------------------------          
#   print('(Step 26A/700) OF MPU2IMAGE4XI5TEST - TRYING TO RECEIVE CONTROL & COMMAND ABOUT IMAGE OPERTION VIA PIPE ', time4xi())      
#   try: 
#       message2readreceive4mpu = channel2readconsume4mpu.recv()
#       print("(Step 26D/700) OF MPU2IMAGE4XI5TEST - RECEIVED FROM CHANNEL2CONSUME4MPU: ", channel2readconsume4mpu, " CONTROL & COMMAND: ", message2readreceive4mpu)                                                                                                   
#       # <ack4xi>ack2imageinfo4mpu</ack4xi>
#   except:
#       print('(Step 26Z/700) OF MPU2IMAGE4XI5TEST *** FAILED TO RECEIVE CONTROL & COMMAND ABOUT IMAGE OPERTION VIA PIPE ', time4xi())               
#   # -------------------------------------------------------------------------------------------------------
#   # (27) LET MANOEUVRE2IMAGE4MPU GO TO STATE 0               
#   # -------------------------------------------------------------------------------------------------------       
#   print("(Step 27A/700) OF MPU2IMAGE4XI5TEST - TRYING TO PERFORM TARGET FUNCTION MANOEUVRE2IMAGE4MPU ", time4xi()) 
#   try:        
#       response2manoeuvre4mpu = manoeuvre2readimage4mpu(queue2read4mpu, channel2readconsume4mpu, channel2readpublish4mpu)
#       print("(Step 27B/700) OF MPU2IMAGE4XI5TEST - GOT RESPONSE2MANOEUVRE4MPU ", response2manoeuvre4mpu)
#   except:
#       print("(Step 27Z/700) OF MPU2IMAGE4XI5TEST *** FAILED TO PERFORM TARGET FUNCTION MANOEUVRE2IMAGE4MPU ", time4xi())   
#">

# -------------------------------------------------------------------------------------------------   
def read2image4mpu(task4imageread2mpu=None):

    '''
    reads 2D images for MPU
    '''
    global mpu2image4xi                                                     # 2D-IMAGES FOR MPU  
    global control2readimage4mpu                                              # COMMAND AND CONTROL FOR MPU IMAGES 
    
    # -----------------------------------------------------------------------------------------
    # (Step 1/33) PARAMETERS           
    # -----------------------------------------------------------------------------------------          
    class2print = "read2image4mpu"
    method2print = "read2image4mpu"

    message2print = "{(Step 1A/33) started" + "; read2image4mpu-started: " + str(read2image4mpu)  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                                     #  
    # <function read2image4mpu at 0x00000000039595C8> 
    
    message2print = "{(Step 1B/33) passed" + "; task4imageread2mpu-to-read2image4mpu: " + str(task4imageread2mpu) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                                                              
    # None
            
    code2ack  = ' '
    code2nack = ' '
    
    # -----------------------------------------------------------------------------------------
    # CREATE MPU2IMAGE4XI IF NECESSARY       
    # ----------------------------------------------------------------------------------------- 
    try:
        if mpu2image4xi is None:
            mpu2image4xi = create2image4mpu()                                                                              # INITIALIZE IMAGE FOR MPU 
            message2print = "{(Step 1C/70) created" + "; mpu2image4xi-created-by-read2image4mpu to: " + str(mpu2image4xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #             
        else:
            message2print = "{(Step 1D/70) skipped" + "; create2image4mpu-skipped-by-read2image4mpu: " + str(read2image4mpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            # skipped                                 
    except:
        try:
            mpu2image4xi = create2image4mpu()
            message2print = "{(Step 1E/70) set" + "; mpu2image4xi-set-by-read2image4mpu to: " + str(mpu2image4xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #             
        except:
            mpu2image4xi = {}
            message2print = "{(Step 1Z/70) yielded" + "; mpu2image4xi-yielded-by-read2image4mpu: " + str(mpu2image4xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            code2nack  = code2nack + '1Z-' 
    
    # -----------------------------------------------------------------------------------------
    # (Step 1/70) - CHECK PARAMETERS           
    # -----------------------------------------------------------------------------------------          
    try:             
        if task4imageread2mpu is None:
            # -----------------------------------------------------------------------------------------
            # NO PARAMETERS GIVEN BY CALLER FOR READ - BUILD TASK! 
            # -----------------------------------------------------------------------------------------    
            
            task4imageread2mpu = {}                                                                                                                                                                                       
            
            message2print = "{(Step 1A/70) initialized" + "; task4imageread2mpu-created-by-read2image4mpu to: " + str(task4imageread2mpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)               
            # 
            
            # ---------------------------------------------------------------------------------------------
            # (1B) TRACE - AUTOMATIC                      
            # ---------------------------------------------------------------------------------------------              
            function2callback4mpu = trace2image4mpu                                                              # CALLBACK FUNCTION FOR MPU
            
            message2print = "{(Step 1B1/70) initialized" + "; function2callback4mpu-created-by-read2image4mpu to: " + str(function2callback4mpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)            
            # <function trace2image4mpu at 0x0000000004737548>;
            
            code2ack  = code2ack + '1B1-'
           
            args2callback = {}                                                                                   # CREATE ARGUMENT DICTIONARY FOR CALLBACK FUNCTION             
            args2callback['CLASS']   = "read2image4mpu"                                                          # MODULE OF MPUIZED OPENCL MODULE
            args2callback['METHOD']  = "read2image4mpu"                                                          # FUNCTION OF MPUIZED OPENCL MODULE
            args2callback['MESSAGE'] = "{(Step 1Z1/33) failed" + "; read2image4mpu-failed " + "; time: " + time4xi() + "}" # ERROR MESSAGE FOR MPUIZED OPENCL TO RETURN    

            message2print = "{(Step 1B2/70) initialized" + "; args2callback-created-by-read2image4mpu to: " + str(args2callback)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)               
            # {'MESSAGE': '{(Step 1Z1/33) failed; read2image4mpu-failed ; time: 05:06:38 10/04/11 Pacific Daylight Time}', 
            #  'CLASS': 'read2image4mpu', 
            #  'METHOD': 'read2image4mpu'}; time: 05:06:38 10/04/11 Pacific Daylight Time} 

            code2ack  = code2ack + '1B2-'
            
            # ---------------------------------------------------------------------------------------------
            # (1C) CALLBACK - AUTOMATIC                      
            # ---------------------------------------------------------------------------------------------              
            task4imageread2mpu['DATA2CALLBACK'] = args2callback                                                   # USER DATA PASSED TO MPUIZED OPENCL - SEVERAL ARGS PASSED IN ONE DICTIONARY
            task4imageread2mpu['FUNCTION2CALLBACK'] = function2callback4mpu                                       # CALLBACK FUNCTION TO CALLED BY OPENCL IN EXCEPTIONS
            
            code2ack  = code2ack + '1C-'
            message2print = "{(Step 1C/70) initialized" + "; task4imageread2mpu-created-by-read2image4mpu to: " + str(task4imageread2mpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            # {'FUNCTION2CALLBACK': <function trace2image4mpu at 0x0000000004737548>, 
            #  'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; read2image4mpu-failed ; time: 05:06:38 10/04/11 Pacific Daylight Time}', 
            #  'CLASS': 'read2image4mpu', 
            #  'METHOD': 'read2image4mpu'}}; time: 05:06:38 10/04/11 Pacific Daylight Time}
             
            
            # ---------------------------------------------------------------------------------------------
            # (2) PLATFORM - AUTOMATIC                   
            # ---------------------------------------------------------------------------------------------             
            platform2xi = {}
            message2print = "{(Step 2A/70) initialized" + "; platform2xi-created-by-read2image4mpu to: " + str(platform2xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)   
            # {}
            
            try:
                platform2xi = get2platformids4xi()                                                                  # PLATFORM IS COMMON TO ALL DEVICES I.E TO DEVICE XI
                message2print = "{(Step 2B/70) got" + "; platform2xi-got-by-read2image4mpu: " + str(platform2xi)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                # {'list2platforms': ['DragonXi'], 'ack2return': ' 2C-2E-2F-2G-', 'nack2return': ' ', 'available2platform': 1, 'max2entries': 1}
                 
                code2ack  = code2ack + '2B-'
            except:
                message2print = "{(Step 2Z/70) failed" + "; platform2xi-failed-by-read2image4mpu: " + str(failed)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                                
                code2nack  = code2nack + '2Z-'
                 
            task4imageread2mpu['PLATFORM'] = platform2xi                                                          # PLATFORM OF SYSTEM 
              
            
            # ---------------------------------------------------------------------------------------------
            # (3) CONTEXT - AUTOMATIC                 
            # ---------------------------------------------------------------------------------------------    
            context2mpu = {}
            message2print = "{(Step 3A/70) trying" + "; context2mpu-trying-by-read2image4mpu: " + str(context2mpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            try:
                context2mpu = get2context4mpu()                                                                     # NO PARAMETERS - THE SYSTEM CREATES CONTEXT IF REQUIRED  
                
                message2print = "{(Step 3B/70) got" + "; context2mpu-got-by-read2image4mpu: " + str(context2mpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # {'COUNT2REFERENCE': 0, 
                #  'DATA2CALLBACK': {'MESSAGE': ...
    
                code2ack  = code2ack + '3B-'         
            except:
                message2print = "{(Step 3Z/70) failed" + "; context2mpu-failed-by-read2image4mpu: " + str(context2mpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '3Z-'
                
            task4imageread2mpu ['CONTEXT'] = context2mpu           
            
            # ---------------------------------------------------------------------------------------------
            # (4) DEVICE - AUTOMATIC                       
            # ---------------------------------------------------------------------------------------------    
            device2mpu = {}
            
            message2print = "{(Step 4A/70) trying" + "; device2mpu-trying-by-read2image4mpu: " + str(device2mpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            try:
                device2mpu = get2mpu4xi()                                                                           # THE SYSTEM CREATES DEVICE INFO, IF NECESSARY    
                
                message2print = "{(Step 4B/70) got" + "; device2mpu-got-by-read2image4mpu: " + str(device2mpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # {'RESOLUTION2TIMER4PROFILING': 318, 
                #  'ID2PLATFORM': '0000000001509568', 
                #  'PROFILING2QUEUE': 'ACK', 
                #  'EXECUTE2KERNELS4OPENCL': 'ACK', 
                #  'NATIVE2WIDTHLONG4VECTOR': 2 ...
                
                code2ack  = code2ack + '4B-'  
            except:
                message2print = "{(Step 4Z/70) failed" + "; device2mpu-failed-by-read2image4mpu: " + str(device2mpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '4Z-'
            
            task4imageread2mpu ['DEVICE'] = device2mpu
            
                   
            # ---------------------------------------------------------------------------------------------
            # (5) IMAGE - REQUIRED                            
            # ---------------------------------------------------------------------------------------------         
            imageinfo2mpu = {}
             
            message2print = "{(Step 5A/70) trying" + "; imageinfo2mpu-trying-by-read2image4mpu: " + str(imageinfo2mpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            try:
                imageinfo2mpu = get2imageinfo4mpu()                                                                         
                
                message2print = "{(Step 5B/70) got" + "; imageinfo2mpu-got-by-read2image4mpu: " + str(imageinfo2mpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # {'SIZE2QUEUE': 2147483646, 
                #  'FORMAT': {'TYPE': 'CL_UNORM_INT8', 
                #  'ORDER': 'CL_RGBA'}, 
                #  'QUEUE': <queue4xi.Queue4Xi object at 0x00000000041F9320>, 
                #  'WIDTH': 800, 
                #  'CONTEXT': {'COUNT2REFERENCE': 0, ...
                
                code2ack  = code2ack + '5B-'  
            except:
                message2print = "{(Step 5Z/70) failed" + "; imageinfo2mpu-failed-by-read2image4mpu: " + str(imageinfo2mpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '5Z-'          
                      
            task4imageread2mpu ['IMAGE'] = imageinfo2mpu         
                        

            # ---------------------------------------------------------------------------------------------
            # (6) QUEUE - REQUIRED BY MPUXI                          
            # ---------------------------------------------------------------------------------------------         
            queue2mpu = None
            
            message2print = "{(Step 6A/70) trying" + "; queue2mpu-trying-by-read2image4mpu: " + str(queue2mpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
            try:
                queue2mpu = Queue4Xi()                                                                                  # THE SYSTEM CREATES QUEUE    
                
                message2print = "{(Step 6B/70) got" + "; queue2mpu-got-by-read2image4mpu: " + str(queue2mpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # <queue4xi.Queue4Xi object at 0x0000000004607160> 
                
                code2ack  = code2ack + '6B-'  
            except:
                message2print = "{(Step 6Z/70) failed" + "; queue2mpu-failed-by-read2image4mpu: " + str(queue2mpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '6Z-'
                
            task4imageread2mpu ['QUEUE'] = queue2mpu

            
            # ---------------------------------------------------------------------------------------------
            # (7) PIPE - REQUIRED BY MPUXI                              
            # ---------------------------------------------------------------------------------------------         
            pipe2mpu = None
           
            message2print = "{(Step 7A/70) trying" + "; pipe2mpu-trying-by-read2image4mpu: " + str(pipe2mpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
            try:
                pipe2mpu = Pipe4Xi()                                                                                    # THE SYSTEM CREATES PIPE    
                
                message2print = "{(Step 7B/70) got" + "; pipe2mpu-got-by-read2image4mpu: " + str(pipe2mpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # (<read-write PipeConnection, handle 408>, <read-write PipeConnection, handle 412>) 
                
                code2ack  = code2ack + '7B-'  
            except:
                message2print = "{(Step 7Z/70) failed" + "; pipe2mpu-failed-by-read2image4mpu: " + str(pipe2mpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '7Z-'
                
            task4imageread2mpu ['PIPE'] = pipe2mpu 
            
            # ---------------------------------------------------------------------------------------------
            # (8) COMMAND QUEUE - REQUIRED                              
            # ---------------------------------------------------------------------------------------------             
            commandqueueinfo2mpu = {}
           
            message2print = "{(Step 8A/70) trying" + "; commandqueueinfo2mpu-trying-by-read2image4mpu: " + str(commandqueueinfo2mpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print)  
                        
            try:
                commandqueueinfo2mpu = get2commandqueueinfo4mpu()                                                    # SYSTEM CREATES COMMAND QUEUE, IF NECESSARY   
                message2print = "{(Step 8B/70) got" + "; commandqueueinfo2mpu-got-by-read2image4mpu: " + str(commandqueueinfo2mpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # {'CODE2ERROR': 0, 
                #  'COUNT2REFERENCE': 0, 
                #  'DATA2CALLBACK': {'MESSAGE': ...
                
                code2ack  = code2ack + '8B-'  
            except:
                message2print = "{(Step 8Z/70) failed" + "; commandqueueinfo2mpu-failed-by-read2image4mpu: " + str(commandqueueinfo2mpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '8Z-'
        
            task4imageread2mpu ['COMMANDQUEUE'] = commandqueueinfo2mpu
            
            
            
            # ---------------------------------------------------------------------------------------------
            # (9) BUFFER - AUTOMATIC BY MPUXI                               
            # ---------------------------------------------------------------------------------------------             
            bufferinfo2mpu = {}
           
            message2print = "{(Step 9A/70) trying" + "; bufferinfo2mpu-trying-by-read2image4mpu: " + str(bufferinfo2mpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                bufferinfo2mpu = get2bufferinfo4mpu()                                                                   # SYSTEM CREATES BUFFER FOR MPU, IF NECESSARY
                message2print = "{(Step 9B/70) got" + "; bufferinfo2mpu-got-by-read2image4mpu: " + str(bufferinfo2mpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # {'CODE2ERROR': 0, 'COUNT2REFERENCE': 0, 'DATA2CALLBACK': {'MESSAGE': ...
                
                code2ack  = code2ack + '9B-' 
            except:
                message2print = "{(Step 9Z/70) failed" + "; bufferinfo2mpu-failed-by-read2image4mpu: " + str(bufferinfo2mpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '9Z-'
        
            task4imageread2mpu ['BUFFER'] = bufferinfo2mpu 
            
            # ---------------------------------------------------------------------------------------------
            # (10) EVENT - REQUIRED                            
            # ---------------------------------------------------------------------------------------------             
            eventinfo2mpu = {}
           
            message2print = "{(Step 10A/70) trying" + "; eventinfo2mpu-trying-by-read2image4mpu: " + str(eventinfo2mpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                eventinfo2mpu = get2eventinfo4mpu()                                                                    
                message2print = "{(Step 10B/70) got" + "; eventinfo2mpu-got-by-read2image4mpu: " + str(eventinfo2mpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # {'CODE2ERROR': 0, 'DATA2CALLBACK': {'MESSAGE': ...
                
                code2ack  = code2ack + '10B-' 
            except:
                message2print = "{(Step 10Z/70) failed" + "; eventinfo2mpu-failed-by-read2image4mpu: " + str(eventinfo2mpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '10Z-'
        
            task4imageread2mpu ['EVENT'] = eventinfo2mpu 
            
            
            # ---------------------------------------------------------------------------------------------
            # (11) EVENTWAITLIST - REQUIRED                              
            # ---------------------------------------------------------------------------------------------             
            eventwaitlist2mpu = {}
           
            message2print = "{(Step 11A/70) trying" + "; eventwaitlist2mpu-trying-by-read2image4mpu: " + str(eventwaitlist2mpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                eventwaitlist2mpu = {}                                                                   
                message2print = "{(Step 11B/70) set" + "; eventwaitlist2mpu-set-by-read2image4mpu: " + str(eventwaitlist2mpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # {}
                
                code2ack  = code2ack + '11B-' 
            except:
                message2print = "{(Step 11Z/70) failed" + "; eventwaitlist2mpu-failed-by-read2image4mpu: " + str(eventwaitlist2mpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '11Z-'
        
            task4imageread2mpu ['EVENTWAITLIST'] = eventwaitlist2mpu                     

            
            # ---------------------------------------------------------------------------------------------
            # (12) EVENTWAITLISTQUANTITY - REQUIRED                               
            # ---------------------------------------------------------------------------------------------             
            eventwaitlistquantity2mpu = 0
           
            message2print = "{(Step 12A/70) trying" + "; eventwaitlistquantity2mpu-trying-by-read2image4mpu: " + str(eventwaitlistquantity2mpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                keys2event = eventwaitlist2mpu.keys()
                for key2event in keys2event:
                    eventwaitlistquantity2mpu = eventwaitlistquantity2mpu + 1                                                                   
                message2print = "{(Step 12B/70) set" + "; eventwaitlistquantity2mpu-set-by-read2image4mpu: " + str(eventwaitlistquantity2mpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 0 
                
                code2ack  = code2ack + '12B-' 
            except:
                message2print = "{(Step 12Z/70) failed" + "; eventwaitlistquantity2mpu-failed-by-read2image4mpu: " + str(eventwaitlistquantity2mpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '12Z-'
        
            task4imageread2mpu ['EVENTWAITLISTQUANTITY'] = eventwaitlistquantity2mpu       
                                                 
            
            # ---------------------------------------------------------------------------------------------
            # (13) BLOCKING - REQUIRED                        
            # ---------------------------------------------------------------------------------------------             
            blocking2mpu = ' '                                                                   
     
            message2print = "{(Step 13A/70) trying" + "; blocking2mpu-trying-by-read2image4mpu: " + str(blocking2mpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                blocking2mpu = 'CL_FALSE'                                                          # the read command is non-blocking                                                              
                message2print = "{(Step 13B/70) set" + "; blocking2mpu-set-by-read2image4mpu to: " + str(blocking2mpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # CL_FALSE
                code2ack  = code2ack + '13B-' 
            except:
                message2print = "{(Step 13Z/70) failed" + "; blocking2mpu-failed-by-read2image4mpu: " + str(blocking2mpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '13Z-'
        
            task4imageread2mpu ['BLOCKING'] = blocking2mpu             
            
            # ---------------------------------------------------------------------------------------------
            # (14) ORIGIN - REQUIRED                             
            # ---------------------------------------------------------------------------------------------             
            origin2mpu = {}
            
            message2print = "{(Step 14A/70) trying" + "; origin2mpu-trying-by-read2image4mpu: " + str(origin2mpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:                                                                                    
                origin2mpu['X'] = 0 
                origin2mpu['Y'] = 0  
                origin2mpu['Z'] = 0                                                                                                                          
                message2print = "{(Step 14B/70) set" + "; origin2mpu-set-by-read2image4mpu to: " + str(origin2mpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # {'Y': 0, 'X': 0, 'Z': 0} 
                code2ack  = code2ack + '14B-' 
            except:
                message2print = "{(Step 14Z/70) failed" + "; origin2mpu-failed-by-read2image4mpu: " + str(origin2mpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '14Z-'
        
            task4imageread2mpu ['ORIGIN'] = origin2mpu
            
            
            # ---------------------------------------------------------------------------------------------
            # (15) REGION - REQUIRED                         
            # ---------------------------------------------------------------------------------------------             
            region2mpu = {}
            
            message2print = "{(Step 15A/70) trying" + "; region2mpu-trying-by-read2image4mpu: " + str(region2mpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:                                                                                    
                region2mpu['X'] = 160                                                                               # WIDTH  IN PIXELS TO READ 
                region2mpu['Y'] = 90                                                                                # HEIGHT IN PIXELS TO READ 
                region2mpu['Z'] = 0                                                                                 # DEPTH  IN PIXELS TO READ                                                                                                                          
                message2print = "{(Step 15B/70) set" + "; region2mpu-set-by-read2image4mpu to: " + str(region2mpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # {'Y': 90, 'X': 160, 'Z': 0}
                code2ack  = code2ack + '15B-' 
            except:
                message2print = "{(Step 15Z/70) failed" + "; region2mpu-failed-by-read2image4mpu: " + str(region2mpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '15Z-'
        
            task4imageread2mpu ['REGION'] = region2mpu
            
            
            # ---------------------------------------------------------------------------------------------
            # (16) ROWPITCH (STRIDE) - REQUIRED                              
            # ---------------------------------------------------------------------------------------------             
            # One can assume that memory representing an image or screen will have data laid out as one row of pixels followed directly by the next row of pixels. 
            # However many APIs (e.g. Direct3D) that return a pointer to image data add an extra block of data at the end of each row. 
            # int offset = x * bytesPerPixel + y * pitch;
            # When a video image is stored in memory, the memory buffer might contain extra padding bytes after each row of pixels. 
            # The padding bytes affect how the image is stored in memory, but do not affect how the image is displayed.
            # The stride is the number of bytes from one row of pixels in memory to the next row of pixels in memory. 
            # Stride is also called pitch. 
            # If padding bytes are present, the stride is wider than the width of the image, as shown in the following illustration.


            rowpitch2mpu = 0
            size2element = 8                                                                                        # VOXEL (X,Y,Z) = SHOT IN PIXELS, R=1 PIXELS
            voxelquantity2layer = 10                                                                                # MAX NUMBER OF VOXELS IN EACH 2D LAYER 
            quantity2direction = 8                                                                                  # ONE DIRECTIONS IN 2D
            width2mpu = 800
            
            message2print = "{(Step 16A/70) trying" + "; rowpitch2mpu-trying-by-read2image4mpu: " + str(rowpitch2mpu)  + "; time: " + time4xi() + "}"  
            print4xi(class2print, method2print, message2print) 
            # 
                     
            try:                                                                                                    # X-AXEL IS THE WORST CASE TO SCAN WITH Y=0, Z=0
                rowpitch2mpu = int(width2mpu * size2element * voxelquantity2layer / quantity2direction)             # (800 * 8 * 10) / 8 = 8K PIXELS MAX TO SCAN 
                                                                                                                    # SPHERE IS SPARSE HAVING FEW SHOTS PER SCAN LINE 
                                                                                                                                         
                message2print = "{(Step 16B/70) set" + "; rowpitch2mpu-set-by-read2image4mpu to: " + str(rowpitch2mpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                #  8000                                                                                          
                 
                code2ack  = code2ack + '16B-' 
            except:
                message2print = "{(Step 16Z/70) failed" + "; rowpitch2mpu-failed-by-read2image4mpu: " + str(rowpitch2mpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                # 
                
                code2nack  = code2nack + '16Z-'
        
            task4imageread2mpu ['ROWPITCH'] = rowpitch2mpu     
            
            # ----------------------------------------------------------------------------------------------------
            # (17) POINTER - REQUIRED - THE POINTER TO A BUFFER IN HOST MEMORY WHERE IMAGE DATA IS TO BE READ FROM                          
            # ----------------------------------------------------------------------------------------------------             
            pointer2mpu = None
            height2mpu = 800
            size2mpu = int(width2mpu * height2mpu)                                                                  # CALCULATED MEMORY
            if size2mpu == 0:
                size2mpu = 4400                                                                                     # MINIMUM MEMORY
            
            message2print = "{(Step 17A/70) trying" + "; pointer2mpu-trying-by-read2image4mpu: " + str(pointer2mpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
            # 
                        
            try:
                arena2mpu = Arena4Xi(size2mpu)                                                                      # SYSTEM RESERVER MEMORY FOR IMAGE                             
                                                                                                                                         
                message2print = "{(Step 17B/70) got" + "; arena2mpu-got-by-read2image4mpu: " + str(arena2mpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # <arena4xi.Arena4Xi object at 0x0000000004449EF0>
                
                size2arena = arena2mpu.size                                                                         # 
                              
                message2print = "{(Step 17C/70) got" + "; size2arena-got-by-read2buffer4mpu: " + str(size2arena)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                # 640000
                
                buffer2arena = arena2mpu.buffer
                
                message2print = "{(Step 17D/70) got" + "; buffer2arena-got-by-read2buffer4mpu: " + str(buffer2arena)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                
                # <mmap.mmap object at 0x00000000041A0C90>
                
                code2ack  = code2ack + '17D-' 
                
                if size2arena < size2mpu:
                    size2mpu = size2arena                                                                           # AVAILABLE MEMORY FOR IMAGE
                    message2print = "{(Step 17E/70) set" + "; size2mpu-set-by-read2buffer4mpu to: " + str(size2mpu)  + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)
                    # 
                    
                    code2ack  = code2ack + '17E-'
                else:
                    pass
                    
                pointer2mpu = buffer2arena
                message2print = "{(Step 17F/70) set" + "; pointer2mpu-set-by-read2buffer4mpu to: " + str(pointer2mpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                # <mmap.mmap object at 0x00000000041A0C90>
                
                code2ack  = code2ack + '17F-'       
            except:
                message2print = "{(Step 17Z/70) failed" + "; pointer2mpu-failed-by-read2image4mpu: " + str(pointer2mpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                # 
                
                code2nack  = code2nack + '17Z-'
        
            task4imageread2mpu ['POINTER'] = pointer2mpu                            
            task4imageread2mpu ['SIZE']    = size2mpu                                                               # AUTOMATIC BY MPUXI
            
            
            # --------------------------------------------------------------------------------------------------------
            # (18) SLICEPITCH - REQUIRED - THE SIZE IN BYTES OF THE 2D SLICE OF THE 3D REGION OF A 3D IMAGE BEING READ                         
            # --------------------------------------------------------------------------------------------------------             
            slicepitch2mpu = None                                                                                   # SLICEPITCH IS ZERO FOR 2D IMAGES                           
           
            message2print = "{(Step 18A/70) trying" + "; slicepitch2mpu-trying-by-read2image4mpu: " + str(slicepitch2mpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                slicepitch2mpu = 0                                                          
                message2print = "{(Step 18B/70) set" + "; slicepitch2mpu-set-by-read2image4mpu to: " + str(slicepitch2mpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 0 

                code2ack  = code2ack + '18B-'                
                 
            except:
                message2print = "{(Step 18Z/70) failed" + "; slicepitch2mpu-failed-by-read2image4mpu: " + str(slicepitch2mpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '18Z-'
            
            task4imageread2mpu ['SLICEPITCH'] = slicepitch2mpu                  
            
            # ---------------------------------------------------------------------------------------------
            # EOF PARAMETER PROVIDING                
            # ---------------------------------------------------------------------------------------------                                                                                                                               
        else:
            pass
        # ---------------------------------------------------------------------------------------------
        # ANALYSE TASKS - WHETHER AUTOMATICALLY GENERATED OR USER GIVEN 
        # ---------------------------------------------------------------------------------------------
        keys2task = task4imageread2mpu.keys()
        message2print = "{(Step 20/70) got" + "; keys2task-got-by-read2image4mpu: " + str(sorted(keys2task))  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
        # ['BLOCKING',
        #  'BUFFER',
        #  'COMMANDQUEUE',
        #  'CONTEXT',
        #  'DATA2CALLBACK',
        #  'DEVICE',
        #  'EVENT',
        #  'EVENTWAITLIST',
        #  'EVENTWAITLISTQUANTITY',
        #  'FUNCTION2CALLBACK',
        #  'IMAGE',
        #  'ORIGIN',
        #  'PIPE',
        #  'PLATFORM',
        #  'POINTER',
        #  'QUEUE',
        #  'REGION',
        #  'ROWPITCH',
        #  'SIZE',
        #  'SLICEPITCH']; time: 05:25:42 10/04/11 Pacific Daylight Time}

        if (('DATA2CALLBACK' in keys2task) and ('FUNCTION2CALLBACK' in keys2task)):
            pass
        else:                
            # ---------------------------------------------------------------------------------------------
            # (21B) TRACE - AUTOMATIC                      
            # ---------------------------------------------------------------------------------------------              
            function2callback4mpu = trace2image4mpu                                                              # CALLBACK FUNCTION FOR MPU
            
            message2print = "{(Step 21B1/70) initialized" + "; function2callback4mpu-created-by-read2image4mpu to: " + str(function2callback4mpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)            
            # 
            
            code2ack  = code2ack + '21B1-'
           
            args2callback = {}                                                                                   # CREATE ARGUMENT DICTIONARY FOR CALLBACK FUNCTION             
            args2callback['CLASS']   = "read2image4mpu"                                                          # MODULE OF MPUIZED OPENCL MODULE
            args2callback['METHOD']  = "read2image4mpu"                                                          # FUNCTION OF MPUIZED OPENCL MODULE
            args2callback['MESSAGE'] = "{(Step 1Z1/33) failed" + "; read2image4mpu-failed " + "; time: " + time4xi() + "}" # ERROR MESSAGE FOR MPUIZED OPENCL TO RETURN    

            message2print = "{(Step 21B2/70) initialized" + "; args2callback-created-by-read2image4mpu to: " + str(args2callback)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)               
            # 

            code2ack  = code2ack + '21B2-'
        
            # ---------------------------------------------------------------------------------------------
            # (21C) CALLBACK - AUTOMATIC                      
            # ---------------------------------------------------------------------------------------------              
            task4imageread2mpu['DATA2CALLBACK'] = args2callback                                                   # USER DATA PASSED TO MPUIZED OPENCL - SEVERAL ARGS PASSED IN ONE DICTIONARY
            task4imageread2mpu['FUNCTION2CALLBACK'] = function2callback4mpu                                       # CALLBACK FUNCTION TO CALLED BY OPENCL IN EXCEPTIONS
            
            code2ack  = code2ack + '21C-'
            message2print = "{(Step 21C/70) initialized" + "; task4imageread2mpu-created-by-read2image4mpu to: " + str(task4imageread2mpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            #
             
        if 'PLATFORM' in keys2task:
            pass
        else:  
            # ---------------------------------------------------------------------------------------------
            # (22) PLATFORM - AUTOMATIC                   
            # ---------------------------------------------------------------------------------------------             
            platform2xi = {}
            message2print = "{(Step 22A/70) initialized" + "; platform2xi-created-by-read2image4mpu to: " + str(platform2xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)   
            # 
            
            try:
                platform2xi = get2platformids4xi()                                                                  # PLATFORM IS COMMON TO ALL DEVICES I.E TO DEVICE XI
                message2print = "{(Step 22B/70) got" + "; platform2xi-got-by-read2image4mpu: " + str(platform2xi)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                # 
                 
                code2ack  = code2ack + '22B-'
            except:
                message2print = "{(Step 22Z/70) failed" + "; platform2xi-failed-by-read2image4mpu: " + str(failed)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                                
                code2nack  = code2nack + '22Z-'
                 
            task4imageread2mpu['PLATFORM'] = platform2xi                                                          # PLATFORM OF SYSTEM 
          
        if 'CONTEXT' in keys2task:
            pass
        else:              
            # ---------------------------------------------------------------------------------------------
            # (23) CONTEXT - AUTOMATIC                 
            # ---------------------------------------------------------------------------------------------    
            context2mpu = {}
            message2print = "{(Step 23A/70) trying" + "; context2mpu-trying-by-read2image4mpu: " + str(context2mpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            try:
                context2mpu = get2context4mpu()                                                                     # NO PARAMETERS - THE SYSTEM CREATES CONTEXT IF REQUIRED  
                
                message2print = "{(Step 23B/70) got" + "; context2mpu-got-by-read2image4mpu: " + str(context2mpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # 
                
                code2ack  = code2ack + '23B-'         
            except:
                message2print = "{(Step 23Z/70) failed" + "; context2mpu-failed-by-read2image4mpu: " + str(context2mpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '23Z-'
                
            task4imageread2mpu ['CONTEXT'] = context2mpu           
        
        if 'DEVICE' in keys2task:
            pass
        else:                   
            # ---------------------------------------------------------------------------------------------
            # (24) DEVICE - AUTOMATIC                       
            # ---------------------------------------------------------------------------------------------    
            device2mpu = {}
            
            message2print = "{(Step 24A/70) trying" + "; device2mpu-trying-by-read2image4mpu: " + str(device2mpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            try:
                device2mpu = get2mpu4xi()                                                                           # THE SYSTEM CREATES DEVICE INFO, IF NECESSARY    
                
                message2print = "{(Step 24B/70) got" + "; device2mpu-got-by-read2image4mpu: " + str(device2mpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # 
                
                code2ack  = code2ack + '24B-'  
            except:
                message2print = "{(Step 24Z/70) failed" + "; device2mpu-failed-by-read2image4mpu: " + str(device2mpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '24Z-'
            
            task4imageread2mpu ['DEVICE'] = device2mpu
        
        if 'IMAGE' in keys2task:
            pass
        else:                      
            # ---------------------------------------------------------------------------------------------
            # (25) IMAGE - REQUIRED                            
            # ---------------------------------------------------------------------------------------------         
            imageinfo2mpu = {}
             
            message2print = "{(Step 25A/70) trying" + "; imageinfo2mpu-trying-by-read2image4mpu: " + str(imageinfo2mpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            try:
                imageinfo2mpu = get2imageinfo4mpu()                                                                         
                
                message2print = "{(Step 25B/70) got" + "; imageinfo2mpu-got-by-read2image4mpu: " + str(imageinfo2mpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                #                 
                code2ack  = code2ack + '25B-'  
            except:
                message2print = "{(Step 25Z/70) failed" + "; imageinfo2mpu-failed-by-read2image4mpu: " + str(imageinfo2mpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '25Z-'          
                      
            task4imageread2mpu ['IMAGE'] = imageinfo2mpu         
                    
        if 'QUEUE' in keys2task:
            pass
        else:   
            # ---------------------------------------------------------------------------------------------
            # (26) QUEUE - REQUIRED BY MPUXI                          
            # ---------------------------------------------------------------------------------------------         
            queue2mpu = None
            
            message2print = "{(Step 26A/70) trying" + "; queue2mpu-trying-by-read2image4mpu: " + str(queue2mpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
            try:
                queue2mpu = Queue4Xi()                                                                                  # THE SYSTEM CREATES QUEUE    
                
                message2print = "{(Step 26B/70) got" + "; queue2mpu-got-by-read2image4mpu: " + str(queue2mpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # 
                
                code2ack  = code2ack + '26B-'  
            except:
                message2print = "{(Step 26Z/70) failed" + "; queue2mpu-failed-by-read2image4mpu: " + str(queue2mpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '26Z-'
                
            task4imageread2mpu ['QUEUE'] = queue2mpu
        
        if 'PIPE' in keys2task:
            pass
        else:              
            # ---------------------------------------------------------------------------------------------
            # (27) PIPE - REQUIRED BY MPUXI                              
            # ---------------------------------------------------------------------------------------------         
            pipe2mpu = None
           
            message2print = "{(Step 27A/70) trying" + "; pipe2mpu-trying-by-read2image4mpu: " + str(pipe2mpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
            try:
                pipe2mpu = Pipe4Xi()                                                                                    # THE SYSTEM CREATES PIPE    
                
                message2print = "{(Step 27B/70) got" + "; pipe2mpu-got-by-read2image4mpu: " + str(pipe2mpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # 
                
                code2ack  = code2ack + '27B-'  
            except:
                message2print = "{(Step 27Z/70) failed" + "; pipe2mpu-failed-by-read2image4mpu: " + str(pipe2mpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '27Z-'
                
            task4imageread2mpu ['PIPE'] = pipe2mpu 
            
        if 'COMMANDQUEUE' in keys2task:
            pass
        else:              
            # ---------------------------------------------------------------------------------------------
            # (28) COMMAND QUEUE - REQUIRED                              
            # ---------------------------------------------------------------------------------------------             
            commandqueueinfo2mpu = {}
           
            message2print = "{(Step 28A/70) trying" + "; commandqueueinfo2mpu-trying-by-read2image4mpu: " + str(commandqueueinfo2mpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print)  
                        
            try:
                commandqueueinfo2mpu = get2commandqueueinfo4mpu()                                                    # SYSTEM CREATES COMMAND QUEUE, IF NECESSARY   
                message2print = "{(Step 28B/70) got" + "; commandqueueinfo2mpu-got-by-read2image4mpu: " + str(commandqueueinfo2mpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                
                code2ack  = code2ack + '28B-'  
            except:
                message2print = "{(Step 28Z/70) failed" + "; commandqueueinfo2mpu-failed-by-read2image4mpu: " + str(commandqueueinfo2mpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '28Z-'
        
            task4imageread2mpu ['COMMANDQUEUE'] = commandqueueinfo2mpu
        
        
        if 'BUFFER' in keys2task:
            pass
        else:               
            # ---------------------------------------------------------------------------------------------
            # (29) BUFFER - AUTOMATIC BY MPUXI                               
            # ---------------------------------------------------------------------------------------------             
            bufferinfo2mpu = {}
           
            message2print = "{(Step 29A/70) trying" + "; bufferinfo2mpu-trying-by-read2image4mpu: " + str(bufferinfo2mpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                bufferinfo2mpu = get2bufferinfo4mpu()                                                                   # SYSTEM CREATES BUFFER FOR MPU, IF NECESSARY
                message2print = "{(Step 29B/70) got" + "; bufferinfo2mpu-got-by-read2image4mpu: " + str(bufferinfo2mpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                
                code2ack  = code2ack + '29B-' 
            except:
                message2print = "{(Step 29Z/70) failed" + "; bufferinfo2mpu-failed-by-read2image4mpu: " + str(bufferinfo2mpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '29Z-'
        
            task4imageread2mpu ['BUFFER'] = bufferinfo2mpu
             
        if 'EVENT' in keys2task:
            pass
        else:               
            # ---------------------------------------------------------------------------------------------
            # (30) EVENT - REQUIRED                            
            # ---------------------------------------------------------------------------------------------             
            eventinfo2mpu = {}
           
            message2print = "{(Step 30A/70) trying" + "; eventinfo2mpu-trying-by-read2image4mpu: " + str(eventinfo2mpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                eventinfo2mpu = get2eventinfo4mpu()                                                                    
                message2print = "{(Step 30B/70) got" + "; eventinfo2mpu-got-by-read2image4mpu: " + str(eventinfo2mpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                
                code2ack  = code2ack + '30B-' 
            except:
                message2print = "{(Step 30Z/70) failed" + "; eventinfo2mpu-failed-by-read2image4mpu: " + str(eventinfo2mpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '30Z-'
        
            task4imageread2mpu ['EVENT'] = eventinfo2mpu 
        
             
        if 'EVENTWAITLIST' in keys2task:
            pass
        else:               
            # ---------------------------------------------------------------------------------------------
            # (31) EVENTWAITLIST - REQUIRED                              
            # ---------------------------------------------------------------------------------------------             
            eventwaitlist2mpu = {}
           
            message2print = "{(Step 31A/70) trying" + "; eventwaitlist2mpu-trying-by-read2image4mpu: " + str(eventwaitlist2mpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                eventwaitlist2mpu = {}                                                                   
                message2print = "{(Step 31B/70) set" + "; eventwaitlist2mpu-set-by-read2image4mpu: " + str(eventwaitlist2mpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                
                code2ack  = code2ack + '31B-' 
            except:
                message2print = "{(Step 31Z/70) failed" + "; eventwaitlist2mpu-failed-by-read2image4mpu: " + str(eventwaitlist2mpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '31Z-'
        
            task4imageread2mpu ['EVENTWAITLIST'] = eventwaitlist2mpu                     
              
        if 'EVENTWAITLISTQUANTITY' in keys2task:
            pass
        else:               
            # ---------------------------------------------------------------------------------------------
            # (32) EVENTWAITLISTQUANTITY - REQUIRED                              
            # ---------------------------------------------------------------------------------------------             
            eventwaitlistquantity2mpu = 0
           
            message2print = "{(Step 32A/70) trying" + "; eventwaitlistquantity2mpu-trying-by-read2image4mpu: " + str(eventwaitlistquantity2mpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                keys2event = eventwaitlist2mpu.keys()
                for key2event in keys2event:
                    eventwaitlistquantity2mpu = eventwaitlistquantity2mpu + 1                                                                      
                message2print = "{(Step 32B/70) set" + "; eventwaitlistquantity2mpu-set-by-read2image4mpu: " + str(eventwaitlistquantity2mpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 0
                
                code2ack  = code2ack + '32B-' 
            except:
                message2print = "{(Step 32Z/70) failed" + "; eventwaitlistquantity2mpu-failed-by-read2image4mpu: " + str(eventwaitlistquantity2mpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '32Z-'
        
            task4imageread2mpu ['EVENTWAITLISTQUANTITY'] = eventwaitlistquantity2mpu       
                                                 
                 
        if 'BLOCKING' in keys2task:
            pass
        else:              
            # ---------------------------------------------------------------------------------------------
            # (33) BLOCKING - REQUIRED                        
            # ---------------------------------------------------------------------------------------------             
            blocking2mpu = ' '                                                                   
            message2print = "{(Step 33A/70) trying" + "; blocking2mpu-trying-by-read2image4mpu: " + str(blocking2mpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                blocking2mpu = 'CL_FALSE'                                                          # the read command is non-blocking                                                              
                message2print = "{(Step 33B/70) set" + "; blocking2mpu-set-by-read2image4mpu to: " + str(blocking2mpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                code2ack  = code2ack + '33B-' 
            except:
                message2print = "{(Step 33Z/70) failed" + "; blocking2mpu-failed-by-read2image4mpu: " + str(blocking2mpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '33Z-'
        
            task4imageread2mpu ['BLOCKING'] = blocking2mpu             
             
        if 'ORIGIN' in keys2task:
            pass
        else:                    
            # ---------------------------------------------------------------------------------------------
            # (34) ORIGIN - REQUIRED                             
            # ---------------------------------------------------------------------------------------------             
            origin2mpu = {}
            
            message2print = "{(Step 34A/70) trying" + "; origin2mpu-trying-by-read2image4mpu: " + str(origin2mpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:                                                                                    
                origin2mpu['X'] = 0 
                origin2mpu['Y'] = 0  
                origin2mpu['Z'] = 0                                                                                                                          
                message2print = "{(Step 34B/70) set" + "; origin2mpu-set-by-read2image4mpu to: " + str(origin2mpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                code2ack  = code2ack + '34B-' 
            except:
                message2print = "{(Step 34Z/70) failed" + "; origin2mpu-failed-by-read2image4mpu: " + str(origin2mpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '34Z-'
        
            task4imageread2mpu ['ORIGIN'] = origin2mpu
            
             
        if 'REGION' in keys2task:
            pass
        else:            
            # ---------------------------------------------------------------------------------------------
            # (35) REGION - REQUIRED                         
            # ---------------------------------------------------------------------------------------------             
            region2mpu = {}
            
            message2print = "{(Step 35A/70) trying" + "; region2mpu-trying-by-read2image4mpu: " + str(region2mpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                            
            try:                                                                                    
                region2mpu['X'] = 160                                                                               # WIDTH  IN PIXELS TO READ 
                region2mpu['Y'] = 90                                                                                # HEIGHT IN PIXELS TO READ 
                region2mpu['Z'] = 0                                                                                 # DEPTH  IN PIXELS TO READ                                                                                                                          
                message2print = "{(Step 35B/70) set" + "; region2mpu-set-by-read2image4mpu to: " + str(region2mpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                code2ack  = code2ack + '35B-' 
            except:
                message2print = "{(Step 35Z/70) failed" + "; region2mpu-failed-by-read2image4mpu: " + str(region2mpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '35Z-'
        
            task4imageread2mpu ['REGION'] = region2mpu
        
            
             
        if 'ROWPITCH' in keys2task:
            pass
        else:               
            # ---------------------------------------------------------------------------------------------
            # (36) ROWPITCH (STRIDE) - REQUIRED                              
            # ---------------------------------------------------------------------------------------------             
            # One can assume that memory representing an image or screen will have data laid out as one row of pixels followed directly by the next row of pixels. 
            # However many APIs (e.g. Direct3D) that return a pointer to image data add an extra block of data at the end of each row. 
            # int offset = x * bytesPerPixel + y * pitch;
            # When a video image is stored in memory, the memory buffer might contain extra padding bytes after each row of pixels. 
            # The padding bytes affect how the image is stored in memory, but do not affect how the image is displayed.
            # The stride is the number of bytes from one row of pixels in memory to the next row of pixels in memory. 
            # Stride is also called pitch. 
            # If padding bytes are present, the stride is wider than the width of the image, as shown in the following illustration.
     
            rowpitch2mpu = 0
            size2element = 1                                                                                        # VOXEL (X,Y,Z) = SHOT IN PIXELS, R=1 PIXELS
            voxelquantity2layer = 1                                                                                 # MAX NUMBER OF VOXELS IN EACH 2D LAYER 
            quantity2direction = 1                                                                                  # ONE DIRECTIONS IN 2D
            
            message2print = "{(Step 36A/70) trying" + "; rowpitch2mpu-trying-by-read2image4mpu: " + str(rowpitch2mpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
            # 
                        
            try:                                                                                                    # X-AXEL IS THE WORST CASE TO SCAN WITH Y=0, Z=0
                rowpitch2mpu = int(width2mpu * size2element * voxelquantity2layer / quantity2direction)             # (800 * 8 * 10) / 8 = 8K PIXELS MAX TO SCAN 
                                                                                                                    # SPHERE IS SPARSE HAVING FEW SHOTS PER SCAN LINE 
                                                                                                                                         
                message2print = "{(Step 36B/70) set" + "; rowpitch2mpu-set-by-read2image4mpu to: " + str(rowpitch2mpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                #                                                                                             
                 
                code2ack  = code2ack + '36B-' 
            except:
                message2print = "{(Step 36Z/70) failed" + "; rowpitch2mpu-failed-by-read2image4mpu: " + str(rowpitch2mpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                # 
                
                code2nack  = code2nack + '36Z-'
        
            task4imageread2mpu ['ROWPITCH'] = rowpitch2mpu     
             
        if 'POINTER' in keys2task:
            pass
        else:                
            # ----------------------------------------------------------------------------------------------------
            # (37) POINTER - REQUIRED - THE POINTER TO A BUFFER IN HOST MEMORY WHERE IMAGE DATA IS TO BE READ FROM                          
            # ----------------------------------------------------------------------------------------------------             
            pointer2mpu = None
            size2mpu = int(width2mpu * height2mpu)                                                                  # CALCULATED MEMORY
            if size2mpu == 0:
                size2mpu = 4400                                                                                     # MINIMUM MEMORY
            
            message2print = "{(Step 37A/70) trying" + "; pointer2mpu-trying-by-read2image4mpu: " + str(pointer2mpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
            # 
                        
            try:
                arena2mpu = Arena4Xi(size2mpu)                                                                      # SYSTEM RESERVER MEMORY FOR IMAGE                             
                                                                                                                                         
                message2print = "{(Step 37B/70) got" + "; arena2mpu-got-by-read2image4mpu: " + str(arena2mpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                
                size2arena = arena2mpu.size                                                                         # 
                              
                message2print = "{(Step 37C/70) got" + "; size2arena-got-by-read2buffer4mpu: " + str(size2arena)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                # 
                
                buffer2arena = arena2mpu.buffer
                
                message2print = "{(Step 37D/70) got" + "; buffer2arena-got-by-read2buffer4mpu: " + str(buffer2arena)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                
                # 
                
                code2ack  = code2ack + '37D-' 
                
                if size2arena < size2mpu:
                    size2mpu = size2arena                                                                           # AVAILABLE MEMORY FOR IMAGE
                    message2print = "{(Step 37E/70) set" + "; size2mpu-set-by-read2buffer4mpu to: " + str(size2mpu)  + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)
                    # 
                    
                    code2ack  = code2ack + '37E-'
                else:
                    pass
                    
                pointer2mpu = buffer2arena
                message2print = "{(Step 37F/70) set" + "; pointer2mpu-set-by-read2buffer4mpu to: " + str(pointer2mpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                # 
                
                code2ack  = code2ack + '37F-'       
            except:
                message2print = "{(Step 37Z/70) failed" + "; pointer2mpu-failed-by-read2image4mpu: " + str(pointer2mpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                # 
                
                code2nack  = code2nack + '37Z-'
        
            task4imageread2mpu ['POINTER'] = pointer2mpu                            
            task4imageread2mpu ['SIZE']    = size2mpu                                                               # AUTOMATIC BY MPUXI
        
             
        if 'SLICEPITCH' in keys2task:
            pass
        else:               
            # --------------------------------------------------------------------------------------------------------
            # (38) SLICEPITCH - REQUIRED - THE SIZE IN BYTES OF THE 2D SLICE OF THE 3D REGION OF A 3D IMAGE BEING READ                         
            # --------------------------------------------------------------------------------------------------------             
            slicepitch2mpu = None                                                                                   # SLICEPITCH IS ZERO FOR 2D IMAGES                           
           
            message2print = "{(Step 38A/70) trying" + "; slicepitch2mpu-trying-by-read2image4mpu: " + str(slicepitch2mpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                slicepitch2mpu = 0                                                          
                message2print = "{(Step 38B/70) set" + "; slicepitch2mpu-set-by-read2image4mpu to: " + str(slicepitch2mpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                code2ack  = code2ack + '38B-'                
                 
            except:
                message2print = "{(Step 38Z/70) failed" + "; slicepitch2mpu-failed-by-read2image4mpu: " + str(slicepitch2mpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '38Z-'
            
            task4imageread2mpu ['SLICEPITCH'] = slicepitch2mpu
                              
        # ---------------------------------------------------------------------------------------------
        # EOF TASK ANALYSIS
        # ---------------------------------------------------------------------------------------------
        keys2task = task4imageread2mpu.keys()
        message2print = "{(Step 39/70) set" + "; keys2task-set-by-read2image4mpu to: " + str(sorted(keys2task))  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
        # ['BLOCKING',
        #  'BUFFER',
        #  'COMMANDQUEUE',
        #  'CONTEXT',
        #  'DATA2CALLBACK',
        #  'DEVICE',
        #  'EVENT',
        #  'EVENTWAITLIST',
        #  'EVENTWAITLISTQUANTITY',
        #  'FUNCTION2CALLBACK',
        #  'IMAGE',
        #  'ORIGIN',
        #  'PIPE',
        #  'PLATFORM',
        #  'POINTER',
        #  'QUEUE',
        #  'REGION',
        #  'ROWPITCH',
        #  'SIZE',
        #  'SLICEPITCH']; time: 05:25:42 10/04/11 Pacific Daylight Time}                
    except: 
        message2print = "{(Step 39Z/70) yielded" + "; task4imageread2mpu-yielded-by-read2image4mpu: " + str(task4imageread2mpu) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # 
        
        code2nack  = code2nack + '39Z-' 

    # ---------------------------------------------------------------------------------------------
    # CONTROL AND COMMAND
    # ---------------------------------------------------------------------------------------------
    control2readimage4mpu = {}       
                                
    # ---------------------------------------------------------------------------------------------
    # CONSUMER AND PUBLISHER CHANNEL FOR CONTROL AND COMMAND
    # ---------------------------------------------------------------------------------------------
    ack4channels2mpu = False
    channels2mpu = ()
    message2print = "{(Step 40A/70) trying" + "channels2mpu-trying-by-read2image4mpu: " + str(channels2mpu)  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)     
    try:
        # (<read-write PipeConnection, handle 520>, <read-write PipeConnection, handle 524>) 
        consumerchannel2mpu, publisherchannel2mpu = task4imageread2mpu ['PIPE']

        message2print = "{(Step 40B/70) got" + "; consumerchannel2mpu-got-by-read2image4mpu: " + str(consumerchannel2mpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
        # 

        message2print = "{(Step 40C/70) got" + "; publisherchannel2mpu-got-by-read2image4mpu: " + str(publisherchannel2mpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
        #                      
        ack4channels2mpu = True
        
        control2readimage4mpu['CONSUMERCHANNEL']  = consumerchannel2mpu
        
        
        control2readimage4mpu['PUBLISHERCHANNEL'] = publisherchannel2mpu
        
        code2ack  = code2ack + '40D-' 
        
    except:
        ack4channels2mpu = False
        code2nack  = code2nack + '40Z-'      
                        
    # ---------------------------------------------------------------------------------------------
    # QUEUE FOR CONTROL AND COMMAND
    # ---------------------------------------------------------------------------------------------
    ack4queue2mpu = False
    queue2mpu = None
    message2print = "{(Step 41A/70) trying" + "; queue2mpu-trying-by-read2image4mpu: " + str(queue2mpu)  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)     
    try:
        queue2mpu = task4imageread2mpu ['QUEUE']

        message2print = "{(Step 41B/70) got" + "; queue2mpu-got-by-read2image4mpu: " + str(queue2mpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
        # 
              
        ack4queue2mpu = True
        control2readimage4mpu['QUEUE']  = queue2mpu        
        code2ack  = code2ack + '41C-'
    except:
        ack4queue2mpu = False  
        code2nack  = code2nack + '41Z-'
    
    
    # -----------------------------------------------------------------------------------------    
    # DUMMY PROCESS OBJECT = THREAD CREATION
    # -----------------------------------------------------------------------------------------           
    # dummy2image4mpu = None
    # message2print = "{(Step 69A/70) trying" + "dummy2image4mpu-trying-by-read2image4mpu: " + str(dummy2image4mpu)  + "; time: " + time4xi() + "}"   
    # print4xi(class2print, method2print, message2print)     
    # try:                            
        # dummy2image4mpu = DummyProcess4Xi(target=manoeuvre2readimage4mpu,                                               # in this module to handle queue and pipe channels
        #                                   args =(queue2mpu,                                                         # caller puts and gets - does NOT call any functions
        #                                          consumerchannel2mpu, publisherchannel2mpu))                        # caller sends and receives - does NOT call any functions
        # message2print = "{(Step 69B/70) created" + "dummy2image4mpu-created-by-read2image4mpu: " + str(dummy2image4mpu)  + "; time: " + time4xi() + "}" 
        # print4xi(class2print, method2print, message2print)  
        # code2ack  = code2ack + '69B-'

    # except:
        # code2nack  = code2nack + '69Z-'
        
    # ---------------------------------------------------------------------------------------------
    # RETURN CONTROL AND COMMAND DICTIONARY
    # ---------------------------------------------------------------------------------------------          
    message2print = "{(Step 70A/70) set" + "; code2ack-set-by-read2image4mpu: " + str(code2ack) + "; time: " + time4xi() + "}"  
    print4xi(class2print, method2print, message2print)                                                             
    # 1B1-1B2-1C-2B-3B-4B-5B-6B-7B-8B-9B-10B-11B-12B-13B-14B-15B-16B-17D-17F-18B-40D-41C-
     
    message2print = "{(Step 70B/70) set" + "; code2nack-set-by-read2image4mpu: " + str(code2nack) + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                             
    #   
        
    message2print = "{(Step 70C/70) returning" + "; control2readimage4mpu-returning-by-read2image4mpu: " + str(control2readimage4mpu) + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                             
    # {'QUEUE': <queue4xi.Queue4Xi object at 0x0000000003FE3B70>,
    #  'PUBLISHERCHANNEL': <read-write PipeConnection, handle 416>, 
    #  'CONSUMERCHANNEL': <read-write PipeConnection, handle 412>}
        
    return control2readimage4mpu
                    
#</function4xi> 

# -------------------------------------------------------------------------------------------------
#<function4xi function = "map2image4mpu" usage = "
#                                                                                                              
#   # ---------------------------------------------------------------------------------------------
#   # ESTABLISH CONTROL & COMMAND                 
#   # ---------------------------------------------------------------------------------------------    
#   print('(Step 92A/700) OF MPU2IMAGE4XI5TEST - TRYING TO ESTABLISH CONTROL & COMMAND WITH MAP2IMAGE4MPU ', time4xi())           
#                    
#   response4map2image = {} 
#   task4map2image = None                                                                                  # WE ONLY WANT TO ESTABLISH CONTROL AND CONTROL FIRST
#
#   try:                               
#       response4map2image = map2image4mpu(task4map2image)                                                                                                          
#       print('(Step 92B/700) OF MPU2IMAGE4XI5TEST - ESTABLISHED CONTROL & COMMAND WITH MAP2IMAGE4MPU : ', response4map2image)
#    
#       queue4map2image4mpu      = response4map2image['QUEUE']
#       channel4map2imagepublish4mpu = response4map2image['PUBLISHERCHANNEL']
#       channel4map2imageconsume4mpu = response4map2image['CONSUMERCHANNEL']
#   except:
#       print('(Step 92Z/700) OF MPU2IMAGE4XI5TEST *** FAILED TO ESTABLISH CONTROL & COMMAND WITH MAP2IMAGE4MPU ', time4xi()) 
#        
#   # -----------------------------------------------------------------------------------------------------------
#   # SEND CONTROL & COMMAND COPY VIA PIPE FOR MANOEUVRE2COPYIMAGE4MPU TO RECEIVE                 
#   # -----------------------------------------------------------------------------------------------------------      
#   print('(Step 93A/700) OF MPU2IMAGE4XI5TEST - TRYING TO SEND CONTROL & COMMAND VIA PIPE WITH MAP2IMAGE4MPU ', time4xi())      
#   try:                               
#       message2send4mpu = "<copy4xi>copy2mapimage4mpu</copy4xi>"
#       channel4map2imagepublish4mpu.send(message2send4mpu)
#       print("(Step 93B/700) OF MPU2IMAGE4XI5TEST - SENT TO CHANNEL2PUBLISH4MPU: ", channel4map2imagepublish4mpu, " MESSAGE2SEND4MPU: " , message2send4mpu)                                                                                                
#   except:
#       print('(Step 93Z1/700) OF MPU2IMAGE4XI5TEST *** FAILED TO SEND CONTROL & COMMAND VIA PIPE WITH MAP2IMAGE4MPU ', time4xi())   
# 
#   # ---------------------------------------------------------------------------------------------------------------------
#   # SIMULATE MANOEUVRE2COPYIMAGE4MPU  - LET IT TO RECEIVE COPY AND TO SEND ROGER VIA PIPE                
#   # ---------------------------------------------------------------------------------------------------------------------       
#   if ack2simulate4mpu is True:
#       print("(Step 94A/700) OF MPU2IMAGE4XI5TEST - TRYING TO PERFORM TARGET FUNCTION MANOEUVRE2COPYIMAGE4MPU ", time4xi())   
#       try:        
#           response2manoeuvre4copy = manoeuvre2mapimage4mpu(queue4map2image4mpu, channel4map2imageconsume4mpu, channel4map2imagepublish4mpu)
#           print("(Step 94B/700) OF MPU2IMAGE4XI5TEST - PERFORMED MANOEUVRE2COPYIMAGE4MPU, RESPONSE2MANOEUVRE4COPY: ", response2manoeuvre4copy)
#        except:
#           print("(Step 94Z/700) OF MPU2IMAGE4XI5TEST *** FAILED TO PERFORM TARGET FUNCTION MANOEUVRE2COPYIMAGE4MPU ", time4xi())     
#   else:
#       pass # trust activated thread    
#
#   # ---------------------------------------------------------------------------------------------
#   # RECEIVE CONTROL & COMMAND ROGER VIA PIPE                  
#   # ---------------------------------------------------------------------------------------------          
#   print('(Step 95A/700) OF MPU2IMAGE4XI5TEST - TRYING TO RECEIVE CONTROL & COMMAND VIA PIPE WITH MAP2IMAGE4MPU ', time4xi())      
#   try:
#       message2receive4mpu = channel4map2imageconsume4mpu.recv()
#       print("(Step 95B/700) OF MPU2IMAGE4XI5TEST - RECEIVED FROM CHANNEL2CONSUME4MPU: ", channel4map2imageconsume4mpu, " CONTROL & COMMAND: ", message2receive4mpu)                                                                                                  
#   except:
#       print('(Step 95Z/700) OF MPU2IMAGE4XI5TEST *** FAILED TO RECEIVE CONTROL & COMMAND VIA PIPE  WITH MAP2IMAGE4MPU ', time4xi())          
#   
#   # ------------------------------------------------------------------------------------------------------------
#   # SEND CONTROL & COMMAND READY VIA PIPE FOR MANOEUVRE2COPYIMAGE4MPU TO RECEIVE                
#   # ------------------------------------------------------------------------------------------------------------      
#   print('(Step 96A/700) OF MPU2IMAGE4XI5TEST - TRYING TO SEND CONTROL & COMMAND VIA PIPE WITH MAP2IMAGE4MPU ', time4xi())     
#   try:                               
#       message2send4mpu = "<ready4xi>ready2mapimage4mpu</ready4xi>"
#       channel4map2imagepublish4mpu.send(message2send4mpu)
#       print("(Step 96B/700) OF MPU2IMAGE4XI5TEST - SENT TO CHANNEL2PUBLISH4MPU: ", channel4map2imagepublish4mpu, " MESSAGE2SEND4MPU: " , message2send4mpu)                                                                                                
#   except:
#       print('(Step 84Z/700) OF MPU2IMAGE4XI5TEST *** FAILED TO SEND CONTROL & COMMAND VIA PIPE WITH MAP2IMAGE4MPU ', time4xi())                                                
# 
#   # -------------------------------------------------------------------------------------------------------
#   # SIMULATE MANOEUVRE2COPYIMAGE4MPU  - LET IT RECEIVE READY AND SEND READY                
#   # -------------------------------------------------------------------------------------------------------       
#   if ack2simulate4mpu is True:
#       print("(Step 97A/700) OF MPU2IMAGE4XI5TEST - TRYING TO PERFORM TARGET FUNCTION MANOEUVRE2COPYIMAGE4MPU ", time4xi())  
#       try:        
#           response2manoeuvre4copy = manoeuvre2mapimage4mpu(queue4map2image4mpu, channel4map2imageconsume4mpu, channel4map2imagepublish4mpu)
#           print("(Step 97B/700) OF MPU2IMAGE4XI5TEST - PERFORMED TARGET FUNCTION MANOEUVRE2COPYIMAGE4MPU ", manoeuvre2copyimage4mpu)
#       except:
#           print("(Step 97Z/700) OF MPU2IMAGE4XI5TEST *** FAILED TO PERFORM TARGET FUNCTION MANOEUVRE2COPYIMAGE4MPU ", time4xi())     
#   else:
#       pass # trust activated thread    
#            
#    
#   # ---------------------------------------------------------------------------------------
#   # RECEIVE CONTROL & COMMAND READY VIA PIPE                
#   # ---------------------------------------------------------------------------------------          
#   print('(Step 98A/700) OF MPU2IMAGE4XI5TEST - TRYING TO RECEIVE CONTROL & COMMAND VIA PIPE WITH MAP2IMAGE4MPU ', time4xi())     
#   try:
#       message2receive4mpu = channel4map2imageconsume4mpu.recv()
#       print("(Step 98B/700) OF MPU2IMAGE4XI5TEST - RECEIVED FROM CHANNEL2CONSUME4MPU: ", channel4map2imageconsume4mpu, " CONTROL & COMMAND: ", message2receive4mpu)                                                                                                  
#   except:
#       print('(Step 86Z/700) OF MPU2IMAGE4XI5TEST *** FAILED TO RECEIVE CONTROL & COMMAND VIA PIPE  WITH MAP2IMAGE4MPU ', time4xi())          
#                         
#   # -----------------------------------------------------------------------------------------
#   # IMAGE MPUXI COPY OPERATIONS 
#   # -----------------------------------------------------------------------------------------         
#   print('(Step 99A/700) OF MPU2IMAGE4XI5TEST - TRYING FILE OPERATIONS WITH IMAGE MPUXI ', time4xi())       
#   try:        
#       path2dragon = '..//..//..//..//..//web//images//png//dragonxi.png'
#       file2dragon = open(path2dragon)
#       print("(Step 99B/700) OF MPU2IMAGE4XI5TEST - FILENO: ", file2mpu.fileno( ))  
#       print("(Step 99C/700) OF MPU2IMAGE4XI5TEST - MODE: ", file2mpu.mode)
#       print("(Step 99D/700) OF MPU2IMAGE4XI5TEST - NAME: ", file2mpu.name)
#       offset2mpu = 10
#       file2mpu.seek(offset2mpu)
#       print("(Step 99E/700) OF MPU2IMAGE4XI5TEST - PERFORMED SEEK WITH OFFSET: ", offset2mpu)
#       file2mpu.close()
#       print("(Step 99F/700) OF DRAGON2IMAGE4XI5TEST - CLOSED IMAGE FILE: ", path2dragon)  
#   except:
#       print("(Step 87Z/700) OF DRAGON2IMAGE4XI5TEST *** FAILED TO PERFORM FILE OPERATIONS WITH IMAGE: ", path2dragon)        
# 
#   # -----------------------------------------------------------------------------------------
#   # PUT IMAGE PATH TO QUEUE
#   # -----------------------------------------------------------------------------------------                          
#   print("(Step 100A/700) OF MPU2IMAGE4XI5TEST - TRYING TO PUT IMAGE PATH TO QUEUE ", time4xi())      
#   try:    
#       queue4map2image4dragon.put(path2dragon)             
#       print("(Step 100B/700) OF DRAGON2IMAGE4XI5TEST - PUT TO QUEUE IMAGE PATH: ", path2dragon) 
#   except:
#       print("(Step 88Z/700) OF MPU2IMAGE4XI5TEST *** FAILED TO PUT IMAGE PATH TO QUEUE ", time4xi()) 
#   
#   # -------------------------------------------------------------------------------------------------------------------------------------
#   # SIMULATE MANOEUVRE2COPYIMAGE4MPU  - STATE III - GETS FROM QUEUE AND SENDS EITHER ACK OR NACK VIA PIPE               
#   # -------------------------------------------------------------------------------------------------------------------------------------       
#   if ack2simulate4mpu is True:
#       print("(Step 101A/700) OF MPU2IMAGE4XI5TEST - TRYING TO PERFORM TARGET FUNCTION MANOEUVRE2COPYIMAGE4MPU ", time4xi()) 
#       try:        
#           response2manoeuvre4copy = manoeuvre2mapimage4mpu(queue4map2image4mpu, channel4map2imageconsume4mpu, channel4map2imagepublish4mpu)
#           print("(Step 101B/700) OF MPU2IMAGE4XI5TEST - PERFORMED TARGET FUNCTION MANOEUVRE2COPYIMAGE4MPU ", manoeuvre2copyimage4mpu)
#       except:
#           print("(Step 101Z/700) OF MPU2IMAGE4XI5TEST *** FAILED TO PERFORM TARGET FUNCTION MANOEUVRE2COPYIMAGE4MPU ", time4xi())     
#   else:
#       pass # trust activated thread                     
#                 
#                                  
#   # ----------------------------------------------------------------------------------------------------
#   # RECEIVE ACK OR NACK ABOUT IMAGE OPERATION CONTROL & COMMAND VIA PIPE                
#   # ----------------------------------------------------------------------------------------------------          
#   print('(Step 102A/700) OF MPU2IMAGE4XI5TEST - TRYING TO RECEIVE CONTROL & COMMAND ABOUT IMAGE OPERTION VIA PIPE ', time4xi())      
#   try: 
#       message2receive4mpu = channel4map2imageconsume4mpu.recv()
#       print("(Step 102B/700) OF MPU2IMAGE4XI5TEST - RECEIVED FROM CHANNEL2CONSUME4MPU: ", channel4map2imageconsume4mpu, " CONTROL & COMMAND: ", message2receive4mpu)                                                                                                   
#   except:
#       print('(Step 90Z/700) OF MPU2IMAGE4XI5TEST *** FAILED TO RECEIVE CONTROL & COMMAND ABOUT IMAGE OPERTION VIA PIPE ', time4xi())               
#
#   
#   # ----------------------------------------------------------------------------------------------------------------
#   # SIMULATE MANOEUVRE2COPYIMAGE4MPU  - LET MANOEUVRE2COPYIMAGE4MPU TO GO TO STATE 0               
#   # ----------------------------------------------------------------------------------------------------------------       
#   if ack2simulate4mpu is True:
#       print("(Step 103A/700) OF MPU2IMAGE4XI5TEST - TRYING TO PERFORM TARGET FUNCTION MANOEUVRE2COPYIMAGE4MPU ", time4xi())  
#       try:        
#           response2manoeuvre4copy = manoeuvre2mapimage4mpu(queue4map2image4mpu, channel4map2imageconsume4mpu, channel4map2imagepublish4mpu)
#           print("(Step 103B/700) OF MPU2IMAGE4XI5TEST - GOT RESPONSE2MANOEUVRE4MPU ", response2manoeuvre4copy)
#       except:
#           print("(Step 103Z/700) OF MPU2IMAGE4XI5TEST *** FAILED TO PERFORM TARGET FUNCTION MANOEUVRE2COPYIMAGE4MPU ", time4xi())     
#   else:
#       pass # trust activated thread  
#                                           
#">

# -------------------------------------------------------------------------------------------------   
def map2image4mpu(task4imagemap2mpu=None):

    '''
    maps 2D images for MPU
    '''
    global mpu2image4xi                                                                                             # 2D-IMAGES FOR MPU  
    global control2mapimage4mpu                                                                                     # COMMAND AND CONTROL FOR MPU IMAGES 
    
    # -----------------------------------------------------------------------------------------
    # (Step 1/33) PARAMETERS           
    # -----------------------------------------------------------------------------------------          
    class2print = "map2image4mpu"
    method2print = "map2image4mpu"

    message2print = "{(Step 1A/33) started" + "; map2image4mpu-started: " + str(map2image4mpu)  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                                     #  
    # 
    
    message2print = "{(Step 1B/33) passed" + "; task4imagemap2mpu-to-map2image4mpu: " + str(task4imagemap2mpu) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                                                              
    # 
            
    code2ack  = ' '
    code2nack = ' '
    
    # -----------------------------------------------------------------------------------------
    # CREATE MPU2IMAGE4XI IF NECESSARY       
    # ----------------------------------------------------------------------------------------- 
    try:
        if mpu2image4xi is None:
            mpu2image4xi = create2image4mpu()                                                                              # INITIALIZE IMAGE FOR MPU 
            message2print = "{(Step 1C/70) created" + "; mpu2image4xi-created-by-map2image4mpu to: " + str(mpu2image4xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #             
        else:
            message2print = "{(Step 1D/70) skipped" + "; create2image4mpu-skipped-by-map2image4mpu: " + str(map2image4mpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #                                
    except:
        try:
            mpu2image4xi = create2image4mpu()
            message2print = "{(Step 1E/70) set" + "; mpu2image4xi-set-by-map2image4mpu to: " + str(mpu2image4xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #             
        except:
            mpu2image4xi = {}
            message2print = "{(Step 1Z/70) yielded" + "; mpu2image4xi-yielded-by-map2image4mpu: " + str(mpu2image4xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            code2nack  = code2nack + '1Z-' 
    
    # -----------------------------------------------------------------------------------------
    # (Step 1/70) - CHECK PARAMETERS           
    # -----------------------------------------------------------------------------------------          
    try:             
        if task4imagemap2mpu is None:
            # -----------------------------------------------------------------------------------------
            # NO PARAMETERS GIVEN BY CALLER FOR MAP - BUILD TASK! 
            # -----------------------------------------------------------------------------------------    
            
            task4imagemap2mpu = {}                                                                                                                                                                                       
            
            message2print = "{(Step 1A/70) initialized" + "; task4imagemap2mpu-created-by-map2image4mpu to: " + str(task4imagemap2mpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)               
            # 
            
            # ---------------------------------------------------------------------------------------------
            # (1B) TRACE - AUTOMATIC                      
            # ---------------------------------------------------------------------------------------------              
            function2callback4mpu = trace2image4mpu                                                              # CALLBACK FUNCTION FOR MPU
            
            message2print = "{(Step 1B1/70) initialized" + "; function2callback4mpu-created-by-map2image4mpu to: " + str(function2callback4mpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)            
            # 
            
            code2ack  = code2ack + '1B1-'
           
            args2callback = {}                                                                                   # CREATE ARGUMENT DICTIONARY FOR CALLBACK FUNCTION             
            args2callback['CLASS']   = "map2image4mpu"                                                          # MODULE OF MPUIZED OPENCL MODULE
            args2callback['METHOD']  = "map2image4mpu"                                                          # FUNCTION OF MPUIZED OPENCL MODULE
            args2callback['MESSAGE'] = "{(Step 1Z1/33) failed" + "; map2image4mpu-failed " + "; time: " + time4xi() + "}" # ERROR MESSAGE FOR MPUIZED OPENCL TO RETURN    

            message2print = "{(Step 1B2/70) initialized" + "; args2callback-created-by-map2image4mpu to: " + str(args2callback)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)               
            # 
            code2ack  = code2ack + '1B2-'
            
            # ---------------------------------------------------------------------------------------------
            # (1C) CALLBACK - AUTOMATIC                      
            # ---------------------------------------------------------------------------------------------              
            task4imagemap2mpu['DATA2CALLBACK'] = args2callback                                                   # USER DATA PASSED TO MPUIZED OPENCL - SEVERAL ARGS PASSED IN ONE DICTIONARY
            task4imagemap2mpu['FUNCTION2CALLBACK'] = function2callback4mpu                                       # CALLBACK FUNCTION TO CALLED BY OPENCL IN EXCEPTIONS
            
            code2ack  = code2ack + '1C-'
            message2print = "{(Step 1C/70) initialized" + "; task4imagemap2mpu-created-by-map2image4mpu to: " + str(task4imagemap2mpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            # 
            
            # ---------------------------------------------------------------------------------------------
            # (2) PLATFORM - AUTOMATIC                   
            # ---------------------------------------------------------------------------------------------             
            platform2xi = {}
            message2print = "{(Step 2A/70) initialized" + "; platform2xi-created-by-map2image4mpu to: " + str(platform2xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)   
            # 
            
            try:
                platform2xi = get2platformids4xi()                                                                  # PLATFORM IS COMMON TO ALL DEVICES I.E TO DEVICE XI
                message2print = "{(Step 2B/70) got" + "; platform2xi-got-by-map2image4mpu: " + str(platform2xi)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                # 
                code2ack  = code2ack + '2B-'
            except:
                message2print = "{(Step 2Z/70) failed" + "; platform2xi-failed-by-map2image4mpu: " + str(failed)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                                
                code2nack  = code2nack + '2Z-'
                 
            task4imagemap2mpu['PLATFORM'] = platform2xi                                                          # PLATFORM OF SYSTEM 
              
            
            # ---------------------------------------------------------------------------------------------
            # (3) CONTEXT - AUTOMATIC                 
            # ---------------------------------------------------------------------------------------------    
            context2mpu = {}
            message2print = "{(Step 3A/70) trying" + "; context2mpu-trying-by-map2image4mpu: " + str(context2mpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            try:
                context2mpu = get2context4mpu()                                                                     # NO PARAMETERS - THE SYSTEM CREATES CONTEXT IF REQUIRED  
                
                message2print = "{(Step 3B/70) got" + "; context2mpu-got-by-map2image4mpu: " + str(context2mpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # 
                
                code2ack  = code2ack + '3B-'         
            except:
                message2print = "{(Step 3Z/70) failed" + "; context2mpu-failed-by-map2image4mpu: " + str(context2mpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '3Z-'
                
            task4imagemap2mpu ['CONTEXT'] = context2mpu           
            
            # ---------------------------------------------------------------------------------------------
            # (4) DEVICE - AUTOMATIC                       
            # ---------------------------------------------------------------------------------------------    
            device2mpu = {}
            
            message2print = "{(Step 4A/70) trying" + "; device2mpu-trying-by-map2image4mpu: " + str(device2mpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            try:
                device2mpu = get2mpu4xi()                                                                           # THE SYSTEM CREATES DEVICE INFO, IF NECESSARY    
                
                message2print = "{(Step 4B/70) got" + "; device2mpu-got-by-map2image4mpu: " + str(device2mpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # 
                code2ack  = code2ack + '4B-'  
            except:
                message2print = "{(Step 4Z/70) failed" + "; device2mpu-failed-by-map2image4mpu: " + str(device2mpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '4Z-'
            
            task4imagemap2mpu ['DEVICE'] = device2mpu
            
                   
            # ---------------------------------------------------------------------------------------------
            # (5) IMAGE - REQUIRED                            
            # ---------------------------------------------------------------------------------------------         
            imageinfo2mpu = {}
             
            message2print = "{(Step 5A/70) trying" + "; imageinfo2mpu-trying-by-map2image4mpu: " + str(imageinfo2mpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            try:
                imageinfo2mpu = get2imageinfo4mpu()                                                                         
                
                message2print = "{(Step 5B/70) got" + "; imageinfo2mpu-got-by-map2image4mpu: " + str(imageinfo2mpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # 
                code2ack  = code2ack + '5B-'  
            except:
                message2print = "{(Step 5Z/70) failed" + "; imageinfo2mpu-failed-by-map2image4mpu: " + str(imageinfo2mpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '5Z-'          
                      
            task4imagemap2mpu ['IMAGE'] = imageinfo2mpu         
                        

            # ---------------------------------------------------------------------------------------------
            # (6) QUEUE - REQUIRED BY MPUXI                          
            # ---------------------------------------------------------------------------------------------         
            queue2mpu = None
            
            message2print = "{(Step 6A/70) trying" + "; queue2mpu-trying-by-map2image4mpu: " + str(queue2mpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
            try:
                queue2mpu = Queue4Xi()                                                                                  # THE SYSTEM CREATES QUEUE    
                
                message2print = "{(Step 6B/70) got" + "; queue2mpu-got-by-map2image4mpu: " + str(queue2mpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # 
                
                code2ack  = code2ack + '6B-'  
            except:
                message2print = "{(Step 6Z/70) failed" + "; queue2mpu-failed-by-map2image4mpu: " + str(queue2mpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '6Z-'
                
            task4imagemap2mpu ['QUEUE'] = queue2mpu

            
            # ---------------------------------------------------------------------------------------------
            # (7) PIPE - REQUIRED BY MPUXI                              
            # ---------------------------------------------------------------------------------------------         
            pipe2mpu = None
           
            message2print = "{(Step 7A/70) trying" + "; pipe2mpu-trying-by-map2image4mpu: " + str(pipe2mpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
            try:
                pipe2mpu = Pipe4Xi()                                                                                    # THE SYSTEM CREATES PIPE    
                
                message2print = "{(Step 7B/70) got" + "; pipe2mpu-got-by-map2image4mpu: " + str(pipe2mpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # 
                
                code2ack  = code2ack + '7B-'  
            except:
                message2print = "{(Step 7Z/70) failed" + "; pipe2mpu-failed-by-map2image4mpu: " + str(pipe2mpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '7Z-'
                
            task4imagemap2mpu ['PIPE'] = pipe2mpu 
            
            # ---------------------------------------------------------------------------------------------
            # (8) COMMAND QUEUE - REQUIRED                              
            # ---------------------------------------------------------------------------------------------             
            commandqueueinfo2mpu = {}
           
            message2print = "{(Step 8A/70) trying" + "; commandqueueinfo2mpu-trying-by-map2image4mpu: " + str(commandqueueinfo2mpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print)  
                        
            try:
                commandqueueinfo2mpu = get2commandqueueinfo4mpu()                                                    # SYSTEM CREATES COMMAND QUEUE, IF NECESSARY   
                message2print = "{(Step 8B/70) got" + "; commandqueueinfo2mpu-got-by-map2image4mpu: " + str(commandqueueinfo2mpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                
                code2ack  = code2ack + '8B-'  
            except:
                message2print = "{(Step 8Z/70) failed" + "; commandqueueinfo2mpu-failed-by-map2image4mpu: " + str(commandqueueinfo2mpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '8Z-'
        
            task4imagemap2mpu ['COMMANDQUEUE'] = commandqueueinfo2mpu
            
            
            
            # ---------------------------------------------------------------------------------------------
            # (9) BUFFER - AUTOMATIC BY MPUXI                               
            # ---------------------------------------------------------------------------------------------             
            bufferinfo2mpu = {}
           
            message2print = "{(Step 9A/70) trying" + "; bufferinfo2mpu-trying-by-map2image4mpu: " + str(bufferinfo2mpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                bufferinfo2mpu = get2bufferinfo4mpu()                                                                   # SYSTEM CREATES BUFFER FOR MPU, IF NECESSARY
                message2print = "{(Step 9B/70) got" + "; bufferinfo2mpu-got-by-map2image4mpu: " + str(bufferinfo2mpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                
                code2ack  = code2ack + '9B-' 
            except:
                message2print = "{(Step 9Z/70) failed" + "; bufferinfo2mpu-failed-by-map2image4mpu: " + str(bufferinfo2mpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '9Z-'
        
            task4imagemap2mpu ['BUFFER'] = bufferinfo2mpu 
            
            # ---------------------------------------------------------------------------------------------
            # (10) EVENT - REQUIRED                            
            # ---------------------------------------------------------------------------------------------             
            eventinfo2mpu = {}
           
            message2print = "{(Step 10A/70) trying" + "; eventinfo2mpu-trying-by-map2image4mpu: " + str(eventinfo2mpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                eventinfo2mpu = get2eventinfo4mpu()                                                                    
                message2print = "{(Step 10B/70) got" + "; eventinfo2mpu-got-by-map2image4mpu: " + str(eventinfo2mpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                
                code2ack  = code2ack + '10B-' 
            except:
                message2print = "{(Step 10Z/70) failed" + "; eventinfo2mpu-failed-by-map2image4mpu: " + str(eventinfo2mpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '10Z-'
        
            task4imagemap2mpu ['EVENT'] = eventinfo2mpu 
            
            
            # ---------------------------------------------------------------------------------------------
            # (11) EVENTWAITLIST - REQUIRED                              
            # ---------------------------------------------------------------------------------------------             
            eventwaitlist2mpu = {}
           
            message2print = "{(Step 11A/70) trying" + "; eventwaitlist2mpu-trying-by-map2image4mpu: " + str(eventwaitlist2mpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                eventwaitlist2mpu = {}                                                                   
                message2print = "{(Step 11B/70) set" + "; eventwaitlist2mpu-set-by-map2image4mpu: " + str(eventwaitlist2mpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                
                code2ack  = code2ack + '11B-' 
            except:
                message2print = "{(Step 11Z/70) failed" + "; eventwaitlist2mpu-failed-by-map2image4mpu: " + str(eventwaitlist2mpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '11Z-'
        
            task4imagemap2mpu ['EVENTWAITLIST'] = eventwaitlist2mpu                     

            
            # ---------------------------------------------------------------------------------------------
            # (12) EVENTWAITLISTQUANTITY - REQUIRED                              
            # ---------------------------------------------------------------------------------------------             
            eventwaitlistquantity2mpu = 0
           
            message2print = "{(Step 12A/70) trying" + "; eventwaitlistquantity2mpu-trying-by-map2image4mpu: " + str(eventwaitlistquantity2mpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                keys2event = eventwaitlist2mpu.keys()
                for key2event in keys2event:
                    eventwaitlistquantity2mpu = eventwaitlistquantity2mpu + 1                                                                      
                message2print = "{(Step 12B/70) set" + "; eventwaitlistquantity2mpu-set-by-map2image4mpu: " + str(eventwaitlistquantity2mpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 0 
                
                code2ack  = code2ack + '12B-' 
            except:
                message2print = "{(Step 12Z/70) failed" + "; eventwaitlistquantity2mpu-failed-by-map2image4mpu: " + str(eventwaitlistquantity2mpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '12Z-'
        
            task4imagemap2mpu ['EVENTWAITLISTQUANTITY'] = eventwaitlistquantity2mpu       
                                                 
            
            # ---------------------------------------------------------------------------------------------
            # (13) BLOCKING - REQUIRED                        
            # ---------------------------------------------------------------------------------------------             
            blocking2mpu = ' '                                                                   
     
            message2print = "{(Step 13A/70) trying" + "; blocking2mpu-trying-by-map2image4mpu: " + str(blocking2mpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                blocking2mpu = 'CL_FALSE'                                                          # the map command is non-blocking                                                              
                message2print = "{(Step 13B/70) set" + "; blocking2mpu-set-by-map2image4mpu to: " + str(blocking2mpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                
                code2ack  = code2ack + '13B-' 
            except:
                message2print = "{(Step 13Z/70) failed" + "; blocking2mpu-failed-by-map2image4mpu: " + str(blocking2mpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '13Z-'
        
            task4imagemap2mpu ['BLOCKING'] = blocking2mpu             
            
            # ---------------------------------------------------------------------------------------------
            # (14) ORIGIN - REQUIRED                             
            # ---------------------------------------------------------------------------------------------             
            origin2mpu = {}
            
            message2print = "{(Step 14A/70) trying" + "; origin2mpu-trying-by-map2image4mpu: " + str(origin2mpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:                                                                                    
                origin2mpu['X'] = 0 
                origin2mpu['Y'] = 0  
                origin2mpu['Z'] = 0                                                                                                                          
                message2print = "{(Step 14B/70) set" + "; origin2mpu-set-by-map2image4mpu to: " + str(origin2mpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                 
                code2ack  = code2ack + '14B-' 
            except:
                message2print = "{(Step 14Z/70) failed" + "; origin2mpu-failed-by-map2image4mpu: " + str(origin2mpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '14Z-'
        
            task4imagemap2mpu ['ORIGIN'] = origin2mpu
            
            
            # ---------------------------------------------------------------------------------------------
            # (15) REGION - REQUIRED                         
            # ---------------------------------------------------------------------------------------------             
            region2mpu = {}
            
            message2print = "{(Step 15A/70) trying" + "; region2mpu-trying-by-map2image4mpu: " + str(region2mpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:                                                                                    
                region2mpu['X'] = 160                                                                               # WIDTH  IN PIXELS TO MAP 
                region2mpu['Y'] = 90                                                                                # HEIGHT IN PIXELS TO MAP 
                region2mpu['Z'] = 0                                                                                 # DEPTH  IN PIXELS TO MAP                                                                                                                          
                message2print = "{(Step 15B/70) set" + "; region2mpu-set-by-map2image4mpu to: " + str(region2mpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                
                code2ack  = code2ack + '15B-' 
            except:
                message2print = "{(Step 15Z/70) failed" + "; region2mpu-failed-by-map2image4mpu: " + str(region2mpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '15Z-'
        
            task4imagemap2mpu ['REGION'] = region2mpu
            
            
            # ---------------------------------------------------------------------------------------------
            # (16) ROWPITCH (STRIDE) - REQUIRED                              
            # ---------------------------------------------------------------------------------------------             
            # One can assume that memory representing an image or screen will have data laid out as one row of pixels followed directly by the next row of pixels. 
            # However many APIs (e.g. Direct3D) that return a pointer to image data add an extra block of data at the end of each row. 
            # int offset = x * bytesPerPixel + y * pitch;
            # When a video image is stored in memory, the memory buffer might contain extra padding bytes after each row of pixels. 
            # The padding bytes affect how the image is stored in memory, but do not affect how the image is displayed.
            # The stride is the number of bytes from one row of pixels in memory to the next row of pixels in memory. 
            # Stride is also called pitch. 
            # If padding bytes are present, the stride is wider than the width of the image, as shown in the following illustration.


            rowpitch2mpu = 0
            size2element = 8                                                                                        # VOXEL (X,Y,Z) = SHOT IN PIXELS, R=1 PIXELS
            voxelquantity2layer = 10                                                                                # MAX NUMBER OF VOXELS IN EACH 2D LAYER 
            quantity2direction = 8                                                                                  # ONE DIRECTIONS IN 2D
            width2mpu = 800
            
            message2print = "{(Step 16A/70) trying" + "; rowpitch2mpu-trying-by-map2image4mpu: " + str(rowpitch2mpu)  + "; time: " + time4xi() + "}"  
            print4xi(class2print, method2print, message2print) 
            # 
                     
            try:                                                                                                    # X-AXEL IS THE WORST CASE TO SCAN WITH Y=0, Z=0
                rowpitch2mpu = int(width2mpu * size2element * voxelquantity2layer / quantity2direction)             # (800 * 8 * 10) / 8 = 8K PIXELS MAX TO SCAN 
                                                                                                                    # SPHERE IS SPARSE HAVING FEW SHOTS PER SCAN LINE 
                                                                                                                                         
                message2print = "{(Step 16B/70) set" + "; rowpitch2mpu-set-by-map2image4mpu to: " + str(rowpitch2mpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                #  8000                                                                                          
                 
                code2ack  = code2ack + '16B-' 
            except:
                message2print = "{(Step 16Z/70) failed" + "; rowpitch2mpu-failed-by-map2image4mpu: " + str(rowpitch2mpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                # 
                
                code2nack  = code2nack + '16Z-'
        
            task4imagemap2mpu ['ROWPITCH'] = rowpitch2mpu     
            
            # --------------------------------------------------------------------------------------------------------
            # (18) SLICEPITCH - REQUIRED - THE SIZE IN BYTES OF THE 2D SLICE OF THE 3D REGION OF A 3D IMAGE BEING MAP                         
            # --------------------------------------------------------------------------------------------------------             
            slicepitch2mpu = None                                                                                   # SLICEPITCH IS ZERO FOR 2D IMAGES                           
           
            message2print = "{(Step 18A/70) trying" + "; slicepitch2mpu-trying-by-map2image4mpu: " + str(slicepitch2mpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                slicepitch2mpu = 0                                                          
                message2print = "{(Step 18B/70) set" + "; slicepitch2mpu-set-by-map2image4mpu to: " + str(slicepitch2mpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 0 

                code2ack  = code2ack + '18B-'                
                 
            except:
                message2print = "{(Step 18Z/70) failed" + "; slicepitch2mpu-failed-by-map2image4mpu: " + str(slicepitch2mpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '18Z-'
            
            task4imagemap2mpu ['SLICEPITCH'] = slicepitch2mpu                  
            
            # ---------------------------------------------------------------------------------------------
            # (19) FLAGS - REQUIRED                        
            # ---------------------------------------------------------------------------------------------             
            flags2mpu = 'CL_MAP_READ'                                                                
     
            message2print = "{(Step 19A/70) trying" + "; flags2mpu-trying-by-map2image4mpu: " + str(flags2mpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                flags2mpu = 'CL_MAP_WRITE'  # THE REGION SPECIFIED BY (ORIGIN, REGION) IN THE IMAGE OBJECT IS BEING MAPPED FOR WRITING                                                                                                                     
                message2print = "{(Step 19B/70) set" + "; flags2mpu-set-by-map2image4mpu to: " + str(flags2mpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                
                code2ack  = code2ack + '19B-' 
            except:
                message2print = "{(Step 19Z/70) failed" + "; flags2mpu-failed-by-map2image4mpu: " + str(flags2mpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '19Z-'
        
            task4imagemap2mpu ['FLAGS'] = flags2mpu 
                                  
            # ---------------------------------------------------------------------------------------------
            # EOF PARAMETER PROVIDING                
            # ---------------------------------------------------------------------------------------------                                                                                                                               
        else:
            pass
            
        # ---------------------------------------------------------------------------------------------
        # ANALYSE TASKS - WHETHER AUTOMATICALLY GENERATED OR USER GIVEN 
        # ---------------------------------------------------------------------------------------------
        keys2task = task4imagemap2mpu.keys()
        message2print = "{(Step 20/70) got" + "; keys2task-got-by-map2image4mpu: " + str(sorted(keys2task))  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
        # ['BLOCKING',
        #  'BUFFER',
        #  'COMMANDQUEUE',
        #  'CONTEXT',
        #  'DATA2CALLBACK',
        #  'DEVICE',
        #  'EVENT',
        #  'EVENTWAITLIST',
        #  'EVENTWAITLISTQUANTITY',
        #  'FLAGS',
        #  'FUNCTION2CALLBACK',
        #  'IMAGE',
        #  'ORIGIN',
        #  'PIPE',
        #  'PLATFORM',
        #  'QUEUE',
        #  'REGION',
        #  'ROWPITCH',
        #  'SLICEPITCH']; time: 01:28:44 10/05/11 Pacific Daylight Time}
 

        if (('DATA2CALLBACK' in keys2task) and ('FUNCTION2CALLBACK' in keys2task)):
            pass
        else:                
            # ---------------------------------------------------------------------------------------------
            # (21B) TRACE - AUTOMATIC                      
            # ---------------------------------------------------------------------------------------------              
            function2callback4mpu = trace2image4mpu                                                              # CALLBACK FUNCTION FOR MPU
            
            message2print = "{(Step 21B1/70) initialized" + "; function2callback4mpu-created-by-map2image4mpu to: " + str(function2callback4mpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)            
            # 
            
            code2ack  = code2ack + '21B1-'
           
            args2callback = {}                                                                                   # CREATE ARGUMENT DICTIONARY FOR CALLBACK FUNCTION             
            args2callback['CLASS']   = "map2image4mpu"                                                          # MODULE OF MPUIZED OPENCL MODULE
            args2callback['METHOD']  = "map2image4mpu"                                                          # FUNCTION OF MPUIZED OPENCL MODULE
            args2callback['MESSAGE'] = "{(Step 1Z1/33) failed" + "; map2image4mpu-failed " + "; time: " + time4xi() + "}" # ERROR MESSAGE FOR MPUIZED OPENCL TO RETURN    

            message2print = "{(Step 21B2/70) initialized" + "; args2callback-created-by-map2image4mpu to: " + str(args2callback)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)               
            # 

            code2ack  = code2ack + '21B2-'
        
            # ---------------------------------------------------------------------------------------------
            # (21C) CALLBACK - AUTOMATIC                      
            # ---------------------------------------------------------------------------------------------              
            task4imagemap2mpu['DATA2CALLBACK'] = args2callback                                                   # USER DATA PASSED TO MPUIZED OPENCL - SEVERAL ARGS PASSED IN ONE DICTIONARY
            task4imagemap2mpu['FUNCTION2CALLBACK'] = function2callback4mpu                                       # CALLBACK FUNCTION TO CALLED BY OPENCL IN EXCEPTIONS
            
            code2ack  = code2ack + '21C-'
            message2print = "{(Step 21C/70) initialized" + "; task4imagemap2mpu-created-by-map2image4mpu to: " + str(task4imagemap2mpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            #
             
        if 'PLATFORM' in keys2task:
            pass
        else:  
            # ---------------------------------------------------------------------------------------------
            # (22) PLATFORM - AUTOMATIC                   
            # ---------------------------------------------------------------------------------------------             
            platform2xi = {}
            message2print = "{(Step 22A/70) initialized" + "; platform2xi-created-by-map2image4mpu to: " + str(platform2xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)   
            # 
            
            try:
                platform2xi = get2platformids4xi()                                                                  # PLATFORM IS COMMON TO ALL DEVICES I.E TO DEVICE XI
                message2print = "{(Step 22B/70) got" + "; platform2xi-got-by-map2image4mpu: " + str(platform2xi)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                # 
                 
                code2ack  = code2ack + '22B-'
            except:
                message2print = "{(Step 22Z/70) failed" + "; platform2xi-failed-by-map2image4mpu: " + str(failed)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                                
                code2nack  = code2nack + '22Z-'
                 
            task4imagemap2mpu['PLATFORM'] = platform2xi                                                          # PLATFORM OF SYSTEM 
          
        if 'CONTEXT' in keys2task:
            pass
        else:              
            # ---------------------------------------------------------------------------------------------
            # (23) CONTEXT - AUTOMATIC                 
            # ---------------------------------------------------------------------------------------------    
            context2mpu = {}
            message2print = "{(Step 23A/70) trying" + "; context2mpu-trying-by-map2image4mpu: " + str(context2mpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            try:
                context2mpu = get2context4mpu()                                                                     # NO PARAMETERS - THE SYSTEM CREATES CONTEXT IF REQUIRED  
                
                message2print = "{(Step 23B/70) got" + "; context2mpu-got-by-map2image4mpu: " + str(context2mpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # 
                
                code2ack  = code2ack + '23B-'         
            except:
                message2print = "{(Step 23Z/70) failed" + "; context2mpu-failed-by-map2image4mpu: " + str(context2mpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '23Z-'
                
            task4imagemap2mpu ['CONTEXT'] = context2mpu           
        
        if 'DEVICE' in keys2task:
            pass
        else:                   
            # ---------------------------------------------------------------------------------------------
            # (24) DEVICE - AUTOMATIC                       
            # ---------------------------------------------------------------------------------------------    
            device2mpu = {}
            
            message2print = "{(Step 24A/70) trying" + "; device2mpu-trying-by-map2image4mpu: " + str(device2mpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            try:
                device2mpu = get2mpu4xi()                                                                           # THE SYSTEM CREATES DEVICE INFO, IF NECESSARY    
                
                message2print = "{(Step 24B/70) got" + "; device2mpu-got-by-map2image4mpu: " + str(device2mpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # 
                
                code2ack  = code2ack + '24B-'  
            except:
                message2print = "{(Step 24Z/70) failed" + "; device2mpu-failed-by-map2image4mpu: " + str(device2mpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '24Z-'
            
            task4imagemap2mpu ['DEVICE'] = device2mpu
        
        if 'IMAGE' in keys2task:
            pass
        else:                      
            # ---------------------------------------------------------------------------------------------
            # (25) IMAGE - REQUIRED                            
            # ---------------------------------------------------------------------------------------------         
            imageinfo2mpu = {}
             
            message2print = "{(Step 25A/70) trying" + "; imageinfo2mpu-trying-by-map2image4mpu: " + str(imageinfo2mpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            try:
                imageinfo2mpu = get2imageinfo4mpu()                                                                         
                
                message2print = "{(Step 25B/70) got" + "; imageinfo2mpu-got-by-map2image4mpu: " + str(imageinfo2mpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                #                 
                code2ack  = code2ack + '25B-'  
            except:
                message2print = "{(Step 25Z/70) failed" + "; imageinfo2mpu-failed-by-map2image4mpu: " + str(imageinfo2mpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '25Z-'          
                      
            task4imagemap2mpu ['IMAGE'] = imageinfo2mpu         
                    
        if 'QUEUE' in keys2task:
            pass
        else:   
            # ---------------------------------------------------------------------------------------------
            # (26) QUEUE - REQUIRED BY MPUXI                          
            # ---------------------------------------------------------------------------------------------         
            queue2mpu = None
            
            message2print = "{(Step 26A/70) trying" + "; queue2mpu-trying-by-map2image4mpu: " + str(queue2mpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
            try:
                queue2mpu = Queue4Xi()                                                                                  # THE SYSTEM CREATES QUEUE    
                
                message2print = "{(Step 26B/70) got" + "; queue2mpu-got-by-map2image4mpu: " + str(queue2mpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # 
                
                code2ack  = code2ack + '26B-'  
            except:
                message2print = "{(Step 26Z/70) failed" + "; queue2mpu-failed-by-map2image4mpu: " + str(queue2mpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '26Z-'
                
            task4imagemap2mpu ['QUEUE'] = queue2mpu
        
        if 'PIPE' in keys2task:
            pass
        else:              
            # ---------------------------------------------------------------------------------------------
            # (27) PIPE - REQUIRED BY MPUXI                              
            # ---------------------------------------------------------------------------------------------         
            pipe2mpu = None
           
            message2print = "{(Step 27A/70) trying" + "; pipe2mpu-trying-by-map2image4mpu: " + str(pipe2mpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
            try:
                pipe2mpu = Pipe4Xi()                                                                                    # THE SYSTEM CREATES PIPE    
                
                message2print = "{(Step 27B/70) got" + "; pipe2mpu-got-by-map2image4mpu: " + str(pipe2mpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # 
                
                code2ack  = code2ack + '27B-'  
            except:
                message2print = "{(Step 27Z/70) failed" + "; pipe2mpu-failed-by-map2image4mpu: " + str(pipe2mpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '27Z-'
                
            task4imagemap2mpu ['PIPE'] = pipe2mpu 
            
        if 'COMMANDQUEUE' in keys2task:
            pass
        else:              
            # ---------------------------------------------------------------------------------------------
            # (28) COMMAND QUEUE - REQUIRED                              
            # ---------------------------------------------------------------------------------------------             
            commandqueueinfo2mpu = {}
           
            message2print = "{(Step 28A/70) trying" + "; commandqueueinfo2mpu-trying-by-map2image4mpu: " + str(commandqueueinfo2mpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print)  
                        
            try:
                commandqueueinfo2mpu = get2commandqueueinfo4mpu()                                                    # SYSTEM CREATES COMMAND QUEUE, IF NECESSARY   
                message2print = "{(Step 28B/70) got" + "; commandqueueinfo2mpu-got-by-map2image4mpu: " + str(commandqueueinfo2mpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                
                code2ack  = code2ack + '28B-'  
            except:
                message2print = "{(Step 28Z/70) failed" + "; commandqueueinfo2mpu-failed-by-map2image4mpu: " + str(commandqueueinfo2mpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '28Z-'
        
            task4imagemap2mpu ['COMMANDQUEUE'] = commandqueueinfo2mpu
        
        
        if 'BUFFER' in keys2task:
            pass
        else:               
            # ---------------------------------------------------------------------------------------------
            # (29) BUFFER - AUTOMATIC BY MPUXI                               
            # ---------------------------------------------------------------------------------------------             
            bufferinfo2mpu = {}
           
            message2print = "{(Step 29A/70) trying" + "; bufferinfo2mpu-trying-by-map2image4mpu: " + str(bufferinfo2mpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                bufferinfo2mpu = get2bufferinfo4mpu()                                                                   # SYSTEM CREATES BUFFER FOR MPU, IF NECESSARY
                message2print = "{(Step 29B/70) got" + "; bufferinfo2mpu-got-by-map2image4mpu: " + str(bufferinfo2mpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                
                code2ack  = code2ack + '29B-' 
            except:
                message2print = "{(Step 29Z/70) failed" + "; bufferinfo2mpu-failed-by-map2image4mpu: " + str(bufferinfo2mpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '29Z-'
        
            task4imagemap2mpu ['BUFFER'] = bufferinfo2mpu
             
        if 'EVENT' in keys2task:
            pass
        else:               
            # ---------------------------------------------------------------------------------------------
            # (30) EVENT - REQUIRED                            
            # ---------------------------------------------------------------------------------------------             
            eventinfo2mpu = {}
           
            message2print = "{(Step 30A/70) trying" + "; eventinfo2mpu-trying-by-map2image4mpu: " + str(eventinfo2mpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                eventinfo2mpu = get2eventinfo4mpu()                                                                    
                message2print = "{(Step 30B/70) got" + "; eventinfo2mpu-got-by-map2image4mpu: " + str(eventinfo2mpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                
                code2ack  = code2ack + '30B-' 
            except:
                message2print = "{(Step 30Z/70) failed" + "; eventinfo2mpu-failed-by-map2image4mpu: " + str(eventinfo2mpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '30Z-'
        
            task4imagemap2mpu ['EVENT'] = eventinfo2mpu 
        
             
        if 'EVENTWAITLIST' in keys2task:
            pass
        else:               
            # ---------------------------------------------------------------------------------------------
            # (31) EVENTWAITLIST - REQUIRED                              
            # ---------------------------------------------------------------------------------------------             
            eventwaitlist2mpu = {}
           
            message2print = "{(Step 31A/70) trying" + "; eventwaitlist2mpu-trying-by-map2image4mpu: " + str(eventwaitlist2mpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                eventwaitlist2mpu = {}                                                                   
                message2print = "{(Step 31B/70) set" + "; eventwaitlist2mpu-set-by-map2image4mpu: " + str(eventwaitlist2mpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                
                code2ack  = code2ack + '31B-' 
            except:
                message2print = "{(Step 31Z/70) failed" + "; eventwaitlist2mpu-failed-by-map2image4mpu: " + str(eventwaitlist2mpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '31Z-'
        
            task4imagemap2mpu ['EVENTWAITLIST'] = eventwaitlist2mpu                     
              
        if 'EVENTWAITLISTQUANTITY' in keys2task:
            pass
        else:               
            # ---------------------------------------------------------------------------------------------
            # (32) EVENTWAITLISTQUANTITY - REQUIRED                             
            # ---------------------------------------------------------------------------------------------             
            eventwaitlistquantity2mpu = 0
           
            message2print = "{(Step 32A/70) trying" + "; eventwaitlistquantity2mpu-trying-by-map2image4mpu: " + str(eventwaitlistquantity2mpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                keys2event = eventwaitlist2mpu.keys()
                for key2event in keys2event:
                    eventwaitlistquantity2mpu = eventwaitlistquantity2mpu + 1                                                                   
                message2print = "{(Step 32B/70) set" + "; eventwaitlistquantity2mpu-set-by-map2image4mpu: " + str(eventwaitlistquantity2mpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                
                code2ack  = code2ack + '32B-' 
            except:
                message2print = "{(Step 32Z/70) failed" + "; eventwaitlistquantity2mpu-failed-by-map2image4mpu: " + str(eventwaitlistquantity2mpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '32Z-'
        
            task4imagemap2mpu ['EVENTWAITLISTQUANTITY'] = eventwaitlistquantity2mpu       
                                                 
                 
        if 'BLOCKING' in keys2task:
            pass
        else:              
            # ---------------------------------------------------------------------------------------------
            # (33) BLOCKING - REQUIRED                        
            # ---------------------------------------------------------------------------------------------             
            blocking2mpu = ' '                                                                   
            message2print = "{(Step 33A/70) trying" + "; blocking2mpu-trying-by-map2image4mpu: " + str(blocking2mpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                blocking2mpu = 'CL_FALSE'                                                          # the map command is non-blocking                                                              
                message2print = "{(Step 33B/70) set" + "; blocking2mpu-set-by-map2image4mpu to: " + str(blocking2mpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                code2ack  = code2ack + '33B-' 
            except:
                message2print = "{(Step 33Z/70) failed" + "; blocking2mpu-failed-by-map2image4mpu: " + str(blocking2mpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '33Z-'
        
            task4imagemap2mpu ['BLOCKING'] = blocking2mpu             
             
        if 'ORIGIN' in keys2task:
            pass
        else:                    
            # ---------------------------------------------------------------------------------------------
            # (34) ORIGIN - REQUIRED                             
            # ---------------------------------------------------------------------------------------------             
            origin2mpu = {}
            
            message2print = "{(Step 34A/70) trying" + "; origin2mpu-trying-by-map2image4mpu: " + str(origin2mpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:                                                                                    
                origin2mpu['X'] = 0 
                origin2mpu['Y'] = 0  
                origin2mpu['Z'] = 0                                                                                                                          
                message2print = "{(Step 34B/70) set" + "; origin2mpu-set-by-map2image4mpu to: " + str(origin2mpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                code2ack  = code2ack + '34B-' 
            except:
                message2print = "{(Step 34Z/70) failed" + "; origin2mpu-failed-by-map2image4mpu: " + str(origin2mpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '34Z-'
        
            task4imagemap2mpu ['ORIGIN'] = origin2mpu
            
             
        if 'REGION' in keys2task:
            pass
        else:            
            # ---------------------------------------------------------------------------------------------
            # (35) REGION - REQUIRED                         
            # ---------------------------------------------------------------------------------------------             
            region2mpu = {}
            
            message2print = "{(Step 35A/70) trying" + "; region2mpu-trying-by-map2image4mpu: " + str(region2mpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                            
            try:                                                                                    
                region2mpu['X'] = 160                                                                               # WIDTH  IN PIXELS TO MAP 
                region2mpu['Y'] = 90                                                                                # HEIGHT IN PIXELS TO MAP 
                region2mpu['Z'] = 0                                                                                 # DEPTH  IN PIXELS TO MAP                                                                                                                          
                message2print = "{(Step 35B/70) set" + "; region2mpu-set-by-map2image4mpu to: " + str(region2mpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                code2ack  = code2ack + '35B-' 
            except:
                message2print = "{(Step 35Z/70) failed" + "; region2mpu-failed-by-map2image4mpu: " + str(region2mpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '35Z-'
        
            task4imagemap2mpu ['REGION'] = region2mpu
        
            
             
        if 'ROWPITCH' in keys2task:
            pass
        else:               
            # ---------------------------------------------------------------------------------------------
            # (36) ROWPITCH (STRIDE) - REQUIRED                              
            # ---------------------------------------------------------------------------------------------             
            # One can assume that memory representing an image or screen will have data laid out as one row of pixels followed directly by the next row of pixels. 
            # However many APIs (e.g. Direct3D) that return a pointer to image data add an extra block of data at the end of each row. 
            # int offset = x * bytesPerPixel + y * pitch;
            # When a video image is stored in memory, the memory buffer might contain extra padding bytes after each row of pixels. 
            # The padding bytes affect how the image is stored in memory, but do not affect how the image is displayed.
            # The stride is the number of bytes from one row of pixels in memory to the next row of pixels in memory. 
            # Stride is also called pitch. 
            # If padding bytes are present, the stride is wider than the width of the image, as shown in the following illustration.
     
            rowpitch2mpu = 0
            size2element = 1                                                                                        # VOXEL (X,Y,Z) = SHOT IN PIXELS, R=1 PIXELS
            voxelquantity2layer = 1                                                                                 # MAX NUMBER OF VOXELS IN EACH 2D LAYER 
            quantity2direction = 1                                                                                  # ONE DIRECTIONS IN 2D
            
            message2print = "{(Step 36A/70) trying" + "; rowpitch2mpu-trying-by-map2image4mpu: " + str(rowpitch2mpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
            # 
                        
            try:                                                                                                    # X-AXEL IS THE WORST CASE TO SCAN WITH Y=0, Z=0
                rowpitch2mpu = int(width2mpu * size2element * voxelquantity2layer / quantity2direction)             # (800 * 8 * 10) / 8 = 8K PIXELS MAX TO SCAN 
                                                                                                                    # SPHERE IS SPARSE HAVING FEW SHOTS PER SCAN LINE 
                                                                                                                                         
                message2print = "{(Step 36B/70) set" + "; rowpitch2mpu-set-by-map2image4mpu to: " + str(rowpitch2mpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                #                                                                                             
                 
                code2ack  = code2ack + '36B-' 
            except:
                message2print = "{(Step 36Z/70) failed" + "; rowpitch2mpu-failed-by-map2image4mpu: " + str(rowpitch2mpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                # 
                
                code2nack  = code2nack + '36Z-'
        
            task4imagemap2mpu ['ROWPITCH'] = rowpitch2mpu     
   
        if 'SLICEPITCH' in keys2task:
            pass
        else:               
            # --------------------------------------------------------------------------------------------------------
            # (38) SLICEPITCH - REQUIRED - THE SIZE IN BYTES OF THE 2D SLICE OF THE 3D REGION OF A 3D IMAGE BEING MAP                         
            # --------------------------------------------------------------------------------------------------------             
            slicepitch2mpu = None                                                                                   # SLICEPITCH IS ZERO FOR 2D IMAGES                           
           
            message2print = "{(Step 38A/70) trying" + "; slicepitch2mpu-trying-by-map2image4mpu: " + str(slicepitch2mpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                slicepitch2mpu = 0                                                          
                message2print = "{(Step 38B/70) set" + "; slicepitch2mpu-set-by-map2image4mpu to: " + str(slicepitch2mpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                code2ack  = code2ack + '38B-'                
                 
            except:
                message2print = "{(Step 38Z/70) failed" + "; slicepitch2mpu-failed-by-map2image4mpu: " + str(slicepitch2mpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '38Z-'
            
            task4imagemap2mpu ['SLICEPITCH'] = slicepitch2mpu
   
        if 'FLAGS' in keys2task:
            pass
        else:               
            # --------------------------------------------------------------------------------------------------------
            # (39) FLAGS - REQUIRED                   
            # --------------------------------------------------------------------------------------------------------             
            flags2mpu = 'CL_MAP_READ'                                                                                                             
           
            message2print = "{(Step 39A/70) trying" + "; flags2mpu-trying-by-map2image4mpu: " + str(flags2mpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                flags2mpu = 'CL_MAP_WRITE'                                                          
                message2print = "{(Step 39B/70) set" + "; flags2mpu-set-by-map2image4mpu to: " + str(flags2mpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                code2ack  = code2ack + '39B-'                
                 
            except:
                message2print = "{(Step 39Z/70) failed" + "; flags2mpu-failed-by-map2image4mpu: " + str(flags2mpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '39Z-'
            
            task4imagemap2mpu ['FLAGS'] = flags2mpu
                                          
        # ---------------------------------------------------------------------------------------------
        # EOF TASK ANALYSIS
        # ---------------------------------------------------------------------------------------------
        keys2task = task4imagemap2mpu.keys()
        message2print = "{(Step 40A/70) set" + "; keys2task-set-by-map2image4mpu to: " + str(sorted(keys2task))  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
        # ['BLOCKING',
        #  'BUFFER',
        #  'COMMANDQUEUE',
        #  'CONTEXT',
        #  'DATA2CALLBACK',
        #  'DEVICE',
        #  'EVENT',
        #  'EVENTWAITLIST',
        #  'EVENTWAITLISTQUANTITY',
        #  'FLAGS',
        #  'FUNCTION2CALLBACK',
        #  'IMAGE',
        #  'ORIGIN',
        #  'PIPE',
        #  'PLATFORM',
        #  'QUEUE',
        #  'REGION',
        #  'ROWPITCH',
        #  'SLICEPITCH']; time: 01:28:44 10/05/11 Pacific Daylight Time}

                       
    except: 
        message2print = "{(Step 40Z/70) yielded" + "; task4imagemap2mpu-yielded-by-map2image4mpu: " + str(task4imagemap2mpu) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # 
        
        code2nack  = code2nack + '40Z-' 

    # ---------------------------------------------------------------------------------------------
    # CONTROL AND COMMAND
    # ---------------------------------------------------------------------------------------------
    control2mapimage4mpu = {}       
                                
    # ---------------------------------------------------------------------------------------------
    # CONSUMER AND PUBLISHER CHANNEL FOR CONTROL AND COMMAND
    # ---------------------------------------------------------------------------------------------
    ack4channels2mpu = False
    channels2mpu = ()
    message2print = "{(Step 40A/70) trying" + "channels2mpu-trying-by-map2image4mpu: " + str(channels2mpu)  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)     
    try:
        # 
        consumerchannel2mpu, publisherchannel2mpu = task4imagemap2mpu ['PIPE']

        message2print = "{(Step 40B/70) got" + "; consumerchannel2mpu-got-by-map2image4mpu: " + str(consumerchannel2mpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
        # <read-write PipeConnection, handle 748>

        message2print = "{(Step 40C/70) got" + "; publisherchannel2mpu-got-by-map2image4mpu: " + str(publisherchannel2mpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
        # <read-write PipeConnection, handle 752>                     
        ack4channels2mpu = True
        
        control2mapimage4mpu['CONSUMERCHANNEL']  = consumerchannel2mpu
        
        
        control2mapimage4mpu['PUBLISHERCHANNEL'] = publisherchannel2mpu
        
        code2ack  = code2ack + '40D-' 
        
    except:
        ack4channels2mpu = False
        code2nack  = code2nack + '40Z-'      
                        
    # ---------------------------------------------------------------------------------------------
    # QUEUE FOR CONTROL AND COMMAND
    # ---------------------------------------------------------------------------------------------
    ack4queue2mpu = False
    queue2mpu = None
    message2print = "{(Step 41A/70) trying" + "queue2mpu-trying-by-map2image4mpu: " + str(queue2mpu)  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)     
    try:
        queue2mpu = task4imagemap2mpu ['QUEUE']

        message2print = "{(Step 41B/70) got" + "; queue2mpu-got-by-map2image4mpu: " + str(queue2mpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
        # <queue4xi.Queue4Xi object at 0x0000000004B3FB70> 
              
        ack4queue2mpu = True
        control2mapimage4mpu['QUEUE']  = queue2mpu        
        code2ack  = code2ack + '41C-'
    except:
        ack4queue2mpu = False  
        code2nack  = code2nack + '41Z-'
    
    
    # -----------------------------------------------------------------------------------------    
    # DUMMY PROCESS OBJECT = THREAD CREATION
    # -----------------------------------------------------------------------------------------           
    # dummy2image4mpu = None
    # message2print = "{(Step 69A/70) trying" + "dummy2image4mpu-trying-by-map2image4mpu: " + str(dummy2image4mpu)  + "; time: " + time4xi() + "}"   
    # print4xi(class2print, method2print, message2print)     
    # try:                            
        # dummy2image4mpu = DummyProcess4Xi(target=manoeuvre2mapimage4mpu,                                            # in this module to handle queue and pipe channels
        #                                   args =(queue2mpu,                                                         # caller puts and gets - does NOT call any functions
        #                                          consumerchannel2mpu, publisherchannel2mpu))                        # caller sends and receives - does NOT call any functions
        # message2print = "{(Step 69B/70) created" + "dummy2image4mpu-created-by-map2image4mpu: " + str(dummy2image4mpu)  + "; time: " + time4xi() + "}" 
        # print4xi(class2print, method2print, message2print)  
        # code2ack  = code2ack + '69B-'
        
    # except:
        # code2nack  = code2nack + '69Z-'
        
    # ---------------------------------------------------------------------------------------------
    # RETURN CONTROL AND COMMAND DICTIONARY
    # ---------------------------------------------------------------------------------------------          
    message2print = "{(Step 70A/70) set" + "; code2ack-set-by-map2image4mpu: " + str(code2ack) + "; time: " + time4xi() + "}"  
    print4xi(class2print, method2print, message2print)                                                             
    # 
    message2print = "{(Step 70B/70) set" + "; code2nack-set-by-map2image4mpu: " + str(code2nack) + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                             
    # 1B1-1B2-1C-2B-3B-4B-5B-6B-7B-8B-9B-10B-11B-12B-13B-14B-15B-16B-18B-19B-40D-41C- 
        
    message2print = "{(Step 70C/70) returning" + "; control2mapimage4mpu-returning-by-map2image4mpu: " + str(control2mapimage4mpu) + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                             
    # {'QUEUE': <queue4xi.Queue4Xi object at 0x0000000004B3FB70>,
    #  'PUBLISHERCHANNEL': <read-write PipeConnection, handle 752>,
    #  'CONSUMERCHANNEL': <read-write PipeConnection, handle 748>}; time: 01:28:44 10/05/11 Pacific Daylight Time}

        
    return control2mapimage4mpu
                    
#</function4xi> 

# -----------------------------------------------------------------------------------------------------------------------
#<function4xi function = "manoeuvre2copyimage4mpu" usage = "
#   # ---------------------------------------------------------------------------------------------------------------------
#   # CALL MANOEUVRE2COPYIMAGE4MPU - LET IT TO RECEIVE COPY MESSAGE AND TO SEND ROGER MESSAGE VIA PIPE                
#   # ---------------------------------------------------------------------------------------------------------------------       
#   if ack2simulate4mpu is True:
#       print("(Step 42A/700) OF MPU2IMAGE4XI5TEST - TRYING TO PERFORM TARGET FUNCTION MANOEUVRE2COPYIMAGE4MPU ", time4xi())   
#       try:        
#           response2manoeuvre4copy = manoeuvre2copyimage4mpu(queue2copy4mpu, channel2copyconsume4mpu, channel2copypublish4mpu)
#           print("(Step 42B/700) OF MPU2IMAGE4XI5TEST - PERFORMED MANOEUVRE2COPYIMAGE4MPU, RESPONSE2MANOEUVRE4COPY: ", response2manoeuvre4copy)
#           # {'MESSAGE2SEND4MPU': '<roger4xi>roger2copy4mpu</roger4xi>', 'ACK2INFO4MPU': False, 'MESSAGE2GET4MPU': '<default4xi>default2copy4mpu</default4xi>', 
#           #  'MESSAGE2RECEIVE4MPU': '<copy4xi>copy2copy4mpu</copy4xi>', 
#           'CODE2ACK4MPU': ' 0D-1C-', 'MESSAGE2PUT4MPU': '<default4xi>default2copy4mpu</default4xi>', 'CODE2NACK4MPU': ' ', 'STATE2MANOEUVRE4MPU': 1}
#        except:
#           print("(Step 42Z/700) OF MPU2IMAGE4XI5TEST *** FAILED TO PERFORM TARGET FUNCTION MANOEUVRE2COPYIMAGE4MPU ", time4xi())     
#   else:
#       pass # trust activated thread
#">            

# ----------------------------------------------------------------------------------------------------------------------- 
def manoeuvre2copyimage4mpu(queue2copymanoeuvre4mpu, consumerchannel2copymanoeuvre4mpu, publisherchannel2copymanoeuvre4mpu):
    '''
    manoeuvres copy control and command
    '''
    global state2copyimage4mpu                                                                                           # STATE FOR COMMAND AND CONTROL
    global max2copyimagestate4mpu                                                                                        # MAX FOR STATE
    global ack2copyimageinfo4mpu                                                                                         # ACK FOR IMAGE INFO
    global message2copyimageget4mpu                                 
    global message2copyimageput4mpu
    global message2copyimagereceive4mpu
    global message2copyimagesend4mpu
    
    # -----------------------------------------------------------------------------------------
    # (Step 0/33) PARAMETERS            
    # -----------------------------------------------------------------------------------------          
    class2print  = "manoeuvre2copyimage4mpu" 
    method2print = "manoeuvre2copyimage4mpu" 

    code2ack  = ' '
    code2nack = ' '
    
    message2print = "{(Step 0A/33) started" + "; manoeuvre2copyimage4mpu-started: " + str(manoeuvre2copyimage4mpu)  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                                     #  
    # <function manoeuvre2copyimage4mpu at 0x0000000003DB8C48>  
    
    message2print = "{(Step 0B/33) passed" + "; queue2copymanoeuvre4mpu-to-manoeuvre2copyimage4mpu: " + str(queue2copymanoeuvre4mpu) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                                                              
    # <queue4xi.Queue4Xi object at 0x0000000003AD9400>
    
    message2print = "{(Step 0C/33) passed" + "; consumerchannel2copymanoeuvre4mpu-to-manoeuvre2copyimage4mpu: " + str(consumerchannel2copymanoeuvre4mpu) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                                                              
    # <copy-write PipeConnection, handle 408>
    
    message2print = "{(Step 0D/33) passed" + "; publisherchannel2copymanoeuvre4mpu-to-manoeuvre2copyimage4mpu: " + str(publisherchannel2copymanoeuvre4mpu) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                                                              
    # <copy-write PipeConnection, handle 412>     
    
    code2ack  = code2ack + '0D-' 
    response2manouvre4copy = {}
                    
    # ------------------------------------------------------------------------------------------------------------------------
    # SETTINGS
    # ------------------------------------------------------------------------------------------------------------------------    
    message2print = "{(Step 0E/70) try" + "; state2copyimage4mpu-try-by-manoeuvre2copyimage4mpu"  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)     
    try:
        if state2copyimage4mpu is None:
            state2copyimage4mpu = 0
            message2print = "{(Step 0F/33) initialized" + "; state2copyimage4mpu-initialized-by-manoeuvre2copyimage4mpu to: " + str(state2copyimage4mpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)    
        else:
            message2print = "{(Step 0G/33) used" + "; state2copyimage4mpu-used-by-manoeuvre2copyimage4mpu: " + str(state2copyimage4mpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # 1; 2; 3;  
    except:
        state2copyimage4mpu = 0
        message2print = "{(Step 0Z1/33) initialized" + "; state2copyimage4mpu-initialized-by-manoeuvre2copyimage4mpu to: " + str(state2copyimage4mpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # 0
        ack2copyimageinfo4mpu        = False
        message2print = "{(Step 0Z2/33) initialized" + "; ack2copyimageinfo4mpu-initialized-by-manoeuvre2copyimage4mpu to: " + str(ack2copyimageinfo4mpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                     
        # False
        message2copyimageget4mpu     = '<default4xi>default2copy4mpu</default4xi>'
        message2print = "{(Step 0Z3/33) initialized" + "; message2copyimageget4mpu-initialized-by-manoeuvre2copyimage4mpu to: " + str(message2copyimageget4mpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                               
        # <default4xi>default2copy4mpu</default4xi>
        message2copyimageput4mpu     = '<default4xi>default2copy4mpu</default4xi>'
        message2print = "{(Step 0Z4/33) initialized" + "; message2copyimageput4mpu-initialized-by-manoeuvre2copyimage4mpu to: " + str(message2copyimageput4mpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
        # <default4xi>default2copy4mpu</default4xi>
        message2copyimagereceive4mpu = '<default4xi>default2copy4mpu</default4xi>'
        message2print = "{(Step 0Z5/33) initialized" + "; message2copyimagereceive4mpu-initialized-by-manoeuvre2copyimage4mpu to: " + str(message2copyimagereceive4mpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
        # <default4xi>default2copy4mpu</default4xi>
        message2copyimagesend4mpu    = '<default4xi>default2copy4mpu</default4xi>'
        message2print = "{(Step 0Z6/33) initialized" + "; message2copyimagesend4mpu-initialized-by-manoeuvre2copyimage4mpu to: " + str(message2copyimagesend4mpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
        # <default4xi>default2copy4mpu</default4xi>
        max2copyimagestate4mpu = 3
        message2print = "{(Step 0Z7/33) initialized" + ";  max2copyimagestate4mpu-initialized-by-manoeuvre2copyimage4mpu to: " + str( max2copyimagestate4mpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # 
                    
    # ------------------------------------------------------------------------------------------------------------------------
    # STATE UPDATE
    # ------------------------------------------------------------------------------------------------------------------------                      
    state2copyimage4mpu = state2copyimage4mpu + 1                                                                   
    message2print = "{(Step 0H/33) added" + "; state2copyimage4mpu-added-by-manoeuvre2copyimage4mpu: " + str(state2copyimage4mpu) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)
    # 1; 2; 3; 4  
    if state2copyimage4mpu > max2copyimagestate4mpu:
        state2copyimage4mpu = 0
        message2print = "{(Step 0I/33) set" + "; state2copyimage4mpu-set-by-manoeuvre2copyimage4mpu to: " + str(state2copyimage4mpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # 0
        ack2copyimageinfo4mpu        = False
        message2print = "{(Step 0J/33) set" + "; ack2copyimageinfo4mpu-set-by-manoeuvre2copyimage4mpu to: " + str(ack2copyimageinfo4mpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                     
        # False
        message2copyimageget4mpu     = '<default4xi>default2copy4mpu</default4xi>'
        message2print = "{(Step 0K/33) set" + "; message2copyimageget4mpu-set-by-manoeuvre2copyimage4mpu to: " + str(message2copyimageget4mpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                               
        # <default4xi>default2copy4mpu</default4xi>
        message2copyimageput4mpu     = '<default4xi>default2copy4mpu</default4xi>'
        message2print = "{(Step 0L/33) set" + "; message2copyimageput4mpu-set-by-manoeuvre2copyimage4mpu to: " + str(message2copyimageput4mpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
        # <default4xi>default2copy4mpu</default4xi>
        message2copyimagereceive4mpu = '<default4xi>default2copy4mpu</default4xi>'
        message2print = "{(Step 0M/33) set" + "; message2copyimagereceive4mpu-set-by-manoeuvre2copyimage4mpu to: " + str(message2copyimagereceive4mpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
        # <default4xi>default2copy4mpu</default4xi>
        message2copyimagesend4mpu    = '<default4xi>default2copy4mpu</default4xi>'
        message2print = "{(Step 0N/33) set" + "; message2copyimagesend4mpu-set-by-manoeuvre2copyimage4mpu to: " + str(message2copyimagesend4mpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
        # <default4xi>default2copy4mpu</default4xi>
        max2copyimagestate4mpu = 3
        message2print = "{(Step 0O/33) initialized" + ";  max2copyimagestate4mpu-set-by-manoeuvre2copyimage4mpu to: " + str( max2copyimagestate4mpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        #     
                    
    # ------------------------------------------------------------------------------------------------------------------------
    # STATE I - RECEIVE CONTROL AND COMMAND COPY MESSAGE VIA PIPE AND SEND ROGER OR NEGATIVE BACK 
    # ------------------------------------------------------------------------------------------------------------------------
    if state2copyimage4mpu == 1:
        message2print = "{(Step 1A/70) try" + "; receive-try-by-manoeuvre2copyimage4mpu"  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        message2copyimagereceive4mpu = ''
        try:
            # ----------------------------------------------------------------------------------------------------------------
            # RECEIVE CONTROL AND COMMAND MESSAGE VIA PIPE
            # ----------------------------------------------------------------------------------------------------------------         
            message2copyimagereceive4mpu = consumerchannel2copymanoeuvre4mpu.recv()
            message2print = "{(Step 1B/33) received" + "; message2copyimagereceive4mpu-received-by-manoeuvre2copyimage4mpu: " + str(message2copyimagereceive4mpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            # <copy4xi>copy2copy4mpu</copy4xi>
                         
            if message2copyimagereceive4mpu == '<copy4xi>copy2copy4mpu</copy4xi>':
                message2copyimagesend4mpu = '<roger4xi>roger2copy4mpu</roger4xi>'
            else:
                message2copyimagesend4mpu = '<negative4xi><negative2copy4mpu</<negative4xi>'
            # ----------------------------------------------------------------------------------------------------------------
            # SEND ROGER OR NEGATIVE BACK 
            # ----------------------------------------------------------------------------------------------------------------                                
            publisherchannel2copymanoeuvre4mpu.send(message2copyimagesend4mpu)
            
            message2print = "{(Step 1C/33) sent" + "; message2copyimagesend4mpu-sent-by-manoeuvre2copyimage4mpu: " + str(message2copyimagesend4mpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)          
            # <roger4xi>roger2copy4mpu</roger4xi>
            
            code2ack  = code2ack + '1C-'
        except:
            message2print = "{(Step 1Z/33) failed" + "; message2copyimagereceive4mpu-failed-by-manoeuvre2copyimage4mpu: " + str(message2copyimagereceive4mpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '1Z-' 
    else:
        pass

                    
    # ------------------------------------------------------------------------------------------------------------------------
    # STATE II - RECEIVE CONTROL AND COMMAND READY MESSAGE VIA PIPE AND SEND READY OR NEGATIVE BACK 
    # ------------------------------------------------------------------------------------------------------------------------
    if state2copyimage4mpu == 2:
        message2print = "{(Step 2A/70) try" + "; receive-try-by-manoeuvre2copyimage4mpu"  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        message2copyimagereceive4mpu = ''
        try:
            # ----------------------------------------------------------------------------------------------------------------
            # RECEIVE CONTROL AND COMMAND MESSAGE VIA PIPE
            # ----------------------------------------------------------------------------------------------------------------         
            message2copyimagereceive4mpu = consumerchannel2copymanoeuvre4mpu.recv()
            message2print = "{(Step 2B/33) received" + "; message2copyimagereceive4mpu-received-by-manoeuvre2copyimage4mpu: " + str(message2copyimagereceive4mpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            # <ready4xi>ready2copy4mpu</ready4xi>
                         
            if message2copyimagereceive4mpu == '<ready4xi>ready2copy4mpu</ready4xi>':
                message2copyimagesend4mpu = '<ready4xi>ready2copy4mpu</ready4xi>'
            else:
                message2copyimagesend4mpu = '<negative4xi><negative2copy4mpu</<negative4xi>'
            # ----------------------------------------------------------------------------------------------------------------
            # SEND READY OR NEGATIVE BACK 
            # ----------------------------------------------------------------------------------------------------------------                                
            publisherchannel2copymanoeuvre4mpu.send(message2copyimagesend4mpu)
            
            message2print = "{(Step 2C/33) sent" + "; message2copyimagesend4mpu-sent-by-manoeuvre2copyimage4mpu: " + str(message2copyimagesend4mpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)          
            # 
            
            code2ack  = code2ack + '2C-'
        except:
            message2print = "{(Step 2Z/33) failed" + "; message2copyimagereceive4mpu-failed-by-manoeuvre2copyimage4mpu: " + str(message2copyimagereceive4mpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '2Z-' 
    else:
        pass        
                
    # ------------------------------------------------------------------------------------------------------------------------
    # STATE III - GET IMAGE INFORMATION SUCH AS IMAGE PATH VIA QUEUE AND SEND ACK OR NACK VIA PIPE
    # ------------------------------------------------------------------------------------------------------------------------
    if state2copyimage4mpu == 3:    
        info2image4mpu = None
        message2print = "{(Step 3A/70) try" + "; info2image4mpu-try-by-manoeuvre2copyimage4mpu"  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        try: 
            # ----------------------------------------------------------------------------------------------------------------
            # GET IMAGE INFORMATION SUCH AS IMAGE PATH VIA QUEUE
            # ----------------------------------------------------------------------------------------------------------------        
            info2image4mpu = queue2copymanoeuvre4mpu.get()
            
            message2print = "{(Step 3B/33) got" + "; info2image4mpu-got-by-manoeuvre2copyimage4mpu: " + str(info2image4mpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            # ..//..//..//..//..//web//images//png//dragonxi.png
            if info2image4mpu is None:
                code2nack  = code2nack + '3Z1-'
                ack2copyimageinfo4mpu = False 
            else:    
                code2ack  = code2ack + '3C-'
                ack2copyimageinfo4mpu = True
        except:
            message2print = "{(Step 3Z1/33) failed" + "; queue2copymanoeuvre4mpu-failed-by-manoeuvre2copyimage4mpu: " + str(queue2copymanoeuvre4mpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '3Z1-'
            
        message2copyimagesend4mpu = ' '
        try:
            # ----------------------------------------------------------------------------------------------------------------
            # SEND CONTROL INFO ABOUT IMAGE OPERATION - EITHER ACK OR NACK  
            # ----------------------------------------------------------------------------------------------------------------         
            if ack2copyimageinfo4mpu is True:
                message2copyimagesend4mpu = '<ack4xi>ack2imageinfo4mpu</ack4xi>'
            else:
                message2copyimagesend4mpu = '<nack4xi>nack2imageinfo4mpu</nack4xi>'
            message2print = "{(Step 4B/33) set" + "; message2copyimagesend4mpu-set-by-manoeuvre2copyimage4mpu to: " + str(message2copyimagesend4mpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # <ack4xi>ack2imageinfo4mpu</ack4xi>       
        except:
            message2print = "{(Step 4Z1/33) failed" + "; message2copyimagesend4mpu-failed-by-manoeuvre2copyimage4mpu: " + str(message2copyimagesend4mpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            
            code2nack  = code2nack + '4Z1-'    
        try:                
            message2print = "{(Step 4C/70) using" + "; publisherchannel2copymanoeuvre4mpu-calling-by-manoeuvre2copyimage4mpu: " + str(publisherchannel2copymanoeuvre4mpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)    
            # <copy-write PipeConnection, handle 412>
            message2print = "{{(Step 4D/70) passing" + "; message2copyimagesend4mpu-passing-to-send-by-manoeuvre2copyimage4mpu: " + str(message2copyimagesend4mpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #              
        except:
            message2print = "{(Step 4Z1/33) failed" + "; message2copyimagesend4mpu-failed-by-manoeuvre2copyimage4mpu: " + str(message2copyimagesend4mpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '4Z2-'                
        try:    
            publisherchannel2copymanoeuvre4mpu.send(message2copyimagesend4mpu)
            
            message2print = "{(Step 4E/33) sent" + "; message2copyimagesend4mpu-sent-by-manoeuvre2copyimage4mpu: " + str(message2copyimagesend4mpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)          
            # <ack4xi>ack2imageinfo4mpu</ack4xi>            
            code2ack  = code2ack + '4E-'
        except:
            message2print = "{(Step 4Z3/33) failed" + "; message2copyimagesend4mpu-failed-by-manoeuvre2copyimage4mpu: " + str(message2copyimagesend4mpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '4Z3-'                
    else:
        pass
        

            
    # ------------------------------------------------------------------------------------------------------------------------
    # RETURN 
    # ------------------------------------------------------------------------------------------------------------------------
    response2manouvre4copy['ACK2INFO4MPU']        = ack2copyimageinfo4mpu
    response2manouvre4copy['CODE2ACK4MPU']        = code2ack
    response2manouvre4copy['CODE2NACK4MPU']       = code2nack
    response2manouvre4copy['MESSAGE2GET4MPU']     = message2copyimageget4mpu
    response2manouvre4copy['MESSAGE2PUT4MPU']     = message2copyimageput4mpu
    response2manouvre4copy['MESSAGE2RECEIVE4MPU'] = message2copyimagereceive4mpu
    response2manouvre4copy['MESSAGE2SEND4MPU']    = message2copyimagesend4mpu
    response2manouvre4copy['STATE2MANOEUVRE4MPU'] = state2copyimage4mpu
                          
    message2print = "{(Step 33/33) returning" + "; response2manouvre4copy-returning-by-manoeuvre2copyimage4mpu: " + str(response2manouvre4copy) + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                             
    #1 {'MESSAGE2SEND4MPU': '<roger4xi>roger2copy4mpu</roger4xi>',
    #   'ACK2INFO4MPU': False,
    #   'MESSAGE2GET4MPU': '<default4xi>default2copy4mpu</default4xi>',
    #   'MESSAGE2RECEIVE4MPU': '<copy4xi>copy2copy4mpu</copy4xi>',
    #   'CODE2ACK4MPU': ' 0D-1C-',
    #   'MESSAGE2PUT4MPU': '<default4xi>default2copy4mpu</default4xi>',
    #   'CODE2NACK4MPU': ' ',
    #   'STATE2MANOEUVRE4MPU': 1}; time: 05:25:15 10/03/11 Pacific Daylight Time}

 
    #2 {'MESSAGE2SEND4MPU': '<ready4xi>ready2copy4mpu</ready4xi>',
    #   'ACK2INFO4MPU': False,
    #   'MESSAGE2GET4MPU': '<default4xi>default2copy4mpu</default4xi>',
    #   'MESSAGE2RECEIVE4MPU': '<ready4xi>ready2copy4mpu</ready4xi>',
    #   'CODE2ACK4MPU': ' 0D-2C-',
    #   'MESSAGE2PUT4MPU': '<default4xi>default2copy4mpu</default4xi>',
    #   'CODE2NACK4MPU': ' ',
    #   'STATE2MANOEUVRE4MPU': 2}; time: 05:25:15 10/03/11 Pacific Daylight Time}

   
    #3 {'MESSAGE2SEND4MPU': '<ack4xi>ack2imageinfo4mpu</ack4xi>',
    #   'ACK2INFO4MPU': True,
    #   'MESSAGE2GET4MPU': '<default4xi>default2copy4mpu</default4xi>',
    #   'MESSAGE2RECEIVE4MPU': '<ready4xi>ready2copy4mpu</ready4xi>',
    #   'CODE2ACK4MPU': ' 0D-3C-4E-',
    #   'MESSAGE2PUT4MPU': '<default4xi>default2copy4mpu</default4xi>',
    #   'CODE2NACK4MPU': ' ', 'STATE2MANOEUVRE4MPU': 3}; time: 05:25:15 10/03/11 Pacific Daylight Time}

    
    #4 {'MESSAGE2SEND4MPU': '<default4xi>default2copy4mpu</default4xi>',
    #   'ACK2INFO4MPU': False,
    #   'MESSAGE2GET4MPU': '<default4xi>default2copy4mpu</default4xi>',
    #   'MESSAGE2RECEIVE4MPU': '<default4xi>default2copy4mpu</default4xi>',
    #   'CODE2ACK4MPU': ' 0D-',
    #   'MESSAGE2PUT4MPU': '<default4xi>default2copy4mpu</default4xi>',
    #   'CODE2NACK4MPU': ' ', 'STATE2MANOEUVRE4MPU': 0}; time: 05:25:15 10/03/11 Pacific Daylight Time}

    return response2manouvre4copy           
          
#</function4xi>


# -----------------------------------------------------------------------------------------------------------------------
#<function4xi function = "manoeuvre2copyimagetobuffer4mpu" usage = "
#   # ---------------------------------------------------------------------------------------------------------------------
#   # CALL MANOEUVRE2COPYIMAGE4MPU  - LET IT TO RECEIVE COPY AND TO SEND ROGER VIA PIPE                
#   # ---------------------------------------------------------------------------------------------------------------------       
#   if ack2simulate4mpu is True:
#       print("(Step 62A/700) OF MPU2IMAGE4XI5TEST - TRYING TO PERFORM TARGET FUNCTION MANOEUVRE2COPYIMAGE4MPU ", time4xi())   
#       try:        
#           response2manoeuvre4copy = manoeuvre2copyimagetobuffer4mpu(queue4copy2imagetobuffer4mpu, channel4copy2imagetobufferconsume4mpu, channel4copy2imagetobufferpublish4mpu)
#           print("(Step 62B/700) OF MPU2IMAGE4XI5TEST - PERFORMED MANOEUVRE2COPYIMAGE4MPU, RESPONSE2MANOEUVRE4COPY: ", response2manoeuvre4copy)
#           # { 'MESSAGE2SEND4MPU': '<roger4xi>roger2copyimagetobuffer4mpu</roger4xi>',
#           #   'ACK2INFO4MPU': False,
#           #   'MESSAGE2GET4MPU': '<default4xi>default2copyimagetobuffer4mpu</default4xi>',
#           #   'MESSAGE2RECEIVE4MPU': '<copy4xi>copy2copyimagetobuffer4mpu</copy4xi>',
#           #   'CODE2ACK4MPU': ' 0D-1C-',
#           #   'MESSAGE2PUT4MPU': '<default4xi>default2copyimagetobuffer4mpu</default4xi>',
#           #   'CODE2NACK4MPU': ' ',
#           #   'STATE2MANOEUVRE4MPU': 1}
#        except:
#           print("(Step 62Z/700) OF MPU2IMAGE4XI5TEST *** FAILED TO PERFORM TARGET FUNCTION MANOEUVRE2COPYIMAGE4MPU ", time4xi())     
#   else:
#       pass # trust activated thread    
#">

# ----------------------------------------------------------------------------------------------------------------------- 
def manoeuvre2copyimagetobuffer4mpu(queue2copyimagetobuffermanoeuvre4mpu, consumerchannel2copyimagetobuffermanoeuvre4mpu, publisherchannel2copyimagetobuffermanoeuvre4mpu):
    '''
    manoeuvres copy control and command for copy2imagetobuffer4mpu
    '''
    global state2copyimagetobuffer4mpu                                                                                           # STATE FOR COMMAND AND CONTROL
    global max2copyimagetobufferstate4mpu                                                                                        # MAX FOR STATE
    global ack2copyimagetobufferinfo4mpu                                                                                         # ACK FOR IMAGE INFO
    global message2copyimagetobufferget4mpu                                 
    global message2copyimagetobufferput4mpu
    global message2copyimagetobufferreceive4mpu
    global message2copyimagetobuffersend4mpu
    
    # -----------------------------------------------------------------------------------------
    # (Step 0/33) PARAMETERS            
    # -----------------------------------------------------------------------------------------          
    class2print  = "manoeuvre2copyimagetobuffer4mpu" 
    method2print = "manoeuvre2copyimagetobuffer4mpu" 

    code2ack  = ' '
    code2nack = ' '
    
    message2print = "{(Step 0A/33) started" + "; manoeuvre2copyimagetobuffer4mpu-started: " + str(manoeuvre2copyimagetobuffer4mpu)  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                                     #  
    # 
    
    message2print = "{(Step 0B/33) passed" + "; queue2copyimagetobuffermanoeuvre4mpu-to-manoeuvre2copyimagetobuffer4mpu: " + str(queue2copyimagetobuffermanoeuvre4mpu) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                                                              
    # 
    
    message2print = "{(Step 0C/33) passed" + "; consumerchannel2copyimagetobuffermanoeuvre4mpu-to-manoeuvre2copyimagetobuffer4mpu: " + str(consumerchannel2copyimagetobuffermanoeuvre4mpu) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                                                              
    # 
    
    message2print = "{(Step 0D/33) passed" + "; publisherchannel2copyimagetobuffermanoeuvre4mpu-to-manoeuvre2copyimagetobuffer4mpu: " + str(publisherchannel2copyimagetobuffermanoeuvre4mpu) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                                                              
    # 
    
    code2ack  = code2ack + '0D-' 
    response2manouvre4copyimagetobuffer = {}
                    
    # ------------------------------------------------------------------------------------------------------------------------
    # SETTINGS
    # ------------------------------------------------------------------------------------------------------------------------    
    message2print = "{(Step 0E/70) try" + "; state2copyimagetobuffer4mpu-try-by-manoeuvre2copyimagetobuffer4mpu"  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)     
    try:
        if state2copyimagetobuffer4mpu is None:
            state2copyimagetobuffer4mpu = 0
            message2print = "{(Step 0F/33) initialized" + "; state2copyimagetobuffer4mpu-initialized-by-manoeuvre2copyimagetobuffer4mpu to: " + str(state2copyimagetobuffer4mpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)    
        else:
            message2print = "{(Step 0G/33) used" + "; state2copyimagetobuffer4mpu-used-by-manoeuvre2copyimagetobuffer4mpu: " + str(state2copyimagetobuffer4mpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # 1; 2; 3;  
    except:
        state2copyimagetobuffer4mpu = 0
        message2print = "{(Step 0Z1/33) initialized" + "; state2copyimagetobuffer4mpu-initialized-by-manoeuvre2copyimagetobuffer4mpu to: " + str(state2copyimagetobuffer4mpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # 0
        ack2copyimagetobufferinfo4mpu        = False
        message2print = "{(Step 0Z2/33) initialized" + "; ack2copyimagetobufferinfo4mpu-initialized-by-manoeuvre2copyimagetobuffer4mpu to: " + str(ack2copyimagetobufferinfo4mpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                     
        # False
        message2copyimagetobufferget4mpu     = '<default4xi>default2copyimagetobuffer4mpu</default4xi>'
        message2print = "{(Step 0Z3/33) initialized" + "; message2copyimagetobufferget4mpu-initialized-by-manoeuvre2copyimagetobuffer4mpu to: " + str(message2copyimagetobufferget4mpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                               
        # 
        message2copyimagetobufferput4mpu     = '<default4xi>default2copyimagetobuffer4mpu</default4xi>'
        message2print = "{(Step 0Z4/33) initialized" + "; message2copyimagetobufferput4mpu-initialized-by-manoeuvre2copyimagetobuffer4mpu to: " + str(message2copyimagetobufferput4mpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
        # 
        message2copyimagetobufferreceive4mpu = '<default4xi>default2copyimagetobuffer4mpu</default4xi>'
        message2print = "{(Step 0Z5/33) initialized" + "; message2copyimagetobufferreceive4mpu-initialized-by-manoeuvre2copyimagetobuffer4mpu to: " + str(message2copyimagetobufferreceive4mpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
        # 
        message2copyimagetobuffersend4mpu    = '<default4xi>default2copyimagetobuffer4mpu</default4xi>'
        message2print = "{(Step 0Z6/33) initialized" + "; message2copyimagetobuffersend4mpu-initialized-by-manoeuvre2copyimagetobuffer4mpu to: " + str(message2copyimagetobuffersend4mpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
        # 
        max2copyimagetobufferstate4mpu = 3
        message2print = "{(Step 0Z7/33) initialized" + ";  max2copyimagetobufferstate4mpu-initialized-by-manoeuvre2copyimagetobuffer4mpu to: " + str( max2copyimagetobufferstate4mpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # 
                    
    # ------------------------------------------------------------------------------------------------------------------------
    # STATE UPDATE
    # ------------------------------------------------------------------------------------------------------------------------                      
    state2copyimagetobuffer4mpu = state2copyimagetobuffer4mpu + 1                                                                   
    message2print = "{(Step 0H/33) added" + "; state2copyimagetobuffer4mpu-added-by-manoeuvre2copyimagetobuffer4mpu: " + str(state2copyimagetobuffer4mpu) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)
    # 1; 2; 3; 4 
     
    if state2copyimagetobuffer4mpu > max2copyimagetobufferstate4mpu:
        state2copyimagetobuffer4mpu = 0
        message2print = "{(Step 0I/33) set" + "; state2copyimagetobuffer4mpu-set-by-manoeuvre2copyimagetobuffer4mpu to: " + str(state2copyimagetobuffer4mpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # 
        ack2copyimagetobufferinfo4mpu        = False
        message2print = "{(Step 0J/33) set" + "; ack2copyimagetobufferinfo4mpu-set-by-manoeuvre2copyimagetobuffer4mpu to: " + str(ack2copyimagetobufferinfo4mpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                     
        # 
        message2copyimagetobufferget4mpu     = '<default4xi>default2copyimagetobuffer4mpu</default4xi>'
        message2print = "{(Step 0K/33) set" + "; message2copyimagetobufferget4mpu-set-by-manoeuvre2copyimagetobuffer4mpu to: " + str(message2copyimagetobufferget4mpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                               
        # 
        message2copyimagetobufferput4mpu     = '<default4xi>default2copyimagetobuffer4mpu</default4xi>'
        message2print = "{(Step 0L/33) set" + "; message2copyimagetobufferput4mpu-set-by-manoeuvre2copyimagetobuffer4mpu to: " + str(message2copyimagetobufferput4mpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
        # 
        message2copyimagetobufferreceive4mpu = '<default4xi>default2copyimagetobuffer4mpu</default4xi>'
        message2print = "{(Step 0M/33) set" + "; message2copyimagetobufferreceive4mpu-set-by-manoeuvre2copyimagetobuffer4mpu to: " + str(message2copyimagetobufferreceive4mpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
        # 
        message2copyimagetobuffersend4mpu    = '<default4xi>default2copyimagetobuffer4mpu</default4xi>'
        message2print = "{(Step 0N/33) set" + "; message2copyimagetobuffersend4mpu-set-by-manoeuvre2copyimagetobuffer4mpu to: " + str(message2copyimagetobuffersend4mpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
        # 
        max2copyimagetobufferstate4mpu = 3
        message2print = "{(Step 0O/33) initialized" + ";  max2copyimagetobufferstate4mpu-set-by-manoeuvre2copyimagetobuffer4mpu to: " + str( max2copyimagetobufferstate4mpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        #     
                    
    # ------------------------------------------------------------------------------------------------------------------------
    # STATE I - RECEIVE CONTROL AND COMMAND COPY MESSAGE VIA PIPE AND SEND ROGER OR NEGATIVE BACK 
    # ------------------------------------------------------------------------------------------------------------------------
    if state2copyimagetobuffer4mpu == 1:
        message2print = "{(Step 1A/70) try" + "; receive-try-by-manoeuvre2copyimagetobuffer4mpu"  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        message2copyimagetobufferreceive4mpu = ''
        try:
            # ----------------------------------------------------------------------------------------------------------------
            # RECEIVE CONTROL AND COMMAND MESSAGE VIA PIPE
            # ----------------------------------------------------------------------------------------------------------------         
            message2copyimagetobufferreceive4mpu = consumerchannel2copyimagetobuffermanoeuvre4mpu.recv()
            message2print = "{(Step 1B/33) received" + "; message2copyimagetobufferreceive4mpu-received-by-manoeuvre2copyimagetobuffer4mpu: " + str(message2copyimagetobufferreceive4mpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            # 
                         
            if message2copyimagetobufferreceive4mpu == '<copy4xi>copy2copyimagetobuffer4mpu</copy4xi>': 
                message2copyimagetobuffersend4mpu = '<roger4xi>roger2copyimagetobuffer4mpu</roger4xi>'
            else:
                message2copyimagetobuffersend4mpu = '<negative4xi><negative2copyimagetobuffer4mpu</<negative4xi>'
            # ----------------------------------------------------------------------------------------------------------------
            # SEND ROGER OR NEGATIVE BACK 
            # ----------------------------------------------------------------------------------------------------------------                                
            publisherchannel2copyimagetobuffermanoeuvre4mpu.send(message2copyimagetobuffersend4mpu)
            
            message2print = "{(Step 1C/33) sent" + "; message2copyimagetobuffersend4mpu-sent-by-manoeuvre2copyimagetobuffer4mpu: " + str(message2copyimagetobuffersend4mpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)          
            # 
            
            code2ack  = code2ack + '1C-'
        except:
            message2print = "{(Step 1Z/33) failed" + "; message2copyimagetobufferreceive4mpu-failed-by-manoeuvre2copyimagetobuffer4mpu: " + str(message2copyimagetobufferreceive4mpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '1Z-' 
    else:
        pass

                    
    # ------------------------------------------------------------------------------------------------------------------------
    # STATE II - RECEIVE CONTROL AND COMMAND READY MESSAGE VIA PIPE AND SEND READY OR NEGATIVE BACK 
    # ------------------------------------------------------------------------------------------------------------------------
    if state2copyimagetobuffer4mpu == 2:
        message2print = "{(Step 2A/70) try" + "; receive-try-by-manoeuvre2copyimagetobuffer4mpu"  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        message2copyimagetobufferreceive4mpu = ''
        try:
            # ----------------------------------------------------------------------------------------------------------------
            # RECEIVE CONTROL AND COMMAND MESSAGE VIA PIPE
            # ----------------------------------------------------------------------------------------------------------------         
            message2copyimagetobufferreceive4mpu = consumerchannel2copyimagetobuffermanoeuvre4mpu.recv()
            message2print = "{(Step 2B/33) received" + "; message2copyimagetobufferreceive4mpu-received-by-manoeuvre2copyimagetobuffer4mpu: " + str(message2copyimagetobufferreceive4mpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            # 
                         
            if message2copyimagetobufferreceive4mpu == '<ready4xi>ready2copyimagetobuffer4mpu</ready4xi>':
                message2copyimagetobuffersend4mpu = '<ready4xi>ready2copyimagetobuffer4mpu</ready4xi>'
            else:
                message2copyimagetobuffersend4mpu = '<negative4xi><negative2copyimagetobuffer4mpu</<negative4xi>'
            # ----------------------------------------------------------------------------------------------------------------
            # SEND READY OR NEGATIVE BACK 
            # ----------------------------------------------------------------------------------------------------------------                                
            publisherchannel2copyimagetobuffermanoeuvre4mpu.send(message2copyimagetobuffersend4mpu)
            
            message2print = "{(Step 2C/33) sent" + "; message2copyimagetobuffersend4mpu-sent-by-manoeuvre2copyimagetobuffer4mpu: " + str(message2copyimagetobuffersend4mpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)          
            # 
            
            code2ack  = code2ack + '2C-'
        except:
            message2print = "{(Step 2Z/33) failed" + "; message2copyimagetobufferreceive4mpu-failed-by-manoeuvre2copyimagetobuffer4mpu: " + str(message2copyimagetobufferreceive4mpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '2Z-' 
    else:
        pass        
                
    # ------------------------------------------------------------------------------------------------------------------------
    # STATE III - GET IMAGE INFORMATION SUCH AS IMAGE PATH VIA QUEUE AND SEND ACK OR NACK VIA PIPE
    # ------------------------------------------------------------------------------------------------------------------------
    if state2copyimagetobuffer4mpu == 3:    
        info2image4mpu = None
        message2print = "{(Step 3A/70) try" + "; info2image4mpu-try-by-manoeuvre2copyimagetobuffer4mpu"  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        try: 
            # ----------------------------------------------------------------------------------------------------------------
            # GET IMAGE INFORMATION SUCH AS IMAGE PATH VIA QUEUE
            # ----------------------------------------------------------------------------------------------------------------        
            info2image4mpu = queue2copyimagetobuffermanoeuvre4mpu.get()
            
            message2print = "{(Step 3B/33) got" + "; info2image4mpu-got-by-manoeuvre2copyimagetobuffer4mpu: " + str(info2image4mpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            # ..//..//..//..//..//web//images//png//dragonxi.png
            if info2image4mpu is None:
                code2nack  = code2nack + '3Z1-'
                ack2copyimagetobufferinfo4mpu = False 
            else:    
                code2ack  = code2ack + '3C-'
                ack2copyimagetobufferinfo4mpu = True
        except:
            message2print = "{(Step 3Z1/33) failed" + "; queue2copyimagetobuffermanoeuvre4mpu-failed-by-manoeuvre2copyimagetobuffer4mpu: " + str(queue2copyimagetobuffermanoeuvre4mpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '3Z1-'
            
        message2copyimagetobuffersend4mpu = ' '
        try:
            # ----------------------------------------------------------------------------------------------------------------
            # SEND CONTROL INFO ABOUT IMAGE OPERATION - EITHER ACK OR NACK  
            # ----------------------------------------------------------------------------------------------------------------         
            if ack2copyimagetobufferinfo4mpu is True:
                message2copyimagetobuffersend4mpu = '<ack4xi>ack2imageinfo4mpu</ack4xi>'
            else:
                message2copyimagetobuffersend4mpu = '<nack4xi>nack2imageinfo4mpu</nack4xi>'
            message2print = "{(Step 4B/33) set" + "; message2copyimagetobuffersend4mpu-set-by-manoeuvre2copyimagetobuffer4mpu to: " + str(message2copyimagetobuffersend4mpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # <ack4xi>ack2imageinfo4mpu</ack4xi>       
        except:
            message2print = "{(Step 4Z1/33) failed" + "; message2copyimagetobuffersend4mpu-failed-by-manoeuvre2copyimagetobuffer4mpu: " + str(message2copyimagetobuffersend4mpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            
            code2nack  = code2nack + '4Z1-'    
        try:                
            message2print = "{(Step 4C/70) using" + "; publisherchannel2copyimagetobuffermanoeuvre4mpu-calling-by-manoeuvre2copyimagetobuffer4mpu: " + str(publisherchannel2copyimagetobuffermanoeuvre4mpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)    
            # <copy-write PipeConnection, handle 412>
            message2print = "{{(Step 4D/70) passing" + "; message2copyimagetobuffersend4mpu-passing-to-send-by-manoeuvre2copyimagetobuffer4mpu: " + str(message2copyimagetobuffersend4mpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #              
        except:
            message2print = "{(Step 4Z1/33) failed" + "; message2copyimagetobuffersend4mpu-failed-by-manoeuvre2copyimagetobuffer4mpu: " + str(message2copyimagetobuffersend4mpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '4Z2-'                
        try:    
            publisherchannel2copyimagetobuffermanoeuvre4mpu.send(message2copyimagetobuffersend4mpu)
            
            message2print = "{(Step 4E/33) sent" + "; message2copyimagetobuffersend4mpu-sent-by-manoeuvre2copyimagetobuffer4mpu: " + str(message2copyimagetobuffersend4mpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)          
            # <ack4xi>ack2imageinfo4mpu</ack4xi>            
            code2ack  = code2ack + '4E-'
        except:
            message2print = "{(Step 4Z3/33) failed" + "; message2copyimagetobuffersend4mpu-failed-by-manoeuvre2copyimagetobuffer4mpu: " + str(message2copyimagetobuffersend4mpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '4Z3-'                
    else:
        pass
        

            
    # ------------------------------------------------------------------------------------------------------------------------
    # RETURN 
    # ------------------------------------------------------------------------------------------------------------------------
    response2manouvre4copyimagetobuffer['ACK2INFO4MPU']        = ack2copyimagetobufferinfo4mpu
    response2manouvre4copyimagetobuffer['CODE2ACK4MPU']        = code2ack
    response2manouvre4copyimagetobuffer['CODE2NACK4MPU']       = code2nack
    response2manouvre4copyimagetobuffer['MESSAGE2GET4MPU']     = message2copyimagetobufferget4mpu
    response2manouvre4copyimagetobuffer['MESSAGE2PUT4MPU']     = message2copyimagetobufferput4mpu
    response2manouvre4copyimagetobuffer['MESSAGE2RECEIVE4MPU'] = message2copyimagetobufferreceive4mpu
    response2manouvre4copyimagetobuffer['MESSAGE2SEND4MPU']    = message2copyimagetobuffersend4mpu
    response2manouvre4copyimagetobuffer['STATE2MANOEUVRE4MPU'] = state2copyimagetobuffer4mpu
                          
    message2print = "{(Step 33/33) returning" + "; response2manouvre4copyimagetobuffer-returning-by-manoeuvre2copyimagetobuffer4mpu: " + str(response2manouvre4copyimagetobuffer) + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                             
    #1 {'MESSAGE2SEND4MPU': '<roger4xi>roger2copyimagetobuffer4mpu</roger4xi>',
    #   'ACK2INFO4MPU': False,
    #   'MESSAGE2GET4MPU': '<default4xi>default2copyimagetobuffer4mpu</default4xi>',
    #   'MESSAGE2RECEIVE4MPU': '<copy4xi>copy2copyimagetobuffer4mpu</copy4xi>',
    #   'CODE2ACK4MPU': ' 0D-1C-',
    #   'MESSAGE2PUT4MPU': '<default4xi>default2copyimagetobuffer4mpu</default4xi>',
    #   'CODE2NACK4MPU': ' ',
    #   'STATE2MANOEUVRE4MPU': 1}; time: 02:00:20 10/04/11 Pacific Daylight Time}

    
    #2 {'MESSAGE2SEND4MPU': '<ready4xi>ready2copyimagetobuffer4mpu</ready4xi>',
    #   'ACK2INFO4MPU': False,
    #   'MESSAGE2GET4MPU': '<default4xi>default2copyimagetobuffer4mpu</default4xi>',
    #   'MESSAGE2RECEIVE4MPU': '<ready4xi>ready2copyimagetobuffer4mpu</ready4xi>',
    #   'CODE2ACK4MPU': ' 0D-2C-',
    #   'MESSAGE2PUT4MPU': '<default4xi>default2copyimagetobuffer4mpu</default4xi>',
    #   'CODE2NACK4MPU': ' ',
    #   'STATE2MANOEUVRE4MPU': 2}; time: 02:10:33 10/04/11 Pacific Daylight Time}


    #3 {'MESSAGE2SEND4MPU': '<ack4xi>ack2imageinfo4mpu</ack4xi>',
    #   'ACK2INFO4MPU': True,
    #   'MESSAGE2GET4MPU': '<default4xi>default2copyimagetobuffer4mpu</default4xi>',
    #   'MESSAGE2RECEIVE4MPU': '<ready4xi>ready2copyimagetobuffer4mpu</ready4xi>',
    #   'CODE2ACK4MPU': ' 0D-3C-4E-',
    #   'MESSAGE2PUT4MPU': '<default4xi>default2copyimagetobuffer4mpu</default4xi>',
    #   'CODE2NACK4MPU': ' ',
    #   'STATE2MANOEUVRE4MPU': 3}; time: 02:10:33 10/04/11 Pacific Daylight Time}


    #4 {'MESSAGE2SEND4MPU': '<default4xi>default2copyimagetobuffer4mpu</default4xi>',
    #   'ACK2INFO4MPU': False,
    #   'MESSAGE2GET4MPU': '<default4xi>default2copyimagetobuffer4mpu</default4xi>',
    #   'MESSAGE2RECEIVE4MPU': '<default4xi>default2copyimagetobuffer4mpu</default4xi>',
    #   'CODE2ACK4MPU': ' 0D-',
    #   'MESSAGE2PUT4MPU': '<default4xi>default2copyimagetobuffer4mpu</default4xi>',
    #   'CODE2NACK4MPU': ' ',
    #   'STATE2MANOEUVRE4MPU': 0}; time: 02:10:33 10/04/11 Pacific Daylight Time}


    return response2manouvre4copyimagetobuffer           
          
#</function4xi>

# -----------------------------------------------------------------------------------------------------------------------
#<function4xi function = "manoeuvre2copybuffertoimage4mpu" usage = "
#   # ---------------------------------------------------------------------------------------------------------------------
#   # SIMULATE MANOEUVRE2COPYIMAGE4MPU - LET IT TO RECEIVE COPY AND TO SEND ROGER VIA PIPE                
#   # ---------------------------------------------------------------------------------------------------------------------       
#   if ack2simulate4mpu is True:
#       print("(Step 82A/700) OF MPU2IMAGE4XI5TEST - TRYING TO PERFORM TARGET FUNCTION MANOEUVRE2COPYIMAGE4MPU ", time4xi())   
#       try:        
#           response2manoeuvre4copy = manoeuvre2copybuffertoimage4mpu(queue4copybuffertoimage4mpu, channel4copybuffertoimageconsume4mpu, channel4copybuffertoimagepublish4mpu)
#           print("(Step 82B/700) OF MPU2IMAGE4XI5TEST - PERFORMED MANOEUVRE2COPYIMAGE4MPU, RESPONSE2MANOEUVRE4COPY: ", response2manoeuvre4copy)
#           # {'MESSAGE2SEND4MPU': '<roger4xi>roger2copybuffertoimage4mpu</roger4xi>',
#           #   'ACK2INFO4MPU': False,
#           #   'MESSAGE2GET4MPU': '<default4xi>default2copybuffertoimage4mpu</default4xi>',
#           #   'MESSAGE2RECEIVE4MPU': '<copy4xi>copy2copybuffertoimage4mpu</copy4xi>',
#           #   'CODE2ACK4MPU': ' 0D-1C-',
#           #   'MESSAGE2PUT4MPU': '<default4xi>default2copybuffertoimage4mpu</default4xi>',
#           #   'CODE2NACK4MPU': ' ',
#           #   'STATE2MANOEUVRE4MPU': 1}
#       except:
#           print("(Step 82Z/700) OF MPU2IMAGE4XI5TEST *** FAILED TO PERFORM TARGET FUNCTION MANOEUVRE2COPYIMAGE4MPU ", time4xi())     
#   else:
#       pass # trust activated thread    
#">

# ----------------------------------------------------------------------------------------------------------------------- 
def manoeuvre2copybuffertoimage4mpu(queue2copybuffertoimagemanoeuvre4mpu, consumerchannel2copybuffertoimagemanoeuvre4mpu, publisherchannel2copybuffertoimagemanoeuvre4mpu):
    '''
    manoeuvres copy control and command for copy2buffertoimage4mpu
    '''
    global state2copybuffertoimage4mpu                                                                                           # STATE FOR COMMAND AND CONTROL
    global max2copybuffertoimagestate4mpu                                                                                        # MAX FOR STATE
    global ack2copybuffertoimageinfo4mpu                                                                                         # ACK FOR IMAGE INFO
    global message2copybuffertoimageget4mpu                                 
    global message2copybuffertoimageput4mpu
    global message2copybuffertoimagereceive4mpu
    global message2copybuffertoimagesend4mpu
    
    # -----------------------------------------------------------------------------------------
    # (Step 0/33) PARAMETERS            
    # -----------------------------------------------------------------------------------------          
    class2print  = "manoeuvre2copybuffertoimage4mpu" 
    method2print = "manoeuvre2copybuffertoimage4mpu" 

    code2ack  = ' '
    code2nack = ' '
    
    message2print = "{(Step 0A/33) started" + "; manoeuvre2copybuffertoimage4mpu-started: " + str(manoeuvre2copybuffertoimage4mpu)  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                                     #  
    # 
    
    message2print = "{(Step 0B/33) passed" + "; queue2copybuffertoimagemanoeuvre4mpu-to-manoeuvre2copybuffertoimage4mpu: " + str(queue2copybuffertoimagemanoeuvre4mpu) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                                                              
    # 
    
    message2print = "{(Step 0C/33) passed" + "; consumerchannel2copybuffertoimagemanoeuvre4mpu-to-manoeuvre2copybuffertoimage4mpu: " + str(consumerchannel2copybuffertoimagemanoeuvre4mpu) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                                                              
    # 
    
    message2print = "{(Step 0D/33) passed" + "; publisherchannel2copybuffertoimagemanoeuvre4mpu-to-manoeuvre2copybuffertoimage4mpu: " + str(publisherchannel2copybuffertoimagemanoeuvre4mpu) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                                                              
    # 
    
    code2ack  = code2ack + '0D-' 
    response2manouvre4copybuffertoimage = {}
                    
    # ------------------------------------------------------------------------------------------------------------------------
    # SETTINGS
    # ------------------------------------------------------------------------------------------------------------------------    
    message2print = "{(Step 0E/70) try" + "; state2copybuffertoimage4mpu-try-by-manoeuvre2copybuffertoimage4mpu"  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)     
    try:
        if state2copybuffertoimage4mpu is None:
            state2copybuffertoimage4mpu = 0
            message2print = "{(Step 0F/33) initialized" + "; state2copybuffertoimage4mpu-initialized-by-manoeuvre2copybuffertoimage4mpu to: " + str(state2copybuffertoimage4mpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)    
        else:
            message2print = "{(Step 0G/33) used" + "; state2copybuffertoimage4mpu-used-by-manoeuvre2copybuffertoimage4mpu: " + str(state2copybuffertoimage4mpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # 1; 2; 3;  
    except:
        state2copybuffertoimage4mpu = 0
        message2print = "{(Step 0Z1/33) initialized" + "; state2copybuffertoimage4mpu-initialized-by-manoeuvre2copybuffertoimage4mpu to: " + str(state2copybuffertoimage4mpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # 0
        ack2copybuffertoimageinfo4mpu        = False
        message2print = "{(Step 0Z2/33) initialized" + "; ack2copybuffertoimageinfo4mpu-initialized-by-manoeuvre2copybuffertoimage4mpu to: " + str(ack2copybuffertoimageinfo4mpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                     
        # False
        message2copybuffertoimageget4mpu     = '<default4xi>default2copybuffertoimage4mpu</default4xi>'
        message2print = "{(Step 0Z3/33) initialized" + "; message2copybuffertoimageget4mpu-initialized-by-manoeuvre2copybuffertoimage4mpu to: " + str(message2copybuffertoimageget4mpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                               
        # 
        message2copybuffertoimageput4mpu     = '<default4xi>default2copybuffertoimage4mpu</default4xi>'
        message2print = "{(Step 0Z4/33) initialized" + "; message2copybuffertoimageput4mpu-initialized-by-manoeuvre2copybuffertoimage4mpu to: " + str(message2copybuffertoimageput4mpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
        # 
        message2copybuffertoimagereceive4mpu = '<default4xi>default2copybuffertoimage4mpu</default4xi>'
        message2print = "{(Step 0Z5/33) initialized" + "; message2copybuffertoimagereceive4mpu-initialized-by-manoeuvre2copybuffertoimage4mpu to: " + str(message2copybuffertoimagereceive4mpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
        # 
        message2copybuffertoimagesend4mpu    = '<default4xi>default2copybuffertoimage4mpu</default4xi>'
        message2print = "{(Step 0Z6/33) initialized" + "; message2copybuffertoimagesend4mpu-initialized-by-manoeuvre2copybuffertoimage4mpu to: " + str(message2copybuffertoimagesend4mpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
        # 
        max2copybuffertoimagestate4mpu = 3
        message2print = "{(Step 0Z7/33) initialized" + ";  max2copybuffertoimagestate4mpu-initialized-by-manoeuvre2copybuffertoimage4mpu to: " + str( max2copybuffertoimagestate4mpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # 
                    
    # ------------------------------------------------------------------------------------------------------------------------
    # STATE UPDATE
    # ------------------------------------------------------------------------------------------------------------------------                      
    state2copybuffertoimage4mpu = state2copybuffertoimage4mpu + 1                                                                   
    message2print = "{(Step 0H/33) added" + "; state2copybuffertoimage4mpu-added-by-manoeuvre2copybuffertoimage4mpu: " + str(state2copybuffertoimage4mpu) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)
    # 1; 2; 3; 4 
     
    if state2copybuffertoimage4mpu > max2copybuffertoimagestate4mpu:
        state2copybuffertoimage4mpu = 0
        message2print = "{(Step 0I/33) set" + "; state2copybuffertoimage4mpu-set-by-manoeuvre2copybuffertoimage4mpu to: " + str(state2copybuffertoimage4mpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # 
        ack2copybuffertoimageinfo4mpu        = False
        message2print = "{(Step 0J/33) set" + "; ack2copybuffertoimageinfo4mpu-set-by-manoeuvre2copybuffertoimage4mpu to: " + str(ack2copybuffertoimageinfo4mpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                     
        # 
        message2copybuffertoimageget4mpu     = '<default4xi>default2copybuffertoimage4mpu</default4xi>'
        message2print = "{(Step 0K/33) set" + "; message2copybuffertoimageget4mpu-set-by-manoeuvre2copybuffertoimage4mpu to: " + str(message2copybuffertoimageget4mpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                               
        # 
        message2copybuffertoimageput4mpu     = '<default4xi>default2copybuffertoimage4mpu</default4xi>'
        message2print = "{(Step 0L/33) set" + "; message2copybuffertoimageput4mpu-set-by-manoeuvre2copybuffertoimage4mpu to: " + str(message2copybuffertoimageput4mpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
        # 
        message2copybuffertoimagereceive4mpu = '<default4xi>default2copybuffertoimage4mpu</default4xi>'
        message2print = "{(Step 0M/33) set" + "; message2copybuffertoimagereceive4mpu-set-by-manoeuvre2copybuffertoimage4mpu to: " + str(message2copybuffertoimagereceive4mpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
        # 
        message2copybuffertoimagesend4mpu    = '<default4xi>default2copybuffertoimage4mpu</default4xi>'
        message2print = "{(Step 0N/33) set" + "; message2copybuffertoimagesend4mpu-set-by-manoeuvre2copybuffertoimage4mpu to: " + str(message2copybuffertoimagesend4mpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
        # 
        max2copybuffertoimagestate4mpu = 3
        message2print = "{(Step 0O/33) initialized" + ";  max2copybuffertoimagestate4mpu-set-by-manoeuvre2copybuffertoimage4mpu to: " + str( max2copybuffertoimagestate4mpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        #     
                    
    # ------------------------------------------------------------------------------------------------------------------------
    # STATE I - RECEIVE CONTROL AND COMMAND COPY MESSAGE VIA PIPE AND SEND ROGER OR NEGATIVE BACK 
    # ------------------------------------------------------------------------------------------------------------------------
    if state2copybuffertoimage4mpu == 1:
        message2print = "{(Step 1A/70) try" + "; receive-try-by-manoeuvre2copybuffertoimage4mpu"  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        message2copybuffertoimagereceive4mpu = ''
        try:
            # ----------------------------------------------------------------------------------------------------------------
            # RECEIVE CONTROL AND COMMAND MESSAGE VIA PIPE
            # ----------------------------------------------------------------------------------------------------------------         
            message2copybuffertoimagereceive4mpu = consumerchannel2copybuffertoimagemanoeuvre4mpu.recv()
            message2print = "{(Step 1B/33) received" + "; message2copybuffertoimagereceive4mpu-received-by-manoeuvre2copybuffertoimage4mpu: " + str(message2copybuffertoimagereceive4mpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            # 
                         
            if message2copybuffertoimagereceive4mpu == '<copy4xi>copy2copybuffertoimage4mpu</copy4xi>':
                message2copybuffertoimagesend4mpu = '<roger4xi>roger2copybuffertoimage4mpu</roger4xi>'
            else:
                message2copybuffertoimagesend4mpu = '<negative4xi><negative2copybuffertoimage4mpu</<negative4xi>'
            # ----------------------------------------------------------------------------------------------------------------
            # SEND ROGER OR NEGATIVE BACK 
            # ----------------------------------------------------------------------------------------------------------------                                
            publisherchannel2copybuffertoimagemanoeuvre4mpu.send(message2copybuffertoimagesend4mpu)
            
            message2print = "{(Step 1C/33) sent" + "; message2copybuffertoimagesend4mpu-sent-by-manoeuvre2copybuffertoimage4mpu: " + str(message2copybuffertoimagesend4mpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)          
            # 
            
            code2ack  = code2ack + '1C-'
        except:
            message2print = "{(Step 1Z/33) failed" + "; message2copybuffertoimagereceive4mpu-failed-by-manoeuvre2copybuffertoimage4mpu: " + str(message2copybuffertoimagereceive4mpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '1Z-' 
    else:
        pass

                    
    # ------------------------------------------------------------------------------------------------------------------------
    # STATE II - RECEIVE CONTROL AND COMMAND READY MESSAGE VIA PIPE AND SEND READY OR NEGATIVE BACK 
    # ------------------------------------------------------------------------------------------------------------------------
    if state2copybuffertoimage4mpu == 2:
        message2print = "{(Step 2A/70) try" + "; receive-try-by-manoeuvre2copybuffertoimage4mpu"  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        message2copybuffertoimagereceive4mpu = ''
        try:
            # ----------------------------------------------------------------------------------------------------------------
            # RECEIVE CONTROL AND COMMAND MESSAGE VIA PIPE
            # ----------------------------------------------------------------------------------------------------------------         
            message2copybuffertoimagereceive4mpu = consumerchannel2copybuffertoimagemanoeuvre4mpu.recv()
            message2print = "{(Step 2B/33) received" + "; message2copybuffertoimagereceive4mpu-received-by-manoeuvre2copybuffertoimage4mpu: " + str(message2copybuffertoimagereceive4mpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            # 
                         
            if message2copybuffertoimagereceive4mpu == '<ready4xi>ready2copybuffertoimage4mpu</ready4xi>':
                message2copybuffertoimagesend4mpu = '<ready4xi>ready2copybuffertoimage4mpu</ready4xi>'
            else:
                message2copybuffertoimagesend4mpu = '<negative4xi><negative2copybuffertoimage4mpu</<negative4xi>'
            # ----------------------------------------------------------------------------------------------------------------
            # SEND READY OR NEGATIVE BACK 
            # ----------------------------------------------------------------------------------------------------------------                                
            publisherchannel2copybuffertoimagemanoeuvre4mpu.send(message2copybuffertoimagesend4mpu)
            
            message2print = "{(Step 2C/33) sent" + "; message2copybuffertoimagesend4mpu-sent-by-manoeuvre2copybuffertoimage4mpu: " + str(message2copybuffertoimagesend4mpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)          
            # 
            
            code2ack  = code2ack + '2C-'
        except:
            message2print = "{(Step 2Z/33) failed" + "; message2copybuffertoimagereceive4mpu-failed-by-manoeuvre2copybuffertoimage4mpu: " + str(message2copybuffertoimagereceive4mpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '2Z-' 
    else:
        pass        
                
    # ------------------------------------------------------------------------------------------------------------------------
    # STATE III - GET IMAGE INFORMATION SUCH AS IMAGE PATH VIA QUEUE AND SEND ACK OR NACK VIA PIPE
    # ------------------------------------------------------------------------------------------------------------------------
    if state2copybuffertoimage4mpu == 3:    
        info2image4mpu = None
        message2print = "{(Step 3A/70) try" + "; info2image4mpu-try-by-manoeuvre2copybuffertoimage4mpu"  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        try: 
            # ----------------------------------------------------------------------------------------------------------------
            # GET IMAGE INFORMATION SUCH AS IMAGE PATH VIA QUEUE
            # ----------------------------------------------------------------------------------------------------------------        
            info2image4mpu = queue2copybuffertoimagemanoeuvre4mpu.get()
            
            message2print = "{(Step 3B/33) got" + "; info2image4mpu-got-by-manoeuvre2copybuffertoimage4mpu: " + str(info2image4mpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            # ..//..//..//..//..//web//images//png//dragonxi.png
            if info2image4mpu is None:
                code2nack  = code2nack + '3Z1-'
                ack2copybuffertoimageinfo4mpu = False 
            else:    
                code2ack  = code2ack + '3C-'
                ack2copybuffertoimageinfo4mpu = True
        except:
            message2print = "{(Step 3Z1/33) failed" + "; queue2copybuffertoimagemanoeuvre4mpu-failed-by-manoeuvre2copybuffertoimage4mpu: " + str(queue2copybuffertoimagemanoeuvre4mpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '3Z1-'
            
        message2copybuffertoimagesend4mpu = ' '
        try:
            # ----------------------------------------------------------------------------------------------------------------
            # SEND CONTROL INFO ABOUT IMAGE OPERATION - EITHER ACK OR NACK  
            # ----------------------------------------------------------------------------------------------------------------         
            if ack2copybuffertoimageinfo4mpu is True:
                message2copybuffertoimagesend4mpu = '<ack4xi>ack2imageinfo4mpu</ack4xi>'
            else:
                message2copybuffertoimagesend4mpu = '<nack4xi>nack2imageinfo4mpu</nack4xi>'
            message2print = "{(Step 4B/33) set" + "; message2copybuffertoimagesend4mpu-set-by-manoeuvre2copybuffertoimage4mpu to: " + str(message2copybuffertoimagesend4mpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # <ack4xi>ack2imageinfo4mpu</ack4xi>       
        except:
            message2print = "{(Step 4Z1/33) failed" + "; message2copybuffertoimagesend4mpu-failed-by-manoeuvre2copybuffertoimage4mpu: " + str(message2copybuffertoimagesend4mpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            
            code2nack  = code2nack + '4Z1-'    
        try:                
            message2print = "{(Step 4C/70) using" + "; publisherchannel2copybuffertoimagemanoeuvre4mpu-calling-by-manoeuvre2copybuffertoimage4mpu: " + str(publisherchannel2copybuffertoimagemanoeuvre4mpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)    
            # <copy-write PipeConnection, handle 412>
            message2print = "{{(Step 4D/70) passing" + "; message2copybuffertoimagesend4mpu-passing-to-send-by-manoeuvre2copybuffertoimage4mpu: " + str(message2copybuffertoimagesend4mpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #              
        except:
            message2print = "{(Step 4Z1/33) failed" + "; message2copybuffertoimagesend4mpu-failed-by-manoeuvre2copybuffertoimage4mpu: " + str(message2copybuffertoimagesend4mpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '4Z2-'                
        try:    
            publisherchannel2copybuffertoimagemanoeuvre4mpu.send(message2copybuffertoimagesend4mpu)
            
            message2print = "{(Step 4E/33) sent" + "; message2copybuffertoimagesend4mpu-sent-by-manoeuvre2copybuffertoimage4mpu: " + str(message2copybuffertoimagesend4mpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)          
            # <ack4xi>ack2imageinfo4mpu</ack4xi>            
            code2ack  = code2ack + '4E-'
        except:
            message2print = "{(Step 4Z3/33) failed" + "; message2copybuffertoimagesend4mpu-failed-by-manoeuvre2copybuffertoimage4mpu: " + str(message2copybuffertoimagesend4mpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '4Z3-'                
    else:
        pass
        

            
    # ------------------------------------------------------------------------------------------------------------------------
    # RETURN 
    # ------------------------------------------------------------------------------------------------------------------------
    response2manouvre4copybuffertoimage['ACK2INFO4MPU']        = ack2copybuffertoimageinfo4mpu
    response2manouvre4copybuffertoimage['CODE2ACK4MPU']        = code2ack
    response2manouvre4copybuffertoimage['CODE2NACK4MPU']       = code2nack
    response2manouvre4copybuffertoimage['MESSAGE2GET4MPU']     = message2copybuffertoimageget4mpu
    response2manouvre4copybuffertoimage['MESSAGE2PUT4MPU']     = message2copybuffertoimageput4mpu
    response2manouvre4copybuffertoimage['MESSAGE2RECEIVE4MPU'] = message2copybuffertoimagereceive4mpu
    response2manouvre4copybuffertoimage['MESSAGE2SEND4MPU']    = message2copybuffertoimagesend4mpu
    response2manouvre4copybuffertoimage['STATE2MANOEUVRE4MPU'] = state2copybuffertoimage4mpu
                          
    message2print = "{(Step 33/33) returning" + "; response2manouvre4copybuffertoimage-returning-by-manoeuvre2copybuffertoimage4mpu: " + str(response2manouvre4copybuffertoimage) + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                             
    #1 {'MESSAGE2SEND4MPU': '<roger4xi>roger2copybuffertoimage4mpu</roger4xi>',
    #   'ACK2INFO4MPU': False,
    #   'MESSAGE2GET4MPU': '<default4xi>default2copybuffertoimage4mpu</default4xi>',
    #   'MESSAGE2RECEIVE4MPU': '<copy4xi>copy2copybuffertoimage4mpu</copy4xi>',
    #   'CODE2ACK4MPU': ' 0D-1C-',
    #   'MESSAGE2PUT4MPU': '<default4xi>default2copybuffertoimage4mpu</default4xi>',
    #   'CODE2NACK4MPU': ' ',
    #   'STATE2MANOEUVRE4MPU': 1}; time: 02:18:55 10/04/11 Pacific Daylight Time}

    
    #2 {'MESSAGE2SEND4MPU': '<ready4xi>ready2copybuffertoimage4mpu</ready4xi>',
    #   'ACK2INFO4MPU': False,
    #   'MESSAGE2GET4MPU': '<default4xi>default2copybuffertoimage4mpu</default4xi>',
    #   'MESSAGE2RECEIVE4MPU': '<ready4xi>ready2copybuffertoimage4mpu</ready4xi>',
    #   'CODE2ACK4MPU': ' 0D-2C-',
    #   'MESSAGE2PUT4MPU': '<default4xi>default2copybuffertoimage4mpu</default4xi>',
    #   'CODE2NACK4MPU': ' ',
    #   'STATE2MANOEUVRE4MPU': 2}; time: 02:18:55 10/04/11 Pacific Daylight Time}

    #3 {'MESSAGE2SEND4MPU': '<ack4xi>ack2imageinfo4mpu</ack4xi>',
    #   'ACK2INFO4MPU': True,
    #   'MESSAGE2GET4MPU': '<default4xi>default2copybuffertoimage4mpu</default4xi>',
    #   'MESSAGE2RECEIVE4MPU': '<ready4xi>ready2copybuffertoimage4mpu</ready4xi>',
    #   'CODE2ACK4MPU': ' 0D-3C-4E-',
    #   'MESSAGE2PUT4MPU': '<default4xi>default2copybuffertoimage4mpu</default4xi>',
    #   'CODE2NACK4MPU': ' ',
    #   'STATE2MANOEUVRE4MPU': 3}; time: 02:18:55 10/04/11 Pacific Daylight Time}
 

    #4 {'MESSAGE2SEND4MPU': '<default4xi>default2copybuffertoimage4mpu</default4xi>',
    #   'ACK2INFO4MPU': False,
    #   'MESSAGE2GET4MPU': '<default4xi>default2copybuffertoimage4mpu</default4xi>',
    #   'MESSAGE2RECEIVE4MPU': '<default4xi>default2copybuffertoimage4mpu</default4xi>',
    #   'CODE2ACK4MPU': ' 0D-',
    #   'MESSAGE2PUT4MPU': '<default4xi>default2copybuffertoimage4mpu</default4xi>',
    #   'CODE2NACK4MPU': ' ',
    #   'STATE2MANOEUVRE4MPU': 0}

    return response2manouvre4copybuffertoimage           
          
#</function4xi>



# -----------------------------------------------------------------------------------------------------------------------
#<function4xi function = "manoeuvre2readimage4mpu" usage = "control function for read2image4mpu function in this module">

# ----------------------------------------------------------------------------------------------------------------------- 
def manoeuvre2readimage4mpu(queue2readmanoeuvre4mpu, consumerchannel2readmanoeuvre4mpu, publisherchannel2readmanoeuvre4mpu):
    '''
    manoeuvres read control and command
    '''
    global state2read4mpu                                                                                           # STATE FOR COMMAND AND CONTROL
    global max2readstate4mpu                                                                                        # MAX FOR STATE
    global ack2readinfo4mpu                                                                                         # ACK FOR IMAGE INFO
    global message2readget4mpu                                 
    global message2readput4mpu
    global message2readreceive4mpu
    global message2readsend4mpu
    
    # -----------------------------------------------------------------------------------------
    # (Step 0/33) PARAMETERS            
    # -----------------------------------------------------------------------------------------          
    class2print  = "manoeuvre2readimage4mpu" 
    method2print = "manoeuvre2readimage4mpu" 

    code2ack  = ' '
    code2nack = ' '
    
    message2print = "{(Step 0A/33) started" + "; manoeuvre2readimage4mpu-started: " + str(manoeuvre2readimage4mpu)  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                                     #  
    # <function manoeuvre2readimage4mpu at 0x0000000003DB8C48>  
    
    message2print = "{(Step 0B/33) passed" + "; queue2readmanoeuvre4mpu-to-manoeuvre2readimage4mpu: " + str(queue2readmanoeuvre4mpu) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                                                              
    # <queue4xi.Queue4Xi object at 0x0000000003AD9400>
    
    message2print = "{(Step 0C/33) passed" + "; consumerchannel2readmanoeuvre4mpu-to-manoeuvre2readimage4mpu: " + str(consumerchannel2readmanoeuvre4mpu) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                                                              
    # <read-write PipeConnection, handle 408>
    
    message2print = "{(Step 0D/33) passed" + "; publisherchannel2readmanoeuvre4mpu-to-manoeuvre2readimage4mpu: " + str(publisherchannel2readmanoeuvre4mpu) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                                                              
    # <read-write PipeConnection, handle 412>     
    
    code2ack  = code2ack + '0D-' 
    response2manouvre4read = {}
                    
    # ------------------------------------------------------------------------------------------------------------------------
    # SETTINGS
    # ------------------------------------------------------------------------------------------------------------------------    
    message2print = "{(Step 0E/70) try" + "; state2read4mpu-try-by-manoeuvre2readimage4mpu"  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)     
    try:
        if state2read4mpu is None:
            state2read4mpu = 0
            message2print = "{(Step 0F/33) initialized" + "; state2read4mpu-initialized-by-manoeuvre2readimage4mpu to: " + str(state2read4mpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)    
        else:
            message2print = "{(Step 0G/33) used" + "; state2read4mpu-used-by-manoeuvre2readimage4mpu: " + str(state2read4mpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # 1; 2; 3;  
    except:
        state2read4mpu = 0
        message2print = "{(Step 0Z1/33) initialized" + "; state2read4mpu-initialized-by-manoeuvre2readimage4mpu to: " + str(state2read4mpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # 0
        ack2readinfo4mpu        = False
        message2print = "{(Step 0Z2/33) initialized" + "; ack2readinfo4mpu-initialized-by-manoeuvre2readimage4mpu to: " + str(ack2readinfo4mpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                     
        # False
        message2readget4mpu     = '<default4xi>default2read4mpu</default4xi>'
        message2print = "{(Step 0Z3/33) initialized" + "; message2readget4mpu-initialized-by-manoeuvre2readimage4mpu to: " + str(message2readget4mpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                               
        # <default4xi>default2read4mpu</default4xi>
        message2readput4mpu     = '<default4xi>default2read4mpu</default4xi>'
        message2print = "{(Step 0Z4/33) initialized" + "; message2readput4mpu-initialized-by-manoeuvre2readimage4mpu to: " + str(message2readput4mpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
        # <default4xi>default2read4mpu</default4xi>
        message2readreceive4mpu = '<default4xi>default2read4mpu</default4xi>'
        message2print = "{(Step 0Z5/33) initialized" + "; message2readreceive4mpu-initialized-by-manoeuvre2readimage4mpu to: " + str(message2readreceive4mpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
        # <default4xi>default2read4mpu</default4xi>
        message2readsend4mpu    = '<default4xi>default2read4mpu</default4xi>'
        message2print = "{(Step 0Z6/33) initialized" + "; message2readsend4mpu-initialized-by-manoeuvre2readimage4mpu to: " + str(message2readsend4mpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
        # <default4xi>default2read4mpu</default4xi>
        max2readstate4mpu = 3
        message2print = "{(Step 0Z7/33) initialized" + ";  max2readstate4mpu-initialized-by-manoeuvre2readimage4mpu to: " + str( max2readstate4mpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # 
                    
    # ------------------------------------------------------------------------------------------------------------------------
    # STATE UPDATE
    # ------------------------------------------------------------------------------------------------------------------------                      
    state2read4mpu = state2read4mpu + 1                                                                   
    message2print = "{(Step 0H/33) added" + "; state2read4mpu-added-by-manoeuvre2readimage4mpu: " + str(state2read4mpu) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)
    # 1; 2; 3; 4  
    if state2read4mpu > max2readstate4mpu:
        state2read4mpu = 0
        message2print = "{(Step 0I/33) set" + "; state2read4mpu-set-by-manoeuvre2readimage4mpu to: " + str(state2read4mpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # 0
        ack2readinfo4mpu        = False
        message2print = "{(Step 0J/33) set" + "; ack2readinfo4mpu-set-by-manoeuvre2readimage4mpu to: " + str(ack2readinfo4mpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                     
        # False
        message2readget4mpu     = '<default4xi>default2read4mpu</default4xi>'
        message2print = "{(Step 0K/33) set" + "; message2readget4mpu-set-by-manoeuvre2readimage4mpu to: " + str(message2readget4mpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                               
        # <default4xi>default2read4mpu</default4xi>
        message2readput4mpu     = '<default4xi>default2read4mpu</default4xi>'
        message2print = "{(Step 0L/33) set" + "; message2readput4mpu-set-by-manoeuvre2readimage4mpu to: " + str(message2readput4mpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
        # <default4xi>default2read4mpu</default4xi>
        message2readreceive4mpu = '<default4xi>default2read4mpu</default4xi>'
        message2print = "{(Step 0M/33) set" + "; message2readreceive4mpu-set-by-manoeuvre2readimage4mpu to: " + str(message2readreceive4mpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
        # <default4xi>default2read4mpu</default4xi>
        message2readsend4mpu    = '<default4xi>default2read4mpu</default4xi>'
        message2print = "{(Step 0N/33) set" + "; message2readsend4mpu-set-by-manoeuvre2readimage4mpu to: " + str(message2readsend4mpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
        # <default4xi>default2read4mpu</default4xi>
        max2readstate4mpu = 3
        message2print = "{(Step 0O/33) initialized" + ";  max2readstate4mpu-set-by-manoeuvre2readimage4mpu to: " + str( max2readstate4mpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        #     
                    
    # ------------------------------------------------------------------------------------------------------------------------
    # STATE I - RECEIVE CONTROL AND COMMAND COPY MESSAGE VIA PIPE AND SEND ROGER OR NEGATIVE BACK 
    # ------------------------------------------------------------------------------------------------------------------------
    if state2read4mpu == 1:
        message2print = "{(Step 1A/70) try" + "; receive-try-by-manoeuvre2readimage4mpu"  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        message2readreceive4mpu = ''
        try:
            # ----------------------------------------------------------------------------------------------------------------
            # RECEIVE CONTROL AND COMMAND MESSAGE VIA PIPE
            # ----------------------------------------------------------------------------------------------------------------         
            message2readreceive4mpu = consumerchannel2readmanoeuvre4mpu.recv()
            message2print = "{(Step 1B/33) received" + "; message2readreceive4mpu-received-by-manoeuvre2readimage4mpu: " + str(message2readreceive4mpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            # <copy4xi>copy2read4mpu</copy4xi>
                         
            if message2readreceive4mpu == '<copy4xi>copy2read4mpu</copy4xi>':
                message2readsend4mpu = '<roger4xi>roger2read4mpu</roger4xi>'
            else:
                message2readsend4mpu = '<negative4xi><negative2read4mpu</<negative4xi>'
            # ----------------------------------------------------------------------------------------------------------------
            # SEND ROGER OR NEGATIVE BACK 
            # ----------------------------------------------------------------------------------------------------------------                                
            publisherchannel2readmanoeuvre4mpu.send(message2readsend4mpu)
            
            message2print = "{(Step 1C/33) sent" + "; message2readsend4mpu-sent-by-manoeuvre2readimage4mpu: " + str(message2readsend4mpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)          
            # <roger4xi>roger2read4mpu</roger4xi>
            
            code2ack  = code2ack + '1C-'
        except:
            message2print = "{(Step 1Z/33) failed" + "; message2readreceive4mpu-failed-by-manoeuvre2readimage4mpu: " + str(message2readreceive4mpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '1Z-' 
    else:
        pass

                    
    # ------------------------------------------------------------------------------------------------------------------------
    # STATE II - RECEIVE CONTROL AND COMMAND READY MESSAGE VIA PIPE AND SEND READY OR NEGATIVE BACK 
    # ------------------------------------------------------------------------------------------------------------------------
    if state2read4mpu == 2:
        message2print = "{(Step 2A/70) try" + "; receive-try-by-manoeuvre2readimage4mpu"  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        message2readreceive4mpu = ''
        try:
            # ----------------------------------------------------------------------------------------------------------------
            # RECEIVE CONTROL AND COMMAND MESSAGE VIA PIPE
            # ----------------------------------------------------------------------------------------------------------------         
            message2readreceive4mpu = consumerchannel2readmanoeuvre4mpu.recv()
            message2print = "{(Step 2B/33) received" + "; message2readreceive4mpu-received-by-manoeuvre2readimage4mpu: " + str(message2readreceive4mpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            # <ready4xi>ready2read4mpu</ready4xi> 
                         
            if message2readreceive4mpu == '<ready4xi>ready2read4mpu</ready4xi>':
                message2readsend4mpu = '<ready4xi>ready2read4mpu</ready4xi>'
            else:
                message2readsend4mpu = '<negative4xi><negative2ready4mpu</<negative4xi>'
            # ----------------------------------------------------------------------------------------------------------------
            # SEND ROGER OR NEGATIVE BACK 
            # ----------------------------------------------------------------------------------------------------------------                                
            publisherchannel2readmanoeuvre4mpu.send(message2readsend4mpu)
            
            message2print = "{(Step 2C/33) sent" + "; message2readsend4mpu-sent-by-manoeuvre2readimage4mpu: " + str(message2readsend4mpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)          
            # <ready4xi>ready2read4mpu</ready4xi>
            
            code2ack  = code2ack + '2C-'
        except:
            message2print = "{(Step 2Z/33) failed" + "; message2readreceive4mpu-failed-by-manoeuvre2readimage4mpu: " + str(message2readreceive4mpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '2Z-' 
    else:
        pass        
                
    # ------------------------------------------------------------------------------------------------------------------------
    # STATE III - GET IMAGE INFORMATION SUCH AS IMAGE PATH VIA QUEUE AND SEND ACK OR NACK VIA PIPE
    # ------------------------------------------------------------------------------------------------------------------------
    if state2read4mpu == 3:    
        info2image4mpu = None
        message2print = "{(Step 3A/70) try" + "; info2image4mpu-try-by-manoeuvre2readimage4mpu"  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        try: 
            # ----------------------------------------------------------------------------------------------------------------
            # GET IMAGE INFORMATION SUCH AS IMAGE PATH VIA QUEUE
            # ----------------------------------------------------------------------------------------------------------------        
            info2image4mpu = queue2readmanoeuvre4mpu.get()
            
            message2print = "{(Step 3B/33) got" + "; info2image4mpu-got-by-manoeuvre2readimage4mpu: " + str(info2image4mpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            # ..//..//..//..//..//web//images//png//dragonxi.png
            if info2image4mpu is None:
                code2nack  = code2nack + '3Z1-'
                ack2readinfo4mpu = False 
            else:    
                code2ack  = code2ack + '3C-'
                ack2readinfo4mpu = True
        except:
            message2print = "{(Step 3Z1/33) failed" + "; queue2readmanoeuvre4mpu-failed-by-manoeuvre2readimage4mpu: " + str(queue2readmanoeuvre4mpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '3Z1-'
            
        message2readsend4mpu = ' '
        try:
            # ----------------------------------------------------------------------------------------------------------------
            # SEND CONTROL INFO ABOUT IMAGE OPERATION - EITHER ACK OR NACK  
            # ----------------------------------------------------------------------------------------------------------------         
            if ack2readinfo4mpu is True:
                message2readsend4mpu = '<ack4xi>ack2imageinfo4mpu</ack4xi>'
            else:
                message2readsend4mpu = '<nack4xi>nack2imageinfo4mpu</nack4xi>'
            message2print = "{(Step 4B/33) set" + "; message2readsend4mpu-set-by-manoeuvre2readimage4mpu to: " + str(message2readsend4mpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # <ack4xi>ack2imageinfo4mpu</ack4xi>       
        except:
            message2print = "{(Step 4Z1/33) failed" + "; message2readsend4mpu-failed-by-manoeuvre2readimage4mpu: " + str(message2readsend4mpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            
            code2nack  = code2nack + '4Z1-'    
        try:                
            message2print = "{(Step 4C/70) using" + "; publisherchannel2readmanoeuvre4mpu-calling-by-manoeuvre2readimage4mpu: " + str(publisherchannel2readmanoeuvre4mpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)    
            # <read-write PipeConnection, handle 412>
            message2print = "{{(Step 4D/70) passing" + "; message2readsend4mpu-passing-to-send-by-manoeuvre2readimage4mpu: " + str(message2readsend4mpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #              
        except:
            message2print = "{(Step 4Z1/33) failed" + "; message2readsend4mpu-failed-by-manoeuvre2readimage4mpu: " + str(message2readsend4mpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '4Z2-'                
        try:    
            publisherchannel2readmanoeuvre4mpu.send(message2readsend4mpu)
            
            message2print = "{(Step 4E/33) sent" + "; message2readsend4mpu-sent-by-manoeuvre2readimage4mpu: " + str(message2readsend4mpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)          
            # <ack4xi>ack2imageinfo4mpu</ack4xi>            
            code2ack  = code2ack + '4E-'
        except:
            message2print = "{(Step 4Z3/33) failed" + "; message2readsend4mpu-failed-by-manoeuvre2readimage4mpu: " + str(message2readsend4mpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '4Z3-'                
    else:
        pass
        

            
    # ------------------------------------------------------------------------------------------------------------------------
    # RETURN 
    # ------------------------------------------------------------------------------------------------------------------------
    response2manouvre4read['ACK2INFO4MPU']        = ack2readinfo4mpu
    response2manouvre4read['CODE2ACK4MPU']        = code2ack
    response2manouvre4read['CODE2NACK4MPU']       = code2nack
    response2manouvre4read['MESSAGE2GET4MPU']     = message2readget4mpu
    response2manouvre4read['MESSAGE2PUT4MPU']     = message2readput4mpu
    response2manouvre4read['MESSAGE2RECEIVE4MPU'] = message2readreceive4mpu
    response2manouvre4read['MESSAGE2SEND4MPU']    = message2readsend4mpu
    response2manouvre4read['STATE2MANOEUVRE4MPU'] = state2read4mpu
                          
    message2print = "{(Step 33/33) returning" + "; response2manouvre4read-returning-by-manoeuvre2readimage4mpu: " + str(response2manouvre4read) + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                             
    #1 {'MESSAGE2SEND4MPU': '<roger4xi>roger2read4mpu</roger4xi>',
    #  'ACK2INFO4MPU': False,
    #  'MESSAGE2GET4MPU': '<default4xi>default2read4mpu</default4xi>',
    #  'MESSAGE2RECEIVE4MPU': '<copy4xi>copy2read4mpu</copy4xi>',
    #  'CODE2ACK4MPU': ' 0D-1C-',
    #  'MESSAGE2PUT4MPU': '<default4xi>default2read4mpu</default4xi>',
    #  'CODE2NACK4MPU': ' ', 'STATE2MANOEUVRE4MPU': 1}; time: 01:44:40 09/29/11 Pacific Daylight Time}
 
    #2 {'MESSAGE2SEND4MPU': '<ready4xi>ready2read4mpu</ready4xi>',
    #  'ACK2INFO4MPU': False,
    #  'MESSAGE2GET4MPU': '<default4xi>default2read4mpu</default4xi>',
    #  'MESSAGE2RECEIVE4MPU': '<ready4xi>ready2read4mpu</ready4xi>',
    #  'CODE2ACK4MPU': ' 0D-2C-',
    #  'MESSAGE2PUT4MPU': '<default4xi>default2read4mpu</default4xi>',
    #  'CODE2NACK4MPU': ' ', 'STATE2MANOEUVRE4MPU': 2}; time: 01:44:40 09/29/11 Pacific Daylight Time}
    
    #3 {'MESSAGE2SEND4MPU': '<ack4xi>ack2imageinfo4mpu</ack4xi>',
    #  'ACK2INFO4MPU': True,
    #  'MESSAGE2GET4MPU': '<default4xi>default2read4mpu</default4xi>',
    #  'MESSAGE2RECEIVE4MPU': '<ready4xi>ready2read4mpu</ready4xi>',
    #  'CODE2ACK4MPU': ' 0D-3C-4E-',
    #  'MESSAGE2PUT4MPU': '<default4xi>default2read4mpu</default4xi>',
    #  'CODE2NACK4MPU': ' ',
    #  'STATE2MANOEUVRE4MPU': 3}; time: 01:44:40 09/29/11 Pacific Daylight Time}
    
    #4 {'MESSAGE2SEND4MPU': '<default4xi>default2read4mpu</default4xi>',
    #  'ACK2INFO4MPU': False,
    #  'MESSAGE2GET4MPU': '<default4xi>default2read4mpu</default4xi>',
    #  'MESSAGE2RECEIVE4MPU': '<default4xi>default2read4mpu</default4xi>',
    #  'CODE2ACK4MPU': ' 0D-',
    #  'MESSAGE2PUT4MPU': '<default4xi>default2read4mpu</default4xi>',
    #  'CODE2NACK4MPU': ' ',
    #  'STATE2MANOEUVRE4MPU': 0}; time: 02:11:49 09/29/11 Pacific Daylight Time}


 
    
    
    return response2manouvre4read           
          
#</function4xi>


# -------------------------------------------------------------------------------------------------------------------------
#<function4xi function = "manoeuvre2writeimage4mpu" usage = "control function for write2image4mpu function in this module">

# ------------------------------------------------------------------------------------------------------------------------- 
def manoeuvre2writeimage4mpu(queue2writemanoeuvre4mpu, consumerchannel2writemanoeuvre4mpu, publisherchannel2writemanoeuvre4mpu):
    '''
    manoeuvres control and command
    '''
    global state2write4mpu                                                       # STATE FOR COMMAND AND CONTROL
    global max2writestate4mpu                                                    # MAX FOR STATE
    global ack2writeinfo4mpu                                                     # ACK FOR IMAGE INFO
    global message2writeget4mpu                                 
    global message2writeput4mpu
    global message2writereceive4mpu
    global message2writesend4mpu
    
    # -----------------------------------------------------------------------------------------
    # (Step 0/33) PARAMETERS            
    # -----------------------------------------------------------------------------------------          
    class2print  = "manoeuvre2writeimage4mpu" 
    method2print = "manoeuvre2writeimage4mpu" 

    code2ack  = ' '
    code2nack = ' '
    
    message2print = "{(Step 0A/33) started" + "; manoeuvre2writeimage4mpu-started: " + str(manoeuvre2writeimage4mpu)  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                                      
    # <function manoeuvre2writeimage4mpu at 0x0000000003DB8C48>  
    
    message2print = "{(Step 0B/33) passed" + "; queue2writemanoeuvre4mpu-to-manoeuvre2writeimage4mpu: " + str(queue2writemanoeuvre4mpu) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                                                              
    # <queue4xi.Queue4Xi object at 0x000000000422D9E8>
    
    message2print = "{(Step 0C/33) passed" + "; consumerchannel2writemanoeuvre4mpu-to-manoeuvre2writeimage4mpu: " + str(consumerchannel2writemanoeuvre4mpu) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                                                              
    # read-write PipeConnection, handle 568>
    
    message2print = "{(Step 0D/33) passed" + "; publisherchannel2writemanoeuvre4mpu-to-manoeuvre2writeimage4mpu: " + str(publisherchannel2writemanoeuvre4mpu) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                                                              
    # <read-write PipeConnection, handle 572> 
    
    code2ack  = code2ack + '0D-' 
    response2manouvre4write = {}
                    
    # ------------------------------------------------------------------------------------------------------------------------
    # SETTINGS
    # ------------------------------------------------------------------------------------------------------------------------    
    message2print = "{(Step 0E/70) try" + "; state2write4mpu-try-by-manoeuvre2writeimage4mpu"  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)     
    try:
        if state2write4mpu is None:
            state2write4mpu = 0
            message2print = "{(Step 0F/33) initialized" + "; state2write4mpu-initialized-by-manoeuvre2writeimage4mpu to: " + str(state2write4mpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)    
        else:
            message2print = "{(Step 0G/33) used" + "; state2write4mpu-used-by-manoeuvre2writeimage4mpu: " + str(state2write4mpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # 1; 2; 3;  
    except:
        state2write4mpu = 0
        message2print = "{(Step 0Z1/33) initialized" + "; state2write4mpu-initialized-by-manoeuvre2writeimage4mpu to: " + str(state2write4mpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # 0
        ack2writeinfo4mpu        = False
        message2print = "{(Step 0Z2/33) initialized" + "; ack2writeinfo4mpu-initialized-by-manoeuvre2writeimage4mpu to: " + str(ack2writeinfo4mpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                     
        # False
        message2writeget4mpu     = '<default4xi>default2write4mpu</default4xi>'
        message2print = "{(Step 0Z3/33) initialized" + "; message2writeget4mpu-initialized-by-manoeuvre2writeimage4mpu to: " + str(message2writeget4mpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                               
        # <default4xi>default2write4mpu</default4xi>
        message2writeput4mpu     = '<default4xi>default2write4mpu</default4xi>'
        message2print = "{(Step 0Z4/33) initialized" + "; message2writeput4mpu-initialized-by-manoeuvre2writeimage4mpu to: " + str(message2writeput4mpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
        # <default4xi>default2write4mpu</default4xi>
        message2writereceive4mpu = '<default4xi>default2write4mpu</default4xi>'
        message2print = "{(Step 0Z5/33) initialized" + "; message2writereceive4mpu-initialized-by-manoeuvre2writeimage4mpu to: " + str(message2writereceive4mpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
        # <default4xi>default2write4mpu</default4xi>
        message2writesend4mpu    = '<default4xi>default2write4mpu</default4xi>'
        message2print = "{(Step 0Z6/33) initialized" + "; message2writesend4mpu-initialized-by-manoeuvre2writeimage4mpu to: " + str(message2writesend4mpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
        # <default4xi>default2write4mpu</default4xi>
        max2writestate4mpu = 3
        message2print = "{(Step 0Z7/33) initialized" + ";  max2writestate4mpu-initialized-by-manoeuvre2writeimage4mpu to: " + str( max2writestate4mpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # 
                    
    # ------------------------------------------------------------------------------------------------------------------------
    # STATE UPDATE
    # ------------------------------------------------------------------------------------------------------------------------                      
    state2write4mpu = state2write4mpu + 1                                                                   
    message2print = "{(Step 0H/33) added" + "; state2write4mpu-added-by-manoeuvre2writeimage4mpu: " + str(state2write4mpu) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)
    # 1; 2; 3; 4  
    if state2write4mpu > max2writestate4mpu:
        state2write4mpu = 0
        message2print = "{(Step 0I/33) set" + "; state2write4mpu-set-by-manoeuvre2writeimage4mpu to: " + str(state2write4mpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # 0
        ack2writeinfo4mpu        = False
        message2print = "{(Step 0J/33) set" + "; ack2writeinfo4mpu-set-by-manoeuvre2writeimage4mpu to: " + str(ack2writeinfo4mpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                     
        # False
        message2writeget4mpu     = '<default4xi>default2write4mpu</default4xi>'
        message2print = "{(Step 0K/33) set" + "; message2writeget4mpu-set-by-manoeuvre2writeimage4mpu to: " + str(message2writeget4mpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                               
        # <default4xi>default2write4mpu</default4xi>
        message2writeput4mpu     = '<default4xi>default2write4mpu</default4xi>'
        message2print = "{(Step 0L/33) set" + "; message2writeput4mpu-set-by-manoeuvre2writeimage4mpu to: " + str(message2writeput4mpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
        # <default4xi>default2write4mpu</default4xi>
        message2writereceive4mpu = '<default4xi>default2write4mpu</default4xi>'
        message2print = "{(Step 0M/33) set" + "; message2writereceive4mpu-set-by-manoeuvre2writeimage4mpu to: " + str(message2writereceive4mpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
        # <default4xi>default2write4mpu</default4xi>
        message2writesend4mpu    = '<default4xi>default2write4mpu</default4xi>'
        message2print = "{(Step 0N/33) set" + "; message2writesend4mpu-set-by-manoeuvre2writeimage4mpu to: " + str(message2writesend4mpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
        # <default4xi>default2write4mpu</default4xi>
        max2writestate4mpu = 3
        message2print = "{(Step 0O/33) initialized" + ";  max2writestate4mpu-set-by-manoeuvre2writeimage4mpu to: " + str( max2writestate4mpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        #     
                    
    # ------------------------------------------------------------------------------------------------------------------------
    # STATE I - RECEIVE CONTROL AND COMMAND COPY MESSAGE VIA PIPE AND SEND ROGER OR NEGATIVE BACK 
    # ------------------------------------------------------------------------------------------------------------------------
    if state2write4mpu == 1:
        message2print = "{(Step 1A/70) try" + "; receive-try-by-manoeuvre2writeimage4mpu"  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        message2writereceive4mpu = ''
        try:
            # ----------------------------------------------------------------------------------------------------------------
            # RECEIVE CONTROL AND COMMAND MESSAGE VIA PIPE
            # ----------------------------------------------------------------------------------------------------------------         
            message2writereceive4mpu = consumerchannel2writemanoeuvre4mpu.recv()
            message2print = "{(Step 1B/33) received" + "; message2writereceive4mpu-received-by-manoeuvre2writeimage4mpu: " + str(message2writereceive4mpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            # <copy4xi>copy2write4mpu</copy4xi>
                         
            if message2writereceive4mpu == '<copy4xi>copy2write4mpu</copy4xi>':
                message2writesend4mpu = '<roger4xi>roger2write4mpu</roger4xi>'
            else:
                message2writesend4mpu = '<negative4xi><negative2write4mpu</<negative4xi>'
            # ----------------------------------------------------------------------------------------------------------------
            # SEND ROGER OR NEGATIVE BACK 
            # ----------------------------------------------------------------------------------------------------------------                                
            publisherchannel2writemanoeuvre4mpu.send(message2writesend4mpu)
            
            message2print = "{(Step 1C/33) sent" + "; message2writesend4mpu-sent-by-manoeuvre2writeimage4mpu: " + str(message2writesend4mpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)          
            # <roger4xi>roger2write4mpu</roger4xi>
            
            code2ack  = code2ack + '1C-'
        except:
            message2print = "{(Step 1Z/33) failed" + "; message2writereceive4mpu-failed-by-manoeuvre2writeimage4mpu: " + str(message2writereceive4mpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '1Z-' 
    else:
        pass

                    
    # ------------------------------------------------------------------------------------------------------------------------
    # STATE II - RECEIVE CONTROL AND COMMAND READY MESSAGE VIA PIPE AND SEND READY OR NEGATIVE BACK 
    # ------------------------------------------------------------------------------------------------------------------------
    if state2write4mpu == 2:
        message2print = "{(Step 2A/70) try" + "; receive-try-by-manoeuvre2writeimage4mpu"  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        message2writereceive4mpu = ''
        try:
            # ----------------------------------------------------------------------------------------------------------------
            # RECEIVE CONTROL AND COMMAND MESSAGE VIA PIPE
            # ----------------------------------------------------------------------------------------------------------------         
            message2writereceive4mpu = consumerchannel2writemanoeuvre4mpu.recv()
            message2print = "{(Step 2B/33) received" + "; message2writereceive4mpu-received-by-manoeuvre2writeimage4mpu: " + str(message2writereceive4mpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            # <ready4xi>ready2write4mpu</ready4xi>
                         
            if message2writereceive4mpu == '<ready4xi>ready2write4mpu</ready4xi>':
                message2writesend4mpu = '<ready4xi>ready2write4mpu</ready4xi>'
            else:
                message2writesend4mpu = '<negative4xi><negative2ready4mpu</<negative4xi>'
            # ----------------------------------------------------------------------------------------------------------------
            # SEND ROGER OR NEGATIVE BACK 
            # ----------------------------------------------------------------------------------------------------------------                                
            publisherchannel2writemanoeuvre4mpu.send(message2writesend4mpu)
            
            message2print = "{(Step 2C/33) sent" + "; message2writesend4mpu-sent-by-manoeuvre2writeimage4mpu: " + str(message2writesend4mpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)          
            # <ready4xi>ready2write4mpu</ready4xi>
            
            code2ack  = code2ack + '2C-'
        except:
            message2print = "{(Step 2Z/33) failed" + "; message2writereceive4mpu-failed-by-manoeuvre2writeimage4mpu: " + str(message2writereceive4mpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '2Z-' 
    else:
        pass        
                
    # ------------------------------------------------------------------------------------------------------------------------
    # STATE III - GET IMAGE INFORMATION SUCH AS IMAGE PATH VIA QUEUE AND SEND ACK OR NACK VIA PIPE
    # ------------------------------------------------------------------------------------------------------------------------
    if state2write4mpu == 3:    
        info2image4mpu = None
        message2print = "{(Step 3A/70) try" + "; info2image4mpu-try-by-manoeuvre2writeimage4mpu"  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        try: 
            # ----------------------------------------------------------------------------------------------------------------
            # GET IMAGE INFORMATION SUCH AS IMAGE PATH VIA QUEUE
            # ----------------------------------------------------------------------------------------------------------------        
            info2image4mpu = queue2writemanoeuvre4mpu.get()
            
            message2print = "{(Step 3B/33) got" + "; info2image4mpu-got-by-manoeuvre2writeimage4mpu: " + str(info2image4mpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            # ..//..//..//..//..//web//images//png//dragonxi.png
            if info2image4mpu is None:
                code2nack  = code2nack + '3Z1-'
                ack2writeinfo4mpu = False 
            else:    
                code2ack  = code2ack + '3C-'
                ack2writeinfo4mpu = True
        except:
            message2print = "{(Step 3Z1/33) failed" + "; queue2writemanoeuvre4mpu-failed-by-manoeuvre2writeimage4mpu: " + str(queue2writemanoeuvre4mpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '3Z1-'
            
        message2writesend4mpu = ' '
        try:
            # ----------------------------------------------------------------------------------------------------------------
            # SEND CONTROL INFO ABOUT IMAGE OPERATION - EITHER ACK OR NACK  
            # ----------------------------------------------------------------------------------------------------------------         
            if ack2writeinfo4mpu is True:
                message2writesend4mpu = '<ack4xi>ack2imageinfo4mpu</ack4xi>'
            else:
                message2writesend4mpu = '<nack4xi>nack2imageinfo4mpu</nack4xi>'
            message2print = "{(Step 4B/33) set" + "; message2writesend4mpu-set-by-manoeuvre2writeimage4mpu to: " + str(message2writesend4mpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # <ack4xi>ack2imageinfo4mpu</ack4xi>       
        except:
            message2print = "{(Step 4Z1/33) failed" + "; message2writesend4mpu-failed-by-manoeuvre2writeimage4mpu: " + str(message2writesend4mpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            
            code2nack  = code2nack + '4Z1-'    
        try:                
            message2print = "{(Step 4C/70) using" + "; publisherchannel2writemanoeuvre4mpu-calling-by-manoeuvre2writeimage4mpu: " + str(publisherchannel2writemanoeuvre4mpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)    
            # <read-write PipeConnection, handle 412>
            message2print = "{{(Step 4D/70) passing" + "; message2writesend4mpu-passing-to-send-by-manoeuvre2writeimage4mpu: " + str(message2writesend4mpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #              
        except:
            message2print = "{(Step 4Z1/33) failed" + "; message2writesend4mpu-failed-by-manoeuvre2writeimage4mpu: " + str(message2writesend4mpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '4Z2-'                
        try:    
            publisherchannel2writemanoeuvre4mpu.send(message2writesend4mpu)
            
            message2print = "{(Step 4E/33) sent" + "; message2writesend4mpu-sent-by-manoeuvre2writeimage4mpu: " + str(message2writesend4mpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)          
            # <ack4xi>ack2imageinfo4mpu</ack4xi>          
            code2ack  = code2ack + '4E-'
        except:
            message2print = "{(Step 4Z3/33) failed" + "; message2writesend4mpu-failed-by-manoeuvre2writeimage4mpu: " + str(message2writesend4mpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '4Z3-'                
    else:
        pass
        

            
    # ------------------------------------------------------------------------------------------------------------------------
    # RETURN 
    # ------------------------------------------------------------------------------------------------------------------------
    response2manouvre4write['ACK2INFO4MPU']        = ack2writeinfo4mpu
    response2manouvre4write['CODE2ACK4MPU']        = code2ack
    response2manouvre4write['CODE2NACK4MPU']       = code2nack
    response2manouvre4write['MESSAGE2GET4MPU']     = message2writeget4mpu
    response2manouvre4write['MESSAGE2PUT4MPU']     = message2writeput4mpu
    response2manouvre4write['MESSAGE2RECEIVE4MPU'] = message2writereceive4mpu
    response2manouvre4write['MESSAGE2SEND4MPU']    = message2writesend4mpu
    response2manouvre4write['STATE2MANOEUVRE4MPU'] = state2write4mpu
                          
    message2print = "{(Step 33/33) returning" + "; response2manouvre4write-returning-by-manoeuvre2writeimage4mpu: " + str(response2manouvre4write) + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                             
    #1
    # {'MESSAGE2SEND4MPU': '<roger4xi>roger2write4mpu</roger4xi>', 
    #  'ACK2INFO4MPU': False, 'MESSAGE2GET4MPU': '<default4xi>default2write4mpu</default4xi>', 
    #  'MESSAGE2RECEIVE4MPU': '<copy4xi>copy2write4mpu</copy4xi>', 
    #  'CODE2ACK4MPU': ' 0D-1C-', 
    #  'MESSAGE2PUT4MPU': '<default4xi>default2write4mpu</default4xi>', 
    #  'CODE2NACK4MPU': ' ', 
    #  'STATE2MANOEUVRE4MPU': 1}; time: 00:11:53 10/03/11 Pacific Daylight Time}
    #2
    # {'MESSAGE2SEND4MPU': '<ready4xi>ready2write4mpu</ready4xi>', 
    #  'ACK2INFO4MPU': False, 'MESSAGE2GET4MPU': '<default4xi>default2write4mpu</default4xi>', 
    #  'MESSAGE2RECEIVE4MPU': '<ready4xi>ready2write4mpu</ready4xi>', 
    #  'CODE2ACK4MPU': ' 0D-2C-', 
    #  'MESSAGE2PUT4MPU': '<default4xi>default2write4mpu</default4xi>', 
    #  'CODE2NACK4MPU': ' ', 
    #  'STATE2MANOEUVRE4MPU': 2}; time: 00:11:53 10/03/11 Pacific Daylight Time}
    #3
    # {'MESSAGE2SEND4MPU': '<ack4xi>ack2imageinfo4mpu</ack4xi>', 
    #  'ACK2INFO4MPU': True, 'MESSAGE2GET4MPU': '<default4xi>default2write4mpu</default4xi>', 
    #  'MESSAGE2RECEIVE4MPU': '<ready4xi>ready2write4mpu</ready4xi>', 
    #  'CODE2ACK4MPU': ' 0D-3C-4E-', 
    #  'MESSAGE2PUT4MPU': '<default4xi>default2write4mpu</default4xi>', 
    #  'CODE2NACK4MPU': ' ', 
    #  'STATE2MANOEUVRE4MPU': 3}; time: 00:11:53 10/03/11 Pacific Daylight Time}

    return response2manouvre4write           
          
#</function4xi>

# -------------------------------------------------------------------------------------------------------------------------
#<function4xi function = "manoeuvre2mapimage4mpu" usage = "control function for map2image4mpu function in this module">

# ------------------------------------------------------------------------------------------------------------------------- 
def manoeuvre2mapimage4mpu(queue2mapmanoeuvre4mpu, consumerchannel2mapmanoeuvre4mpu, publisherchannel2mapmanoeuvre4mpu):
    '''
    manoeuvres control and command
    '''
    global state2map4mpu                                                       # STATE FOR COMMAND AND CONTROL
    global max2mapstate4mpu                                                    # MAX FOR STATE
    global ack2mapinfo4mpu                                                     # ACK FOR IMAGE INFO
    global message2mapget4mpu                                 
    global message2mapput4mpu
    global message2mapreceive4mpu
    global message2mapsend4mpu
    
    # -----------------------------------------------------------------------------------------
    # (Step 0/33) PARAMETERS            
    # -----------------------------------------------------------------------------------------          
    class2print  = "manoeuvre2mapimage4mpu" 
    method2print = "manoeuvre2mapimage4mpu" 

    code2ack  = ' '
    code2nack = ' '
    
    message2print = "{(Step 0A/33) started" + "; manoeuvre2mapimage4mpu-started: " + str(manoeuvre2mapimage4mpu)  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                                      
    #  
    
    message2print = "{(Step 0B/33) passed" + "; queue2mapmanoeuvre4mpu-to-manoeuvre2mapimage4mpu: " + str(queue2mapmanoeuvre4mpu) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                                                              
    # 
    
    message2print = "{(Step 0C/33) passed" + "; consumerchannel2mapmanoeuvre4mpu-to-manoeuvre2mapimage4mpu: " + str(consumerchannel2mapmanoeuvre4mpu) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                                                              
    # 
    
    message2print = "{(Step 0D/33) passed" + "; publisherchannel2mapmanoeuvre4mpu-to-manoeuvre2mapimage4mpu: " + str(publisherchannel2mapmanoeuvre4mpu) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                                                              
    # 
    
    code2ack  = code2ack + '0D-' 
    response2manouvre4map = {}
                    
    # ------------------------------------------------------------------------------------------------------------------------
    # SETTINGS
    # ------------------------------------------------------------------------------------------------------------------------    
    message2print = "{(Step 0E/70) try" + "; state2map4mpu-try-by-manoeuvre2mapimage4mpu"  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)     
    try:
        if state2map4mpu is None:
            state2map4mpu = 0
            message2print = "{(Step 0F/33) initialized" + "; state2map4mpu-initialized-by-manoeuvre2mapimage4mpu to: " + str(state2map4mpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)    
        else:
            message2print = "{(Step 0G/33) used" + "; state2map4mpu-used-by-manoeuvre2mapimage4mpu: " + str(state2map4mpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # 1; 2; 3;  
    except:
        state2map4mpu = 0
        message2print = "{(Step 0Z1/33) initialized" + "; state2map4mpu-initialized-by-manoeuvre2mapimage4mpu to: " + str(state2map4mpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # 0
        ack2mapinfo4mpu        = False
        message2print = "{(Step 0Z2/33) initialized" + "; ack2mapinfo4mpu-initialized-by-manoeuvre2mapimage4mpu to: " + str(ack2mapinfo4mpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                     
        # False
        message2mapget4mpu     = '<default4xi>default2map4mpu</default4xi>'
        message2print = "{(Step 0Z3/33) initialized" + "; message2mapget4mpu-initialized-by-manoeuvre2mapimage4mpu to: " + str(message2mapget4mpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                               
        # <default4xi>default2map4mpu</default4xi>
        message2mapput4mpu     = '<default4xi>default2map4mpu</default4xi>'
        message2print = "{(Step 0Z4/33) initialized" + "; message2mapput4mpu-initialized-by-manoeuvre2mapimage4mpu to: " + str(message2mapput4mpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
        # <default4xi>default2map4mpu</default4xi>
        message2mapreceive4mpu = '<default4xi>default2map4mpu</default4xi>'
        message2print = "{(Step 0Z5/33) initialized" + "; message2mapreceive4mpu-initialized-by-manoeuvre2mapimage4mpu to: " + str(message2mapreceive4mpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
        # <default4xi>default2map4mpu</default4xi>
        message2mapsend4mpu    = '<default4xi>default2map4mpu</default4xi>'
        message2print = "{(Step 0Z6/33) initialized" + "; message2mapsend4mpu-initialized-by-manoeuvre2mapimage4mpu to: " + str(message2mapsend4mpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
        # <default4xi>default2map4mpu</default4xi>
        max2mapstate4mpu = 3
        message2print = "{(Step 0Z7/33) initialized" + ";  max2mapstate4mpu-initialized-by-manoeuvre2mapimage4mpu to: " + str( max2mapstate4mpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # 
                    
    # ------------------------------------------------------------------------------------------------------------------------
    # STATE UPDATE
    # ------------------------------------------------------------------------------------------------------------------------                      
    state2map4mpu = state2map4mpu + 1                                                                   
    message2print = "{(Step 0H/33) added" + "; state2map4mpu-added-by-manoeuvre2mapimage4mpu: " + str(state2map4mpu) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)
    # 1; 2; 3; 4  
    if state2map4mpu > max2mapstate4mpu:
        state2map4mpu = 0
        message2print = "{(Step 0I/33) set" + "; state2map4mpu-set-by-manoeuvre2mapimage4mpu to: " + str(state2map4mpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # 0
        ack2mapinfo4mpu        = False
        message2print = "{(Step 0J/33) set" + "; ack2mapinfo4mpu-set-by-manoeuvre2mapimage4mpu to: " + str(ack2mapinfo4mpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                     
        # False
        message2mapget4mpu     = '<default4xi>default2map4mpu</default4xi>'
        message2print = "{(Step 0K/33) set" + "; message2mapget4mpu-set-by-manoeuvre2mapimage4mpu to: " + str(message2mapget4mpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                               
        # <default4xi>default2map4mpu</default4xi>
        message2mapput4mpu     = '<default4xi>default2map4mpu</default4xi>'
        message2print = "{(Step 0L/33) set" + "; message2mapput4mpu-set-by-manoeuvre2mapimage4mpu to: " + str(message2mapput4mpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
        # <default4xi>default2map4mpu</default4xi>
        message2mapreceive4mpu = '<default4xi>default2map4mpu</default4xi>'
        message2print = "{(Step 0M/33) set" + "; message2mapreceive4mpu-set-by-manoeuvre2mapimage4mpu to: " + str(message2mapreceive4mpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
        # <default4xi>default2map4mpu</default4xi>
        message2mapsend4mpu    = '<default4xi>default2map4mpu</default4xi>'
        message2print = "{(Step 0N/33) set" + "; message2mapsend4mpu-set-by-manoeuvre2mapimage4mpu to: " + str(message2mapsend4mpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
        # <default4xi>default2map4mpu</default4xi>
        max2mapstate4mpu = 3
        message2print = "{(Step 0O/33) initialized" + ";  max2mapstate4mpu-set-by-manoeuvre2mapimage4mpu to: " + str( max2mapstate4mpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        #     
                    
    # ------------------------------------------------------------------------------------------------------------------------
    # STATE I - RECEIVE CONTROL AND COMMAND COPY MESSAGE VIA PIPE AND SEND ROGER OR NEGATIVE BACK 
    # ------------------------------------------------------------------------------------------------------------------------
    if state2map4mpu == 1:
        message2print = "{(Step 1A/70) try" + "; receive-try-by-manoeuvre2mapimage4mpu"  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        message2mapreceive4mpu = ''
        try:
            # ----------------------------------------------------------------------------------------------------------------
            # RECEIVE CONTROL AND COMMAND MESSAGE VIA PIPE
            # ----------------------------------------------------------------------------------------------------------------         
            message2mapreceive4mpu = consumerchannel2mapmanoeuvre4mpu.recv()
            message2print = "{(Step 1B/33) received" + "; message2mapreceive4mpu-received-by-manoeuvre2mapimage4mpu: " + str(message2mapreceive4mpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            # 
                         
            if message2mapreceive4mpu == '<copy4xi>copy2mapimage4mpu</copy4xi>':
                message2mapsend4mpu = '<roger4xi>roger2mapimage4mpu</roger4xi>'
            else:
                message2mapsend4mpu = '<negative4xi><negative2mapimage4mpu</<negative4xi>'
            # ----------------------------------------------------------------------------------------------------------------
            # SEND ROGER OR NEGATIVE BACK 
            # ----------------------------------------------------------------------------------------------------------------                                
            publisherchannel2mapmanoeuvre4mpu.send(message2mapsend4mpu)
            
            message2print = "{(Step 1C/33) sent" + "; message2mapsend4mpu-sent-by-manoeuvre2mapimage4mpu: " + str(message2mapsend4mpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)          
            # 
            
            code2ack  = code2ack + '1C-'
        except:
            message2print = "{(Step 1Z/33) failed" + "; message2mapreceive4mpu-failed-by-manoeuvre2mapimage4mpu: " + str(message2mapreceive4mpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '1Z-' 
    else:
        pass

                    
    # ------------------------------------------------------------------------------------------------------------------------
    # STATE II - RECEIVE CONTROL AND COMMAND READY MESSAGE VIA PIPE AND SEND READY OR NEGATIVE BACK 
    # ------------------------------------------------------------------------------------------------------------------------
    if state2map4mpu == 2:
        message2print = "{(Step 2A/70) try" + "; receive-try-by-manoeuvre2mapimage4mpu"  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        message2mapreceive4mpu = ''
        try:
            # ----------------------------------------------------------------------------------------------------------------
            # RECEIVE CONTROL AND COMMAND MESSAGE VIA PIPE
            # ----------------------------------------------------------------------------------------------------------------         
            message2mapreceive4mpu = consumerchannel2mapmanoeuvre4mpu.recv()
            message2print = "{(Step 2B/33) received" + "; message2mapreceive4mpu-received-by-manoeuvre2mapimage4mpu: " + str(message2mapreceive4mpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            # 
                         
            if message2mapreceive4mpu == '<ready4xi>ready2mapimage4mpu</ready4xi>':
                message2mapsend4mpu = '<ready4xi>ready2mapimage4mpu</ready4xi>'
            else:
                message2mapsend4mpu = '<negative4xi><negative2ready4mpu</<negative4xi>'
            # ----------------------------------------------------------------------------------------------------------------
            # SEND ROGER OR NEGATIVE BACK 
            # ----------------------------------------------------------------------------------------------------------------                                
            publisherchannel2mapmanoeuvre4mpu.send(message2mapsend4mpu)
            
            message2print = "{(Step 2C/33) sent" + "; message2mapsend4mpu-sent-by-manoeuvre2mapimage4mpu: " + str(message2mapsend4mpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)          
            # 
            
            code2ack  = code2ack + '2C-'
        except:
            message2print = "{(Step 2Z/33) failed" + "; message2mapreceive4mpu-failed-by-manoeuvre2mapimage4mpu: " + str(message2mapreceive4mpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '2Z-' 
    else:
        pass        
                
    # ------------------------------------------------------------------------------------------------------------------------
    # STATE III - GET IMAGE INFORMATION SUCH AS IMAGE PATH VIA QUEUE AND SEND ACK OR NACK VIA PIPE
    # ------------------------------------------------------------------------------------------------------------------------
    if state2map4mpu == 3:    
        info2image4mpu = None
        message2print = "{(Step 3A/70) try" + "; info2image4mpu-try-by-manoeuvre2mapimage4mpu"  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        try: 
            # ----------------------------------------------------------------------------------------------------------------
            # GET IMAGE INFORMATION SUCH AS IMAGE PATH VIA QUEUE
            # ----------------------------------------------------------------------------------------------------------------        
            info2image4mpu = queue2mapmanoeuvre4mpu.get()
            
            message2print = "{(Step 3B/33) got" + "; info2image4mpu-got-by-manoeuvre2mapimage4mpu: " + str(info2image4mpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            # 
            if info2image4mpu is None:
                code2nack  = code2nack + '3Z1-'
                ack2mapinfo4mpu = False 
            else:    
                code2ack  = code2ack + '3C-'
                ack2mapinfo4mpu = True
        except:
            message2print = "{(Step 3Z1/33) failed" + "; queue2mapmanoeuvre4mpu-failed-by-manoeuvre2mapimage4mpu: " + str(queue2mapmanoeuvre4mpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '3Z1-'
            
        message2mapsend4mpu = ' '
        try:
            # ----------------------------------------------------------------------------------------------------------------
            # SEND CONTROL INFO ABOUT IMAGE OPERATION - EITHER ACK OR NACK  
            # ----------------------------------------------------------------------------------------------------------------         
            if ack2mapinfo4mpu is True:
                message2mapsend4mpu = '<ack4xi>ack2imageinfo4mpu</ack4xi>'
            else:
                message2mapsend4mpu = '<nack4xi>nack2imageinfo4mpu</nack4xi>'
            message2print = "{(Step 4B/33) set" + "; message2mapsend4mpu-set-by-manoeuvre2mapimage4mpu to: " + str(message2mapsend4mpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            #      
        except:
            message2print = "{(Step 4Z1/33) failed" + "; message2mapsend4mpu-failed-by-manoeuvre2mapimage4mpu: " + str(message2mapsend4mpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            
            code2nack  = code2nack + '4Z1-'    
        try:                
            message2print = "{(Step 4C/70) using" + "; publisherchannel2mapmanoeuvre4mpu-calling-by-manoeuvre2mapimage4mpu: " + str(publisherchannel2mapmanoeuvre4mpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)    
            # 
            message2print = "{{(Step 4D/70) passing" + "; message2mapsend4mpu-passing-to-send-by-manoeuvre2mapimage4mpu: " + str(message2mapsend4mpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #              
        except:
            message2print = "{(Step 4Z1/33) failed" + "; message2mapsend4mpu-failed-by-manoeuvre2mapimage4mpu: " + str(message2mapsend4mpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '4Z2-'                
        try:    
            publisherchannel2mapmanoeuvre4mpu.send(message2mapsend4mpu)
            
            message2print = "{(Step 4E/33) sent" + "; message2mapsend4mpu-sent-by-manoeuvre2mapimage4mpu: " + str(message2mapsend4mpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)          
            #          
            code2ack  = code2ack + '4E-'
        except:
            message2print = "{(Step 4Z3/33) failed" + "; message2mapsend4mpu-failed-by-manoeuvre2mapimage4mpu: " + str(message2mapsend4mpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '4Z3-'                
    else:
        pass
        

            
    # ------------------------------------------------------------------------------------------------------------------------
    # RETURN 
    # ------------------------------------------------------------------------------------------------------------------------
    response2manouvre4map['ACK2INFO4MPU']        = ack2mapinfo4mpu
    response2manouvre4map['CODE2ACK4MPU']        = code2ack
    response2manouvre4map['CODE2NACK4MPU']       = code2nack
    response2manouvre4map['MESSAGE2GET4MPU']     = message2mapget4mpu
    response2manouvre4map['MESSAGE2PUT4MPU']     = message2mapput4mpu
    response2manouvre4map['MESSAGE2RECEIVE4MPU'] = message2mapreceive4mpu
    response2manouvre4map['MESSAGE2SEND4MPU']    = message2mapsend4mpu
    response2manouvre4map['STATE2MANOEUVRE4MPU'] = state2map4mpu
                          
    message2print = "{(Step 33/33) returning" + "; response2manouvre4map-returning-by-manoeuvre2mapimage4mpu: " + str(response2manouvre4map) + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                             
    #1 {'MESSAGE2SEND4MPU': '<roger4xi>roger2mapimage4mpu</roger4xi>', 
    #  'ACK2INFO4MPU': False, 
    #  'MESSAGE2GET4MPU': '<default4xi>default2map4mpu</default4xi>', 
    #  'MESSAGE2RECEIVE4MPU': '<copy4xi>copy2mapimage4mpu</copy4xi>', 
    #  'CODE2ACK4MPU': ' 0D-1C-', 
    #  'MESSAGE2PUT4MPU': '<default4xi>default2map4mpu</default4xi>', 
    #  'CODE2NACK4MPU': ' ', 
    #  'STATE2MANOEUVRE4MPU': 1}
     
    #2 {'MESSAGE2SEND4MPU': '<ready4xi>ready2mapimage4mpu</ready4xi>', 
    #  'ACK2INFO4MPU': False, 
    #  'MESSAGE2GET4MPU': '<default4xi>default2map4mpu</default4xi>', 
    #  'MESSAGE2RECEIVE4MPU': '<ready4xi>ready2mapimage4mpu</ready4xi>', 
    #  'CODE2ACK4MPU': ' 0D-2C-', 
    #  'MESSAGE2PUT4MPU': '<default4xi>default2map4mpu</default4xi>', 
    #  'CODE2NACK4MPU': ' ', 
    #  'STATE2MANOEUVRE4MPU': 2}; time: 06:49:48 10/04/11 Pacific Daylight Time}
     
    #3 {'MESSAGE2SEND4MPU': '<ack4xi>ack2imageinfo4mpu</ack4xi>', 
    #  'ACK2INFO4MPU': True, 
    #  'MESSAGE2GET4MPU': '<default4xi>default2map4mpu</default4xi>', 
    #  'MESSAGE2RECEIVE4MPU': '<ready4xi>ready2mapimage4mpu</ready4xi>', 
    #  'CODE2ACK4MPU': ' 0D-3C-4E-', 
    #  'MESSAGE2PUT4MPU': '<default4xi>default2map4mpu</default4xi>', 
    #  'CODE2NACK4MPU': ' ', 
    #  'STATE2MANOEUVRE4MPU': 3}; time: 06:49:48 10/04/11 Pacific Daylight Time}
    
    #4 {'MESSAGE2SEND4MPU': '<default4xi>default2map4mpu</default4xi>', 
    #  'ACK2INFO4MPU': False, 
    #  'MESSAGE2GET4MPU': '<default4xi>default2map4mpu</default4xi>', 
    #  'MESSAGE2RECEIVE4MPU': '<default4xi>default2map4mpu</default4xi>', 
    #  'CODE2ACK4MPU': ' 0D-', 
    #  'MESSAGE2PUT4MPU': '<default4xi>default2map4mpu</default4xi>', 
    #  'CODE2NACK4MPU': ' ', 'STATE2MANOEUVRE4MPU': 0}; time: 06:49:48 10/04/11 Pacific Daylight Time}

    return response2manouvre4map           
          
#</function4xi>

# -------------------------------------------------------------------------------------------------
#<function4xi function = "write2image4mpu" usage = "
#                                                                                                              
#   # ---------------------------------------------------------------------------------------------
#   # (16) ESTABLISH CONTROL & COMMAND WITH READ2IMAGE4MPU                      
#   # ---------------------------------------------------------------------------------------------    
#   print('(Step 16A/700) OF MPU2IMAGE4XI5TEST - TRYING TO ESTABLISH CONTROL & COMMAND WITH READ2IMAGE4MPU ', time4xi())    
#   respon4write2image = {} 
#   task4write2image = None
#   try:                               
#       response4write2image = write2image4mpu(task4write2image)                                                                                                          
#       print('(Step 16B/700) OF MPU2IMAGE4XI5TEST - ESTABLISHED CONTROL & COMMAND WITH READ2IMAGE4MPU : ', response4write2image)
#       queue2write4mpu      = response4write2image['QUEUE']
#       channel2publish4mpu = response4write2image['PUBLISHERCHANNEL']
#       channel2consume4mpu = response4write2image['CONSUMERCHANNEL']
#   except:
#       print('(Step 16Z/700) OF MPU2IMAGE4XI5TEST *** FAILED TO ESTABLISH CONTROL & COMMAND WITH READ2IMAGE4MPU ', time4xi()) 
#   # --------------------------------------------------------------------------------------------------
#   # (17) SEND CONTROL & COMMAND COPY VIA PIPE FOR MANOEUVRE2IMAGE4MPU TO RECEIVE                 
#   # --------------------------------------------------------------------------------------------------      
#   print('(Step 17A/700) OF MPU2IMAGE4XI5TEST - TRYING TO SEND CONTROL & COMMAND VIA PIPE WITH READ2IMAGE4MPU ', time4xi())     
#   try:                               
#       message2writesend4mpu = "<copy4xi>copy2write4mpu</copy4xi>"
#       channel2publish4mpu = response4write2image['PUBLISHERCHANNEL']
#       channel2publish4mpu.send(message2writesend4mpu)
#       print("(Step 17B/700) OF MPU2IMAGE4XI5TEST - SENT TO CHANNEL2PUBLISH4MPU: ", channel2publish4mpu, " MESSAGE2SEND4MPU: " , message2writesend4mpu)                                                                                                
#   except:
#       print('(Step 17Z1/700) OF MPU2IMAGE4XI5TEST *** FAILED TO SEND CONTROL & COMMAND VIA PIPE WITH READ2IMAGE4MPU ', time4xi())                                                
#   # ------------------------------------------------------------------------------------------
#   # (18) LET MANOEUVRE2IMAGE4MPU TO RECEIVE COPY AND AND SEND ROGER VIA PIPE                
#   # ------------------------------------------------------------------------------------------       
#   print("(Step 18C/700) OF MPU2IMAGE4XI5TEST - TRYING TO PERFORM TARGET FUNCTION MANOEUVRE2IMAGE4MPU ", time4xi()) 
#   try:        
#       response2manoeuvre4mpu = manoeuvre2readimage4mpu(queue2write4mpu, channel2consume4mpu, channel2publish4mpu)
#       print("(Step 18D/700) OF MPU2IMAGE4XI5TEST - PERFORMED TARGET FUNCTION MANOEUVRE2IMAGE4MPU ", manoeuvre2readimage4mpu)
#   except:
#       print("(Step 18Z/700) OF MPU2IMAGE4XI5TEST *** FAILED TO PERFORM TARGET FUNCTION MANOEUVRE2IMAGE4MPU ", time4xi())     
#   else:
#       pass # trust activated thread    
#   # ---------------------------------------------------------------------------------------------
#   # (19) RECEIVE CONTROL & COMMAND ROGER VIA PIPE                   
#   # ---------------------------------------------------------------------------------------------          
#   print('(Step 19A/700) OF MPU2IMAGE4XI5TEST - TRYING TO RECEIVE CONTROL & COMMAND VIA PIPE WITH READ2IMAGE4MPU ', time4xi())     
#   try:
#       channel2consume4mpu = response4write2image['CONSUMERCHANNEL']                              
#       message2writereceive4mpu = channel2consume4mpu.recv()
#       print("(Step 19B/700) OF MPU2IMAGE4XI5TEST - RECEIVED FROM CHANNEL2CONSUME4MPU: ", channel2consume4mpu, " CONTROL & COMMAND: ", message2writereceive4mpu)                                                                                                  
#       # <roger4xi>roger2write4mpu</roger4xi>
#   except:
#       print('(Step 19Z/700) OF MPU2IMAGE4XI5TEST *** FAILED TO RECEIVE CONTROL & COMMAND VIA PIPE  WITH READ2IMAGE4MPU ', time4xi())         
#   # ---------------------------------------------------------------------------------------------------
#   # (20) SEND CONTROL & COMMAND READY VIA PIPE FOR MANOEUVRE2IMAGE4MPU TO RECEIVE                
#   # ---------------------------------------------------------------------------------------------------      
#   print('(Step 20A/700) OF MPU2IMAGE4XI5TEST - TRYING TO SEND CONTROL & COMMAND VIA PIPE WITH READ2IMAGE4MPU ', time4xi())     
#   try:                               
#       message2writesend4mpu = "<write4xi>write2write4mpu</write4xi>"
#       channel2publish4mpu = response4write2image['PUBLISHERCHANNEL']
#       channel2publish4mpu.send(message2writesend4mpu)
#       print("(Step 20B/700) OF MPU2IMAGE4XI5TEST - SENT TO CHANNEL2PUBLISH4MPU: ", channel2publish4mpu, " MESSAGE2SEND4MPU: " , message2writesend4mpu)                                                                                                
#       # <write4xi>write2write4mpu</write4xi> 
#   except:
#        print('(Step 20Z/700) OF MPU2IMAGE4XI5TEST *** FAILED TO SEND CONTROL & COMMAND VIA PIPE WITH READ2IMAGE4MPU ', time4xi())                                                
#   # -----------------------------------------------------------------------------------
#   # (21) LET MANOEUVRE2IMAGE4MPU TO RECEIVE READY AND TO SEND READY                
#   # -----------------------------------------------------------------------------------       
#   print("(Step 21A/700) OF MPU2IMAGE4XI5TEST - TRYING TO PERFORM TARGET FUNCTION MANOEUVRE2IMAGE4MPU ", time4xi()) 
#   try:        
#       response2manoeuvre4mpu = manoeuvre2readimage4mpu(queue2write4mpu, channel2consume4mpu, channel2publish4mpu)
#       print("(Step 21B/700) OF MPU2IMAGE4XI5TEST - PERFORMED TARGET FUNCTION MANOEUVRE2IMAGE4MPU ", manoeuvre2readimage4mpu)
#   except:
#       print("(Step 21Z/700) OF MPU2IMAGE4XI5TEST *** FAILED TO PERFORM TARGET FUNCTION MANOEUVRE2IMAGE4MPU ", time4xi())     
#   # ---------------------------------------------------------------------------------------
#   # (22) RECEIVE CONTROL & COMMAND READY VIA PIPE                      
#   # ---------------------------------------------------------------------------------------          
#   print('(Step 22A/700) OF MPU2IMAGE4XI5TEST - TRYING TO RECEIVE CONTROL & COMMAND VIA PIPE WITH READ2IMAGE4MPU ', time4xi())     
#   try:
#       channel2consume4mpu = response4write2image['CONSUMERCHANNEL']                              
#       message2writereceive4mpu = channel2consume4mpu.recv()
#       print("(Step 22B/700) OF MPU2IMAGE4XI5TEST - RECEIVED FROM CHANNEL2CONSUME4MPU: ", channel2consume4mpu, " CONTROL & COMMAND: ", message2writereceive4mpu)                                                                                                  
#       # <write4xi>write2write4mpu</write4xi> 
#   except:
#       print('(Step 22Z/700) OF MPU2IMAGE4XI5TEST *** FAILED TO RECEIVE CONTROL & COMMAND VIA PIPE  WITH READ2IMAGE4MPU ', time4xi())          
#   # -----------------------------------------------------------------------------------------
#   # (24) PUT IMAGE PATH TO QUEUE
#   # -----------------------------------------------------------------------------------------                          
#   print("(Step 24A/700) OF MPU2IMAGE4XI5TEST - TRYING TO PUT IMAGE PATH TO QUEUE ", time4xi())      
#   try:    
#       queue2write4dragon.put(path2dragon)             
#       print("(Step 24B/700) OF DRAGON2IMAGE4XI5TEST - PUT TO QUEUE IMAGE PATH: ", path2dragon) 
#       # ..//..//..//..//..//web//images//png//dragonxi.png  
#   except:
#       print("(Step 24Z/700) OF MPU2IMAGE4XI5TEST *** FAILED TO PUT IMAGE PATH TO QUEUE ", time4xi()) 
#   # ------------------------------------------------------------------------------------------------------
#   # (25) LET MANOEUVRE2IMAGE4MPU TO GET FROM QUEUE AND TO SEND EITHER ACK OR NACK VIA PIPE               
#   # -------------------------------------------------------------------------------------------------------       
#   print("(Step 25A/700) OF MPU2IMAGE4XI5TEST - TRYING TO PERFORM TARGET FUNCTION MANOEUVRE2IMAGE4MPU ", time4xi()) 
#   try:        
#       response2manoeuvre4mpu = manoeuvre2readimage4mpu(queue2write4mpu, channel2consume4mpu, channel2publish4mpu)
#       print("(Step 25B/700) OF MPU2IMAGE4XI5TEST - PERFORMED TARGET FUNCTION MANOEUVRE2IMAGE4MPU ", manoeuvre2readimage4mpu)
#   except:
#       print("(Step 25Z/700) OF MPU2IMAGE4XI5TEST *** FAILED TO PERFORM TARGET FUNCTION MANOEUVRE2IMAGE4MPU ", time4xi())     
#   # -----------------------------------------------------------------------------------------------
#   # (26) RECEIVE ACK OR NACK ABOUT IMAGE OPERATION VIA PIPE                
#   # -----------------------------------------------------------------------------------------------          
#   print('(Step 26A/700) OF MPU2IMAGE4XI5TEST - TRYING TO RECEIVE CONTROL & COMMAND ABOUT IMAGE OPERTION VIA PIPE ', time4xi())      
#   try: 
#       message2writereceive4mpu = channel2consume4mpu.recv()
#       print("(Step 26D/700) OF MPU2IMAGE4XI5TEST - RECEIVED FROM CHANNEL2CONSUME4MPU: ", channel2consume4mpu, " CONTROL & COMMAND: ", message2writereceive4mpu)                                                                                                   
#       # <ack4xi>ack2imageinfo4mpu</ack4xi>
#   except:
#       print('(Step 26Z/700) OF MPU2IMAGE4XI5TEST *** FAILED TO RECEIVE CONTROL & COMMAND ABOUT IMAGE OPERTION VIA PIPE ', time4xi())               
#   # -------------------------------------------------------------------------------------------------------
#   # (27) LET MANOEUVRE2IMAGE4MPU GO TO STATE 0               
#   # -------------------------------------------------------------------------------------------------------       
#   print("(Step 27A/700) OF MPU2IMAGE4XI5TEST - TRYING TO PERFORM TARGET FUNCTION MANOEUVRE2IMAGE4MPU ", time4xi()) 
#   try:        
#       response2manoeuvre4mpu = manoeuvre2readimage4mpu(queue2write4mpu, channel2consume4mpu, channel2publish4mpu)
#       print("(Step 27B/700) OF MPU2IMAGE4XI5TEST - GOT RESPONSE2MANOEUVRE4MPU ", response2manoeuvre4mpu)
#   except:
#       print("(Step 27Z/700) OF MPU2IMAGE4XI5TEST *** FAILED TO PERFORM TARGET FUNCTION MANOEUVRE2IMAGE4MPU ", time4xi())   

#">

# -------------------------------------------------------------------------------------------------   
def write2image4mpu(task4imagewrite2mpu=None):

    '''
    writes 2D images for MPU
    '''
    global mpu2image4xi                                                     # 2D-IMAGES FOR MPU  
    global control2writeimage4mpu                                              # COMMAND AND CONTROL FOR MPU IMAGES 
    
    # -----------------------------------------------------------------------------------------
    # (Step 1/33) PARAMETERS        
    # -----------------------------------------------------------------------------------------          
    class2print = "write2image4mpu"
    method2print = "write2image4mpu"

    message2print = "{(Step 1A/33) started" + "; write2image4mpu-started: " + str(write2image4mpu)  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                                     #  
    # <function write2image4mpu at 0x00000000038E8EC8>
    
    message2print = "{(Step 1B/33) passed" + "; task4imagewrite2mpu-to-write2image4mpu: " + str(task4imagewrite2mpu) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                                                              
    # None
            
    code2ack  = ' '
    code2nack = ' '
    
    # -----------------------------------------------------------------------------------------
    # CREATE MPU2IMAGE4XI IF NECESSARY       
    # ----------------------------------------------------------------------------------------- 
    try:
        if mpu2image4xi is None:
            mpu2image4xi = create2image4mpu()                                                                              # INITIALIZE IMAGE FOR MPU 
            message2print = "{(Step 1C/70) created" + "; mpu2image4xi-created-by-write2image4mpu to: " + str(mpu2image4xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #             
        else:
            message2print = "{(Step 1D/70) skipped" + "; create2image4mpu-skipped-by-write2image4mpu: " + str(write2image4mpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            # create2image4mpu-skipped-by-write2image4mpu: <function write2image4mpu at 0x00000000038E8EC8>                                 
    except:
        try:
            mpu2image4xi = create2image4mpu()
            message2print = "{(Step 1E/70) set" + "; mpu2image4xi-set-by-write2image4mpu to: " + str(mpu2image4xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #             
        except:
            mpu2image4xi = {}
            message2print = "{(Step 1Z/70) yielded" + "; mpu2image4xi-yielded-by-write2image4mpu: " + str(mpu2image4xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            code2nack  = code2nack + '1Z-' 
    
    # -----------------------------------------------------------------------------------------
    # (Step 1/70) - CHECK PARAMETERS           
    # -----------------------------------------------------------------------------------------          
    try:             
        if task4imagewrite2mpu is None:
            # -----------------------------------------------------------------------------------------
            # NO PARAMETERS GIVEN BY CALLER FOR READ - BUILD TASK! 
            # -----------------------------------------------------------------------------------------    
            
            task4imagewrite2mpu = {}                                                                                                                                                                                       
            
            message2print = "{(Step 1A/70) initialized" + "; task4imagewrite2mpu-created-by-write2image4mpu to: " + str(task4imagewrite2mpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)               
            # 
            
            # ---------------------------------------------------------------------------------------------
            # (1B) TRACE - AUTOMATIC                      
            # ---------------------------------------------------------------------------------------------              
            function2callback4mpu = trace2image4mpu                                                                 # CALLBACK FUNCTION FOR MPU
            
            message2print = "{(Step 1B1/70) initialized" + "; function2callback4mpu-created-by-write2image4mpu to: " + str(function2callback4mpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)            
            # <function trace2image4mpu at 0x00000000038E2A48>
            
            code2ack  = code2ack + '1B1-'
           
            args2callback = {}                                                                                      # CREATE ARGUMENT DICTIONARY FOR CALLBACK FUNCTION             
            args2callback['CLASS']   = "write2image4mpu"                                                            # MODULE OF MPUIZED OPENCL MODULE
            args2callback['METHOD']  = "write2image4mpu"                                                            # FUNCTION OF MPUIZED OPENCL MODULE
            args2callback['MESSAGE'] = "{(Step 1Z1/33) failed" + "; write2image4mpu-failed " + "; time: " + time4xi() + "}" # ERROR MESSAGE FOR MPUIZED OPENCL TO RETURN    

            message2print = "{(Step 1B2/70) initialized" + "; args2callback-created-by-write2image4mpu to: " + str(args2callback)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)               
            # {'MESSAGE': '{(Step 1Z1/33) failed; write2image4mpu-failed ; time: 02:32:29 10/05/11 Pacific Daylight Time}', 
            #  'CLASS': 'write2image4mpu', 
            #  'METHOD': 'write2image4mpu'}; time: 02:32:29 10/05/11 Pacific Daylight Time}

            code2ack  = code2ack + '1B2-'
            
            # ---------------------------------------------------------------------------------------------
            # (1C) CALLBACK - AUTOMATIC                      
            # ---------------------------------------------------------------------------------------------              
            task4imagewrite2mpu['DATA2CALLBACK'] = args2callback                                                    # USER DATA PASSED TO MPUIZED OPENCL - SEVERAL ARGS PASSED IN ONE DICTIONARY
            task4imagewrite2mpu['FUNCTION2CALLBACK'] = function2callback4mpu                                        # CALLBACK FUNCTION TO CALLED BY OPENCL IN EXCEPTIONS
            
            code2ack  = code2ack + '1C-'
            message2print = "{(Step 1C/70) initialized" + "; task4imagewrite2mpu-created-by-write2image4mpu to: " + str(task4imagewrite2mpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            # {'FUNCTION2CALLBACK': <function trace2image4mpu at 0x00000000038E2A48>,
            #  'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; write2image4mpu-failed ; time: 02:32:29 10/05/11 Pacific Daylight Time}',
            #  'CLASS': 'write2image4mpu',
            #  'METHOD': 'write2image4mpu'}}; time: 02:32:29 10/05/11 Pacific Daylight Time}

            
            # ---------------------------------------------------------------------------------------------
            # (2) PLATFORM - AUTOMATIC                   
            # ---------------------------------------------------------------------------------------------             
            platform2xi = {}
            message2print = "{(Step 2A/70) initialized" + "; platform2xi-created-by-write2image4mpu to: " + str(platform2xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)   
            # {}
            
            try:
                platform2xi = get2platformids4xi()                                                                  # PLATFORM IS COMMON TO ALL DEVICES I.E TO DEVICE XI
                message2print = "{(Step 2B/70) got" + "; platform2xi-got-by-write2image4mpu: " + str(platform2xi)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                # {'list2platforms': ['DragonXi'], 'ack2return': ' 2C-2E-2F-2G-', 'nack2return': ' ', 'available2platform': 1, 'max2entries': 1}; time: 02:32:29 10/05/11 Pacific Daylight Time}

                code2ack  = code2ack + '2B-'
            except:
                message2print = "{(Step 2Z/70) failed" + "; platform2xi-failed-by-write2image4mpu: " + str(failed)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                                
                code2nack  = code2nack + '2Z-'
                 
            task4imagewrite2mpu['PLATFORM'] = platform2xi                                                           # PLATFORM OF SYSTEM 
              
            
            # ---------------------------------------------------------------------------------------------
            # (3) CONTEXT - AUTOMATIC                 
            # ---------------------------------------------------------------------------------------------    
            context2mpu = {}
            message2print = "{(Step 3A/70) trying" + "; context2mpu-trying-by-write2image4mpu: " + str(context2mpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            try:
                context2mpu = get2context4mpu()                                                                     # NO PARAMETERS - THE SYSTEM CREATES CONTEXT IF REQUIRED  
                
                message2print = "{(Step 3B/70) got" + "; context2mpu-got-by-write2image4mpu: " + str(context2mpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # {'COUNT2REFERENCE': 0, 
                #  'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2context4mpu-failed ; time: 02:32:29 10/05/11 Pacific Daylight Time}', 
                #  'CLASS': 'create2context4mpu', ...
   
                code2ack  = code2ack + '3B-'         
            except:
                message2print = "{(Step 3Z/70) failed" + "; context2mpu-failed-by-write2image4mpu: " + str(context2mpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '3Z-'
                
            task4imagewrite2mpu ['CONTEXT'] = context2mpu           
            
            # ---------------------------------------------------------------------------------------------
            # (4) DEVICE - AUTOMATIC                       
            # ---------------------------------------------------------------------------------------------    
            device2mpu = {}
            
            message2print = "{(Step 4A/70) trying" + "; device2mpu-trying-by-write2image4mpu: " + str(device2mpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            try:
                device2mpu = get2mpu4xi()                                                                           # THE SYSTEM CREATES DEVICE INFO, IF NECESSARY    
                
                message2print = "{(Step 4B/70) got" + "; device2mpu-got-by-write2image4mpu: " + str(device2mpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # {'RESOLUTION2TIMER4PROFILING': 318, 'ID2PLATFORM': '0000000001509568' ...
               
                code2ack  = code2ack + '4B-'  
            except:
                message2print = "{(Step 4Z/70) failed" + "; device2mpu-failed-by-write2image4mpu: " + str(device2mpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '4Z-'
            
            task4imagewrite2mpu ['DEVICE'] = device2mpu
            
                   
            # ---------------------------------------------------------------------------------------------
            # (5) IMAGE - REQUIRED                            
            # ---------------------------------------------------------------------------------------------         
            imageinfo2mpu = {}
             
            message2print = "{(Step 5A/70) trying" + "; imageinfo2mpu-trying-by-write2image4mpu: " + str(imageinfo2mpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            try:
                imageinfo2mpu = get2imageinfo4mpu()                                                                         
                
                message2print = "{(Step 5B/70) got" + "; imageinfo2mpu-got-by-write2image4mpu: " + str(imageinfo2mpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # {'SIZE2QUEUE': 2147483646, 
                #  'FORMAT': {'TYPE': 'CL_UNORM_INT8', 
                #  'ORDER': 'CL_RGBA'}, 
                #  'QUEUE': <queue4xi.Queue4Xi object at 0x0000000004494E10>, 
                #  'WIDTH': 800, 
                #  'CONTEXT': {'COUNT2REFERENCE': 0, 
                #  'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2context4mpu-failed ; time: 02:32:29 10/05/11 Pacific Daylight Time}', 
                #  'CLASS': 'create2context4mpu', 
                #  'METHOD': 'create2context4mpu'}, 
                #  'FUNCTION2CALLBACK': <function trace2context4mpu at 0x000000000387E348>, 
                #  'ACK2RETURN': ' 2A-2C-2D-2E-2B-3B-4B-5B-6B-7B-7B-', 
                #  'INFO2PRIVATE': None, 
                #  'PLATFORM': {'list2platforms': ['DragonXi'], 'ack2return': ' 2C-2E-2F-2G-', 'nack2return': ' ', 'available2platform': 1, 'max2entries': 1}, 
                #  'NACK2RETURN': ' ', 
                #  'SIZE2PRIVATE': 0, 
                #  'PROPERTIES': None}, 
                #  'PIPE': (<read-write PipeConnection, handle 520>, <read-write PipeConnection, handle 524>), 
                #  'COUNT2REFERENCE': 1, 
                #  'FUNCTION2CALLBACK': <function trace2image4mpu at 0x00000000038E2A48>, 
                #  'COMMANDQUEUE': ...

                code2ack  = code2ack + '5B-'  
            except:
                message2print = "{(Step 5Z/70) failed" + "; imageinfo2mpu-failed-by-write2image4mpu: " + str(imageinfo2mpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '5Z-'          
                      
            task4imagewrite2mpu ['IMAGE'] = imageinfo2mpu         
                        

            # ---------------------------------------------------------------------------------------------
            # (6) QUEUE - REQUIRED BY MPUXI                          
            # ---------------------------------------------------------------------------------------------         
            queue2mpu = None
            
            message2print = "{(Step 6A/70) trying" + "; queue2mpu-trying-by-write2image4mpu: " + str(queue2mpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
            try:
                queue2mpu = Queue4Xi()                                                                              # THE SYSTEM CREATES QUEUE    
                
                message2print = "{(Step 6B/70) got" + "; queue2mpu-got-by-write2image4mpu: " + str(queue2mpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # <queue4xi.Queue4Xi object at 0x0000000003D4FA90>
                
                code2ack  = code2ack + '6B-'  
            except:
                message2print = "{(Step 6Z/70) failed" + "; queue2mpu-failed-by-write2image4mpu: " + str(queue2mpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '6Z-'
                
            task4imagewrite2mpu ['QUEUE'] = queue2mpu

            
            # ---------------------------------------------------------------------------------------------
            # (7) PIPE - REQUIRED BY MPUXI                              
            # ---------------------------------------------------------------------------------------------         
            pipe2mpu = None
           
            message2print = "{(Step 7A/70) trying" + "; pipe2mpu-trying-by-write2image4mpu: " + str(pipe2mpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
            try:
                pipe2mpu = Pipe4Xi()                                                                                # THE SYSTEM CREATES PIPE    
                
                message2print = "{(Step 7B/70) got" + "; pipe2mpu-got-by-write2image4mpu: " + str(pipe2mpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # (<read-write PipeConnection, handle 568>, <read-write PipeConnection, handle 572>)
                
                code2ack  = code2ack + '7B-'  
            except:
                message2print = "{(Step 7Z/70) failed" + "; pipe2mpu-failed-by-write2image4mpu: " + str(pipe2mpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '7Z-'
                
            task4imagewrite2mpu ['PIPE'] = pipe2mpu 
            
            # ---------------------------------------------------------------------------------------------
            # (8) COMMAND QUEUE - REQUIRED                              
            # ---------------------------------------------------------------------------------------------             
            commandqueueinfo2mpu = {}
           
            message2print = "{(Step 8A/70) trying" + "; commandqueueinfo2mpu-trying-by-write2image4mpu: " + str(commandqueueinfo2mpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print)  
                        
            try:
                commandqueueinfo2mpu = get2commandqueueinfo4mpu()                                                   # SYSTEM CREATES COMMAND QUEUE, IF NECESSARY   
                message2print = "{(Step 8B/70) got" + "; commandqueueinfo2mpu-got-by-write2image4mpu: " + str(commandqueueinfo2mpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # {'CODE2ERROR': 0, 
                #  'COUNT2REFERENCE': 0, 
                #  'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2commandqueue4mpu-failed ; time: 02:32:29 10/05/11 Pacific Daylight Time}', 
                #  'CLASS': 'create2commandqueue4mpu', 
                #  'METHOD': 'create2commandqueue4mpu'}, 
                #  'FUNCTION2CALLBACK': <function trace2commandqueue4mpu at 0x000000000388DA48> ...
                
                code2ack  = code2ack + '8B-'  
            except:
                message2print = "{(Step 8Z/70) failed" + "; commandqueueinfo2mpu-failed-by-write2image4mpu: " + str(commandqueueinfo2mpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '8Z-'
        
            task4imagewrite2mpu ['COMMANDQUEUE'] = commandqueueinfo2mpu
            
            
            
            # ---------------------------------------------------------------------------------------------
            # (9) BUFFER - AUTOMATIC BY MPUXI                               
            # ---------------------------------------------------------------------------------------------             
            bufferinfo2mpu = {}
           
            message2print = "{(Step 9A/70) trying" + "; bufferinfo2mpu-trying-by-write2image4mpu: " + str(bufferinfo2mpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                bufferinfo2mpu = get2bufferinfo4mpu()                                                               # SYSTEM CREATES BUFFER FOR MPU, IF NECESSARY
                message2print = "{(Step 9B/70) got" + "; bufferinfo2mpu-got-by-write2image4mpu: " + str(bufferinfo2mpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # {'CODE2ERROR': 0, 
                #  'COUNT2REFERENCE': 0, 
                #  'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2buffer4mpu-failed ; time: 02:32:28 10/05/11 Pacific Daylight Time}', 
                #  'CLASS': 'create2buffer4mpu', 
                #  'METHOD': 'create2buffer4mpu'}, ...
                
                code2ack  = code2ack + '9B-' 
            except:
                message2print = "{(Step 9Z/70) failed" + "; bufferinfo2mpu-failed-by-write2image4mpu: " + str(bufferinfo2mpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '9Z-'
        
            task4imagewrite2mpu ['BUFFER'] = bufferinfo2mpu 
            
            # ---------------------------------------------------------------------------------------------
            # (10) EVENT - REQUIRED                            
            # ---------------------------------------------------------------------------------------------             
            eventinfo2mpu = {}
           
            message2print = "{(Step 10A/70) trying" + "; eventinfo2mpu-trying-by-write2image4mpu: " + str(eventinfo2mpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                eventinfo2mpu = get2eventinfo4mpu()                                                                    
                message2print = "{(Step 10B/70) got" + "; eventinfo2mpu-got-by-write2image4mpu: " + str(eventinfo2mpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                #  {'CODE2ERROR': 0, 
                #  'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2event4mpu-failed ; time: 02:32:28 10/05/11 Pacific Daylight Time}', 
                #  'CLASS': 'create2event4mpu', 
                #  'METHOD': 'create2event4mpu'}, 
                #  'FUNCTION2CALLBACK': <function trace2event4mpu at 0x00000000038A3F48>, ...
                
                code2ack  = code2ack + '10B-' 
            except:
                message2print = "{(Step 10Z/70) failed" + "; eventinfo2mpu-failed-by-write2image4mpu: " + str(eventinfo2mpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '10Z-'
        
            task4imagewrite2mpu ['EVENT'] = eventinfo2mpu 
            
            
            # ---------------------------------------------------------------------------------------------
            # (11) EVENTWAITLIST - REQUIRED                              
            # ---------------------------------------------------------------------------------------------             
            eventwaitlist2mpu = {}
           
            message2print = "{(Step 11A/70) trying" + "; eventwaitlist2mpu-trying-by-write2image4mpu: " + str(eventwaitlist2mpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                eventwaitlist2mpu = {}                                                                   
                message2print = "{(Step 11B/70) set" + "; eventwaitlist2mpu-set-by-write2image4mpu: " + str(eventwaitlist2mpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # {} 
                
                code2ack  = code2ack + '11B-' 
            except:
                message2print = "{(Step 11Z/70) failed" + "; eventwaitlist2mpu-failed-by-write2image4mpu: " + str(eventwaitlist2mpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '11Z-'
        
            task4imagewrite2mpu ['EVENTWAITLIST'] = eventwaitlist2mpu                     

            
            # ---------------------------------------------------------------------------------------------
            # (12) EVENTWAITLISTQUANTITY - REQUIRED                               
            # ---------------------------------------------------------------------------------------------             
            eventwaitlistquantity2mpu = 0
           
            message2print = "{(Step 12A/70) trying" + "; eventwaitlistquantity2mpu-trying-by-write2image4mpu: " + str(eventwaitlistquantity2mpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                keys2event = eventwaitlist2mpu.keys()
                for key2event in keys2event:
                    eventwaitlistquantity2mpu = eventwaitlistquantity2mpu + 1                                                                   
                message2print = "{(Step 12B/70) set" + "; eventwaitlistquantity2mpu-set-by-write2image4mpu: " + str(eventwaitlistquantity2mpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 0 
                
                code2ack  = code2ack + '12B-' 
            except:
                message2print = "{(Step 12Z/70) failed" + "; eventwaitlistquantity2mpu-failed-by-write2image4mpu: " + str(eventwaitlistquantity2mpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '12Z-'
        
            task4imagewrite2mpu ['EVENTWAITLISTQUANTITY'] = eventwaitlistquantity2mpu       
                                                 
            
            # ---------------------------------------------------------------------------------------------
            # (13) BLOCKING - REQUIRED                        
            # ---------------------------------------------------------------------------------------------             
            blocking2mpu = ' '                                                                   
     
            message2print = "{(Step 13A/70) trying" + "; blocking2mpu-trying-by-write2image4mpu: " + str(blocking2mpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                blocking2mpu = 'CL_FALSE'                                                                           # the write command is non-blocking                                                              
                message2print = "{(Step 13B/70) set" + "; blocking2mpu-set-by-write2image4mpu to: " + str(blocking2mpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # CL_FALSE 
                code2ack  = code2ack + '13B-' 
            except:
                message2print = "{(Step 13Z/70) failed" + "; blocking2mpu-failed-by-write2image4mpu: " + str(blocking2mpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '13Z-'
        
            task4imagewrite2mpu ['BLOCKING'] = blocking2mpu             
            
            # ---------------------------------------------------------------------------------------------
            # (14) ORIGIN - REQUIRED                             
            # ---------------------------------------------------------------------------------------------             
            origin2mpu = {}
            
            message2print = "{(Step 14A/70) trying" + "; origin2mpu-trying-by-write2image4mpu: " + str(origin2mpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:                                                                                    
                origin2mpu['X'] = 0 
                origin2mpu['Y'] = 0  
                origin2mpu['Z'] = 0                                                                                                                          
                message2print = "{(Step 14B/70) set" + "; origin2mpu-set-by-write2image4mpu to: " + str(origin2mpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # {'Y': 0, 'X': 0, 'Z': 0} 
                code2ack  = code2ack + '14B-' 
            except:
                message2print = "{(Step 14Z/70) failed" + "; origin2mpu-failed-by-write2image4mpu: " + str(origin2mpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '14Z-'
        
            task4imagewrite2mpu ['ORIGIN'] = origin2mpu
            
            
            # ---------------------------------------------------------------------------------------------
            # (15) REGION - REQUIRED                         
            # ---------------------------------------------------------------------------------------------             
            region2mpu = {}
            
            message2print = "{(Step 15A/70) trying" + "; region2mpu-trying-by-write2image4mpu: " + str(region2mpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:                                                                                    
                region2mpu['X'] = 160                                                                               # WIDTH  IN PIXELS TO READ 
                region2mpu['Y'] = 90                                                                                # HEIGHT IN PIXELS TO READ 
                region2mpu['Z'] = 0                                                                                 # DEPTH  IN PIXELS TO READ                                                                                                                          
                message2print = "{(Step 15B/70) set" + "; region2mpu-set-by-write2image4mpu to: " + str(region2mpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # {'Y': 90, 'X': 160, 'Z': 0}
                code2ack  = code2ack + '15B-' 
            except:
                message2print = "{(Step 15Z/70) failed" + "; region2mpu-failed-by-write2image4mpu: " + str(region2mpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '15Z-'
        
            task4imagewrite2mpu ['REGION'] = region2mpu
            
            
            # ---------------------------------------------------------------------------------------------
            # (16) ROWPITCH (STRIDE) - REQUIRED                              
            # ---------------------------------------------------------------------------------------------             
            # One can assume that memory representing an image or screen will have data laid out as one row of pixels followed directly by the next row of pixels. 
            # However many APIs (e.g. Direct3D) that return a pointer to image data add an extra block of data at the end of each row. 
            # int offset = x * bytesPerPixel + y * pitch;
            # When a video image is stored in memory, the memory buffer might contain extra padding bytes after each row of pixels. 
            # The padding bytes affect how the image is stored in memory, but do not affect how the image is displayed.
            # The stride is the number of bytes from one row of pixels in memory to the next row of pixels in memory. 
            # Stride is also called pitch. 
            # If padding bytes are present, the stride is wider than the width of the image, as shown in the following illustration.

            height2mpu = 800
            width2mpu = 800
            rowpitch2mpu = 0
            size2element = 1                                                                                        # VOXEL (X,Y,Z) = SHOT IN PIXELS, R=1 PIXELS
            voxelquantity2layer = 1                                                                                 # MAX NUMBER OF VOXELS IN EACH 2D LAYER 
            quantity2direction = 1                                                                                  # ONE DIRECTIONS IN 2D
            
            message2print = "{(Step 16A/70) trying" + "; rowpitch2mpu-trying-by-write2image4mpu: " + str(rowpitch2mpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
            #  
                        
            try:                                                                                                    # X-AXEL IS THE WORST CASE TO SCAN WITH Y=0, Z=0
                rowpitch2mpu = int(width2mpu * size2element * voxelquantity2layer / quantity2direction)             # (800 * 8 * 10) / 8 = 8K PIXELS MAX TO SCAN 
                                                                                                                    # SPHERE IS SPARSE HAVING FEW SHOTS PER SCAN LINE 
                                                                                                                                         
                message2print = "{(Step 16B/70) set" + "; rowpitch2mpu-set-by-write2image4mpu to: " + str(rowpitch2mpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 800                                                                                            
                 
                code2ack  = code2ack + '16B-' 
            except:
                message2print = "{(Step 16Z/70) failed" + "; rowpitch2mpu-failed-by-write2image4mpu: " + str(rowpitch2mpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                # 
                
                code2nack  = code2nack + '16Z-'
        
            task4imagewrite2mpu ['ROWPITCH'] = rowpitch2mpu     
            
            # ----------------------------------------------------------------------------------------------------
            # (17) POINTER - REQUIRED - THE POINTER TO A BUFFER IN HOST MEMORY WHERE IMAGE DATA IS TO BE READ FROM                          
            # ----------------------------------------------------------------------------------------------------             
            pointer2mpu = None
            size2mpu = int(width2mpu * height2mpu)                                                                  # CALCULATED MEMORY
            if size2mpu == 0:
                size2mpu = 4400                                                                                     # MINIMUM MEMORY
            
            message2print = "{(Step 17A/70) trying" + "; pointer2mpu-trying-by-write2image4mpu: " + str(pointer2mpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
            # 
                        
            try:
                arena2mpu = Arena4Xi(size2mpu)                                                                      # SYSTEM RESERVER MEMORY FOR IMAGE                             
                                                                                                                                         
                message2print = "{(Step 17B/70) got" + "; arena2mpu-got-by-write2image4mpu: " + str(arena2mpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # <arena4xi.Arena4Xi object at 0x00000000041348D0>
                
                size2arena = arena2mpu.size                                                                         # 
                              
                message2print = "{(Step 17C/70) got" + "; size2arena-got-by-write2buffer4mpu: " + str(size2arena)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                # 640000 
                
                buffer2arena = arena2mpu.buffer
                
                message2print = "{(Step 17D/70) got" + "; buffer2arena-got-by-write2buffer4mpu: " + str(buffer2arena)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                
                # <mmap.mmap object at 0x000000000421A450> 
                
                code2ack  = code2ack + '17D-' 
                
                if size2arena < size2mpu:
                    size2mpu = size2arena                                                                           # AVAILABLE MEMORY FOR IMAGE
                    message2print = "{(Step 17E/70) set" + "; size2mpu-set-by-write2buffer4mpu to: " + str(size2mpu)  + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)
                    # 
                    
                    code2ack  = code2ack + '17E-'
                else:
                    pass
                    
                pointer2mpu = buffer2arena
                message2print = "{(Step 17F/70) set" + "; pointer2mpu-set-by-write2buffer4mpu to: " + str(pointer2mpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                # <mmap.mmap object at 0x000000000421A450>
                
                code2ack  = code2ack + '17F-'       
            except:
                message2print = "{(Step 17Z/70) failed" + "; pointer2mpu-failed-by-write2image4mpu: " + str(pointer2mpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                # 
                
                code2nack  = code2nack + '17Z-'
        
            task4imagewrite2mpu ['POINTER'] = pointer2mpu                            
            task4imagewrite2mpu ['SIZE']    = size2mpu                                                               # AUTOMATIC BY MPUXI
            
            
            # --------------------------------------------------------------------------------------------------------
            # (18) SLICEPITCH - REQUIRED - THE SIZE IN BYTES OF THE 2D SLICE OF THE 3D REGION OF A 3D IMAGE BEING READ                         
            # --------------------------------------------------------------------------------------------------------             
            slicepitch2mpu = None                                                                                   # SLICEPITCH IS ZERO FOR 2D IMAGES                           
           
            message2print = "{(Step 18A/70) trying" + "; slicepitch2mpu-trying-by-write2image4mpu: " + str(slicepitch2mpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                slicepitch2mpu = 0                                                          
                message2print = "{(Step 18B/70) set" + "; slicepitch2mpu-set-by-write2image4mpu to: " + str(slicepitch2mpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 0

                code2ack  = code2ack + '18B-'                
                 
            except:
                message2print = "{(Step 18Z/70) failed" + "; slicepitch2mpu-failed-by-write2image4mpu: " + str(slicepitch2mpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '18Z-'
            
            task4imagewrite2mpu ['SLICEPITCH'] = slicepitch2mpu                  
            
            # ---------------------------------------------------------------------------------------------
            # EOF PARAMETER PROVIDING                
            # ---------------------------------------------------------------------------------------------                                                                                                                               
        else:
            pass
            
        # ---------------------------------------------------------------------------------------------
        # CHECK PARAMETERS - WHETHER AUTOMATICALLY GENERATED OR USER GIVEN 
        # ---------------------------------------------------------------------------------------------
        keys2task = task4imagewrite2mpu.keys()
        message2print = "{(Step 20/70) using" + "; keys2task-using-by-write2image4mpu: " + str(sorted(keys2task))  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
        # ['BLOCKING',
        #  'BUFFER',
        #  'COMMANDQUEUE',
        #  'CONTEXT',
        #  'DATA2CALLBACK',
        #  'DEVICE',
        #  'EVENT',
        #  'EVENTWAITLIST',
        #  'EVENTWAITLISTQUANTITY',
        #  'FUNCTION2CALLBACK',
        #  'IMAGE',
        #  'ORIGIN',
        #  'PIPE',
        #  'PLATFORM',
        #  'POINTER',
        #  'QUEUE',
        #  'REGION',
        #  'ROWPITCH',
        #  'SIZE',
        #  'SLICEPITCH']; time: 02:45:31 10/05/11 Pacific Daylight Time}

        if (('DATA2CALLBACK' in keys2task) and ('FUNCTION2CALLBACK' in keys2task)):
            pass
        else:                
            # ---------------------------------------------------------------------------------------------
            # (21B) TRACE - AUTOMATIC                      
            # ---------------------------------------------------------------------------------------------              
            function2callback4mpu = trace2image4mpu                                                             # CALLBACK FUNCTION FOR MPU
            
            message2print = "{(Step 21B1/70) initialized" + "; function2callback4mpu-created-by-write2image4mpu to: " + str(function2callback4mpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)            
            # 
            
            code2ack  = code2ack + '21B1-'
           
            args2callback = {}                                                                                  # CREATE ARGUMENT DICTIONARY FOR CALLBACK FUNCTION             
            args2callback['CLASS']   = "write2image4mpu"                                                        # MODULE OF MPUIZED OPENCL MODULE
            args2callback['METHOD']  = "write2image4mpu"                                                        # FUNCTION OF MPUIZED OPENCL MODULE
            args2callback['MESSAGE'] = "{(Step 1Z1/33) failed" + "; write2image4mpu-failed " + "; time: " + time4xi() + "}" # ERROR MESSAGE FOR MPUIZED OPENCL TO RETURN    
            message2print = "{(Step 21B2/70) initialized" + "; args2callback-created-by-write2image4mpu to: " + str(args2callback)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)               
            # 
    
            code2ack  = code2ack + '21B2-'
        
            # ---------------------------------------------------------------------------------------------
            # (21C) CALLBACK - AUTOMATIC                      
            # ---------------------------------------------------------------------------------------------              
            task4imagewrite2mpu['DATA2CALLBACK'] = args2callback                                                # USER DATA PASSED TO MPUIZED OPENCL - SEVERAL ARGS PASSED IN ONE DICTIONARY
            task4imagewrite2mpu['FUNCTION2CALLBACK'] = function2callback4mpu                                    # CALLBACK FUNCTION TO CALLED BY OPENCL IN EXCEPTIONS
            
            code2ack  = code2ack + '21C-'
            message2print = "{(Step 21C/70) initialized" + "; task4imagewrite2mpu-created-by-write2image4mpu to: " + str(task4imagewrite2mpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            #
             
        if 'PLATFORM' in keys2task:
            pass
        else:  
            # ---------------------------------------------------------------------------------------------
            # (22) PLATFORM - AUTOMATIC                   
            # ---------------------------------------------------------------------------------------------             
            platform2xi = {}
            message2print = "{(Step 22A/70) initialized" + "; platform2xi-created-by-write2image4mpu to: " + str(platform2xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)   
            # 
                
            try:
                platform2xi = get2platformids4xi()                                                              # PLATFORM IS COMMON TO ALL DEVICES I.E TO DEVICE XI
                message2print = "{(Step 22B/70) got" + "; platform2xi-got-by-write2image4mpu: " + str(platform2xi)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                # 
                     
                code2ack  = code2ack + '22B-'
            except:
                message2print = "{(Step 22Z/70) failed" + "; platform2xi-failed-by-write2image4mpu: " + str(failed)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                                
                code2nack  = code2nack + '22Z-'
                     
            task4imagewrite2mpu['PLATFORM'] = platform2xi                                                       # PLATFORM OF SYSTEM 
              
        if 'CONTEXT' in keys2task:
            pass
        else:              
            # ---------------------------------------------------------------------------------------------
            # (23) CONTEXT - AUTOMATIC                 
            # ---------------------------------------------------------------------------------------------    
            context2mpu = {}
            message2print = "{(Step 23A/70) trying" + "; context2mpu-trying-by-write2image4mpu: " + str(context2mpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            try:
                context2mpu = get2context4mpu()                                                                 # NO PARAMETERS - THE SYSTEM CREATES CONTEXT IF REQUIRED  
                
                message2print = "{(Step 23B/70) got" + "; context2mpu-got-by-write2image4mpu: " + str(context2mpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # 
                
                code2ack  = code2ack + '23B-'         
            except:
                message2print = "{(Step 23Z/70) failed" + "; context2mpu-failed-by-write2image4mpu: " + str(context2mpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '23Z-'
                
            task4imagewrite2mpu ['CONTEXT'] = context2mpu           
        
        if 'DEVICE' in keys2task:
            pass
        else:                   
            # ---------------------------------------------------------------------------------------------
            # (24) DEVICE - AUTOMATIC                       
            # ---------------------------------------------------------------------------------------------    
            device2mpu = {}
            
            message2print = "{(Step 24A/70) trying" + "; device2mpu-trying-by-write2image4mpu: " + str(device2mpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            try:
                device2mpu = get2mpu4xi()                                                                       # THE SYSTEM CREATES DEVICE INFO, IF NECESSARY    
                
                message2print = "{(Step 24B/70) got" + "; device2mpu-got-by-write2image4mpu: " + str(device2mpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # 
                
                code2ack  = code2ack + '24B-'  
            except:
                message2print = "{(Step 24Z/70) failed" + "; device2mpu-failed-by-write2image4mpu: " + str(device2mpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '24Z-'
            
            task4imagewrite2mpu ['DEVICE'] = device2mpu
        
        if 'IMAGE' in keys2task:
            pass
        else:                      
            # ---------------------------------------------------------------------------------------------
            # (25) IMAGE - REQUIRED                            
            # ---------------------------------------------------------------------------------------------         
            imageinfo2mpu = {}
             
            message2print = "{(Step 25A/70) trying" + "; imageinfo2mpu-trying-by-write2image4mpu: " + str(imageinfo2mpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            try:
                imageinfo2mpu = get2imageinfo4mpu()                                                                         
                
                message2print = "{(Step 25B/70) got" + "; imageinfo2mpu-got-by-write2image4mpu: " + str(imageinfo2mpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                #                 
                code2ack  = code2ack + '25B-'  
            except:
                message2print = "{(Step 25Z/70) failed" + "; imageinfo2mpu-failed-by-write2image4mpu: " + str(imageinfo2mpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '25Z-'          
                      
            task4imagewrite2mpu ['IMAGE'] = imageinfo2mpu         
                    
        if 'QUEUE' in keys2task:
            pass
        else:   
            # ---------------------------------------------------------------------------------------------
            # (26) QUEUE - REQUIRED BY MPUXI                          
            # ---------------------------------------------------------------------------------------------         
            queue2mpu = None
            
            message2print = "{(Step 26A/70) trying" + "; queue2mpu-trying-by-write2image4mpu: " + str(queue2mpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
            try:
                queue2mpu = Queue4Xi()                                                                          # THE SYSTEM CREATES QUEUE    
                
                message2print = "{(Step 26B/70) got" + "; queue2mpu-got-by-write2image4mpu: " + str(queue2mpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # 
                
                code2ack  = code2ack + '26B-'  
            except:
                message2print = "{(Step 26Z/70) failed" + "; queue2mpu-failed-by-write2image4mpu: " + str(queue2mpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '26Z-'
                
            task4imagewrite2mpu ['QUEUE'] = queue2mpu

        if 'PIPE' in keys2task:
            pass
        else:              
            # ---------------------------------------------------------------------------------------------
            # (27) PIPE - REQUIRED BY MPUXI                              
            # ---------------------------------------------------------------------------------------------         
            pipe2mpu = None
           
            message2print = "{(Step 27A/70) trying" + "; pipe2mpu-trying-by-write2image4mpu: " + str(pipe2mpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
            try:
                pipe2mpu = Pipe4Xi()                                                                            # THE SYSTEM CREATES PIPE    
                
                message2print = "{(Step 27B/70) got" + "; pipe2mpu-got-by-write2image4mpu: " + str(pipe2mpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # 
                
                code2ack  = code2ack + '27B-'  
            except:
                message2print = "{(Step 27Z/70) failed" + "; pipe2mpu-failed-by-write2image4mpu: " + str(pipe2mpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '27Z-'
                
            task4imagewrite2mpu ['PIPE'] = pipe2mpu 
            
        if 'COMMANDQUEUE' in keys2task:
            pass
        else:              
            # ---------------------------------------------------------------------------------------------
            # (28) COMMAND QUEUE - REQUIRED                              
            # ---------------------------------------------------------------------------------------------             
            commandqueueinfo2mpu = {}
           
            message2print = "{(Step 28A/70) trying" + "; commandqueueinfo2mpu-trying-by-write2image4mpu: " + str(commandqueueinfo2mpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print)  
                        
            try:
                commandqueueinfo2mpu = get2commandqueueinfo4mpu()                                               # SYSTEM CREATES COMMAND QUEUE, IF NECESSARY   
                message2print = "{(Step 28B/70) got" + "; commandqueueinfo2mpu-got-by-write2image4mpu: " + str(commandqueueinfo2mpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                
                code2ack  = code2ack + '28B-'  
            except:
                message2print = "{(Step 28Z/70) failed" + "; commandqueueinfo2mpu-failed-by-write2image4mpu: " + str(commandqueueinfo2mpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '28Z-'
        
            task4imagewrite2mpu ['COMMANDQUEUE'] = commandqueueinfo2mpu
            
            
        if 'BUFFER' in keys2task:
            pass
        else:               
            # ---------------------------------------------------------------------------------------------
            # (29) BUFFER - AUTOMATIC BY MPUXI                               
            # ---------------------------------------------------------------------------------------------             
            bufferinfo2mpu = {}
           
            message2print = "{(Step 29A/70) trying" + "; bufferinfo2mpu-trying-by-write2image4mpu: " + str(bufferinfo2mpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                bufferinfo2mpu = get2bufferinfo4mpu()                                                           # SYSTEM CREATES BUFFER FOR MPU, IF NECESSARY
                message2print = "{(Step 29B/70) got" + "; bufferinfo2mpu-got-by-write2image4mpu: " + str(bufferinfo2mpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                
                code2ack  = code2ack + '29B-' 
            except:
                message2print = "{(Step 29Z/70) failed" + "; bufferinfo2mpu-failed-by-write2image4mpu: " + str(bufferinfo2mpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '29Z-'
        
            task4imagewrite2mpu ['BUFFER'] = bufferinfo2mpu
             
        if 'EVENT' in keys2task:
            pass
        else:               
            # ---------------------------------------------------------------------------------------------
            # (30) EVENT - REQUIRED                            
            # ---------------------------------------------------------------------------------------------             
            eventinfo2mpu = {}
           
            message2print = "{(Step 30A/70) trying" + "; eventinfo2mpu-trying-by-write2image4mpu: " + str(eventinfo2mpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                eventinfo2mpu = get2eventinfo4mpu()                                                                    
                message2print = "{(Step 30B/70) got" + "; eventinfo2mpu-got-by-write2image4mpu: " + str(eventinfo2mpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                
                code2ack  = code2ack + '30B-' 
            except:
                message2print = "{(Step 30Z/70) failed" + "; eventinfo2mpu-failed-by-write2image4mpu: " + str(eventinfo2mpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '30Z-'
        
            task4imagewrite2mpu ['EVENT'] = eventinfo2mpu 
        
             
        if 'EVENTWAITLIST' in keys2task:
            pass
        else:               
            # ---------------------------------------------------------------------------------------------
            # (31) EVENTWAITLIST - REQUIRED                              
            # ---------------------------------------------------------------------------------------------             
            eventwaitlist2mpu = {}
           
            message2print = "{(Step 31A/70) trying" + "; eventwaitlist2mpu-trying-by-write2image4mpu: " + str(eventwaitlist2mpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                eventwaitlist2mpu = {}                                                                   
                message2print = "{(Step 31B/70) set" + "; eventwaitlist2mpu-set-by-write2image4mpu: " + str(eventwaitlist2mpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                
                code2ack  = code2ack + '31B-' 
            except:
                message2print = "{(Step 31Z/70) failed" + "; eventwaitlist2mpu-failed-by-write2image4mpu: " + str(eventwaitlist2mpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '31Z-'
        
            task4imagewrite2mpu ['EVENTWAITLIST'] = eventwaitlist2mpu                     

                 
        if 'EVENTWAITLISTQUANTITY' in keys2task:
            pass
        else:               
            # ---------------------------------------------------------------------------------------------
            # (32) EVENTWAITLISTQUANTITY - REQUIRED                               
            # ---------------------------------------------------------------------------------------------             
            eventwaitlistquantity2mpu = 0
           
            message2print = "{(Step 32A/70) trying" + "; eventwaitlistquantity2mpu-trying-by-write2image4mpu: " + str(eventwaitlistquantity2mpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                keys2event = eventwaitlist2mpu.keys()
                for key2event in keys2event:
                    eventwaitlistquantity2mpu = eventwaitlistquantity2mpu + 1                                                                     
                message2print = "{(Step 32B/70) set" + "; eventwaitlistquantity2mpu-set-by-write2image4mpu: " + str(eventwaitlistquantity2mpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                
                code2ack  = code2ack + '32B-' 
            except:
                message2print = "{(Step 32Z/70) failed" + "; eventwaitlistquantity2mpu-failed-by-write2image4mpu: " + str(eventwaitlistquantity2mpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '32Z-'
        
            task4imagewrite2mpu ['EVENTWAITLISTQUANTITY'] = eventwaitlistquantity2mpu       
                                                 
                 
        if 'BLOCKING' in keys2task:
            pass
        else:              
            # ---------------------------------------------------------------------------------------------
            # (33) BLOCKING - REQUIRED                        
            # ---------------------------------------------------------------------------------------------             
            blocking2mpu = ' '                                                                   
            message2print = "{(Step 33A/70) trying" + "; blocking2mpu-trying-by-write2image4mpu: " + str(blocking2mpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                blocking2mpu = 'CL_FALSE'                                                                       # the write command is non-blocking                                                              
                message2print = "{(Step 33B/70) set" + "; blocking2mpu-set-by-write2image4mpu to: " + str(blocking2mpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                code2ack  = code2ack + '33B-' 
            except:
                message2print = "{(Step 33Z/70) failed" + "; blocking2mpu-failed-by-write2image4mpu: " + str(blocking2mpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '33Z-'
        
            task4imagewrite2mpu ['BLOCKING'] = blocking2mpu             
                 
        if 'ORIGIN' in keys2task:
            pass
        else:                    
            # ---------------------------------------------------------------------------------------------
            # (34) ORIGIN - REQUIRED                             
            # ---------------------------------------------------------------------------------------------             
            origin2mpu = {}
            
            message2print = "{(Step 34A/70) trying" + "; origin2mpu-trying-by-write2image4mpu: " + str(origin2mpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:                                                                                    
                origin2mpu['X'] = 0 
                origin2mpu['Y'] = 0  
                origin2mpu['Z'] = 0                                                                                                                          
                message2print = "{(Step 34B/70) set" + "; origin2mpu-set-by-write2image4mpu to: " + str(origin2mpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                code2ack  = code2ack + '34B-' 
            except:
                message2print = "{(Step 34Z/70) failed" + "; origin2mpu-failed-by-write2image4mpu: " + str(origin2mpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '34Z-'
        
            task4imagewrite2mpu ['ORIGIN'] = origin2mpu
                
                 
        if 'REGION' in keys2task:
            pass
        else:            
            # ---------------------------------------------------------------------------------------------
            # (35) REGION - REQUIRED                         
            # ---------------------------------------------------------------------------------------------             
            region2mpu = {}
            
            message2print = "{(Step 35A/70) trying" + "; region2mpu-trying-by-write2image4mpu: " + str(region2mpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:                                                                                    
                region2mpu['X'] = 160                                                                               # WIDTH  IN PIXELS TO WRITE 
                region2mpu['Y'] = 90                                                                                # HEIGHT IN PIXELS TO WRITE
                region2mpu['Z'] = 0                                                                                 # DEPTH  IN PIXELS TO WRITE                                                                                                                          
                message2print = "{(Step 35B/70) set" + "; region2mpu-set-by-write2image4mpu to: " + str(region2mpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                code2ack  = code2ack + '35B-' 
            except:
                message2print = "{(Step 35Z/70) failed" + "; region2mpu-failed-by-write2image4mpu: " + str(region2mpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '35Z-'
        
            task4imagewrite2mpu ['REGION'] = region2mpu
            
                
                 
        if 'ROWPITCH' in keys2task:
            pass
        else:               
            # ---------------------------------------------------------------------------------------------
            # (36) ROWPITCH (STRIDE) - REQUIRED                              
            # ---------------------------------------------------------------------------------------------             
            # One can assume that memory representing an image or screen will have data laid out as one row of pixels followed directly by the next row of pixels. 
            # However many APIs (e.g. Direct3D) that return a pointer to image data add an extra block of data at the end of each row. 
            # int offset = x * bytesPerPixel + y * pitch;
            # When a video image is stored in memory, the memory buffer might contain extra padding bytes after each row of pixels. 
            # The padding bytes affect how the image is stored in memory, but do not affect how the image is displayed.
            # The stride is the number of bytes from one row of pixels in memory to the next row of pixels in memory. 
            # Stride is also called pitch. 
            # If padding bytes are present, the stride is wider than the width of the image, as shown in the following illustration.
     
            rowpitch2mpu = 0
            size2element = 1                                                                                        # VOXEL (X,Y,Z) = SHOT IN PIXELS, R=1 PIXELS
            voxelquantity2layer = 1                                                                                 # MAX NUMBER OF VOXELS IN EACH 2D LAYER 
            quantity2direction = 1                                                                                  # ONE DIRECTIONS IN 2D
            
            message2print = "{(Step 36A/70) trying" + "; rowpitch2mpu-trying-by-write2image4mpu: " + str(rowpitch2mpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
            # 
                        
            try:                                                                                                    # X-AXEL IS THE WORST CASE TO SCAN WITH Y=0, Z=0
                rowpitch2mpu = int(width2mpu * size2element * voxelquantity2layer / quantity2direction)             # (800 * 8 * 10) / 8 = 8K PIXELS MAX TO SCAN 
                                                                                                                    # SPHERE IS SPARSE HAVING FEW SHOTS PER SCAN LINE 
                                                                                                                                         
                message2print = "{(Step 36B/70) set" + "; rowpitch2mpu-set-by-write2image4mpu to: " + str(rowpitch2mpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                #                                                                                             
                 
                code2ack  = code2ack + '36B-' 
            except:
                message2print = "{(Step 36Z/70) failed" + "; rowpitch2mpu-failed-by-write2image4mpu: " + str(rowpitch2mpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                # 
                
                code2nack  = code2nack + '36Z-'
        
            task4imagewrite2mpu ['ROWPITCH'] = rowpitch2mpu     
                 
        if 'POINTER' in keys2task:
            pass
        else:                
            # ----------------------------------------------------------------------------------------------------
            # (37) POINTER - REQUIRED - THE POINTER TO A BUFFER IN HOST MEMORY WHERE IMAGE DATA IS TO BE READ FROM                          
            # ----------------------------------------------------------------------------------------------------             
            pointer2mpu = None
            size2mpu = int(width2mpu * height2mpu)                                                                  # CALCULATED MEMORY
            if size2mpu == 0:
                size2mpu = 4400                                                                                     # MINIMUM MEMORY
            
            message2print = "{(Step 37A/70) trying" + "; pointer2mpu-trying-by-write2image4mpu: " + str(pointer2mpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
            # 
                        
            try:
                arena2mpu = Arena4Xi(size2mpu)                                                                      # SYSTEM RESERVER MEMORY FOR IMAGE                             
                                                                                                                                         
                message2print = "{(Step 37B/70) got" + "; arena2mpu-got-by-write2image4mpu: " + str(arena2mpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                
                size2arena = arena2mpu.size                                                                         # 
                              
                message2print = "{(Step 37C/70) got" + "; size2arena-got-by-write2buffer4mpu: " + str(size2arena)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                # 
                
                buffer2arena = arena2mpu.buffer
                
                message2print = "{(Step 37D/70) got" + "; buffer2arena-got-by-write2buffer4mpu: " + str(buffer2arena)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                
                # 
                
                code2ack  = code2ack + '37D-' 
                
                if size2arena < size2mpu:
                    size2mpu = size2arena                                                                           # AVAILABLE MEMORY FOR IMAGE
                    message2print = "{(Step 37E/70) set" + "; size2mpu-set-by-write2buffer4mpu to: " + str(size2mpu)  + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)
                    # 
                    
                    code2ack  = code2ack + '37E-'
                else:
                    pass
                    
                pointer2mpu = buffer2arena
                message2print = "{(Step 37F/70) set" + "; pointer2mpu-set-by-write2buffer4mpu to: " + str(pointer2mpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                # 
                
                code2ack  = code2ack + '37F-'       
            except:
                message2print = "{(Step 37Z/70) failed" + "; pointer2mpu-failed-by-write2image4mpu: " + str(pointer2mpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                # 
                
                code2nack  = code2nack + '37Z-'
        
            task4imagewrite2mpu ['POINTER'] = pointer2mpu                            
            task4imagewrite2mpu ['SIZE']    = size2mpu                                                               # AUTOMATIC BY MPUXI
        
             
        if 'SLICEPITCH' in keys2task:
            pass
        else:               
            # --------------------------------------------------------------------------------------------------------
            # (38) SLICEPITCH - REQUIRED - THE SIZE IN BYTES OF THE 2D SLICE OF THE 3D REGION OF A 3D IMAGE BEING READ                         
            # --------------------------------------------------------------------------------------------------------             
            slicepitch2mpu = None                                                                                   # SLICEPITCH IS ZERO FOR 2D IMAGES                           
           
            message2print = "{(Step 38A/70) trying" + "; slicepitch2mpu-trying-by-write2image4mpu: " + str(slicepitch2mpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                slicepitch2mpu = 0                                                          
                message2print = "{(Step 38B/70) set" + "; slicepitch2mpu-set-by-write2image4mpu to: " + str(slicepitch2mpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                code2ack  = code2ack + '38B-'                
                 
            except:
                message2print = "{(Step 38Z/70) failed" + "; slicepitch2mpu-failed-by-write2image4mpu: " + str(slicepitch2mpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '38Z-'
            
            task4imagewrite2mpu ['SLICEPITCH'] = slicepitch2mpu                  

        # ---------------------------------------------------------------------------------------------
        # EOF PARAMETER CHECKING AND PROVIDING FOR MISSING ONES
        # ---------------------------------------------------------------------------------------------
        keys2task = task4imagewrite2mpu.keys()
        message2print = "{(Step 39/70) set" + "; keys2task-set-by-write2image4mpu to: " + str(sorted(keys2task))  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
        # ['BLOCKING',
        #  'BUFFER',
        #  'COMMANDQUEUE',
        #  'CONTEXT',
        #  'DATA2CALLBACK',
        #  'DEVICE',
        #  'EVENT',
        #  'EVENTWAITLIST',
        #  'EVENTWAITLISTQUANTITY',
        #  'FUNCTION2CALLBACK',
        #  'IMAGE',
        #  'ORIGIN',
        #  'PIPE',
        #  'PLATFORM',
        #  'POINTER',
        #  'QUEUE',
        #  'REGION',
        #  'ROWPITCH',
        #  'SIZE',
        #  'SLICEPITCH']; time: 02:45:31 10/05/11 Pacific Daylight Time}
             
    except: 
        message2print = "{(Step 39Z/70) yielded" + "; task4imagewrite2mpu-yielded-by-write2image4mpu: " + str(task4imagewrite2mpu) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # 
        
        code2nack  = code2nack + '39Z-' 

    # ---------------------------------------------------------------------------------------------
    # CONTROL AND COMMAND
    # ---------------------------------------------------------------------------------------------
    control2writeimage4mpu = {}       
                                
    # ---------------------------------------------------------------------------------------------
    # CONSUMER AND PUBLISHER CHANNEL FOR CONTROL AND COMMAND
    # ---------------------------------------------------------------------------------------------
    ack4channels2mpu = False
    channels2mpu = ()
    message2print = "{(Step 40A/70) trying" + "channels2mpu-trying-by-write2image4mpu: " + str(channels2mpu)  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)     
    try:
        # 
        consumerchannel2mpu, publisherchannel2mpu = task4imagewrite2mpu ['PIPE']

        message2print = "{(Step 40B/70) got" + "; consumerchannel2mpu-got-by-write2image4mpu: " + str(consumerchannel2mpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
        # <read-write PipeConnection, handle 568>

        message2print = "{(Step 40C/70) got" + "; publisherchannel2mpu-got-by-write2image4mpu: " + str(publisherchannel2mpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
        # <read-write PipeConnection, handle 572>                     
        ack4channels2mpu = True
        
        control2writeimage4mpu['CONSUMERCHANNEL']  = consumerchannel2mpu
        
        
        control2writeimage4mpu['PUBLISHERCHANNEL'] = publisherchannel2mpu
        
        code2ack  = code2ack + '40D-' 
        
    except:
        ack4channels2mpu = False
        code2nack  = code2nack + '40Z-'      
                        
    # ---------------------------------------------------------------------------------------------
    # QUEUE FOR CONTROL AND COMMAND
    # ---------------------------------------------------------------------------------------------
    ack4queue2mpu = False
    queue2mpu = None
    message2print = "{(Step 41A/70) trying" + "queue2mpu-trying-by-write2image4mpu: " + str(queue2mpu)  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)     
    try:
        queue2mpu = task4imagewrite2mpu ['QUEUE']

        message2print = "{(Step 41B/70) got" + "; queue2mpu-got-by-write2image4mpu: " + str(queue2mpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
        # <queue4xi.Queue4Xi object at 0x0000000003C83A90> 
              
        ack4queue2mpu = True
        control2writeimage4mpu['QUEUE']  = queue2mpu        
        code2ack  = code2ack + '41C-'
    except:
        ack4queue2mpu = False  
        code2nack  = code2nack + '41Z-'
    
    
    # -----------------------------------------------------------------------------------------    
    # DUMMY PROCESS OBJECT = THREAD CREATION
    # -----------------------------------------------------------------------------------------           
    # dummy2image4mpu = None
    # message2print = "{(Step 69A/70) trying" + "dummy2image4mpu-trying-by-write2image4mpu: " + str(dummy2image4mpu)  + "; time: " + time4xi() + "}"   
    # print4xi(class2print, method2print, message2print)     
    # try:                            
        # dummy2image4mpu = DummyProcess4Xi(target=manoeuvre2readimage4mpu,                                               # in this module to handle queue and pipe channels
        #                                   args =(queue2mpu,                                                         # caller puts and gets - does NOT call any functions
        #                                          consumerchannel2mpu, publisherchannel2mpu))                        # caller sends and receives - does NOT call any functions
        # message2print = "{(Step 69B/70) created" + "dummy2image4mpu-created-by-write2image4mpu: " + str(dummy2image4mpu)  + "; time: " + time4xi() + "}" 
        # print4xi(class2print, method2print, message2print)  
        # code2ack  = code2ack + '69B-'
        
    # except:
        # code2nack  = code2nack + '69Z-'
        
    # ---------------------------------------------------------------------------------------------
    # RETURN CONTROL AND COMMAND DICTIONARY
    # ---------------------------------------------------------------------------------------------          
    message2print = "{(Step 70A/70) set" + "; code2ack-set-by-write2image4mpu: " + str(code2ack) + "; time: " + time4xi() + "}"  
    print4xi(class2print, method2print, message2print)                                                             
    # 1B1-1B2-1C-2B-3B-4B-5B-6B-7B-8B-9B-10B-11B-12B-13B-14B-15B-16B-17D-17F-18B-40D-41C-
     
    message2print = "{(Step 70B/70) set" + "; code2nack-set-by-write2image4mpu: " + str(code2nack) + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                             
    #  
        
    message2print = "{(Step 70C/70) returning" + "; control2writeimage4mpu-returning-by-write2image4mpu: " + str(control2writeimage4mpu) + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                             
    # {'QUEUE': <queue4xi.Queue4Xi object at 0x0000000003C83A90>,
    #  'PUBLISHERCHANNEL': <read-write PipeConnection, handle 572>,
    #  'CONSUMERCHANNEL': <read-write PipeConnection, handle 568>}; time: 02:45:31 10/05/11 Pacific Daylight Time}

    return control2writeimage4mpu
                    
#</function4xi> 


 
# -------------------------------------------------------------------------------------------------
#<function4xi function = "release2image4mpu" usage = "
#
#   task4release2image  = {} 
#   task4release2image  ['COUNT2REFERENCE'] = 1 
#   response4release2image = {}
#    
#   print('(Step 12A/33) OF MPU2IMAGE4XI5TEST - TRYING TO RELEASE IMAGE OF MPU DEVICE ', time4xi())   
#   try:                               
#       response4release2image = release2image4mpu(task4release2image)                                                                                             
#       print('(Step 12B/33) OF MPU2IMAGE4XI5TEST - RESPONSE4RELEASE2IMAGE : ', response4release2image)                           
#   except:
#       print('(Step 12Z1/33) OF MPU2IMAGE4XI5TEST *** FAILED TO RELEASE IMAGE OF MPU DEVICE ', time4xi())  
#   
#                 
#">

# -------------------------------------------------------------------------------------------------   
def release2image4mpu(task4release2mpu=None):

    '''
    releases mpu2image4xi
    '''
    global mpu2image4xi  
    global code2ack
    global code2nack
    
    # -----------------------------------------------------------------------------------------
    # (Step 1/33) PARAMETERS           
    # -----------------------------------------------------------------------------------------          
    class2print = "release2image4mpu"
    method2print = "release2image4mpu" 

    message2print = "{(Step 1A/33) started" + "; release2image4mpu-started: " + str(release2image4mpu)  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                                     #  
    # <function release2image4mpu at 0x0000000003A9E8C8>
    
    message2print = "{(Step 1B/33) passed" + "; task4release2mpu-to-release2image4mpu: " + str(task4release2mpu) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                                                              
    # {'COUNT2REFERENCE': 1} 

             
    code2ack  = ' '
    code2nack = ' '

    try:
        if mpu2image4xi is None:
            mpu2image4xi = create2image4mpu()                                                                       # INITIALIZE IMAGE                 
    except:
        try:
            mpu2image4xi = create2image4mpu()
        except:
            mpu2image4xi = {}
            message2print = "{(Step 2Z0/70) yielded" + "; mpu2image4xi-yielded-by-release2image4mpu: " + str(mpu2image4xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            code2nack  = code2nack + '2Z0-' 
                
    message2print = "{(Step 1A/70) try" + "; release-try-by-release2image4mpu: None"  + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)
    
    if task4release2mpu is None:
        task4release2mpu = {}
        task4release2mpu['COUNT2REFERENCE'] = 1
           
    try:
        # -----------------------------------------------------------------------------------------
        # RELEASE
        # -----------------------------------------------------------------------------------------               
        mpu2image4xi['COUNT2REFERENCE'] = mpu2image4xi['COUNT2REFERENCE'] - task4release2mpu['COUNT2REFERENCE']
        
        message2print = "{(Step 1A/70) calculated" + "; count2reference-calculated-by-release2image4mpu: " + str(mpu2image4xi['COUNT2REFERENCE']) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # 
         
        if mpu2image4xi['COUNT2REFERENCE'] < 0:
            mpu2image4xi['COUNT2REFERENCE'] = 0
            
            message2print = "{(Step 1B/70) warning" + "; count2reference-warning-by-release2image4mpu: " + " ** " + str(mpu2image4xi['COUNT2REFERENCE'])  + " ** " + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)      
            # { release2image4mpu : release2image4mpu ; (Step 8Z1/70) warning; count2reference-warning-by-release2image4mpu:  ** 0 ** ; time: 23:41:37 09/06/11 Pacific Daylight Time}    
            code2nack  = code2nack + '1B-'
        else:
            code2ack  = code2ack + '1B-'
    except:
        try:
            mpu2image4xi['COUNT2REFERENCE'] = 0
            
            message2print = "{(Step 8Z1/70) warning" + "; count2reference-warning-by-release2image4mpu: " + " ** " + str(mpu2image4xi['COUNT2REFERENCE'])  + " ** " + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)   
            # 
            code2nack  = code2nack + '8Z1-'
            
        except:
            message2print = "{(Step 8Z2/70) yielded" + "; mpu2image4xi-yielded-by-release2image4mpu: " + str(mpu2image4xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            code2nack  = code2nack + '8Z2-'
                                                                                          
            
    # -----------------------------------------------------------------------------------------
    # (Step 70/70)           
    # -----------------------------------------------------------------------------------------
    mpu2image4xi ['ACK2RETURN']  = code2ack
    mpu2image4xi ['NACK2RETURN'] = code2nack  
    
    message2print = "{(Step 70/70) returning" + "; mpu2image4xi-returning-by-release2image4mpu: " + str(mpu2image4xi) + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                             
    # {'CODE2ERROR': 0,
    #  'ACK2RETURN': ' ',
    #  'ROWPITCH': 800,
    #  'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2image4mpu-failed ; time: 23:41:37 09/06/11 Pacific Daylight Time}',
    #  'CLASS': 'create2image4mpu',
    #  'METHOD': 'create2image4mpu'},
    #  'WIDTH': 800,
    #  'BUFFER': {'CODE2ERROR': 0, 
    #  'COUNT2REFERENCE': 0,
    
    #  'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2buffer4mpu-failed ; time: 23:41:37 09/06/11 Pacific Daylight Time}', 'CLASS': 'create2buffer4mpu', 'METHOD': 'create2buffer4mpu'},
    #  'FUNCTION2CALLBACK': <function trace2buffer4mpu at 0x0000000003A97348>,
    #  'POINTER2BUFFER': <mmap.mmap object at 0x0000000003CDE298>,
    #  'FLAGS': 0, 'CONTEXT': {'COUNT2REFERENCE': 0, ...
    
    
    return mpu2image4xi

#</function4xi> 

# -------------------------------------------------------------------------------------------------
#<function4xi function = "retain2image4mpu" usage = "
#   task4retain2image  = {}
#   task4retain2image  ['DIMENSION'] = 2 
#   task4retain2image  ['COUNT2REFERENCE'] = 1 
#   response4retain2Dimage = {}
#   print('(Step 13A/700) OF MPU2IMAGE4XI5TEST - TRYING TO RETAIN 2D IMAGE OF MPU DEVICE ', time4xi())  
#   try:                               
#       response4retain2Dimage = retain2image4mpu(task4retain2image)                                                                                                 
#       print('(Step 13B/700) OF MPU2IMAGE4XI5TEST - RESPONSE4RETAIN2DIMAGE : ', response4retain2Dimage)                          
#       # 
#   except:
#       print('(Step 13Z1/700) OF MPU2IMAGE4XI5TEST *** FAILED TO RETAIN 2D IMAGE OF MPU DEVICE ', time4xi()) 
#
#">

# -------------------------------------------------------------------------------------------------   
def retain2image4mpu(task4retain2mpu=None):

    '''
    retains either mpu2image4xi
    '''
    global mpu2image4xi 
    
     
    global code2ack
    global code2nack
    
    # -----------------------------------------------------------------------------------------
    # (Step 1/70) - PARAMETERS           
    # -----------------------------------------------------------------------------------------          
    class2print = "retain2image4mpu"
    method2print = "retain2image4mpu"

    message2print = "{(Step 1A/33) started" + "; retain2image4mpu-started: " + str(retain2image4mpu)  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                                     #  
    # <function retain2image4mpu at 0x0000000003A9E948>   
    
    message2print = "{(Step 1B/33) passed" + "; task4retain2mpu-to-retain2image4mpu: " + str(task4retain2mpu) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                                                              
    # {'COUNT2REFERENCE': 1, 'DIMENSION': 2}
    # {'COUNT2REFERENCE': 1, 'DIMENSION': 3}
    code2ack  = ' '
    code2nack = ' '
    
    # -----------------------------------------------------------------------------------------
    # (Step 2/70) - PROVIDE TASKS, IF NECESSARY AND CHECK GIVEN TASKS 
    # -----------------------------------------------------------------------------------------    
    if task4retain2mpu is None:
        task4retain2mpu = {}
        task4retain2mpu['DIMENSION'] = 2
        task4retain2mpu['COUNT2REFERENCE'] = 1
        message2print = "{(Step 2A/33) set" + "; task4retain2mpu-set-by-retain2image4mpu to: " + str(task4retain2mpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        #           
    else:
        pass
        
    if 'DIMENSION' in task4retain2mpu.keys():
        if task4retain2mpu['DIMENSION'] == 2:
            pass
        else:
            task4retain2mpu['DIMENSION'] = 2 
            message2print = "{(Step 2B/33) set" + "; dimension-set-by-retain2image4mpu to: " + str(task4retain2mpu['DIMENSION'])  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #                       
    else:
        task4retain2mpu['DIMENSION'] = 2
    
    # -----------------------------------------------------------------------------------------
    # (Step 3/70) - CHECK COUNT2REFERENCE
    # -----------------------------------------------------------------------------------------         
    if 'COUNT2REFERENCE' in task4retain2mpu.keys():
        pass
    else:
        task4retain2mpu['COUNT2REFERENCE'] = 1
        message2print = "{(Step 3A/33) set" + "; count2reference-set-by-retain2image4mpu to: " + str(task4retain2mpu['COUNT2REFERENCE'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # 
                                   
    if task4retain2mpu['COUNT2REFERENCE'] < 1:
        task4retain2mpu['COUNT2REFERENCE'] = 1
        message2print = "{(Step 3B/33) set" + "; count2reference-set-by-retain2image4mpu to: " + str(task4retain2mpu['COUNT2REFERENCE'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        #           
        
    # -----------------------------------------------------------------------------------------
    # (Step 4/70) - INITIALIZE 2D IMAGE IF NECESSARY
    # -----------------------------------------------------------------------------------------
    try:
        if mpu2image4xi is None:
            mpu2image4xi = create2image4mpu()
            message2print = "{(Step 4A/70) created" + "; mpu2image4xi-created-by-retain2image4mpu: " + str(mpu2image4xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)                                                                                                          
    except:
        try:
            mpu2image4xi = create2image4mpu()
            message2print = "{(Step 4B/70) created" + "; mpu2image4xi-created-by-retain2image4mpu: " + str(mpu2image4xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)                 
        except:
            mpu2image4xi = {}
            message2print = "{(Step 4Z/70) yielded" + "; mpu2image4xi-yielded-by-retain2image4mpu: " + str(mpu2image4xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            code2nack  = code2nack + '4Z-' 


    # -----------------------------------------------------------------------------------------
    # (Step 5/70) - RETAIN
    # -----------------------------------------------------------------------------------------             
    try:
        mpu2image4xi['COUNT2REFERENCE'] = mpu2image4xi['COUNT2REFERENCE'] + task4retain2mpu['COUNT2REFERENCE']
        message2print = "{(Step 6A/70) calculated" + "; count2reference-calculated-by-retain2image4mpu: " + str(mpu2image4xi['COUNT2REFERENCE']) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        #              
        if mpu2image4xi['COUNT2REFERENCE'] <= 0:
            mpu2image4xi['COUNT2REFERENCE'] = 1
            message2print = "{(Step 6Z1/70) warning" + "; count2reference-warning-by-retain2image4mpu: " + " ** " + str(mpu2image4xi['COUNT2REFERENCE'])  + " ** " + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)      
            code2nack  = code2nack + '6Z1-'
        else:
            code2ack  = code2ack + '6C-'

    except:
        try:
            mpu2image4xi['COUNT2REFERENCE'] = 1
        
            message2print = "{(Step 7Z1/70) warning" + "; count2reference-warning-by-retain2image4mpu: " + " ** " + str(mpu2image4xi['COUNT2REFERENCE'])  + " ** " + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)   
            code2nack  = code2nack + '7Z1-'
        
        except:
            message2print = "{(Step 7Z2/70) yielded" + "; mpu2image4xi-yielded-by-retain2image4mpu: " + str(mpu2image4xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            code2nack  = code2nack + '7Z2-'
        
                                                                                          
            
    # -----------------------------------------------------------------------------------------
    # (Step 70/70) - RETURN           
    # -----------------------------------------------------------------------------------------
    
    mpu2image4xi ['ACK2RETURN']  = code2ack
    mpu2image4xi ['NACK2RETURN'] = code2nack     
    message2print = "{(Step 70A/70) returning" + "; mpu2image4xi-returning-by-retain2image4mpu: " + str(mpu2image4xi) + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                             
    # {'CODE2ERROR': 0, 'ACK2RETURN': ' 6C-', 'ROWPITCH': 800, 'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2image4mpu-failed ; time: 23:25:02 10/05/11 Pacific Daylight Time}', 'CLASS': 'create2image4mpu', 'METHOD': 'create2image4mpu'}, 'WIDTH': 800, 'BUFFER': {'CODE2ERROR': 0, 
    #                   'COUNT2REFERENCE': 0, ...
    # 'COUNT2REFERENCE': 1, 
    # 'FLAGS': 0, 'CONTEXT': ...
    
    return mpu2image4xi

#</function4xi> 


# -------------------------------------------------------------------------------------------------
#<function4xi function = "get2imageinfo4mpu" usage = "
#   #1
#   print('(Step 3A/33) OF MPU2IMAGE4XI5TEST - TRYING TO GET 2D IMAGE INFO FOR MPU', time4xi())  
#   try:
#       imageinfo2mpu = get2imageinfo4mpu()                                                                       
#       print('(Step 3B/33) OF MPU2IMAGE4XI5TEST - GOT 2D IMAGE INFO FOR MPU: ', imageinfo2mpu) 
#   except:
#       print('(Step 3Z/33) OF MPU2IMAGE4XI5TEST *** FAILED TO GET 2D IMAGE INFO FOR MPU', time4xi())
#   #2
#   task4getinfo2image = {}
#   task4getinfo2image ['DIMENSION'] = 2
#   task4getinfo2image ['WIDTH'] = 0
#   task4getinfo2image ['HEIGHT'] = 0
#   response4getinfo2image = {}
#   print('(Step 12C/33) OF MPU2IMAGE4XI5TEST - TRYING TO GET SELECTED 3D IMAGE INFO OF MPU DEVICE ', time4xi())    
# 
#   try:                               
#       response4getinfo2image = get2imageinfo4mpu (task4getinfo2image)                                                                                             
#       print('(Step 12D/33) OF MPU2IMAGE4XI5TEST - RESPONSE4RELEASE2IMAGE : ', response4getinfo2image)                           
#       #
#   except:
#       print('(Step 12Z2/33) OF MPU2IMAGE4XI5TEST *** FAILED TO GET SELECTED 3D IMAGE INFO OF MPU DEVICE ', time4xi()) 
#            
#">

# -------------------------------------------------------------------------------------------------   
def get2imageinfo4mpu(task4getinfo2image=None):

    '''
    gets mpu2image4xi
    '''
    
    global mpu2image4xi
    
    
    # -----------------------------------------------------------------------------------------
    # (Step 1/33)           
    # -----------------------------------------------------------------------------------------          
    class2print = "get2imageinfo4mpu"
    method2print = "get2imageinfo4mpu"

    message2print = "{(Step 0A/33) started" + "; get2imageinfo4mpu-started: " + str(get2imageinfo4mpu)  + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)
    # <function get2imageinfo4mpu at 0x0000000003A9E9C8>                                                              
    
    message2print = "{(Step 0A/33) passed" + "; task4getinfo2image-passed-to-get2imageinfo4mpu: " + str(task4getinfo2image) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)   
    # {'WIDTH': 0, 'DIMENSION': 2, 'HEIGHT': 0}
    # {'WIDTH': 0, 'DEPTH': 0, 'DIMENSION': 3, 'HEIGHT': 0}
    # {'DOMAIN': 'ALL', 'DIMENSION': 2}
    
    image2mpu = {}
    try: 
        if mpu2image4xi is None:
            # -------------------------------------------------------------------------------------
            # INITIALIZE 2D IMAGE FOR MPU           
            # -------------------------------------------------------------------------------------           
            image2mpu = create2image4mpu()
            message2print = "{(Step 1B2/33) initialized" + "; image2mpu-initialized-by-get2imageinfo4mpu to: " + str(image2mpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            # 
        else:
            message2print = "{(Step 1B2/33) using" + "; mpu2image4xi-using-by-get2imageinfo4mpu: " + str(mpu2image4xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # {'CODE2ERROR': 0, 
            #  'ACK2RETURN': ' 1D-1E-2C-2B-3B-4B-5B-6B-7B-8B-9B-10B-11B-14B-15B-16B-17B-17D-18B-18C-21A1-22B1-22C1-23B-24B-25B-26B-27B-28B-29B-30B-31B-34H-35H-36B-37B-38B-', 
            #  'ROWPITCH': 800, 
            #  'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2image4mpu-failed ; time: 09:05:31 10/05/11 Pacific Daylight Time}', 
            #  'CLASS': 'create2image4mpu', 
            #  'METHOD': 'create2image4mpu'}, 
            #  'WIDTH': 800, 
            #  'BUFFER': ...
            
            
        # -------------------------------------------------------------------------------------
        # RETURN THE WHOLE OR PART OF THE IMAGE INFO OF MPU          
        # -------------------------------------------------------------------------------------     
        dimension2xi = 0
        
        if task4getinfo2image is None:
            task4getinfo2image = {}
            task4getinfo2image ['DIMENSION'] = 2 
            task4getinfo2image ['DOMAIN'] = 'ALL'
            image2mpu = mpu2image4xi
            message2print = "{(Step 1C/33) using" + "; image2mpu-got-by-get2imageinfo4mpu: " + str(image2mpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            # {'CODE2ERROR': 0, 
            #  'DOMAIN': 'ALL', 
            #  'CONTEXT': ...
            
        else:
            pass    

        keys2task4imageinfo = task4getinfo2image.keys()
        if 'DIMENSION' in keys2task4imageinfo:
            dimension2xi = task4getinfo2image['DIMENSION']
            if dimension2xi == 2:
                pass
            else:
                dimension2xi = 2    
        else:
            pass
            
        if 'DOMAIN' in keys2task4imageinfo:
            domain2xi = task4getinfo2image['DOMAIN']
            if domain2xi == 'ALL':
                pass
            else:
                domain2xi = 'ALL'

            keys2domain = mpu2image4xi.keys()
            for key2domain in keys2domain:
                task4getinfo2image[key2domain] = mpu2image4xi[key2domain]
                    
            message2print = "{(Step 1C/33) using" + "; task4getinfo2image-using-by-get2imageinfo4mpu: " + str(task4getinfo2image)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)                                          
        else:
            pass            
            
                    

        keys2task = task4getinfo2image.keys()
        for key2task in keys2task:
            try:
                message2print = "{(Step 2D2/33) using" + "; key2task-using-by-get2image4mpu: " + str(key2task)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)  
                # WIDTH; DIMENSION; HEIGHT  
                image2mpu [key2task] = mpu2image4xi[key2task]
                
                message2print = "{(Step 2D2/33) set" + "; info-of-image2mpu-set-by-get2imageinfo4mpu to: " + str(image2mpu [key2task])  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                # 800; 800  
            except:
                message2print = "{(Step 2Z/70) yielded" + "; key2task-yielded-by-get2image4mpu: " + str(key2task) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                               
                # DIMENSION;
                                   
                message2print = "{(Step 2Z2/70) using" + "; task4getinfo2image-using-by-get2image4mpu: " + str(task4getinfo2image)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)  
                # 
                
                message2print = "{(Step 2Z3/70) using" + "; mpu2image4xi-using-by-get2image4mpu: " + str(mpu2image4xi)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                #                           
    except:
        try:
            # -------------------------------------------------------------------------------------
            # INITIALIZE           
            # -------------------------------------------------------------------------------------           
            image2mpu = create2image4mpu()
            message2print = "{(Step 1F/33) initialized" + "; image2mpu-initialized-by-get2imageinfo4mpu to: " + str(image2mpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #
            if task4getinfo2image is None:
                image2mpu = mpu2image4xi
                message2print = "{(Step 1G/33) got" + "; image2mpu-got-by-get2imageinfo4mpu: " + str(image2mpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                # 
            else:
                keys2task = task4getinfo2image.keys()
                for key2task in keys2task:
                    try:
                        message2print = "{(Step HD2/33) using" + "; key2task-using-by-get2image4mpu: " + str(key2task)  + "; time: " + time4xi() + "}"
                        print4xi(class2print, method2print, message2print)  
                        # 
                        image2mpu [key2task] = mpu2image4xi[key2task]
                    
                        message2print = "{(Step HD2/33) set" + "; info-of-image2mpu-set-by-get2imageinfo4mpu to: " + str(image2mpu [key2task])  + "; time: " + time4xi() + "}"
                        print4xi(class2print, method2print, message2print)
                        #   
                    except:
                        message2print = "{(Step HZ/70) yielded" + "; key2task-yielded-by-get2image4mpu: " + str(key2task) + "; time: " + time4xi() + "}"
                        print4xi(class2print, method2print, message2print)                            
        except:
            image2mpu = {}
            message2print = "{(Step 1Z3/70) yielded" + "; image2mpu-yielded-by-get2image4mpu: " + str(image2mpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
                        
    message2print = "{(Step 33/33) returning" + "; image2mpu-returning-by-get2imageinfo4mpu: " + str(image2mpu) + "; time: " + time4xi() + "}"  
    print4xi(class2print, method2print, message2print)
    # 2D: {'WIDTH': 800, 'HEIGHT': 800}
    # 3D: {'WIDTH': 800, 'DEPTH': 800, 'HEIGHT': 800}
    # 2D ALL: {'CODE2ERROR': 0, 
    #  'ROWPITCH': 800, 
    #  'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2image4mpu-failed ; time: 09:05:31 10/05/11 Pacific Daylight Time}', 
    #  'CLASS': 'create2image4mpu', 
    #  'METHOD': 'create2image4mpu'}, 
    #  'FORMAT': {'TYPE': 'CL_UNORM_INT8', 
    #  'ORDER': 'CL_RGBA'}, 
    #  'BUFFER': ...
    
    # 3D ALL: {'PIPE': (<read-write PipeConnection, handle 520>, <read-write PipeConnection, handle 524>), 
    #  'ROWPITCH': 800, 
    #  'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create3image4mpu-failed ; time: 09:05:31 10/05/11 Pacific Daylight Time}', 
    #  'CLASS': 'create3image4mpu', 
    #  'METHOD': 'create3image4mpu'}, 
    #  'FORMAT': {'TYPE': 'CL_UNORM_INT8', 
    #  'ORDER': 'CL_RGBA'}, 
    #  'BUFFER': ....
    #  'HEIGHT': 800, 
    #  'QUEUE': <queue4xi.Queue4Xi object at 0x0000000004504400>, 
    #  'WIDTH': 800, 
    #  'DEPTH': 800, 
    #  'FLAGS': 0, 
    #  'CONTEXT': ...
    
    return image2mpu
            
#</function4xi> 
 

# -------------------------------------------------------------------------------------------------
#<function4xi function = "get2supportedimageformats4mpu" usage = "
#
#   print('(Step 3A/33) OF MPU2IMAGE4XI5TEST - TRYING TO GET SUPPORTED IMAGE FORMATS FOR MPU', time4xi())  
#   try:
#       imageformats = get2supportedimageformats4mpu()                                                                       
#       print('(Step 3B/33) OF MPU2IMAGE4XI5TEST - GOT SUPPORTED IMAGE FORMATS FOR MPU: ', imageformats) 
#   except:
#       print('(Step 3Z/33) OF MPU2IMAGE4XI5TEST *** FAILED TO GET SUPPORTED IMAGE FORMATS FOR MPU', time4xi())
#            
#">

# -------------------------------------------------------------------------------------------------   
def get2supportedimageformats4mpu(task4supportedimageformats2mpu=None):  

    '''
    gets supported image formats for mpu
    '''
    
    global mpu2image4xi

    
    # -----------------------------------------------------------------------------------------
    # (Step 1/33)           
    # -----------------------------------------------------------------------------------------          
    class2print = "get2supportedimageformats4mpu"
    method2print = "get2supportedimageformats4mpu"

    message2print = "{(Step 0A/33) started" + "; get2supportedimageformats4mpu-started: " + str(get2supportedimageformats4mpu)  + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print) 
    # <function get2supportedimageformats4mpu at 0x0000000003BBE8C8>                                                          
    
    message2print = "{(Step 0B/33) passed" + "; task4supportedimageformats2mpu-passed-to-get2supportedimageformats4mpu: " + str(task4supportedimageformats2mpu) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)   
    # None; {'DIMENSION': 3} 
    
    
    if task4supportedimageformats2mpu is None:
        task4supportedimageformats2mpu = {} 
        task4supportedimageformats2mpu['DIMENSION'] = 2                                                             # 2D image is the default
    
    dimension2xi = 2
    try:
        keys2supportedimageformats = task4supportedimageformats2mpu.keys()    
        if 'DIMENSION' in keys2supportedimageformats:
            dimension2xi = task4supportedimageformats2mpu['DIMENSION']
            if ((dimension2xi <=0) or (dimension2xi > 2)): 
                dimension2xi = 2
        else:
            dimension2xi = 2
    except:
        pass        
    
    message2print = "{(Step 1A/33) using" + "; dimension2xi-using-by-dimension2xi: " + str(dimension2xi)  + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)    
    # 2; 3
                        
    supportedimageformats2mpu = {} 
    try: 
        if mpu2image4xi is None:
            # -------------------------------------------------------------------------------------
            # INITIALIZE           
            # -------------------------------------------------------------------------------------           
            image2mpu = create2image4mpu()                                                                          # create mpu2image4xi
            message2print = "{(Step 1B/33) initialized" + "; image2mpu-initialized-by-get2supportedimageformats4mpu to: " + str(image2mpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #                
        else:
            pass

                        
        # -------------------------------------------------------------------------------------
        # SUPPORTED IMAGE FORMATS FOR MPU       
        # -------------------------------------------------------------------------------------     
        supportedimageformats2mpu = mpu2image4xi['FORMAT']
        message2print = "{(Step 1E/33) got" + "; supportedimageformats2mpu-got-by-get2supportedimageformats4mpu: " + str(supportedimageformats2mpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)           
        # {'TYPE': 'CL_UNORM_INT8', 'ORDER': 'CL_RGBA'} 
            
    except:
        try:
            # -------------------------------------------------------------------------------------
            # INITIALIZE           
            # -------------------------------------------------------------------------------------           
            image2mpu = create2image4mpu()
            message2print = "{(Step 1F1/33) initialized" + "; image2mpu-initialized-by-get2supportedimageformats4mpu to: " + str(image2mpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            # 
     
            
            # -------------------------------------------------------------------------------------
            # SUPPORTED IMAGE FORMATS FOR MPU       
            # -------------------------------------------------------------------------------------     
            supportedimageformats2mpu = mpu2image4xi['FORMAT']
            message2print = "{(Step 1H/33) got" + "; supportedimageformats2mpu-got-by-get2supportedimageformats4mpu: " + str(supportedimageformats2mpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)           
            #  
                                              
        except:
            message2print = "{(Step 1Z/70) yielded" + "; supportedimageformats2mpu-yielded-by-get2image4mpu: " + str(supportedimageformats2mpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
                        
    message2print = "{(Step 33/33) returning" + "; supportedimageformats2mpu-returning-by-get2supportedimageformats4mpu: " + str(supportedimageformats2mpu) + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)
    # 2: {'TYPE': 'CL_UNORM_INT8', 'ORDER': 'CL_RGBA'}
    # 3: {'TYPE': 'CL_UNORM_INT8', 'ORDER': 'CL_RGBA'}
    
    return supportedimageformats2mpu
            
#</function4xi> 

 



# -------------------------------------------------------------------------------------------------
#<function4xi function = "wait2image4mpu" usage = "
#   print('=' *33)          
#   print('(Step 16A/700) OF MPU2IMAGE4XI5TEST - TRYING TO WAIT 2D IMAGE FOR MPU ', time4xi())   
#   print('=' *33)      
#   try:
#       response4wait2mpu = wait2image4mpu()
#       keys2response = response4wait2mpu.keys()
#       print('(Step 16B/700) OF MPU2IMAGE4XI5TEST - WAITED 2D IMAGE FOR MPU ', time4xi()) 
#       for key2response in sorted(keys2response):
#           print(key2response, ' : ', response4wait2mpu[key2response])
#           # ACK2RETURN  :   6C-
#           # BUFFER  :  {'CODE2ERROR': 0 ...
#           # CALLBACK  :  <function trace2image4mpu at 0x0000000004E93B48>
#           # CODE2ERROR  :  0
#           # COMMANDQUEUE  :  {'CODE2ERROR': 0, ...
#           # CONTEXT  :  {'COUNT2REFERENCE': 0 ...
#           # COUNT2REFERENCE  :  1
#           # DATA2CALLBACK  :  {'MESSAGE': '{(Step 1Z1/33) failed; create2image4mpu-failed ; time: 02:00:43 10/06/11 Pacific Daylight Time}', 'CLASS': 'create2image4mpu', 'METHOD': 'create2image4mpu'}
#           # DEPTH  :  0
#           # DEVICE  :  {'RESOLUTION2TIMER4PROFILING': 318, 'ID2PLATFORM': '0000000001509568', 'PROFILING2QUEUE': 'ACK',...
#           # EMPTY2QUEUE  :  True
#           # EVENT  :  {'CODE2ERROR': 0, 'DATA2CALLBACK':...
#           # FLAGS  :  0
#           # FORMAT  :  {'TYPE': 'CL_UNORM_INT8', 'ORDER': 'CL_RGBA'}
#           # FULL2QUEUE  :  False
#           # FUNCTION2CALLBACK  :  <function trace2image4mpu at 0x0000000004E93B48>
#           # HEIGHT  :  800
#           # NACK2RETURN  :   
#           # PIPE  :  (<read-write PipeConnection, handle 556>, <read-write PipeConnection, handle 560>)
#           # PLATFORM  :  {'list2platforms': ['DragonXi'], 'ack2return': ' 2C-2E-2F-2G-', 'nack2return': ' ', 'available2platform': 1, 'max2entries': 1}
#           # POINTER  :  <mmap.mmap object at 0x0000000004351500>
#           # QUEUE  :  <queue4xi.Queue4Xi object at 0x00000000044E95C0>
#           # ROWPITCH  :  800
#           # SIZE2QUEUE  :  2147483646 
#   except:
#       print('(Step 16Z1/700) OF MPU2IMAGE4XI5TEST *** FAILED TO WAIT 2D IMAGE FOR MPU: ', wait2mpu)            
#">

# -------------------------------------------------------------------------------------------------   
def wait2image4mpu(task4wait2image=None):

    '''
    waits mpu2image4mpu
    '''
    
    global mpu2image4xi
    
    # -----------------------------------------------------------------------------------------
    # (Step 1/33)           
    # -----------------------------------------------------------------------------------------          
    class2print = "wait2image4mpu"
    method2print = "wait2image4mpu"

    message2print = "{(Step 1A1/33) started" + "; wait2image4mpu-started: " + str(wait2image4mpu)  + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)
    #  
    
    message2print = "{(Step 1A2/33) passed" + "; task4wait2image-passed-by-wait2image4mpu to: " + str(task4wait2image)  + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)
    # 
    
    try: 
        if mpu2image4xi is None:
            # -------------------------------------------------------------------------------------
            # INITIALIZE - KEEP SYSTEM RUNNING           
            # -------------------------------------------------------------------------------------           
            mpu2image4xi = create2image4mpu()
            
            message2print = "{(Step 1B/33) created" + "; mpu2image4xi-created-by-wait2image4mpu: " + str(mpu2image4xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #
                                                
            # -------------------------------------------------------------------------------------
            # WAIT         
            # -------------------------------------------------------------------------------------             
            queue2image4mpu = None
            try:
                queue2image4mpu = mpu2image4xi['QUEUE']
                message2print = "{(Step 1C/33) got" + "; queue2image4mpu-got-by-wait2image4mpu to: " + str(queue2image4mpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                #
            except:     
                message2print = "{(Step 1Z1/70) yielded" + "; mpu2xi-yielded-by-wait2image4mpu: " + str(mpu2xi) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)    
            
            size2image4mpu = 0
            try:                    
                size2image4mpu = queue2image4mpu.qsize()
                message2print = "{(Step 1D/33) got" + "; size2image4mpu-got-by-wait2image4mpu to: " + str(size2image4mpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                
                #
                               
                
            except:     
                message2print = "{(Step 1Z2/70) yielded" + "; mpu2xi-yielded-by-wait2image4mpu: " + str(mpu2xi) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
             
            empty2image4mpu = True       
            try:                    
                empty2image4mpu= queue2image4mpu.empty()
                message2print = "{(Step 1E/33) got" + "; empty2image4mpu-got-by-wait2image4mpu to: " + str(empty2image4mpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                
                #
            except:     
                message2print = "{(Step 1Z3/70) yielded" + "; empty2image4mpu-yielded-by-wait2image4mpu: " + str(empty2image4mpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)    
             
            full2image4mpu = False       
            try:                                
                full2image4mpu = queue2image4mpu.full()
                message2print = "{(Step 1F/33) got" + "; full2image4mpu-got-by-wait2image4mpu to: " + str(full2image4mpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                
                #                
            except:     
                message2print = "{(Step 1Z4/70) yielded" + "; full2image4mpu-yielded-by-wait2image4mpu: " + str(full2image4mpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)   
                            
            message2print = "{(Step 1G/33) finished" + "; mpu2image4xi-finished-by-wait2image4mpu to: " + str(mpu2image4xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #
                            
        else:
            # -------------------------------------------------------------------------------------
            # USE THE EXISTING IMAGE FOR MPU          
            # -------------------------------------------------------------------------------------     
            
            message2print = "{(Step 1H/33) used" + "; mpu2image4xi-used-by-wait2image4mpu: " + str(mpu2image4xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            # 
            
            # -------------------------------------------------------------------------------------
            # WAIT         
            # -------------------------------------------------------------------------------------             
            queue2image4mpu = None
            try:
                queue2image4mpu = mpu2image4xi['QUEUE']
                message2print = "{(Step 1I/33) got" + "; queue2image4mpu-got-by-wait2image4mpu to: " + str(queue2image4mpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                # 
            except:     
                message2print = "{(Step 1Z5/70) yielded" + "; mpu2xi-yielded-by-wait2image4mpu: " + str(mpu2xi) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                
            if task4wait2image is None:
                pass
            else:
                # -------------------------------------------------------------------------------------
                # IMAGE GIVEN AS PARAMETER          
                # -------------------------------------------------------------------------------------               
                message2print = "{(Step 1J1/33) passed" + "; task4wait2image-passed-by-wait2image4mpu to: " + str(task4wait2image)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                # 
                queue2task = {}    
                try:
                    queue2task = task4wait2image ['QUEUE']
                    message2print = "{(Step 1J2/33) set" + "; queue2task-set-by-wait2image4mpu to: " + str(queue2task)  + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)
                    # 
                except:     
                    message2print = "{(Step 1Z61/70) yielded" + "; queue2task-yielded-by-wait2image4mpu: " + str(queue2task) + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)
                
                try:                        
                    if queue2task == queue2image4mpu:
                        # -------------------------------------------------------------------------------------
                        # ACCEPTED - GIVEN QUEUE IS THE SAME          
                        # -------------------------------------------------------------------------------------                     
                        message2print = "{(Step 1K/33) accepted" + "; queue2task-accepted-by-wait2image4mpu: " + str(queue2task) + "; time: " + time4xi() + "}"
                        print4xi(class2print, method2print, message2print)
                        # 
                    else:
                        try:
                            message2print = "{(Step 1L/33) warned" + "; queue2task-warned-by-wait2image4mpu: " + " ** " + str(queue2task)  + " ** " + "; time: " + time4xi() + "}"
                            print4xi(class2print, method2print, message2print) 
                            # 
                        
                            message2print = "{(Step 1M/33) warned" + "; queue2image4mpu-warned-by-wait2image4mpu: " + " ** " + str(queue2image4mpu)  + " ** " + "; time: " + time4xi() + "}"
                            print4xi(class2print, method2print, message2print)    
                            # 
                        except:
                            pass
                                                     
                except:     
                    message2print = "{(Step 1Z62/70) yielded" + "; queue2task-yielded-by-wait2image4mpu: " + str(queue2task) + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)                   
            
            size2queue4mpu = 0        
            try:                    
                size2queue4mpu = queue2image4mpu.qsize()
                message2print = "{(Step 1N1/33) got" + "; size2queue4mpu-got-by-wait2image4mpu to: " + str(size2queue4mpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                
                # 
                
                mpu2image4xi ['SIZE2QUEUE'] = size2queue4mpu
                message2print = "{(Step 1N2/33) set" + "; size2queue4image-got-by-wait2image4mpu to: " + str(mpu2image4xi ['SIZE2QUEUE'])  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                  
                # 
            except:     
                message2print = "{(Step 1Z7/70) yielded" + "; size2queue4mpu-yielded-by-wait2image4mpu: " + str(size2queue4mpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)    
            
            empty2queue4mpu = False
            try:                    
                empty2queue4mpu = queue2image4mpu.empty()
                message2print = "{(Step 1O1/33) got" + "; empty2queue4mpu-got-by-wait2image4mpu to: " + str(empty2queue4mpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                
                # 
                
                mpu2image4xi ['EMPTY2QUEUE'] = empty2queue4mpu
                message2print = "{(Step 1O2/33) set" + "; empty2queue4image-got-by-wait2image4mpu to: " + str(mpu2image4xi ['EMPTY2QUEUE'])  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                #               
            except:     
                message2print = "{(Step 1Z8/70) yielded" + "; empty2queue4mpu-yielded-by-wait2image4mpu: " + str(empty2queue4mpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)    
            
            full2queue4mpu = False
            try:                                
                full2queue4mpu = queue2image4mpu.full()
                message2print = "{(Step 1P1/33) got" + "; full2queue4mpu-got-by-wait2image4mpu to: " + str(full2queue4mpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                
                # 
                 
                mpu2image4xi ['FULL2QUEUE'] = full2queue4mpu
                message2print = "{(Step 1P2/33) set" + "; full2queue4image-got-by-wait2image4mpu to: " + str(mpu2image4xi ['FULL2QUEUE'])  + "; time: " + time4xi() + "}" 
                print4xi(class2print, method2print, message2print)
                #                          
            except:     
                message2print = "{(Step 1Z9/70) yielded" + "; full2queue4mpu-yielded-by-wait2image4mpu: " + str(full2queue4mpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)    
            
            state2queue4mpu = None
            try:                                
                state2queue4mpu = queue2image4mpu.__getstate__()
                message2print = "{(Step 1P3/33) got" + "; state2queue4mpu-got-by-wait2image4mpu to: " + str(state2queue4mpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                
                # 
                 
                mpu2image4xi ['STATE2QUEUE'] = state2queue4mpu
                message2print = "{(Step 1P4/33) set" + "; state2queue4image-set-by-wait2image4mpu to: " + str(mpu2image4xi ['STATE2QUEUE'])  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                #           
            except:     
                message2print = "{(Step 1Z10/70) yielded" + "; state2queue4mpu-yielded-by-wait2image4mpu: " + str(state2queue4mpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)               
                
                                                            
            message2print = "{(Step 1Q/33) finished" + "; mpu2image4xi-finished-by-wait2image4mpu: " + str(mpu2image4xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            # 
    except:
        try:
            # -------------------------------------------------------------------------------------
            # INITIALIZE           
            # -------------------------------------------------------------------------------------           
            mpu2image4mpu = create2image4mpu()
            message2print = "{(Step 1R/33) got" + "; mpu2image4xi-got-by-wait2image4mpu to: " + str(mpu2image4xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #  
            
            # -------------------------------------------------------------------------------------
            # WAIT         
            # -------------------------------------------------------------------------------------             
            queue2image4mpu = None
            try:
                queue2image4mpu = mpu2image4xi['QUEUE']
                message2print = "{(Step 1S/33) got" + "; queue2image4mpu-got-by-wait2image4mpu to: " + str(queue2image4mpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                #
            except:     
                message2print = "{(Step 1Z10/70) yielded" + "; queue2image4mpu-yielded-by-wait2image4mpu: " + str(queue2image4mpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                
            if task4wait2image is None:
                pass
            else:
                # -------------------------------------------------------------------------------------
                # IMAGE QUEUE GIVEN AS PARAMETER          
                # -------------------------------------------------------------------------------------               
                queue2task = None    
                try:
                    queue2task = task4wait2image ['QUEUE']
                    message2print = "{(Step 1T/33) passed" + "; queue2task-passed-by-wait2image4mpu to: " + str(queue2task)  + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)
                    #
                    
                    if queue2task == queue2image4mpu:
                        # -------------------------------------------------------------------------------------
                        # ACCEPTED - GIVEN QUEUE IS THE SAME          
                        # -------------------------------------------------------------------------------------                     
                        message2print = "{(Step 1U/33) accepted" + "; queue2task-accepted-by-wait2image4mpu: " + str(queue2task) + "; time: " + time4xi() + "}"
                        print4xi(class2print, method2print, message2print)
                        # 
                    else:
                        message2print = "{(Step 1X/33) warned" + "; queue2task-warned-by-wait2image4mpu: " + " ** " + str(queue2task)  + " ** " + "; time: " + time4xi() + "}"
                        print4xi(class2print, method2print, message2print) 
                        # 
                        
                        message2print = "{(Step 1Y1/33) warned" + "; queue2image4mpu-warned-by-wait2image4mpu: " + " ** " + str(queue2image4mpu)  + " ** " + "; time: " + time4xi() + "}" 
                        print4xi(class2print, method2print, message2print)    
                        #                      
                except:     
                    message2print = "{(Step 1Z11/70) yielded" + "; queue2task-yielded-by-wait2image4mpu: " + str(queue2task) + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print) 
            
            size2image4mpu = 0                             
            try:                    
                size2image4mpu = queue2image4mpu.qsize()
                message2print = "{(Step 1Y2/33) got" + "; size2image4mpu-got-by-wait2image4mpu to: " + str(size2image4mpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                
                #
            except:     
                message2print = "{(Step 1Z12/70) yielded" + "; size2image4mpu-yielded-by-wait2image4mpu: " + str(size2image4mpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
             
            empty2image4mpu = True          
            try:                    
                empty2image4mpu = queue2image4mpu.empty()
                message2print = "{(Step 1Y3/33) got" + "; empty2image4mpu-got-by-wait2image4mpu to: " + str(empty2image4mpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                
                #
            except:     
                message2print = "{(Step 1Z13/70) yielded" + "; mpu2xi-yielded-by-wait2image4mpu: " + str(mpu2xi) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
             
            full2image4mpu = False          
            try:                                
                full2image4mpu= queue2image4mpu.full()
                message2print = "{(Step 1Y4/33) got" + "; full2image4mpu-got-by-wait2image4mpu to: " + str(full2image4mpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                
                #                
            except:     
                message2print = "{(Step 1Z14/70) yielded" + "; full2image4mpu-yielded-by-wait2image4mpu: " + str(full2image4mpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)    
                                                        
            message2print = "{(Step 1Y5/33) finished" + "; mpu2image4xi-finished-by-wait2image4mpu: " + str(mpu2image4xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)                                    
        except:
            mpu2image4mpu= {}
            message2print = "{(Step 1Z15/70) yielded" + "; mpu2image4xi-yielded-by-wait2image4mpu: " + str(mpu2image4xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
                        
    message2print = "{(Step 33/33) returning" + "; mpu2image4xi-returning-by-wait2image4mpu: " + str(mpu2image4xi) + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)
    # {'SIZE2QUEUE': 2147483646, 
    #  'FORMAT': {'TYPE': 'CL_UNORM_INT8', 
    #  'ORDER': 'CL_RGBA'}, 
    #  'QUEUE': <queue4xi.Queue4Xi object at 0x0000000004420E10>, 
    #  'WIDTH': 800, 
    #  'CONTEXT': ...
    
    return mpu2image4xi
    
   
#</function4xi> 
                                               
# =================================================================================================
#<class4xi class = "MPU2Image4Xi" base = "object" child = "None"
#   synopsis = "offers methods for MPU 2D images" usage = "2D Image4Xi of MPU"
#>

# =================================================================================================
class MPU2Image4Xi(object): 
    ''' 
    MPU2Image4Xi offers methods for MPU 2D images
    '''            

    #                                        
    # ---------------------------------------------------------------------------------------------
    # sharable class variables
    # ---------------------------------------------------------------------------------------------      
    #<share4xi share = " 
    classvariable2mpu = cpu_corecount4xi()
    #"
    #/>
        
    # ---------------------------------------------------------------------------------------------
    # Public API of this class
    # ---------------------------------------------------------------------------------------------    
    #<expose4xi expose = "
    _exposed_ = (
            'create2image',                    # ACK 5.31   p76    
            'copy2image',                      # ACK 5.31   p88
            'copy2imagetobuffer',              # ACK 5.3.4  p90
            'copy2buffertoimage',              # ACK 5.3.4  p92
            'read2image',                      # ACK 5.3.3  p84
            'write2image',                     # ACK 5.3.3  p84            
            'map2image',                       # ACK 5.3.5  p94            
            'get2imageinfo',                   # ACK 5.3.6  p97
            'get2supportedimageformats',       # ACK 5.3.2  p82 nb: a separate method to get format whereas using get2imageinfo you can access any element by key such as 'FORMAT'   
            'retain2image',                    # ACK 5.4.1  p99 nb: for compability to OpenCL - you can get any element of dictionary by get2imageinfo
            'release2image',                   # ACK 5.4.1  p99
            'trace2image',                     # ACK modernized 
            'set2imagecallback',               # ACK modernized
            'set2image',                       # ACK modernized nb: with memorymap features you can process 2D images as files!                    
            'wait2image',                      # ACK modernized nb: checks Queue object and thereby generates delay   
            'sense',                           # ACK modernized  
            'reason',                          # ACK modernized  
            'ace')                             # ACK modernized           
    #"
    #/>
 
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "init-of-MPU2Image4Xi" usage = "
    #   print('(Step 5A/33) OF MPU2IMAGE4XI5TEST - CREATING IMAGE4MPU OBJECT ', time4xi())     
    #   try:
    #       imageobject2mpu = MPU2Image4Xi()
    #       print('(Step 5B/33) OF MPU2IMAGE4XI5TEST - CREATED IMAGE4MPU OBJECT: ', imageobject2mpu)                                  
    #   except:
    #       print('(Step 5Z/33) OF MPU2IMAGE4XI5TEST *** FAILED TO CREATE IMAGE4MPU OBJECT')
    #">
    
    # ---------------------------------------------------------------------------------------------   
    def __init__(self, task4imagecreate2mpu = None):
        '''
        _init__ method initializes MPU2Image4Xi
        ''' 
        
        # -----------------------------------------------------------------------------------------
        # (Step 1/33)           
        # -----------------------------------------------------------------------------------------
                    
        class2print = "MPU2Image4Xi"
        method2print = "__init__"
                     
        message2print = "{(Step 1A/33) started" + "; init-of-MPU2Image4Xi-started: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print)                                                          
        # 23:41:37 09/06/11 Pacific Daylight Time 

        message2print = "{(Step 1B/33) borning" + "; mpu2image2xi-of-MPU2Image4Xi-borning: None" + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                          
        # None 
                
        message2print = "{(Step 1C/33) inheriting" + "; object-inheriting-by-mpu2image2xi-of-MPU2Image4Xi: " + str(object)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                          
        # <class 'object'>
        
        message2print = "{(Step 1D/33) sharing" + "; classvariable2mpu-sharing-by-mpu2image2xi-of-MPU2Image4Xi: " + str(MPU2Image4Xi.classvariable2mpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                          
        # 4             
                                
        message2print = "{(Step 1E/33) passed" + "; task4imagecreate2mpu-passed-to-init-of-MPU2Image4Xi: " + str(task4imagecreate2mpu) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                          
        # None                            
        
        # -----------------------------------------------------------------------------------------
        # (Step 2/33)             
        # -----------------------------------------------------------------------------------------
        
        message2print = "{(Step 2A/33) passing" + "; task4imagecreate2mpu-passing-to-create2image4mpu-by-init-of-MPU2Image4Xi: " + str(task4imagecreate2mpu) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                          
        # None  
        
        mpu2image2xi = create2image4mpu(task4imagecreate2mpu)
        
        message2print = "{(Step 2B/33) performed" + "; create2image4mpu-performed-by-init-of-MPU2Image4Xi: " + str(create2image4mpu) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                          
        # <function create2image4mpu at 0x0000000003A9E7C8>
                        
        # -----------------------------------------------------------------------------------------
        # (Step 3/33)             
        # -----------------------------------------------------------------------------------------
        message2print = "{(Step 3A/33) activated" + "; mpu2image2xi-of-MPU2Image4Xi-activated: " + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                          
        # { MPU2Image4Xi : __init__ ; (Step 3A/33) activated; mpu2image2xi-of-MPU2Image4Xi-activated: ; time: 00:14:05 09/07/11 Pacific Daylight Time} 
                
        message2print = "{(Step 3B/33) inherited" + "; object-inherited-by-mpu2image2xi-of-MPU2Image4Xi: " + str(object)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                          
        # <class 'object'> 
        
        message2print = "{(Step 3C/33) shared" + "; classvariable2mpu-shared-by-mpu2image2xi-of-MPU2Image4Xi: " + str(MPU2Image4Xi.classvariable2mpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                          
        # 4           
       
        message2print = "{(Step 3D/33) returning" + "; init-of-MPU2Image4Xi-returning: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print)                                                          
        # 00:14:05 09/07/11 Pacific Daylight Time
        

    #</methodize4xi>
    

    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "create2image-of-MPU2Image4Xi" usage = "
    #   print('(Step 23A/33) OF MPU2IMAGE4XI5TEST - TRYING TO CREATE MPU IMAGE', time4xi()) 
    #   try:
    #       xi2mpu = MPU2Image4Xi()                            
    #       image2mpu = xi2mpu.create2image()                                  
    #       print('(Step 23B/33) OF MPU2IMAGE4XI5TEST - CREATED MPU IMAGE: ', image2mpu)
    #   except:
    #       print('(Step 23Z/33) OF MPU2IMAGE4XI5TEST *** FAILED TO CREATE MPU IMAGE ***', time4xi())                  
    #                     
    #">   
    
    # ---------------------------------------------------------------------------------------------   
    def create2image(self, task4image2create = None):
        '''
        create2image method creates 2D image objects of MPU2Image4Xi
        ''' 
        
        # -----------------------------------------------------------------------------------------
        # (Step 1/33)           
        # -----------------------------------------------------------------------------------------              
        class2print = "MPU2Image4Xi"
        method2print = "create2image"
                     
        message2print = "{(Step 1A/33) started" + "; create2image-of-MPU2Image4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
        # 00:14:05 09/07/11 Pacific Daylight Time 
            
        message2print = "{(Step 1B/33) passed" + "; task4image2create-passed-to-create-of-MPU2Image4Xi: " + str(task4image2create) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                      
        # None 
          
        # -----------------------------------------------------------------------------------------
        # (Step 2/33)           
        # -----------------------------------------------------------------------------------------
        message2print = "{(Step 2A/33) passing" + "; task4image2create-passing-to-create2image4mpu-by-create2image-of-MPU2Image4Xi: " + str(task4image2create) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
        # None      
  
        mpu2image2xi = create2image4mpu(task4image2create)

        message2print = "{(Step 2B/33) performed" + "; create2image4mpu-performed-by-create2image-of-MPU2Image4Xi: " + str(create2image4mpu) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)             
        # <function create2image4mpu at 0x00000000039DF7C8>
                                
        # -----------------------------------------------------------------------------------------
        # (Step 3/33)           
        # ----------------------------------------------------------------------------------------- 
        message2print = "{(Step 3/33) returning" + "; mpu2image2xi-returning-by-create2image-of-MPU2Image4Xi: " + str(mpu2image2xi) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print)                                                          # 
        # {'CODE2ERROR': 0,
        #  'ACK2RETURN': ' 1D-1E-2C-2B-3B-4B-5B-6B-7B-8B-9B-10B-11B-14B-15B-16B-17B-17D-18B-18C-21A1-22B1-22C1-23B-24B-25B-26B-27B-28B-29B-30B-31B-34H-35H-36B-37B-38B-',
        #  'ROWPITCH': 800,
        #  'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2image4mpu-failed ; time: 00:53:55 10/05/11 Pacific Daylight Time}',
        #  'CLASS': 'create2image4mpu',
        #  'METHOD': 'create2image4mpu'},
        #  'WIDTH': 800 ...

        return mpu2image2xi
                                
    #                        
    #</methodize4xi>
    

    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "set2image-of-MPU2Image4Xi" usage = "
    #   print('(Step 1A/33) OF MPU2IMAGE4XI - TRYING TO SET MPU IMAGE')
    #   try:
    #       cc2xi = MPU2Image4Xi()                            
    #       image2mpu = cc2xi.set2image()                                  
    #       print('(Step 1B/33) OF MPU4XI5TEST - SET MPU IMAGE: ', image2mpu)                                                  
    #   except:
    #       print('(Step 1Z/33) OF MPU4XI5TEST *** FAILED TO SET MPU IMAGE ***')  
    #">
    
    # ---------------------------------------------------------------------------------------------   
    def set2image(self, task4set2image = None):
        '''
        sets 2D image MPU objects
        ''' 
        
        # -----------------------------------------------------------------------------------------
        # (Step 1/33)           
        # -----------------------------------------------------------------------------------------              
        class2print = "MPU2Image4Xi"
        method2print = "set2image"
                     
        message2print = "{(Step 1A/33) started" + "; set2image-of-MPU2Image4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
        # 01:02:22 09/07/11 Pacific Daylight Time 
            
        message2print = "{(Step 1B/33) passed" + "; task4set2image-passed-to-set2image-of-MPU2Image4Xi: " + str(task4set2image) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                          
        # {'CODE2ERROR': 0,
        #  'QUEUE': <queue4xi.Queue4Xi object at 0x0000000004191E10>,
        #  'CONTEXT': {'COUNT2REFERENCE': 0, ...
        
          
        # -----------------------------------------------------------------------------------------
        # (Step 2/33)           
        # -----------------------------------------------------------------------------------------
        message2print = "{(Step 2A/33) passing" + "; task4set2image-passing-to-set2image4mpu-by-set2image-of-MPU2Image4Xi: " + str(task4set2image) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                               
        # {'CODE2ERROR': 0, 
        #  'QUEUE': <queue4xi.Queue4Xi object at 0x0000000004191E10>, 
        #  'CONTEXT': {'COUNT2REFERENCE': 0 ...

        mpu2image2xi = set2image4mpu(task4set2image)

        message2print = "{(Step 2B/33) performed" + "; set2image4mpu-performed-by-set2image-of-MPU2Image4Xi: " + str(set2image4mpu) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)             
        # <function set2image4mpu at 0x0000000003AB36C8> 
                                
        # -----------------------------------------------------------------------------------------
        # (Step 3/33)           
        # ----------------------------------------------------------------------------------------- 
       
        message2print = "{(Step 3A/33) returning" + "; mpu2image2xi-returning-by-set2image-of-MPU2Image4Xi: " + str(mpu2image2xi) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
        # {'CODE2ERROR': 0,
        #  'ACK2RETURN': ' 23B-24B-25B-26B-27B-28B-',
        #  'ROWPITCH': 800,
        #  'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2image4mpu-failed ; time: 01:02:22 09/07/11 Pacific Daylight Time}', 'CLASS': 'create2image4mpu', 'METHOD': 'create2image4mpu'},
        #  'WIDTH': 800,
        #  'BUFFER': {'CODE2ERROR': 0, 'COUNT2REFERENCE': 0, 'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2buffer4mpu-failed ; time: 01:02:22 09/07/11 Pacific Daylight Time}',
        #  'CLASS': 'create2buffer4mpu',
        #  'METHOD': 'create2buffer4mpu'},
        #  'FUNCTION2CALLBACK': <function trace2buffer4mpu at 0x0000000003AA81C8>,
        #  'POINTER2BUFFER': <mmap.mmap object at 0x0000000003CC92F0>,
        #  'FLAGS': 0,
        #  'CONTEXT': {'COUNT2REFERENCE': 0 ...
                    
                
        return mpu2image2xi
                                
                            
    #</methodize4xi>

    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "get2imageinfo-of-MPU2Image4Xi" usage = "
    #   print('(Step 1A/33) OF MPU2IMAGE4XI - TRYING TO GET MPU 2D IMAGE')
    #   try:
    #       cc2xi = MPU2Image4Xi()                            
    #       image2mpu = cc2xi.get2imageinfo()                                  
    #       print('(Step 1B/33) OF MPU4XI5TEST - GOT MPU IMAGE: ', image2mpu)                                                  
    #   except:
    #       print('(Step 1Z/33) OF MPU4XI5TEST *** FAILED TO GET MPU 2D IMAGE ***')   
    # ">
    
    # ---------------------------------------------------------------------------------------------   
    def get2imageinfo(self, task4getinfo2image=None):
        '''
        get2imageinfo method gets 2D objects of MPU2Image4Xi
        ''' 
        
        # -----------------------------------------------------------------------------------------
        # (Step 1/33)           
        # -----------------------------------------------------------------------------------------              
        class2print = "MPU2Image4Xi"
        method2print = "get2imageinfo"
                     
        message2print = "{(Step 1A/33) started" + "; get2imageinfo-of-MPU2Image4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
        # 00:14:05 09/07/11 Pacific Daylight Time 

        # -----------------------------------------------------------------------------------------
        # (Step 2/33)           
        # ----------------------------------------------------------------------------------------- 
        mpu2image2xi = get2imageinfo4mpu(task4getinfo2image)
        
        message2print = "{(Step 2A/33) returning" + "; mpu2image2xi-returning-by-get2imageinfo-of-MPU2Image4Xi: " + str(mpu2image2xi) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
        # {'CODE2ERROR': 0,
        #  'ACK2RETURN': ' 1D-1E-2C-2B-3B-4B-5B-6B-7B-8B-9B-10B-11B-14B-15B-16B-17B-17D-18B-18C-21A1-22B1-22C1-23B-24B-25B-26B-27B-28B-29B-30B-31B-34H-35H-36B-37B-38B-',
        #  'ROWPITCH': 800,
        #  'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2image4mpu-failed ; time: 00:14:05 09/07/11 Pacific Daylight Time}', 'CLASS': 'create2image4mpu', 'METHOD': 'create2image4mpu'},
        #  'WIDTH': 800,
        #  'BUFFER': {'CODE2ERROR': 0, 'COUNT2REFERENCE': 0, 'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2buffer4mpu-failed ; time: 00:14:05 09/07/11 Pacific Daylight Time}',
        #  'CLASS': 'create2buffer4mpu',
        #  'METHOD': 'create2buffer4mpu'},
        #  'FUNCTION2CALLBACK': <function trace2buffer4mpu at 0x00000000039D8348>,
        #  'POINTER2BUFFER': <mmap.mmap object at 0x00000000037D32F0>,
        #  'FLAGS': 0,
        #  'CONTEXT': {'COUNT2REFERENCE': 0, ...
        # or when only two elemets required
        # {'WIDTH': 800, 'HEIGHT': 800}
        
        return mpu2image2xi
                            
    #</methodize4xi>

    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "get2supportedimageformats-of-MPU2Image4Xi" usage = "
    #  print('(Step 6B1/33) OF MPU2IMAGE4XI5TEST - TRYING TO GET SUPPORTED IMAGE FORMATS FOR MPU')
    #  try:
    #      cc2xi = MPU2Image4Xi()                            
    #      supportedimageformats2mpu = cc2xi.get2supportedimageformats()                                  
    #      print('(Step 6B2/33) OF MPU4XI5TEST - GOT SUPPORTED IMAGE FORMATS FOR MPU: ', supportedimageformats2mpu)
    #                                                   
    #  except:
    #      print('(Step 2Z/33) OF MPU4XI5TEST *** FAILED TO GET SUPPORTED IMAGE FORMATS FOR MPU***')     
    # ">
    
    # ---------------------------------------------------------------------------------------------   
    def get2supportedimageformats(self):
        '''
        get2supportedimageformats method gets supported imageformats by objects of MPU2Image4Xi
        ''' 
        
        # -----------------------------------------------------------------------------------------
        # (Step 1/33)           
        # -----------------------------------------------------------------------------------------              
        class2print = "MPU2Image4Xi"
        method2print = "get2supportedimageformats"
                     
        message2print = "{(Step 1A/33) started" + "; get2supportedimageformats-of-MPU2Image4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
        # 01:43:00 09/07/11 Pacific Daylight Time

        # -----------------------------------------------------------------------------------------
        # (Step 2/33)           
        # ----------------------------------------------------------------------------------------- 
        supportedimageformats2mpu = get2supportedimageformats4mpu()
        
        message2print = "{(Step 2A/33) returning" + "; supportedimageformats2mpu-returning-by-get2imageinfo-of-MPU2Image4Xi: " + str(supportedimageformats2mpu) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
        # {'TYPE': 'CL_UNORM_INT8', 'ORDER': 'CL_RGBA'} 
        
        return supportedimageformats2mpu
                            
    #</methodize4xi>    
    
    
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "read2image-of-MPU2Image4Xi" usage = "
    #   print('(Step 127A/700) OF MPU2IMAGE4XI5TEST - TRYING TO ESTABLISH CONTROL AND COMMAND WITH READ IMAGE OBJECT ', time4xi()) 
    #   response4read2mpu = {}
    #   try:
    #       xi2mpu = MPU2Image4Xi()    
    #       response4read2mpu = xi2mpu.read2image()
    #       print('(Step 127B/700) OF MPU2IMAGE4XI5TEST - ESTABLISHED CONTROL AND COMMAND WITH READ IMAGE OBJECT, RESPONSE4READ2MPU: ', response4read2mpu) 
    #   except:
    #       print('(Step 127Z1/700) OF MPU2IMAGE4XI5TEST *** FAILED TO ESTABLISH CONTROL AND COMMAND WITH READ IMAGE OBJECT ', time4xi())
    #   ... continue as with read2image4mpu function
    #">
    
    # ---------------------------------------------------------------------------------------------   
    def read2image(self, task4read2image = None):
        '''
        reads MPU2Image4Xi objects
        ''' 
        
        # -----------------------------------------------------------------------------------------
        # (Step 1/33)           
        # -----------------------------------------------------------------------------------------              
        class2print = "MPU2Image4Xi"
        method2print = "read2image"
                     
        message2print = "{(Step 1A/33) started" + "; read2image-of-MPU2Image4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
        # 00:20:02 10/05/11 Pacific Daylight Time 
            
        message2print = "{(Step 1B/33) passed" + "; task4read2image-passed-to-read2image-of-MPU2Image4Xi: " + str(task4read2image) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                          
        # None 
          
        # -----------------------------------------------------------------------------------------
        # (Step 2/33)           
        # -----------------------------------------------------------------------------------------
        message2print = "{(Step 2A/33) passing" + "; task4read2image-passing-to-read2image4mpu-by-read2image-of-MPU2Image4Xi: " + str(task4read2image) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                               
        # None
  
        response4readimage2mpu = read2image4mpu(task4read2image)

        message2print = "{(Step 2B/33) performed" + "; read2image4mpu-performed-by-read2image-of-MPU2Image4Xi: " + str(read2image4mpu) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)             
        # <function read2image4mpu at 0x000000000385DA48> 
                                
        # -----------------------------------------------------------------------------------------
        # (Step 3/33)           
        # ----------------------------------------------------------------------------------------- 
       
        message2print = "{(Step 3A/33) returning" + "; response4readimage2mpu-returning-by-read2image-of-MPU2Image4Xi: " + str(response4readimage2mpu) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
        # {'QUEUE': <queue4xi.Queue4Xi object at 0x000000000417DD68>,
        #  'PUBLISHERCHANNEL': <read-write PipeConnection, handle 936>,
        #  'CONSUMERCHANNEL': <read-write PipeConnection, handle 932>}; time: 00:20:02 10/05/11 Pacific Daylight Time}
  
        return response4readimage2mpu
                                                           
    #</methodize4xi>
    
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "map2image-of-MPU2Image4Xi" usage = "
    #   print('(Step 127A/700) OF MPU2IMAGE4XI5TEST - TRYING TO ESTABLISH CONTROL AND COMMAND WITH MAP IMAGE OBJECT ', time4xi()) 
    #   xi2mpu = MPU2Image4Xi() 
    #   ...
    #   response4map2mpu = {}
    #   try:
    #       response4map2mpu = xi2mpu.map2image()
    #       print('(Step 127B/700) OF MPU2IMAGE4XI5TEST - ESTABLISHED CONTROL AND COMMAND WITH MAP IMAGE OBJECT, RESPONSE4READ2MPU: ', response4map2mpu) 
    #   except:
    #       print('(Step 127Z1/700) OF MPU2IMAGE4XI5TEST *** FAILED TO ESTABLISH CONTROL AND COMMAND WITH MAP IMAGE OBJECT ', time4xi())
    #   ... continue as with map2image4mpu function
    #">
    
    # ---------------------------------------------------------------------------------------------   
    def map2image(self, task4map2image = None):
        '''
        maps MPU2Image4Xi objects
        ''' 
        
        # -----------------------------------------------------------------------------------------
        # (Step 1/33)           
        # -----------------------------------------------------------------------------------------              
        class2print = "MPU2Image4Xi"
        method2print = "map2image"
                     
        message2print = "{(Step 1A/33) started" + "; map2image-of-MPU2Image4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
        # 01:28:44 10/05/11 Pacific Daylight Time 
            
        message2print = "{(Step 1B/33) passed" + "; task4map2image-passed-to-map2image-of-MPU2Image4Xi: " + str(task4map2image) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                          
        # None
          
        # -----------------------------------------------------------------------------------------
        # (Step 2/33)           
        # -----------------------------------------------------------------------------------------
        message2print = "{(Step 2A/33) passing" + "; task4map2image-passing-to-map2image4mpu-by-map2image-of-MPU2Image4Xi: " + str(task4map2image) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                               
        # None
  
        response4mapimage2mpu = map2image4mpu(task4map2image)

        message2print = "{(Step 2B/33) performed" + "; map2image4mpu-performed-by-map2image-of-MPU2Image4Xi: " + str(map2image4mpu) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)             
        # <function map2image4mpu at 0x000000000390BAC8> 
                                
        # -----------------------------------------------------------------------------------------
        # (Step 3/33)           
        # ----------------------------------------------------------------------------------------- 
       
        message2print = "{(Step 3A/33) returning" + "; response4mapimage2mpu-returning-by-map2image-of-MPU2Image4Xi: " + str(response4mapimage2mpu) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
        # {'QUEUE': <queue4xi.Queue4Xi object at 0x00000000050E9C18>,
        #  'PUBLISHERCHANNEL': <read-write PipeConnection, handle 1100>,
        #  'CONSUMERCHANNEL': <read-write PipeConnection, handle 1096>}; time: 01:28:44 10/05/11 Pacific Daylight Time}

        
        return response4mapimage2mpu
                                                           
    #</methodize4xi>
            
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "release2image-of-MPU2Image4Xi" usage = "
    #
    # ---------------------------------------------------------------------------------------------
    # RELEASE 2D IMAGE BY DECREMENTING REFERENCE COUNT WITH 1             
    # ---------------------------------------------------------------------------------------------    
    #   task4release2image  = {} 
    #   task4release2image  ['COUNT2REFERENCE'] = 1 
    #   response4release2image = {}
    #   print('(Step 133A/700) OF MPU2IMAGE4XI5TEST - TRYING TO RELEASE 2D IMAGE OF MPU DEVICE ', time4xi())    
    # 
    #   try:                               
    #       response4release2Dimage = xi2mpu.release2image(task4release2image)                                                                                             
    #       print('(Step 133B/700) OF MPU2IMAGE4XI5TEST - RELEASED 2D IMAGE OF MPU DEVICE ', time4xi())                           
    #       keys2response = response4release2Dimage.keys()
    #       for key2response in sorted(keys2response):
    #           print(key2response, ' : ', response4release2Dimage[key2response])
    #           #
    #   except:
    #       print('(Step 133Z1/700) OF MPU2IMAGE4XI5TEST *** FAILED TO RELEASE IMAGE OF MPU DEVICE ', time4xi())  
    #   
    #   task4getinfo2image = {}
    #   task4getinfo2image ['COUNT2REFERENCE'] = 0
    #   response4getinfo2image = {}
    #   print('(Step 133C/700) OF MPU2IMAGE4XI5TEST - TRYING TO GET SELECTED 2D IMAGE INFO OF MPU DEVICE ', time4xi())     
    #   # 
    #   try:                               
    #       response4getinfo2Dimage = xi2mpu.get2imageinfo (task4getinfo2image)                                                                                             
    #       keys2response = response4getinfo2Dimage.keys()
    #       print('(Step 133D/700) OF MPU2IMAGE4XI5TEST - GOT SELECTED 2D IMAGE INFO OF MPU DEVICE ', time4xi())   
    #       for key2response in sorted(keys2response):
    #           print(key2response, ' : ', response4getinfo2Dimage[key2response])
    #           # 
    #   except:
    #       print('(Step 133Z2/700) OF MPU2IMAGE4XI5TEST *** FAILED TO GET SELECTED 2D IMAGE INFO OF MPU DEVICE ', time4xi())  
    #">
    
    # ---------------------------------------------------------------------------------------------   
    def release2image(self, task4release2image = None):
        '''
        releases MPU2Image4Xi objects
        ''' 
        
        # -----------------------------------------------------------------------------------------
        # (Step 1/33)           
        # -----------------------------------------------------------------------------------------              
        class2print = "MPU2Image4Xi"
        method2print = "release2image"
                     
        message2print = "{(Step 1A/33) started" + "; release2image-of-MPU2Image4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
        # 
            
        message2print = "{(Step 1B/33) passed" + "; task4release2image-passed-to-release2image-of-MPU2Image4Xi: " + str(task4release2image) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                          
        # 
          
        # -----------------------------------------------------------------------------------------
        # (Step 2/33)           
        # -----------------------------------------------------------------------------------------
        message2print = "{(Step 2A/33) passing" + "; task4release2image-passing-to-release2image4mpu-by-release2image-of-MPU2Image4Xi: " + str(task4release2image) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                               
        # 
  
        mpu2image2xi = release2image4mpu(task4release2image)

        message2print = "{(Step 2B/33) performed" + "; release2image4mpu-performed-by-release2image-of-MPU2Image4Xi: " + str(release2image4mpu) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)             
        # 
                                
        # -----------------------------------------------------------------------------------------
        # (Step 3/33)           
        # ----------------------------------------------------------------------------------------- 
       
        message2print = "{(Step 3A/33) returning" + "; mpu2image2xi-returning-by-release2image-of-MPU2Image4Xi: " + str(mpu2image2xi) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
        #                          
                
        return mpu2image2xi
                                                           
    #</methodize4xi>
    
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "retain2image-of-MPU2Image4Xi" usage = "
    #
    # ---------------------------------------------------------------------------------------------
    # RETAIN 2D IMAGE BY DECREASING REFERENCE WITH 1             
    # ---------------------------------------------------------------------------------------------    
    #   task4retain2image  = {}
    #   task4retain2image  ['COUNT2REFERENCE'] = 1 
    #   response4retain2Dimage = {}
    #   print('(Step 134A/700) OF MPU2IMAGE4XI5TEST - TRYING TO RETAIN 2D IMAGE OF MPU DEVICE ', time4xi())   
    #   try:                               
    #       response4retain2Dimage = xi2mpu.retain2image(task4retain2image)                                                                                                 
    #       print('(Step 134B/700) OF MPU2IMAGE4XI5TEST - RESPONSE4RETAIN2DIMAGE : ', time4xi())                         
    #       keys2response = response4retain2Dimage.keys()
    #       for key2response in sorted(keys2response):
    #           print(key2response, ' : ', response4retain2Dimage[key2response])
    #           # 
    #   except:
    #       print('(Step 134Z1/700) OF MPU2IMAGE4XI5TEST *** FAILED TO RETAIN 2D IMAGE OF MPU DEVICE ', time4xi())
    #   task4getinfo2image = {}
    #   task4getinfo2image ['COUNT2REFERENCE'] = 0
    #   response4getinfo2image = {}
    #   print('(Step 134C/700) OF MPU2IMAGE4XI5TEST - TRYING TO GET SELECTED 2D IMAGE INFO OF MPU DEVICE ', time4xi())     
    #   # 
    #   try:                               
    #       response4getinfo2Dimage = xi2mpu.get2imageinfo (task4getinfo2image)                                                                                             
    #       keys2response = response4getinfo2Dimage.keys()
    #       print('(Step 134D/700) OF MPU2IMAGE4XI5TEST - GOT SELECTED 2D IMAGE INFO OF MPU DEVICE ', time4xi())   
    #       for key2response in sorted(keys2response):
    #           print(key2response, ' : ', response4getinfo2Dimage[key2response])
    #           # 
    #   except:
    #       print('(Step 134Z2/700) OF MPU2IMAGE4XI5TEST *** FAILED TO GET SELECTED 2D IMAGE INFO OF MPU DEVICE ', time4xi())  
    #">
    
    # ---------------------------------------------------------------------------------------------   
    def retain2image(self, task4retain2image = None):
        '''
        retains MPU2Image4Xi objects
        ''' 
        
        # -----------------------------------------------------------------------------------------
        # (Step 1/33)           
        # -----------------------------------------------------------------------------------------              
        class2print = "MPU2Image4Xi"
        method2print = "retain2image"
                     
        message2print = "{(Step 1A/33) started" + "; retain2image-of-MPU2Image4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
        # 00:14:05 09/07/11 Pacific Daylight Time 
            
        message2print = "{(Step 1B/33) passed" + "; task4retain-passed-to-retain2image-of-MPU2Image4Xi: " + str(task4retain2image) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                          
        # {'COUNT2REFERENCE': 1}
          
        # -----------------------------------------------------------------------------------------
        # (Step 2/33)           
        # -----------------------------------------------------------------------------------------
        message2print = "{(Step 2A/33) passing" + "; task4retain-passing-to-retain2image4mpu-by-retain-of-MPU2Image4Xi: " + str(task4retain2image) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                               
        # {'COUNT2REFERENCE': 1}
  
        mpu2image2xi = retain2image4mpu(task4retain2image)

        message2print = "{(Step 2B/33) performed" + "; retain2image4mpu-performed-by-retain2image-of-MPU2Image4Xi: " + str(retain2image4mpu) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)             
        # <function retain2image4mpu at 0x00000000039DF948>
                                
        # -----------------------------------------------------------------------------------------
        # (Step 3/33)           
        # ----------------------------------------------------------------------------------------- 
       
        message2print = "{(Step 3A/33) returning" + "; mpu2image2xi-returning-by-retain2image-of-MPU2Image4Xi: " + str(mpu2image2xi) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
        # {'CODE2ERROR': 0, 
        #  'ACK2RETURN': ' 1D-', 
        #  'ROWPITCH': 800, 
        #  'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2image4mpu-failed ; time: 00:14:05 09/07/11 Pacific Daylight Time}', 
        #  'CLASS': 'create2image4mpu', 
        #  'METHOD': 'create2image4mpu'}, 
        #  'WIDTH': 800, 
        #  'BUFFER': {'CODE2ERROR': 0, 
        #  ...
        #  'COUNT2REFERENCE': 1,
        #  'FLAGS': 0,
        #  'CONTEXT': {'COUNT2REFERENCE': 0 ...
                         
                
        return mpu2image2xi
                                                           
    #</methodize4xi>   
    
    
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "wait2image-of-MPU2Image4Xi" usage = "
    #   print('=' *33)          
    #   print('(Step 126A/700) OF MPU2IMAGE4XI5TEST - TRYING TO WAIT 2D IMAGE OPERATION FOR MPU ', time4xi()) 
    #   print('=' *33)      
    #   wait2mpu = {}
    #   try:
    #       response4wait2Dimage = xi2mpu.wait2image()
    #       print('(Step 122B/700) OF MPU2IMAGE4XI5TEST - WAITED 2D IMAGE OPERATION FOR MPU ', time4xi()) 
    #       keys2response = response4wait2Dimage.keys()
    #       for key2response in sorted(keys2response):
    #           print(key2response, ' : ', response4wait2Dimage[key2response])
    #           # 
    #   except:
    #       print('(Step 126Z1/700) OF MPU2IMAGE4XI5TEST *** FAILED TO 2D IMAGE OPERATION FOR MPU ', time4xi())    
    #">
    
    # ---------------------------------------------------------------------------------------------   
    def wait2image(self, task4wait2image = None):
        '''
        waits MPU2Image4Xi objects
        ''' 
        
        # -----------------------------------------------------------------------------------------
        # (Step 1/33)           
        # -----------------------------------------------------------------------------------------              
        class2print = "MPU2Image4Xi"
        method2print = "wait2image"
                     
        message2print = "{(Step 1A/33) started" + "; wait2image-of-MPU2Image4Xi-started: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print)   
        # 00:14:05 09/07/11 Pacific Daylight Time 
            
        message2print = "{(Step 1B/33) passed" + "; task4wait2image-passed-to-wait2image-of-MPU2Image4Xi: " + str(task4wait2image) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                          
        # None 
          
        # -----------------------------------------------------------------------------------------
        # (Step 2/33)           
        # -----------------------------------------------------------------------------------------
        message2print = "{(Step 2A/33) passing" + "; task4wait-passing-to-wait2image4mpu-by-wait2image-of-MPU2Image4Xi: " + str(task4wait2image) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                               
        # None
  
        mpu2image2xi = wait2image4mpu(task4wait2image)

        message2print = "{(Step 2B/33) performed" + "; wait2image4mpu-performed-by-wait2image-of-MPU2Image4Xi: " + str(wait2image4mpu) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)             
        # semaphore and lock analysis by multiprocessing4xi
        # <function wait2image4mpu at 0x00000000039DFA48> 
                                
        # -----------------------------------------------------------------------------------------
        # (Step 3/33)           
        # ----------------------------------------------------------------------------------------- 
       
        message2print = "{(Step 3A/33) returning" + "; mpu2image2xi-returning-by-wait2image-of-MPU2Image4Xi: " + str(mpu2image2xi) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
        # {'SIZE2QUEUE': 2147483646,
        #  'FORMAT': {'TYPE': 'CL_UNORM_INT8', 'ORDER': 'CL_RGBA'},
        #  'QUEUE': <queue4xi.Queue4Xi object at 0x000000000434DDA0>,
        #  'WIDTH': 800,
        #  'CONTEXT': ...
                
        return mpu2image2xi
                                                           
    #</methodize4xi> 
    
    
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "write2image-of-MPU2Image4Xi" usage = "
    #   print('(Step 127A/700) OF MPU2IMAGE4XI5TEST - TRYING TO ESTABLISH CONTROL AND COMMAND WITH WRITE IMAGE OBJECT', time4xi()) 
    #   response4write2mpu = {}
    #   try:
    #       response4write2mpu = xi2mpu.write2image()
    #       print('(Step 127B/700) OF MPU2IMAGE4XI5TEST - ESTABLISHED CONTROL AND COMMAND WITH WRITE IMAGE OBJECT, RESPONSE4WRITE2MPU: ', response4write2mpu) 
    #   except:
    #       print('(Step 127Z1/700) OF MPU2IMAGE4XI5TEST *** FAILED TO ESTABLISH CONTROL AND COMMAND WITH WRITE IMAGE OBJECT', time4xi()) 
    #
    #   ... continue as with write2image4mpu function
    #">
    
    # ---------------------------------------------------------------------------------------------   
    def write2image(self, task4write2image = None):
        '''
        writes MPU2Image4Xi objects
        ''' 
        
        # -----------------------------------------------------------------------------------------
        # (Step 1/33)           
        # -----------------------------------------------------------------------------------------              
        class2print = "MPU2Image4Xi"
        method2print = "write2image"
                     
        message2print = "{(Step 1A/33) started" + "; write2image-of-MPU2Image4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
        # 02:45:31 10/05/11 Pacific Daylight Time
            
        message2print = "{(Step 1B/33) passed" + "; task4write2image-passed-to-write2image-of-MPU2Image4Xi: " + str(task4write2image) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                          
        # None
          
        # -----------------------------------------------------------------------------------------
        # (Step 2/33)           
        # -----------------------------------------------------------------------------------------
        message2print = "{(Step 2A/33) passing" + "; task4write2image-passing-to-write2image4mpu-by-write2image-of-MPU2Image4Xi: " + str(task4write2image) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                               
        # None
  
        response4writeimage2mpu = write2image4mpu(task4write2image)

        message2print = "{(Step 2B/33) performed" + "; write2image4mpu-performed-by-write2image-of-MPU2Image4Xi: " + str(write2image4mpu) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)             
        # <function write2image4mpu at 0x000000000380AEC8>
                                
        # -----------------------------------------------------------------------------------------
        # (Step 3/33)           
        # ----------------------------------------------------------------------------------------- 
       
        message2print = "{(Step 3A/33) returning" + "; response4writeimage2mpu-returning-by-write2image-of-MPU2Image4Xi: " + str(response4writeimage2mpu) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
        # {'QUEUE': <queue4xi.Queue4Xi object at 0x0000000004D10E10>,
        #  'PUBLISHERCHANNEL': <read-write PipeConnection, handle 968>,
        #  'CONSUMERCHANNEL': <read-write PipeConnection, handle 964>}; time: 02:45:31 10/05/11 Pacific Daylight Time}

                     
                
        return response4writeimage2mpu
                                                           
    #</methodize4xi>
    
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "copy2image-of-MPU2Image4Xi" usage = "
    #   print('(Step 127A/700) OF MPU2IMAGE4XI5TEST - TRYING TO ESTABLISH CONTROL AND COMMAND WITH WRITE IMAGE OBJECT', time4xi()) 
    #   response4copy2mpu = {}
    #   try:
    #       response4copy2mpu = xi2mpu.copy2image()
    #       print('(Step 127B/700) OF MPU2IMAGE4XI5TEST - ESTABLISHED CONTROL AND COMMAND WITH WRITE IMAGE OBJECT, RESPONSE4WRITE2MPU: ', response4copy2mpu) 
    #   except:
    #       print('(Step 127Z1/700) OF MPU2IMAGE4XI5TEST *** FAILED TO ESTABLISH CONTROL AND COMMAND WITH WRITE IMAGE OBJECT', time4xi()) 
    #
    #   ... continue as with copy2image4mpu function
    #">
    
    # ---------------------------------------------------------------------------------------------   
    def copy2image(self, task4copy2image = None):
        '''
        copies MPU2Image4Xi objects
        ''' 
        
        # -----------------------------------------------------------------------------------------
        # (Step 1/33)           
        # -----------------------------------------------------------------------------------------              
        class2print = "MPU2Image4Xi"
        method2print = "copy2image"
                     
        message2print = "{(Step 1A/33) started" + "; copy2image-of-MPU2Image4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
        # 01:28:44 10/05/11 Pacific Daylight Time 
            
        message2print = "{(Step 1B/33) passed" + "; task4copy2image-passed-to-copy2image-of-MPU2Image4Xi: " + str(task4copy2image) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                          
        # None 
          
        # -----------------------------------------------------------------------------------------
        # (Step 2/33)           
        # -----------------------------------------------------------------------------------------
        message2print = "{(Step 2A/33) passing" + "; task4copy2image-passing-to-copy2image4mpu-by-copy2image-of-MPU2Image4Xi: " + str(task4copy2image) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                               
        # None 
  
        response4copyimage2mpu = copy2image4mpu(task4copy2image)

        message2print = "{(Step 2B/33) performed" + "; copy2image4mpu-performed-by-copy2image-of-MPU2Image4Xi: " + str(copy2image4mpu) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)             
        # <function copy2image4mpu at 0x000000000390B8C8>
                                
        # -----------------------------------------------------------------------------------------
        # (Step 3/33)           
        # ----------------------------------------------------------------------------------------- 
       
        message2print = "{(Step 3A/33) returning" + "; response4copyimage2mpu-returning-by-copy2image-of-MPU2Image4Xi: " + str(response4copyimage2mpu) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
        # {'QUEUE': <queue4xi.Queue4Xi object at 0x0000000004EC4128>,
        #  'PUBLISHERCHANNEL': <read-write PipeConnection, handle 1000>,
        #  'CONSUMERCHANNEL': <read-write PipeConnection, handle 996>}; time: 01:28:44 10/05/11 Pacific Daylight Time}
        
        return response4copyimage2mpu
                                                           
    #</methodize4xi>
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "copy2imagetobuffer-of-MPU2Image4Xi" usage = "
    #   print('(Step 127A/700) OF MPU2IMAGE4XI5TEST - TRYING TO ESTABLISH CONTROL AND COMMAND WITH COPY IMAGE TO BUFFER OBJECT', time4xi()) 
    #   response4copyimagetobuffer = {}
    #   try:
    #       response4copyimagetobuffer = xi2mpu.copy2imagetobuffer()
    #       print('(Step 127B/700) OF MPU2IMAGE4XI5TEST - ESTABLISHED CONTROL AND COMMAND WITH COPY IMAGE TO BUFFER  OBJECT, RESPONSE4WRITE2MPU: ', response4copyimagetobuffer) 
    #   except:
    #       print('(Step 127Z1/700) OF MPU2IMAGE4XI5TEST *** FAILED TO ESTABLISH CONTROL AND COMMAND WITH COPY IMAGE TO BUFFER ', time4xi()) 
    #
    #   ... continue as with copy2imagetobuffer4mpu function
    #">
    
    # ---------------------------------------------------------------------------------------------   
    def copy2imagetobuffer(self, task4copy2imagetobuffer = None):
        '''
        copies 2D images to buffer
        ''' 
        
        # -----------------------------------------------------------------------------------------
        # (Step 1/33)           
        # -----------------------------------------------------------------------------------------              
        class2print = "MPU2Image4Xi"
        method2print = "copy2imagetobuffer"
                     
        message2print = "{(Step 1A/33) started" + "; copy2imagetobuffer-of-MPU2Image4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
        # 01:28:44 10/05/11 Pacific Daylight Time 
            
        message2print = "{(Step 1B/33) passed" + "; task4copy2imagetobuffer-passed-to-copy2imagetobuffer-of-MPU2Image4Xi: " + str(task4copy2imagetobuffer) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                          
        # None
          
        # -----------------------------------------------------------------------------------------
        # (Step 2/33)           
        # -----------------------------------------------------------------------------------------
        message2print = "{(Step 2A/33) passing" + "; task4copy2imagetobuffer-passing-to-copy2imagetobuffer4mpu-by-copy2imagetobuffer-of-MPU2Image4Xi: " + str(task4copy2imagetobuffer) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                               
        # None
  
        response4copyimage2mpu = copy2imagetobuffer4mpu(task4copy2imagetobuffer)

        message2print = "{(Step 2B/33) performed" + "; copy2imagetobuffer4mpu-performed-by-copy2imagetobuffer-of-MPU2Image4Xi: " + str(copy2imagetobuffer4mpu) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)             
        # <function copy2imagetobuffer4mpu at 0x000000000390B948>
                                
        # -----------------------------------------------------------------------------------------
        # (Step 3/33)           
        # ----------------------------------------------------------------------------------------- 
       
        message2print = "{(Step 3A/33) returning" + "; response4copyimage2mpu-returning-by-copy2imagetobuffer-of-MPU2Image4Xi: " + str(response4copyimage2mpu) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
        # {'QUEUE': <queue4xi.Queue4Xi object at 0x0000000004FE5FD0>,
        #  'PUBLISHERCHANNEL': <read-write PipeConnection, handle 1036>,
        #  'CONSUMERCHANNEL': <read-write PipeConnection, handle 1032>}; time: 01:28:44 10/05/11 Pacific Daylight Time}
          
        return response4copyimage2mpu
                                                           
    #</methodize4xi>
    
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "copy2buffertoimage-of-MPU2Image4Xi" usage = "
    #   print('(Step 127A/700) OF MPU2IMAGE4XI5TEST - TRYING TO ESTABLISH CONTROL AND COMMAND WITH COPY BUFFER TO IMAGE OBJECT', time4xi()) 
    #   response4copybuffertoimage = {}
    #   try:
    #       response4copybuffertoimage = xi2mpu.copy2buffertoimage()
    #       print('(Step 127B/700) OF MPU2IMAGE4XI5TEST - ESTABLISHED CONTROL AND COMMAND WITH COPY BUFFER TO IMAGE  OBJECT, RESPONSE4WRITE2MPU: ', response4copybuffertoimage) 
    #   except:
    #       print('(Step 127Z1/700) OF MPU2IMAGE4XI5TEST *** FAILED TO ESTABLISH CONTROL AND COMMAND WITH COPY BUFFER TO IMAGE ', time4xi()) 
    #
    #   ... continue as with copy2buffertoimage4mpu function
    #">
    
    # ---------------------------------------------------------------------------------------------   
    def copy2buffertoimage(self, task4copy2buffertoimage = None):
        '''
        copies 2D images to buffer
        ''' 
        
        # -----------------------------------------------------------------------------------------
        # (Step 1/33)           
        # -----------------------------------------------------------------------------------------              
        class2print = "MPU2Image4Xi"
        method2print = "copy2buffertoimage"
                     
        message2print = "{(Step 1A/33) started" + "; copy2buffertoimage-of-MPU2Image4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
        # 01:28:44 10/05/11 Pacific Daylight Time 
            
        message2print = "{(Step 1B/33) passed" + "; task4copy2buffertoimage-passed-to-copy2buffertoimage-of-MPU2Image4Xi: " + str(task4copy2buffertoimage) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                          
        # None
          
        # -----------------------------------------------------------------------------------------
        # (Step 2/33)           
        # -----------------------------------------------------------------------------------------
        message2print = "{(Step 2A/33) passing" + "; task4copy2buffertoimage-passing-to-copy2buffertoimage4mpu-by-copy2buffertoimage-of-MPU2Image4Xi: " + str(task4copy2buffertoimage) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                               
        # None
  
        response4copyimage2mpu = copy2buffertoimage4mpu(task4copy2buffertoimage)

        message2print = "{(Step 2B/33) performed" + "; copy2buffertoimage4mpu-performed-by-copy2buffertoimage-of-MPU2Image4Xi: " + str(copy2buffertoimage4mpu) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)             
        # <function copy2buffertoimage4mpu at 0x000000000390B9C8> 
                                
        # -----------------------------------------------------------------------------------------
        # (Step 3/33)           
        # ----------------------------------------------------------------------------------------- 
       
        message2print = "{(Step 3A/33) returning" + "; response4copyimage2mpu-returning-by-copy2buffertoimage-of-MPU2Image4Xi: " + str(response4copyimage2mpu) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
        # {'QUEUE': <queue4xi.Queue4Xi object at 0x000000000505A6D8>,
        #  'PUBLISHERCHANNEL': <read-write PipeConnection, handle 1068>,
        #  'CONSUMERCHANNEL': <read-write PipeConnection, handle 1064>}; time: 01:28:44 10/05/11 Pacific Daylight Time}

                
        return response4copyimage2mpu
                                                           
    #</methodize4xi>    
                    
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "trace2image-of-MPU2Image4Xi" usage = "
    #   print('=' *33)          
    #   print('(Step 15A/25) OF MPU2IMAGE4XI5TEST - TRYING TO TRACE MPU: ', time4xi())      
    #   print('=' *33)      
    #   trace2mpu = None
    #   try:
    #       trace2mpu = trace2image4mpu() 
    #       print('(Step 15B/25) OF MPU2IMAGE4XI5TEST - TRACE2MPU: ', trace2mpu)  
    #       # {'MESSAGE': '{(Step 2Z0/33) failed; trace2image4mpu-failed {}; time: 01:39:15 10/06/11 Pacific Daylight Time}', 'CLASS': 'trace2image4mpu', 'METHOD': 'trace2image4mpu'}
    #    except:
    #       print('(Step 15Z1/25) OF MPU2IMAGE4XI5TEST *** FAILED TO TRACE MPU: ', trace2mpu)     
    #">
    
    # ---------------------------------------------------------------------------------------------   
    def trace2image(self, task4trace2image = None):
        '''
        traces MPU2Image4Xi objects
        ''' 
        
        # -----------------------------------------------------------------------------------------
        # (Step 1/33)           
        # -----------------------------------------------------------------------------------------              
        class2print = "MPU2Image4Xi"
        method2print = "trace2image"
                     
        message2print = "{(Step 1A/33) started" + "; trace2image-of-MPU2Image4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
        # 00:14:05 09/07/11 Pacific Daylight Time 
            
        message2print = "{(Step 1B/33) passed" + "; task4trace2image-passed-to-trace-of-MPU2Image4Xi: " + str(task4trace2image) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                          
        # None   
          
        # -----------------------------------------------------------------------------------------
        # (Step 2/33)           
        # -----------------------------------------------------------------------------------------
        message2print = "{(Step 2A/33) passing" + "; task4trace2image-passing-to-trace2image4mpu-by-trace2image-of-MPU2Image4Xi: " + str(task4trace2image) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                               
        # None  
  
        trace2xi = trace2image4mpu(task4trace2image)

        message2print = "{(Step 2B/33) performed" + "; trace2image4mpu-performed-by-trace2image-of-MPU2Image4Xi: " + str(trace2image4mpu) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)             
        # <function trace2image4mpu at 0x00000000039DF348>
                                
        # -----------------------------------------------------------------------------------------
        # (Step 3/33)           
        # ----------------------------------------------------------------------------------------- 
       
        message2print = "{(Step 3A/33) returning" + "; trace2xi-returning-by-trace2image-of-MPU2Image4Xi: " + str(trace2xi) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
        # {'MESSAGE': '{(Step 2Z0/33) failed; trace2image4mpu-failed {}; time: 01:05:20 09/08/11 Pacific Daylight Time}',
        #  'CLASS': 'trace2image4mpu',
        #  'METHOD': 'trace2image4mpu'}; time: 01:05:20 09/08/11 Pacific Daylight Time}
                 
                
        return trace2xi
                                                           
    #</methodize4xi>       


    
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "set2imagecallback-of-MPU2Image4Xi" usage = "
    #   print('(Step 1A/33) OF MPU2IMAGE4XI - TRYING TO SET MPU IMAGE')
    #   try:
    #       cc2xi = MPU2Image4Xi()                            
    #       image2mpu = cc2xi.set2imagecallback()                                  
    #       print('(Step 1B/33) OF MPU4XI5TEST - SET MPU IMAGE: ', image2mpu)                                                  
    #   except:
    #       print('(Step 1Z/33) OF MPU4XI5TEST *** FAILED TO SET MPU IMAGE ***')  
    #">
    
    # ---------------------------------------------------------------------------------------------   
    def set2imagecallback(self, task4set2imagecallback = None): 
        '''
        set2imagecallback of MPU2Image4Xi objects
        ''' 
        
        # -----------------------------------------------------------------------------------------
        # (Step 1/33)           
        # -----------------------------------------------------------------------------------------              
        class2print = "MPU2Image4Xi"
        method2print = "set2imagecallback"
                     
        message2print = "{(Step 1A/33) started" + "; set2imagecallback-of-MPU2Image4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
        # 00:14:05 09/07/11 Pacific Daylight Time
            
        message2print = "{(Step 1B/33) passed" + "; task4set2imagecallback-passed-to-set2imagecallback-of-MPU2Image4Xi: " + str(task4set2imagecallback) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                          
        # None    
          
        # -----------------------------------------------------------------------------------------
        # (Step 2/33)           
        # -----------------------------------------------------------------------------------------
        message2print = "{(Step 2A/33) passing" + "; task4set2imagecallback-passing-to-set2imagecallback -by-set2imagecallback-of-MPU2Image4Xi: " + str(task4set2imagecallback) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                               
        # None
        
        callback2mpu4image = {} 
        try:
            callback2mpu4image = set2imagecallback4mpu (task4set2imagecallback)

            message2print = "{(Step 2B/33) callback2mpu4image" + "; callback2mpu4image-set-by-set2imagecallback-of-MPU2Image4Xi to: " + str(callback2mpu4image) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            # <function trace2image4mpu at 0x00000000039DF348>
        except:
            message2print = "{(Step 4A/33) failed" + "; callback2mpu4image-failed-by-set2imagecallback-of-MPU2Image4Xi: " + str(callback2mpu4image) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
                                            
        # -----------------------------------------------------------------------------------------
        # (Step 3/33)           
        # ----------------------------------------------------------------------------------------- 
       
        message2print = "{(Step 3A/33) returning" + "; callback2mpu4image-returning-by-set2imagecallback-of-MPU2Image4Xi: " + str(callback2mpu4image) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
        # <function trace2image4mpu at 0x0000000003AB31C8> 
                                             
        return callback2mpu4image
                                                           
    #</methodize4xi>     
    
                                                              
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "sense-of-MPU2Image4Xi" usage = "
    #   try:
    #       print(xi2mpu.sense())
    #       print('(Step 24B/33) OF MPU2IMAGE4XI5TEST - SENSED ', time4xi()) 
    #   except:
    #       print('(Step 24Z/33) OF MPU2IMAGE4XI5TEST *** FAILED TO SENSE ', time4xi())  
    # ">
    
    # ---------------------------------------------------------------------------------------------
    def sense(self):
        '''
        method sense: senses MPU2Image4Xi class objects
        '''
        
        # -----------------------------------------------------------------------------------------
        #
        # ----------------------------------------------------------------------------------------- 
        class2print = 'MPU2Image4Xi'
        method2print = 'sense'
        
                     
        message2print = "{(Step 1A/33) started" + "; sense-of-MPU2Image4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
        # 01:02:22 09/07/11 Pacific Daylight Time 
        
        # -----------------------------------------------------------------------------------------
        #
        # -----------------------------------------------------------------------------------------  
        
        message2print = "{(Step 2A/33) sensing" + "; MPU2Image4Xi-sensing-by-sense-of-MPU2Image4Xi: " + str(MPU2Image4Xi)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)         
        # <class 'mpu2image4xi.MPU2Image4Xi'>         
        
        # -----------------------------------------------------------------------------------------
        # get sensed knowledge for this object
        # -----------------------------------------------------------------------------------------        
        
        kb2xi = sense_object4xi(self)
        
        message2print = "{(Step 3A/33) got" + "; kb2xi-got-by-sense-of-MPU2Image4Xi: " + str(kb2xi)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # {'object': <mpu2image4xi.MPU2Image4Xi object at 0x0000000004188400>}   
        
        # -----------------------------------------------------------------------------------------
        # add knowledge specific to this object
        # format example: kb2xi['time2xi'] = time4xi() #adding time 
        # -----------------------------------------------------------------------------------------         
        try:
            kb2xi['classvariable2mpu-of-MPU2Image4Xi'] = MPU2Image4Xi.classvariable2mpu
        except:
            message2print = "{(Step 4A/33) yielded" + "; kb2xi-yielded-by-sense-of-MPU2Image4Xi: " + str(kb2xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            
      
        message2print = "{(Step 5A/33) sensed" + "; MPU2Image4Xi-sensed-by-sense-of-MPU2Image4Xi: " + str(MPU2Image4Xi)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)    
        # <class 'mpu2image4xi.MPU2Image4Xi'>
        
        # -----------------------------------------------------------------------------------------
        # 
        # -----------------------------------------------------------------------------------------         
        
        message2print = "{(Step 6A/33) returning" + "; kb2xi-returning-by-sense-of-MPU2Image4Xi: " + str(kb2xi) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print)                                                          
        # {'object': <mpu2image4xi.MPU2Image4Xi object at 0x0000000004188400>, 'classvariable2mpu-of-MPU2Image4Xi': 4}             
          
        return kb2xi
        
        # -----------------------------------------------------------------------------------------
        #
        # -----------------------------------------------------------------------------------------        
    #</methodize4xi>
    
        
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "reason-of-MPU2Image4Xi" usage = "
    #   print('(Step 25A/33) OF MPU2IMAGE4XI5TEST - REASONING ', time4xi())
    #   try:
    #       print(xi2mpu.reason())
    #       print('(Step 25B/33) OF MPU2IMAGE4XI5TEST - REASONED ', time4xi())
    #   except:
    #       print('(Step 25Z/33) OF MPU2IMAGE4XI5TEST *** FAILED TO REASON ', time4xi())   
    #">
    
    # ---------------------------------------------------------------------------------------------
    def reason(self):
        '''
        method reason: reasons MPU2Image4Xi class objects
        '''
        
        # -----------------------------------------------------------------------------------------
        #
        # ----------------------------------------------------------------------------------------- 
        class2print = 'MPU2Image4Xi'
        method2print = 'reason'

                     
        message2print = "{(Step 1A/33) started" + "; reason-of-MPU2Image4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
        # 01:02:22 09/07/11 Pacific Daylight Time 
        
        # -----------------------------------------------------------------------------------------
        #
        # -----------------------------------------------------------------------------------------
        
        message2print = "{(Step 2A/33) reasoning" + "; MPU2Image4Xi-reasoning-by-reason-of-MPU2Image4Xi: " + str(MPU2Image4Xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        # <class 'mpu2image4xi.MPU2Image4Xi'>  
        
        # -----------------------------------------------------------------------------------------
        # get reasoned knowledge for this object
        # -----------------------------------------------------------------------------------------         
        
        kb4xi = reason_object4xi(self)
        
        
        message2print = "{(Step 3A/33) got" + "; kb4xi-got-by-reason-of-MPU2Image4Xi: " + str(kb4xi)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # {'object-reason-object4xi': <mpu2image4xi.MPU2Image4Xi object at 0x0000000004188400>}      
        
        # -----------------------------------------------------------------------------------------
        # add knowledge specific to this object
        # format example: kb4xi['time2xi'] = time4xi() #adding time 
        # -----------------------------------------------------------------------------------------          
        try:
            kb4xi['time-reason-of-MPU2Image4Xi'] = time4xi() #adding time
        except:
            message2print = "{(Step 4Z/33) yielded" + "; kb4xi-yielded-by-reason-of-MPU2Image4Xi: " + str(kb4xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
        
          
        
        message2print = "{(Step 5A/33) reasoned" + "; MPU2Image4Xi-reasoned-by-reason-of-MPU2Image4Xi: " + str(MPU2Image4Xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)        
        # <class 'mpu2image4xi.MPU2Image4Xi'>
        
        # -----------------------------------------------------------------------------------------
        #
        # -----------------------------------------------------------------------------------------        
        
        message2print = "{(Step 6A/33) returning" + "; kb4xi-returning-by-reason-of-MPU2Image4Xi: " + str(kb4xi) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print)                                                          
        # {'object-reason-object4xi': <mpu2image4xi.MPU2Image4Xi object at 0x0000000004188400>, 'time-reason-of-MPU2Image4Xi': '01:02:22 09/07/11 Pacific Daylight Time'}                
          
        return kb4xi
        
        # -----------------------------------------------------------------------------------------
        #
        # -----------------------------------------------------------------------------------------            
    #</methodize4xi>
    
    
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "ace-of-MPU2Image4Xi" usage = "
    #   print('(Step 26A/33) OF MPU2IMAGE4XI5TEST - ACING ', time4xi())   
    #   try:
    #       print(xi2mpu.ace())
    #       print('(Step 26B/33) OF MPU2IMAGE4XI5TEST - ACED ', time4xi())                                                 
    #   except:
    #       print('(Step 26Z/33) OF MPU2IMAGE4XI5TEST *** FAILED TO ACE ', time4xi())                                       
    #    
    #">
    
    # ---------------------------------------------------------------------------------------------
    def ace(self):
        '''
        method ace: aces MPU2Image4Xi class objects 
        '''
        
        # -----------------------------------------------------------------------------------------
        #
        # -----------------------------------------------------------------------------------------       
        
        
        class2print = 'MPU2Image4Xi'
        method2print = 'ace'
        
                     
        message2print = "{(Step 1A/33) started" + "; ace-of-MPU2Image4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
        # 01:02:22 09/07/11 Pacific Daylight Time   
        
        # -----------------------------------------------------------------------------------------
        #
        # -----------------------------------------------------------------------------------------
        
        message2print = "{(Step 2A/33) acing" + "; MPU2Image4Xi-acing-by-ace-of-MPU2Image4Xi: " + str(MPU2Image4Xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)         
        # <class 'mpu2image4xi.MPU2Image4Xi'>           
        
        # -----------------------------------------------------------------------------------------
        # get aced knowledge for this object
        # -----------------------------------------------------------------------------------------        
        
        kb8xi = ace_object4xi(self)  
        
        message2print = "{(Step 3A/33) got" + "; kb8xi-got-by-ace-of-MPU2Image4Xi: " + str(kb8xi)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # {'object8xi': <mpu2image4xi.MPU2Image4Xi object at 0x0000000004188400>}   


        # -----------------------------------------------------------------------------------------
        # add knowledge specific to this object
        # format example: kb8xi['time2xi'] = time4xi() #adding time 
        # -----------------------------------------------------------------------------------------              
        try:
             kb8xi['cores-of-MPU2Image4Xi'] = MPU2Image4Xi.classvariable2mpu
        except:
             message2print = "{(Step 4A/33) yielded" + "; kb8xi-yielded-by-ace-of-MPU2Image4Xi: " + str(kb8xi)  + "; time: " + time4xi() + "}"
             print4xi(class2print, method2print, message2print) 
  
        message2print = "{(Step 5A/33) aced" + "; MPU2Image4Xi-aced-by-ace-of-MPU2Image4Xi: " + str(MPU2Image4Xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)         
        # <class 'mpu2image4xi.MPU2Image4Xi'>  
        
        # -----------------------------------------------------------------------------------------
        #
        # -----------------------------------------------------------------------------------------        
        message2print = "{(Step 6A/33) returning" + "; kb8xi-returning-by-ace-of-MPU2Image4Xi: " + str(kb8xi) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print)                                                          
        # {'cores-of-MPU2Image4Xi': 4, 'object8xi': <mpu2image4xi.MPU2Image4Xi object at 0x0000000004188400>}
        
        return kb8xi
        
        # -----------------------------------------------------------------------------------------
        #
        # -----------------------------------------------------------------------------------------            
    #</methodize4xi>
#</class4xi>
#</program4xi>

# -------------------------------------------------------------------------------------------------
# Recorded History                             
# ------------------------------------------------------------------------------------------------- 
#<record4xi record = "
#   mpu2image4xi5test.txt
#   sense-mpu2image4xi5test.xml
#   reason-mpu2image4xi5test.xml
#   ace-mpu2image4xi5test.xml
#"
#>

#<acknowledge4xi acknowledgement = "33" tester = "Small & Smart"/>

# -------------------------------------------------------------------------------------------------
# to-be-done                              
# -------------------------------------------------------------------------------------------------    
#<do4xi do = "
# nothing
#"
#/>

#<note4xi note = "
# parse4xi.py - to parse this module
# "/>

# -------------------------------------------------------------------------------------------------
# Legal Notice                             
# ------------------------------------------------------------------------------------------------- 
#<protect4xi protection = "copyright"
#   owner = "Small & Smart"
#   rights = "All rights reserved"
#   state = "California"
#   country = "United States of America"
#   codes = "Codes established in California and applicable to corporations in California"
#   compliance = "The California Law"
#   court = "Supreme Court of California, 350 McAllister Street, San Francisco, CA 94102-4783"
#/>

#<end4xi end = "
# -------------------------------------------------------------------------------------------------
# end of mpu2image4xi.py                              
# ------------------------------------------------------------------------------------------------- 
#"
#/>
